package de.bsvrz.buv.plugin.bmvew;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IBenutzerZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Betriebsmeldungsverwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Das Verbindungsobjekt
   */
  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;

  /**
   * Liste der {@link IBenutzerZuhoerer}, die ber einen neue angemeldeten Benuter informiert werden mchten.
   */
  private LinkedList<IBetriebsmeldungListener> _betriebsMeldungsListener = new LinkedList<IBetriebsmeldungListener>();

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
//    if (_instanz == null) { 
//      synchronized(_syncObject) { 
//          if (_instanz == null) { 
//              _instanz = new DaVKommunikation();
//          }
//      }
//    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {
    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {
      // Anmelden, wenn die Verbindung vorhanden ist.
      // Ansonsten wird dies dem Listener berlassen.
      SystemObjectType type = _connection.getDataModel().getType("typ.informationsKanal");
      List<SystemObject> list = type.getElements();
      if (list.size() == 0)
        _debug.config("Liste der Informationskanaele ist null!");
      for (SystemObject so : list)
      {
        _debug.fine("Anmeldung auf Informationskanal :" + so.getPid());
      }

      anmeldeLeseDaV(list, "atg.betriebsMeldung", "asp.information", ReceiverRole.receiver());
    }
    else
      _debug.error("Plugin Betriebsmeldungsverwaltung : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Fgt einen Listener zur Liste der Betriebsmeldungslistener hinzu.
   * 
   * @param listener
   */
  public void hinzufuegenBetriebsmeldungsListener(IBetriebsmeldungListener listener)
  {
    if (!_betriebsMeldungsListener.contains(listener))
    {
      _betriebsMeldungsListener.add(listener);
    }
    else
    {
      _debug
          .error("Der BetriebsmeldungsZuhrer ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt.");
    }
  }

  /**
   * Entfernt einen {@link IBenutzerZuhoerer} von der Liste der Zuhrer.
   * 
   * @param benutzerZuhoerer
   *          Zu entfernender {@link IBenutzerZuhoerer}.
   */
  public void entferneBetriebsmeldungsListener(IBetriebsmeldungListener listener)
  {
    if (_betriebsMeldungsListener.contains(listener))
    {
      _betriebsMeldungsListener.remove(listener);
    }
    else
    {
      _debug
          .error("Der BetriebsmeldungsZuhrer ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }

  /**
   * Benachrichtigt alle registrierten {@link IBenutzerZuhoerer} darber, dass ein neuer Benutzer sich angemeldet hat.
   * 
   */
  public void benachrichtigeBetriebsmeldungsListener(List<Meldung> meldungen)
  {
    BetriebsmeldungEreignis ereignis = new BetriebsmeldungEreignis(this, meldungen);

    Iterator<IBetriebsmeldungListener> it = _betriebsMeldungsListener.iterator();
    while (it.hasNext())
    {
      IBetriebsmeldungListener listener = it.next();
      listener.betriebsmeldungEingang(ereignis);
    }
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
    final List<Meldung> meldungen = new ArrayList<Meldung>();

    for (int i = 0; i < results.length; i++)
    {

      final ResultData result = results[i];
      final Data data = result.getData();

      if (data != null)
      {

        Meldung meldung = new Meldung();

        meldung.setPid(result.getObject().getPid());

        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
        Date d = new Date();
        d.setTime(result.getDataTime());
        String s = sdf.format(d);
        meldung.setZeitStempel(s);

        meldung.setApplikationsID(data.getItem("ApplikationsID").asTextValue().getText());
        meldung.setLaufendeNummer(data.getItem("LaufendeNummer").asTextValue().getText());
        meldung.setApplikationsTyp(data.getItem("ApplikationsTyp").asTextValue().getText());
        meldung.setApplikationsKennung(data.getItem("ApplikationsKennung").asTextValue().getText());
        meldung.setID(data.getItem("ID").asTextValue().getText());
        meldung.setMeldungsTyp(data.getItem("MeldungsTyp").asTextValue().getText());
        meldung.setMeldungsTypZusatz(data.getItem("MeldungsTypZusatz").asTextValue().getText());
        meldung.setMeldungsKlasse(data.getItem("MeldungsKlasse").asTextValue().getText());

        if (data.getItem("Referenz").asArray().getLength() == 1)
        {
          meldung.setReferenz(data.getItem("Referenz").asArray().getItem(0).asReferenceValue().getText());
        }
        else
        {
          meldung.setReferenz("");
        }

        meldung.setStatus(data.getItem("Status").asTextValue().getText());
        meldung.setUrlasserBenutzerReferenz(data.getItem("Urlasser").getItem("BenutzerReferenz").asTextValue()
            .getText());
        meldung.setUrlasserUrsache(data.getItem("Urlasser").getItem("Ursache").asTextValue().getText());
        meldung.setUrlasserVeranlasser(data.getItem("Urlasser").getItem("Veranlasser").asTextValue().getText());
        meldung.setMeldungsText(data.getItem("MeldungsText").asTextValue().getText());

        meldungen.add(meldung);

      }
    }

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        benachrichtigeBetriebsmeldungsListener(meldungen);
      }
    });

  }

}
