package de.bsvrz.buv.plugin.bmvew;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * "Editor" fr regulre Ausdrcke. Liefert ein "groes" Textfeld, in dem der
 * regulre Ausdruck bearbeitet werden kann. <br>
 * 
 * Die Darstellung besteht aus:
 * <ul>
 * <li>Einem Textfeld, in dem der regulre Ausdruck bearbeitet werden kann.</li>
 * <li>Einem OK-Button, der die nderungen am regulren Ausdruck bernimmt.</li>
 * <li>Einem Abbrechen-Button, der die nderungen am regulren Ausruck verwirft.
 * </li>
 * </ul>
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:06:11 $ / ($Author: Pittner
 *          $)
 * @author HofmannJ
 * 
 */
public class EditorRegulaererAusdruck extends Dialog
{
	/**
	 * Speichert den regulren Ausdruck.
	 */
	private FilterElement _filterElement = null;

	private String _regAusdruck = null;

	private Shell shell = null; // @jve:decl-index=0:visual-constraint="204,120"

	private Composite compositeTop = null;

	private Composite compositeBottom = null;

	private Button buttonOk = null;

	private Button buttonAbbrechen = null;

	private Group group = null;

	private Text textArea = null;

	private boolean _modified = false;

	/**
	 * Konstruktor, liefert eine neue Instanz der Klasse
	 * {@link EditorRegulaererAusdruck}.
	 * 
	 * @param parent
	 *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
	 */
	public EditorRegulaererAusdruck(Shell parent)
	{
		super(parent);
	}

	/**
	 * ffnet den Dialog {@link EditorRegulaererAusdruck} zum Editieren/Anlegen
	 * eines neuen regulren Ausdrucks.
	 * 
	 * @return Regulrer Ausdruck als String.
	 */
	public String oeffne()
	{
		// shell = new Shell();
		shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN
				| SWT.MAX | SWT.CLOSE);
		shell.setText("Editor regulrer Ausdruck");

		shell.setLayout(new GridLayout());
		createCompositeTop();
		createCompositeBottom();
		shell.setSize(new Point(436, 259));
		shell.open();

		Display display = getParent().getDisplay();

		while (!shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return _regAusdruck;

	}

	/**
	 * ffnet den Dialog {@link EditorRegulaererAusdruck} zum Editieren eines
	 * bereits vorhandenen regulren Ausdrucks.
	 * 
	 * @param regAusdruck
	 *          Regulrer Ausdruck, der bearbeitet werden soll.
	 * 
	 * @return Liefert den bearbeiteten, regulren Ausduck. Der Dialog
	 *         {@link FilterDefinitionsdialog} kann die Darstellung des regulren
	 *         Ausdrucks des jeweiligen Filterelements hiermit aktuallisieren.
	 * 
	 */
	public String oeffne(String regAusdruck)
	{

		_regAusdruck = regAusdruck;

		// shell = new Shell();
		shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN
				| SWT.MAX | SWT.CLOSE);
		shell.setText("Editor regulrer Ausdruck");

		shell.setLayout(new GridLayout());
		createCompositeTop();
		createCompositeBottom();
		shell.setSize(new Point(436, 259));
		shell.open();
		textArea.setText(_regAusdruck);

		Display display = getParent().getDisplay();

		while (!shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return _regAusdruck;
	}

	/**
	 * Generiert die Elemente des Dialogs {@link EditorRegulaererAusdruck}.
	 * 
	 */
	private void generiereGui()
	{
	}

	/**
	 * bernimmt die nderungen am regulren Ausdruck und schliet den Dialog.
	 * 
	 */
	private void aktionOk()
	{
		if (_modified)
		{
			_regAusdruck = textArea.getText();
			_modified = false;
		}
		shell.dispose();
	}

	/**
	 * Verwirft die nderungen am regulren Ausdruck und schliet den Dialog.
	 * 
	 */
	private void aktionAbbrechen()
	{
		// _regAusdruck = null;
		shell.dispose();
	}

	/**
	 * This method initializes compositeTop
	 * 
	 */
	private void createCompositeTop()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeTop = new Composite(shell, SWT.NONE);
		compositeTop.setLayout(new GridLayout());
		createGroup();
		compositeTop.setLayoutData(gridData);
	}

	/**
	 * This method initializes compositeBottom
	 * 
	 */
	private void createCompositeBottom()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.wrap = true;
		rowLayout.pack = false;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.CENTER;
		gridData1.grabExcessHorizontalSpace = false;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeBottom = new Composite(shell, SWT.NONE);
		compositeBottom.setLayoutData(gridData1);
		compositeBottom.setLayout(rowLayout);
		buttonOk = new Button(compositeBottom, SWT.NONE);
		buttonOk.setText("Ok");		
		buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});

		buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}			
		});
	}

	/**
	 * This method initializes group
	 * 
	 */
	private void createGroup()
	{
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		group = new Group(compositeTop, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setText("Regulrer Ausdruck");
		group.setLayoutData(gridData2);
		textArea = new Text(group, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		textArea.setLayoutData(gridData3);
		textArea.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				_modified = true;

			}
		});

	}

}
