package de.bsvrz.buv.plugin.bmvew;

import java.util.LinkedList;
import java.util.List;

/**
 * Diese Klasse reprsentiert die Datenstruktur eines Filters von Betriebsmeldungen. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @version $Revision: 1.1 $ / $Date: 2010/02/02 15:14:30 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class Filter implements Cloneable
{
  /**
   * Name des Filters.
   */
  private String _name = null;

  /*
   * Elemente des Filters, der Default RegEx ist "*".
   */
  private FilterElement _feApplikationsID = new FilterElement(Feldbezeichner.ApplikationsID, ".*");

  private FilterElement _feLaufendeNummer = new FilterElement(Feldbezeichner.LaufendeNummer, ".*");

  private FilterElement _feApplikationsTyp = new FilterElement(Feldbezeichner.ApplikationsTyp, ".*");

  private FilterElement _feApplikationsKennung = new FilterElement(Feldbezeichner.ApplikationsKennung, ".*");

  private FilterElement _feID = new FilterElement(Feldbezeichner.ID, ".*");

  private FilterElement _feMeldungsTyp = new FilterElement(Feldbezeichner.MeldungsTyp, ".*");

  private FilterElement _feMeldungsTypZusatz = new FilterElement(Feldbezeichner.MeldungsTypZusatz, ".*");

  private FilterElement _feMeldungsKlasse = new FilterElement(Feldbezeichner.MeldungsKlasse, ".*");

  private FilterElement _feReferenz = new FilterElement(Feldbezeichner.Referenz, ".*");

  private FilterElement _feStatus = new FilterElement(Feldbezeichner.Status, ".*");

  private FilterElement _feUrlasserBenutzerReferenz = new FilterElement(Feldbezeichner.UrlasserBenutzerReferenz, ".*");

  private FilterElement _feUrlasserUrsache = new FilterElement(Feldbezeichner.UrlasserUrsache, ".*");

  private FilterElement _feUrlasserVeranlasser = new FilterElement(Feldbezeichner.UrlasserVeranlasser, ".*");

  private FilterElement _feMeldungsText = new FilterElement(Feldbezeichner.MeldungsText, ".*");

  /**
   * Liste aller Filter
   */
  private List<FilterElement> _listeFilter = new LinkedList<FilterElement>();

  /**
   * ffentlicher, parameterloser Konstruktor.
   * 
   */
  public Filter()
  {
    _name = "";
    init();
  }

  /**
   * Konstruktor, setzt den Namen des Filters.
   * 
   * @param name
   */
  public Filter(String name)
  {
    _name = name;
    init();
  }

  /**
   * Fhrt die initialisierung durch.
   * 
   */
  private void init()
  {
    /*
     * Initialisierung der Spalten-Filter, die in die Liste der Filter kommen.
     */
    _listeFilter.add(_feApplikationsID);
    _listeFilter.add(_feLaufendeNummer);
    _listeFilter.add(_feApplikationsTyp);
    _listeFilter.add(_feApplikationsKennung);
    _listeFilter.add(_feID);
    _listeFilter.add(_feMeldungsTyp);
    _listeFilter.add(_feMeldungsTypZusatz);
    _listeFilter.add(_feMeldungsKlasse);
    _listeFilter.add(_feReferenz);
    _listeFilter.add(_feStatus);
    _listeFilter.add(_feUrlasserBenutzerReferenz);
    _listeFilter.add(_feUrlasserUrsache);
    _listeFilter.add(_feUrlasserVeranlasser);
    _listeFilter.add(_feMeldungsText);
  }

  /**
   * Setzt den Namen des Filters.
   * 
   * @param _name
   *          Name des Filters.
   */
  public void setName(String _name)
  {
    this._name = _name;
  }

  /**
   * Liefert den Namen dieses Filters.
   * 
   * @return Namen des Filters.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Liefert die Liste aller Spalten-Filter.
   * 
   * @return Liste aller Spalten-Filter.
   */
  public List<FilterElement> getFilterElemente()
  {
    return this._listeFilter;
  }

  /*
   * Hier folgen nun die SETTER der einzelnen Filter. Es ist nicht notwendig, GETTER fr diese Filter zu definieren,
   * denn ber die Methode <code>getFilter</code> wird die Liste aller Filter geliefert.
   */

  /**
   * Setzt das Filterelement fr die Spalte ApplikationsId.
   * 
   * @param feApplikationsID
   *          ApplikationsId des Filters.
   */
  public void setApplikationsID(FilterElement feApplikationsID)
  {
    this._feApplikationsID = feApplikationsID;
  }

  /**
   * Setzt das Filterelement fr die Spalte ApplikationsKennung.
   * 
   * @param feApplikationsKennung
   *          Applikationskennung des Filters.
   */
  public void setApplikationsKennung(FilterElement feApplikationsKennung)
  {
    this._feApplikationsKennung = feApplikationsKennung;
  }

  /**
   * Setzt das Filterelement fr die Spalte ApplikationsTyp.
   * 
   * @param feApplikationsTyp
   *          ApplikationsTyp des Filters.
   */
  public void setApplikationsTyp(FilterElement feApplikationsTyp)
  {
    this._feApplikationsTyp = feApplikationsTyp;
  }

  /**
   * Setzt das Filterelement fr die Spalte Id.
   * 
   * @param feID
   *          Id des Filters.
   */
  public void setID(FilterElement feID)
  {
    this._feID = feID;
  }

  /**
   * Setzt das Filterelement fr die Spalte LaufendeNummer.
   * 
   * @param feLaufendeNummer
   *          Laufende Nummer des Filters.
   */
  public void setLaufendeNummer(FilterElement feLaufendeNummer)
  {
    this._feLaufendeNummer = feLaufendeNummer;
  }

  /**
   * Setzt das Filterelement fr die Spalte Meldungsklasse.
   * 
   * @param feMeldungsKlasse
   *          Meldungsklasse des Filters.
   */
  public void setMeldungsKlasse(FilterElement feMeldungsKlasse)
  {
    this._feMeldungsKlasse = feMeldungsKlasse;
  }

  /**
   * Setzt das Filterelement fr die Spalte.
   * 
   * @param feMeldungsText
   *          Melngstext des Filters.
   */
  public void setMeldungsText(FilterElement feMeldungsText)
  {
    this._feMeldungsText = feMeldungsText;
  }

  /**
   * Setzt das Filterelement fr die Spalte MeldungsTyp.
   * 
   * @param feMeldungsTyp
   *          MeldungsTyp des Filters.
   */
  public void setMeldungsTyp(FilterElement feMeldungsTyp)
  {
    this._feMeldungsTyp = feMeldungsTyp;
  }

  /**
   * Setzt das Filterelement fr die Spalte MeldungsTypZusatz.
   * 
   * @param feMeldungsTypZusatz
   *          MeldungsTypZusatz des Filters.
   */
  public void setMeldungsTypZusatz(FilterElement feMeldungsTypZusatz)
  {
    this._feMeldungsTypZusatz = feMeldungsTypZusatz;
  }

  /**
   * Setzt das Filterelement fr die Spalte Referenz.
   * 
   * @param feReferenz
   *          Referenz des Filters.
   */
  public void setReferenz(FilterElement feReferenz)
  {
    this._feReferenz = feReferenz;
  }

  /**
   * Setzt das Filterelement fr die Spalte Status.
   * 
   * @param feStatus
   *          Status des Filters.
   */
  public void setStatus(FilterElement feStatus)
  {
    this._feStatus = feStatus;
  }

  /**
   * Setzt das Filterelement fr die Spalte UrlasserBenutzerReferenz.
   * 
   * @param feUrlasserBenutzerReferenz
   *          UrlasserBenutzerReferenz des Filters.
   */
  public void setUrlasserBenutzerReferenz(FilterElement feUrlasserBenutzerReferenz)
  {
    this._feUrlasserBenutzerReferenz = feUrlasserBenutzerReferenz;
  }

  /**
   * Setzt das Filterelement fr die Spalte UrlasserUrsache.
   * 
   * @param feUrlasserUrsache
   *          UrlasserUrsache des Filters.
   */
  public void setUrlasserUrsache(FilterElement feUrlasserUrsache)
  {
    this._feUrlasserUrsache = feUrlasserUrsache;
  }

  /**
   * Setzt das Filterelement fr die Spalte UrlasserVeranlasser.
   * 
   * @param feUrlasserVeranlasser
   *          UrlasserVeranlasser des Filters.
   */
  public void setUrlasserVeranlasser(FilterElement feUrlasserVeranlasser)
  {
    this._feUrlasserVeranlasser = feUrlasserVeranlasser;
  }

  /**
   * Liefert einen Clone dieser Klasse und setzt den Prfix "Kopie von " vor den Namen.
   * 
   * @return Clone dieser Klasse.
   */
  public Filter clone()
  {
    String name = "Kopie von " + _name;

    // Kopieren des Namens
    Filter betriebsMvFilter = new Filter(name);

    // Kopieren der Elemente
    betriebsMvFilter.setApplikationsID(_feApplikationsID.clone());
    betriebsMvFilter.setApplikationsKennung(_feApplikationsKennung.clone());
    betriebsMvFilter.setApplikationsTyp(_feApplikationsTyp.clone());
    betriebsMvFilter.setID(_feID.clone());
    betriebsMvFilter.setLaufendeNummer(_feLaufendeNummer.clone());
    betriebsMvFilter.setMeldungsKlasse(_feMeldungsKlasse.clone());
    betriebsMvFilter.setMeldungsText(_feMeldungsText.clone());
    betriebsMvFilter.setMeldungsTyp(_feMeldungsTyp.clone());
    betriebsMvFilter.setMeldungsTypZusatz(_feMeldungsTypZusatz.clone());
    betriebsMvFilter.setReferenz(_feReferenz.clone());
    betriebsMvFilter.setStatus(_feStatus.clone());
    betriebsMvFilter.setUrlasserBenutzerReferenz(_feUrlasserBenutzerReferenz.clone());
    betriebsMvFilter.setUrlasserUrsache(_feUrlasserUrsache.clone());
    betriebsMvFilter.setUrlasserVeranlasser(_feUrlasserVeranlasser.clone());

    return betriebsMvFilter;
  }

  public FilterElement getApplikationsID()
  {
    return _feApplikationsID;
  }

  public FilterElement getLaufendeNummer()
  {
    return _feLaufendeNummer;
  }

  public FilterElement getApplikationsTyp()
  {
    return _feApplikationsTyp;
  }

  public FilterElement getApplikationsKennung()
  {
    return _feApplikationsKennung;
  }

  public FilterElement getID()
  {
    return _feID;
  }

  public FilterElement getMeldungsTyp()
  {
    return _feMeldungsTyp;
  }

  public FilterElement getMeldungsTypZusatz()
  {
    return _feMeldungsTypZusatz;
  }

  public FilterElement getMeldungsKlasse()
  {
    return _feMeldungsKlasse;
  }

  public FilterElement getReferenz()
  {
    return _feReferenz;
  }

  public FilterElement getStatus()
  {
    return _feStatus;
  }

  public FilterElement getUrlasserBenutzerReferenz()
  {
    return _feUrlasserBenutzerReferenz;
  }

  public FilterElement getUrlasserUrsache()
  {
    return _feUrlasserUrsache;
  }

  public FilterElement getUrlasserVeranlasser()
  {
    return _feUrlasserVeranlasser;
  }

  public FilterElement getMeldungsText()
  {
    return _feMeldungsText;
  }

}
