package de.bsvrz.buv.plugin.bmvew;

import java.util.LinkedList;
import java.util.List;

/**
 * Speichert mehrere {@link Filter}, die auf die Betriebsmeldungen im Dialog MeldungsanzeigeDialog angewendet werden
 * sollen.
 * 
 * @author HofmannJ
 * 
 */
public class FilterSpeicher
{
  /**
   * Liste der Filter.
   */
  private List<Filter> _lFilter = new LinkedList<Filter>();
 
  /**
  * Speichert die einzigste Instanz der Klasse {@link FilterSpeicher}.
  */
 private static FilterSpeicher _instanz = null;
  
  
  /**
   * Liefert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   * 
   * @return Einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  public static FilterSpeicher getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new FilterSpeicher();
    }
    return _instanz;
  }
  
  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   */
  private FilterSpeicher()
  {
    
  }

  /**
   * Fgt einen Filter zur Liste der Filter hinzu
   * 
   * @param filter
   *          Filter, der hinzugefgt werden soll
   */
  public void filterHinzufuegen(Filter filter)
  {
    _lFilter.add(filter);
  }

  /**
   * Einen Filter von der Liste der Filter entfernen.
   * 
   * @param filter
   *          Zu entfernender Filter.
   */
  public void filterEntfernen(Filter filter)
  {
    _lFilter.remove(filter);
  }

  /**
   * Liefert die Liste aller Filter.
   * 
   * @return Filterliste.
   */
  public List<Filter> getFilterListe()
  {
    return this._lFilter;
  }

}
