package de.bsvrz.buv.plugin.bmvew;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * In diesem Dialog sind die aktuell definierten Eintrge darzustellen. Fr jeden Eintrag ist die Eigenschaft Name
 * darzustellen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:06:11 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class FilterVerwaltungsdialog extends Dialog
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Ist <code>true</code>, wenn sich etwas an den Filtern gendert hat, sonst <code>false</code>;
   */
  private boolean _geandert = false;

  /**
   * FilterSpeicher.
   */
  private FilterSpeicher _filterSpeicher = null;

  /**
   * Filterverwaltungsdialog
   */
  private FilterVerwaltungsdialog _filterVerwaltungsdialog = null;

  private Shell shell = null; // @jve:decl-index=0:visual-constraint="144,147"

  private Composite compositeTop = null;

  private Composite compositeBottom = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private List _list = null;

  private Composite compositeRight = null;

  private Button buttonLoeschen = null;

  private Button buttonKopieren = null;

  private Button buttonBearbeiten = null;

  private Button buttonNeu = null;
    
  private Filter _filter = null;

  private XmlFilterVerwaltung _xmvSchreiben;




  /**
   * Konstruktor fr neue Instanzen der Klasse {@link FilterVerwaltungsdialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public FilterVerwaltungsdialog(Shell parent)
  {
    super(parent);
  }

  /**
   * ffnet den Dialog {@link FilterVerwaltungsdialog}.
   * 
   * @param filterSpeicher
   *          Instanz der Klasse {@link FilterSpeicher}, die die Liste aller {@link Filter} enthlt.
   * 
   * @return Liefert <code>true</code>, wenn sich etwas im {@link FilterSpeicher} gendert hat, sonst
   *         <code>false</code>;
   */
  public FilterSpeicher oeffne(FilterSpeicher filterSpeicher)
  {

    _filterSpeicher = filterSpeicher;
        
    // Neue Shell anlegen, Wait & Dispose, evtl. Rckgabewerte.
    Shell parent = getParent();
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    // shell = new Shell();
    shell.setText("Verwaltung Filter Betriebsmeldungen");
    shell.setSize(new Point(400, 300));
    shell.setLayout(new GridLayout());
    createCompositeTop();
    createCompositeBot();
    
    shell.open();

    Display display = parent.getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _filterSpeicher;
  }

  /**
   * Generiert die Elemente des Dialogs {@link FilterVerwaltungsdialog}.
   * 
   */
  private void generiereGui()
  {
  }

  /**
   * Definiert einen neuen Eintrag in der Liste von Filtern fr Betriebsmeldungen.
   * 
   */
  private void aktionNeu()
  {
    FilterDefinitionsdialog filterDefinitionsdialog = new FilterDefinitionsdialog(shell);
    Filter filter = filterDefinitionsdialog.oeffne(new Filter(), false);
    if (filter != null)
    {
      
      java.util.List<Filter> liste = _filterSpeicher.getFilterListe();
      Filter tmp = null;
      for (Filter f : liste)
      {
        if (f.getName().equals(filter.getName())){
            tmp = f; 
        }
      }
      
      if (tmp != null)
        _filterSpeicher.filterEntfernen(tmp);
      
      _filterSpeicher.filterHinzufuegen(filter);
      _list.add(filter.getName());
    }
  }

  /**
   * Bearbeitet einen Eintrag in der Liste von Filtern fr Betriebsmeldungen. Es ffnet sich der Definitionsdialg ({@link FilterDefinitionsdialog})
   * zum Bearbeiten des Eintrags.
   * 
   */
  private void aktionBearbeiten()
  {
    if (_filter != null){
    
      FilterDefinitionsdialog filterDefinitionsdialog = new FilterDefinitionsdialog(shell);
  
      String alt = _filter.getName();     
      
      Filter neu = filterDefinitionsdialog.oeffne(_filter, true);    
      
      if (neu != null){
        
        _list.remove(alt);
        _list.add(neu.getName());  
          
        _filterSpeicher.filterEntfernen(_filter);
        
        _filterSpeicher.filterHinzufuegen(neu);
        
      }
    

    }else{
      MessageBox mb = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Verwaltung");
      mb.setMessage("Es muss ein Filter selektiert werden");
      mb.open();
      return;
    }
      
  }

  /**
   * Kopiert einen Eintrag in der Liste von Filtern fr Betriebsmeldungen. Dieser erhlt den Prfix "Kopie von",
   * anschlieend ffnet sich der Definitionsdialg ({@link FilterDefinitionsdialog}) zum Bearbeiten des kopierten
   * Eintrags.
   * 
   */
  private void aktionKopieren()
  {
    if (_filter != null){    
        Filter clone = _filter.clone();         
        clone.setName("Kopie von " + _filter.getName());                
        _list.add(clone.getName());
        
        _filterSpeicher.filterHinzufuegen(clone);      
    }
    else{
      MessageBox mb = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Verwaltung");
      mb.setMessage("Es muss ein Filter selektiert werden");
      mb.open();
      return;
    }

  }

  /**
   * Lscht einen selektierten Eintrag nach vorhergehender "Sind sie sicher?"-Abfrage.
   * 
   */
  private void aktionLoeschen()
  {

    if (_filter != null){  
           
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
      mb.setText("Filter lschen");
      mb.setMessage("Wollen Sie den Filter: " + _filter.getName() + " wirklich lschen?");

      if (mb.open() == SWT.OK)
      {
        _filterSpeicher.filterEntfernen(_filter);
        _list.remove(_filter.getName());
        
      }
        
    }else{
      MessageBox mb = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Verwaltung");
      mb.setMessage("Es muss ein Filter selektiert werden");
      mb.open();
      return;
    
    }
      
  
  }

  /**
   * Schliet den Verwaltungsdialog {@link FilterVerwaltungsdialog}.
   * 
   */
  private void aktionSchliessen()
  {    
    shell.dispose();
  }

  /**
   * Zeigt die Hilfe zum Verwaltungsdialog {@link FilterVerwaltungsdialog} an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Aktivator.PLUGIN_ID + ".betriebsmeldungsverwaltung";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.grabExcessVerticalSpace = true;
    gridData5.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(shell, SWT.NONE);
    compositeTop.setLayoutData(gridData3);
    compositeTop.setLayout(gridLayout);
    _list = new List(compositeTop, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    _list.setLayoutData(gridData5);
    _list.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
        super.widgetDefaultSelected(e);
        FilterDefinitionsdialog filterDefinitionsdialog = new FilterDefinitionsdialog(shell);

        Filter filter = null;
        for (Filter f : _filterSpeicher.getFilterListe())
        {
          if (f.getName().equals(_list.getSelection()[0]))
          {
            filter = f;
            break;
          }
        }

        if (filter != null)
          filterDefinitionsdialog.oeffne(filter, true);

      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        super.widgetSelected(e);
        Filter filter = null;
        for (Filter f : _filterSpeicher.getFilterListe())
        {
          if (f.getName().equals(_list.getSelection()[0]))
          {
            filter = f;            
            break;
          }
        }

        if (filter != null)
          _filter = filter;
      }

    });
    
//    _list.addFocusListener(new FocusListener(){
//
//      @Override
//      public void focusGained(FocusEvent e)
//      {
//        
//      }
//
//      @Override
//      public void focusLost(FocusEvent e)
//      {
//        _filter = null;
//        
//      }
//      
//    });

    for (Filter f : _filterSpeicher.getFilterListe())
    {
      _list.add(f.getName());
    }

    createCompositeRight();

  }

  /**
   * This method initializes compositeBot
   * 
   */
  private void createCompositeBot()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    rowLayout.fill = false;
    rowLayout.wrap = false;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.END;
    gridData4.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(shell, SWT.NONE);
    compositeBottom.setLayoutData(gridData4);
    compositeBottom.setLayout(rowLayout);
    buttonSchliessen = new Button(compositeBottom, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});

    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeRight()
  {
    RowLayout rowLayout2 = new RowLayout();
    rowLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout2.wrap = false;
    rowLayout2.pack = false;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.CENTER;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    compositeRight = new Composite(compositeTop, SWT.NONE);
    compositeRight.setLayoutData(gridData6);
    compositeRight.setLayout(rowLayout2);
    buttonNeu = new Button(compositeRight, SWT.NONE);
    buttonNeu.setText("Neu");    
    buttonNeu.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionNeu();
			}			
		});

    buttonBearbeiten = new Button(compositeRight, SWT.NONE);
    buttonBearbeiten.setText("Bearbeiten...");
    buttonBearbeiten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionBearbeiten();
			}			
		});
    
    
    buttonKopieren = new Button(compositeRight, SWT.NONE);
    buttonKopieren.setText("Kopieren...");
    buttonKopieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionKopieren();
			}			
		});
    
    
    
    buttonLoeschen = new Button(compositeRight, SWT.NONE);
    buttonLoeschen.setText("Lschen");    
    buttonLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionLoeschen();
			}			
		});
  }

}
