package de.bsvrz.buv.plugin.ereigniskal;

import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert die Reprsentation eines Ereigniskalender Eintrages.
 * 
 * @version $Revision: 1.1 $ / $Date: 2010/02/02 15:21:42 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class Eintrag implements Cloneable
{
  /**
   * Pid des Ereigniskalendereintrags.
   */
  private String _pid = null;

  /**
   * Name des Ereigniskalendereintrags.
   */
  private String _name = null;

  /**
   * Beschreibung des Ereigniskalendereintrags.
   */
  private String _beschreibung = null;

  /**
   * EreignisTyp des Ereigniskalendereintrags.
   */
  private SystemObject _ereignisTyp = null;

  /**
   * Quelle des Ereigniskalendereintrags.
   */
  private String _quelle = null;

  /**
   * Enumerator fr die Art der Zeitbereichsangabe.
   * 
   * @author HofmannJ
   */
  public enum _artDerZeitspezifikation {
    Zeitbereichsangabe, Systemkalendereintrag;
  };

  /**
   * Legt die Art der Zeitspezifikation ber den Enumerator <code>enumArtDerZeitspezifikation</code> fest.
   */
  private _artDerZeitspezifikation _gewaehlteArtDerZeitspezifikation = null;

  /**
   * Speichert den Anfangszeitpunkt in Millisekunden seit dem 1.1.1970 bei der Zeitbereichsangabe.
   */
  private Long _anfangsZeitpunkt = null;

  /**
   * Speichert den Endzeitpunkt in Millisekunden seit dem 1.1.1970 bei der Zeitbereichsangabe.
   */
  private Long _endZeitpunkt = null;

  /**
   * Speichert das Datum in Millisekunden seit dem 1.1.1970 bei der Datumseingabe
   */
  private SystemObject _systemKalenderEintrag = null;

  /**
   * TableViewer fr die verkehrliche Gueltigkeit
   */
  private List<EintragVerkehrlicheGueltigkeit> _lEintragVerkehrlicheGueltigkeit = null;

  /**
   * TableViewer fr die raeumliche Gueltigkeit
   */
  private List<EintragRaeumlicheGueltigkeit> _lEintragRaeumlicheGueltigkeit = null;

  /**
   * Setzt die Art der Zeitspezifikation auf Zeitbereichsangabe
   * 
   * @param name
   *          Name des Systemkalendereintrags.
   * @param anfangsZeitpunkt
   *          Anfangszeitpunkt des Systemkalendereintrags in Millisekunden seit dem 1.1.1970.
   * @param endZeitpunkt
   *          Endzeitpunkt des Systemkalendereintrags in Millisekunden seit dem 1.1.1970.
   */
  public void setZeitbereich(String name, Long anfangsZeitpunkt, Long endZeitpunkt)
  {
    this._name = name;
    this._gewaehlteArtDerZeitspezifikation = _gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe;
    this._anfangsZeitpunkt = anfangsZeitpunkt;
    this._endZeitpunkt = endZeitpunkt;
  }

  /**
   * Liefert einen Clone dieser Klasse und setzt den Prfix "Kopie von " vor den Namen.
   * 
   * @return Clone dieser Klasse.
   */
  public Eintrag clone()
  {
    Eintrag ereigniskalenderDatensatz = new Eintrag();
    String neuerName = "Kopie von " + _name;

    ereigniskalenderDatensatz.setName(neuerName);
    ereigniskalenderDatensatz.setBeschreibung(_beschreibung);
    ereigniskalenderDatensatz.setEreignisTyp(_ereignisTyp);
    ereigniskalenderDatensatz.setArtDerZeitspezifikation(_gewaehlteArtDerZeitspezifikation);
    ereigniskalenderDatensatz.setQuelle(_quelle);
    ereigniskalenderDatensatz.setAnfangsZeitpunkt(_anfangsZeitpunkt);
    ereigniskalenderDatensatz.setEndZeitpunkt(_endZeitpunkt);
    ereigniskalenderDatensatz.setSystemKalenderEintrag(_systemKalenderEintrag);
    ereigniskalenderDatensatz.setListeEintragRaeumlicheGueltigkeit(_lEintragRaeumlicheGueltigkeit);
    ereigniskalenderDatensatz.setListeEintragVerkehrlicheGueltigkeit(_lEintragVerkehrlicheGueltigkeit);

    return ereigniskalenderDatensatz;
  }

  /**
   * Liefert den Namen des Ereigniskalender Datensatzes.
   * 
   * @return Name des Ereigniskalender Datensatzes.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Setzt den Namen des Ereigniskalender Datensatzes.
   * 
   * @param name
   *          Name des Ereigniskalender Datensatzes.
   */
  public void setName(String name)
  {
    this._name = name;
  }

  /**
   * Liefert die Art der Zeitspezifikation
   * 
   * @return Art der Zeitspezifikation.
   */
  public _artDerZeitspezifikation getArtDerZeitspezifikation()
  {
    return _gewaehlteArtDerZeitspezifikation;
  }

  /**
   * Setzt die Art der Zeitspezifikation.
   * 
   * @param artDerZeitspezifikation
   *          Art der Zeitspezifikation.
   */
  public void setArtDerZeitspezifikation(_artDerZeitspezifikation artDerZeitspezifikation)
  {
    _gewaehlteArtDerZeitspezifikation = artDerZeitspezifikation;
  }

  /**
   * Liefert den Anfangszeitpunkt bei Zeitbereichseingabe. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf Zeitbereichseingabe gesetzt, wird -1 geliefert.
   * 
   * @return Anfangszeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public Long getAnfangsZeitpunkt()
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe)
        || _gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Systemkalendereintrag))
    {
      return _anfangsZeitpunkt;
    }
    else
    {
      return -1L;
    }

  }

  /**
   * Setzt den Anfangszeitpunkt bei Zeitbereichseingabe. Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf
   * Zeitbereichseingabe gesetzt, wird der Anfangszeitpunkt nicht gesetzt.
   * 
   * @param zeitpunkt
   *          Anfangszeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public void setAnfangsZeitpunkt(Long zeitpunkt)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe)
        || _gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Systemkalendereintrag))
    {
      _anfangsZeitpunkt = zeitpunkt;
    }

  }

  /**
   * Liefert den Endzeitpunkt bei Zeitbereichseingabe. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf Zeitbereichseingabe gesetzt, wird -1 geliefert.
   * 
   * @return Endzeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public Long getEndZeitpunkt()
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe)
        || _gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Systemkalendereintrag))
    {
      return _endZeitpunkt;
    }
    else
    {
      return -1L;
    }
  }

  /**
   * Setzt den Endzeitpunkt bei Zeitbereichseingabe. Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf
   * Zeitbereichseingabe gesetzt, wird der Endzeitpunkt nicht gesetzt.
   * 
   * @param zeitpunkt
   *          Endzeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public void setEndZeitpunkt(Long zeitpunkt)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe)
        || _gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Systemkalendereintrag))
    {
      _endZeitpunkt = zeitpunkt;
    }
  }

  /**
   * Setzt die freie Definition der Zeitspezifikation. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf freie Definition gesetzt, wird die freie Definition
   * der Zeitspezifikation nicht gesetzt.
   * 
   * @param definition
   *          Freie Definition der Zeitspezifikation.
   */
  public void setSystemKalenderEintrag(SystemObject systemObjekt)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Systemkalendereintrag)
        || _gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe))
    {
      _systemKalenderEintrag = systemObjekt;
    }
  }

  /**
   * Liefert die Pid des Ereigniskalender.
   * 
   * @return Pid des Ereigniskalender.
   */
  public String getPid()
  {
    return _pid;
  }

  /**
   * Setzt die Pid des Ereigniskalender.
   * 
   * @param pid
   *          Pid des Ereigniskalender.
   */
  public void setPid(String pid)
  {
    this._pid = pid;
  }

  /**
   * Holt die Liste der verkehrlichen Gueltigkeiten
   * 
   * @return
   */
  public List<EintragVerkehrlicheGueltigkeit> getListeEintragVerkehrlicheGueltigkeit()
  {
    return _lEintragVerkehrlicheGueltigkeit;
  }

  /**
   * Setzt die Liste der verkehrlichen Gueltigkeiten
   * 
   * @param eintragVerkehrlicheGueltigkeit
   */
  public void setListeEintragVerkehrlicheGueltigkeit(List<EintragVerkehrlicheGueltigkeit> eintragVerkehrlicheGueltigkeit)
  {
    _lEintragVerkehrlicheGueltigkeit = eintragVerkehrlicheGueltigkeit;
  }

  /**
   * Holt die Liste der raeumlichen Gueltigkeiten
   * 
   * @return
   */
  public List<EintragRaeumlicheGueltigkeit> getListeEintragRaeumlicheGueltigkeit()
  {
    return _lEintragRaeumlicheGueltigkeit;
  }

  /**
   * Holt die Liste der raeumlichen Gueltigkeiten
   * 
   * @param
   */
  public void setListeEintragRaeumlicheGueltigkeit(List<EintragRaeumlicheGueltigkeit> eintragRaeumlicheGueltigkeit)
  {
    _lEintragRaeumlicheGueltigkeit = eintragRaeumlicheGueltigkeit;
  }

  /**
   * Holt die Beschreibung
   * 
   * @return
   */
  public String getBeschreibung()
  {
    return _beschreibung;
  }

  /**
   * Setzt die Beschreibung
   * 
   * @param beschreibung
   */
  public void setBeschreibung(String beschreibung)
  {
    _beschreibung = beschreibung;
  }

  /**
   * Holt den EreignisTyp
   * 
   * @return
   */
  public SystemObject getEreignisTyp()
  {
    return _ereignisTyp;
  }

  /**
   * Setzt den EreignisTyp
   * 
   * @param ereignisTyp
   */
  public void setEreignisTyp(SystemObject ereignisTyp)
  {
    _ereignisTyp = ereignisTyp;
  }

  /**
   * Holt die Quelle
   * 
   * @return
   */
  public String getQuelle()
  {
    return _quelle;
  }

  /**
   * Setzt die Quelle
   * 
   * @param quelle
   */
  public void setQuelle(String quelle)
  {
    _quelle = quelle;
  }

  /**
   * Setzt des SystemkalenderEintrag
   * 
   * @return
   */
  public SystemObject getSystemKalenderEintrag()
  {
    return _systemKalenderEintrag;
  }

}
