/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.orte;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer;
import de.bsvrz.buv.rw.basislib.ort.FilterKnoten;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeComposite;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeEnumeratorAuswahl;
import de.bsvrz.buv.rw.basislib.ort.FilterKnoten.Richtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.EnumAsbVerkehrsRichtung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dieser {@link ViewPart} liefert ein Beispiel, wie sich die Klasse
 * {@link OrtsangabeComposite} verwenden lsst. <br>
 * <br>
 * Die hinter den {@link Button}s hinterlegten Aktionen liefern die Daten der
 * im {@link OrtsangabeComposite} gettigten Selektion in einem Textfenster.
 * 
 * 
 * @author HofmannJ
 * 
 */
public class ViewOrte extends ViewPart
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * ID dieser View
	 */
	public static final String ID = "de.bsvrz.buv.plugin.beispiele.orte.viewOrte";

	private Composite top = null;
	private Group groupOben = null;

	/**
	 * {@link OrtsangabeComposite}, in dem die Ortsauswahl erfolgt.
	 */
	private OrtsangabeComposite _ortsangabeComposite = null;

	private Group groupUnten = null;

	private Group groupUntenLinks = null;

	private Composite compositeUntenRechts = null;

	private Button buttonStrassensegmentOffset = null;

	private Button buttonKnotenOffset = null;

	private Text textAreaAusgabe = null;

	private Button buttonBetriebskilometer = null;

	private Button buttonASBOrtsangabe = null;

	private Button buttonUmschalten = null;

	private Button buttonStrassenSegmentOffsetTest = null;

	private Button buttonKnotenOffsetSetzen = null;

	private Button buttonBetriebskilometertest = null;

	private Button buttonAsbOrtsangabeTest = null;

	@Override
	public void createPartControl(Composite parent)
	{
		top = new Composite(parent, SWT.NONE);
		top.setLayout(new GridLayout());
		createGroupOben();

		createGroupUnten();
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * Erzeugen von groupOben. In dieser Gruppe ist das
	 * {@link OrtsangabeComposite} enthalten.
	 * 
	 */
	private void createGroupOben()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		groupOben = new Group(top, SWT.NONE);
		groupOben.setLayout(new GridLayout());
		groupOben.setLayoutData(gridData);
		groupOben.setText("Orstserfassung");

		ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();

		if (con != null)
		{
			_ortsangabeComposite = new OrtsangabeComposite(groupOben, SWT.NONE, con);
		} else
		{
			_ortsangabeComposite = new OrtsangabeComposite(groupOben, SWT.NONE, null);
		}

		_ortsangabeComposite.setLayoutData(gridData1);
	}

	/**
	 * This method initializes groupUnten
	 * 
	 */
	private void createGroupUnten()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		groupUnten = new Group(top, SWT.NONE);
		groupUnten.setLayoutData(gridData2);
		createGroupUntenLinks();
		groupUnten.setLayout(gridLayout);
		createCompositeUntenRechts();
		groupUnten.setText("Sonstiges");
	}

	/**
	 * This method initializes groupUntenLinks
	 * 
	 */
	private void createGroupUntenLinks()
	{
		GridData gridData7 = new GridData();
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.verticalAlignment = GridData.FILL;
		gridData7.grabExcessVerticalSpace = true;
		gridData7.horizontalAlignment = GridData.FILL;
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		groupUntenLinks = new Group(groupUnten, SWT.NONE);
		groupUntenLinks.setLayout(new GridLayout());
		groupUntenLinks.setLayoutData(gridData3);
		groupUntenLinks.setText("Info");
		textAreaAusgabe = new Text(groupUntenLinks, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		textAreaAusgabe.setEditable(false);
		textAreaAusgabe.setLayoutData(gridData7);
	}

	private void aktionInformationenStrassensegmentHolen()
	{
		OrtsangabeEnumeratorAuswahl auswahlArt = _ortsangabeComposite
				.getAuswahlArt();
		if (auswahlArt.equals(OrtsangabeEnumeratorAuswahl.strassensegment))
		{
			String text = _ortsangabeComposite.getFilterStrassenSegmentOffset()
					.toString();
			textAreaAusgabe.setText(text);
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
			mb.setText("Info:");
			mb
					.setMessage("Es ist zur Zeit kein Reiter zur Auswahl von Straensegmenten aktiv.");
			mb.open();
		}
	}

	/**
	 * Holen und ausgeben von Informationen zu Betriebskilometern
	 */
	protected void aktionInformationenBetriebskilometerHolen()
	{
		OrtsangabeEnumeratorAuswahl auswahlArt = _ortsangabeComposite
				.getAuswahlArt();
		if (auswahlArt.equals(OrtsangabeEnumeratorAuswahl.betriebskilometer))
		{
			String text = _ortsangabeComposite.getFilterBetriebskilometer()
					.toString();
			textAreaAusgabe.setText(text);

		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
			mb.setText("Info:");
			mb
					.setMessage("Es ist zur Zeit kein Reiter zur Auswahl von Betriebskilometern aktiv.");
			mb.open();
		}
	}

	protected void aktionInformationenASBHolen()
	{
		OrtsangabeEnumeratorAuswahl auswahlArt = _ortsangabeComposite
				.getAuswahlArt();
		if (auswahlArt.equals(OrtsangabeEnumeratorAuswahl.asborstangabe))
		{
			textAreaAusgabe.setText(""
					+ _ortsangabeComposite.getFilterAsbOrtsangabe());
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
			mb.setText("Info:");
			mb
					.setMessage("Es ist zur Zeit kein Reiter zur Auswahl von ASB-Ortsangaben aktiv.");
			mb.open();
		}
	}

	private void aktionInformationenKnotenHolen()
	{
		OrtsangabeEnumeratorAuswahl auswahlArt = _ortsangabeComposite
				.getAuswahlArt();
		if (auswahlArt.equals(OrtsangabeEnumeratorAuswahl.knotenoffset))
		{
			FilterKnoten filterKnoten = _ortsangabeComposite.getFilterKnoten();
			if (filterKnoten != null)
			{
				String text = _ortsangabeComposite.getFilterKnoten().toString();
				// _debug.fine("===> " + text);
				textAreaAusgabe.setText(text);
			} else
			{
				_debug.fine("NULL!!!");
			}
		} else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
			mb.setText("Info:");
			mb
					.setMessage("Es ist zur Zeit kein Reiter zur Auswahl von Knoten aktiv.");
			mb.open();
		}
	}

	/**
	 * This method initializes compositeUntenRechts
	 * 
	 */
	private void createCompositeUntenRechts()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout.fill = true;
		rowLayout.wrap = true;
		rowLayout.justify = false;
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = false;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		compositeUntenRechts = new Composite(groupUnten, SWT.NONE);
		compositeUntenRechts.setLayoutData(gridData4);
		compositeUntenRechts.setLayout(rowLayout);
		buttonStrassensegmentOffset = new Button(compositeUntenRechts, SWT.NONE);
		buttonStrassensegmentOffset.setText("Straensegment-Offset");
		buttonStrassensegmentOffset
				.setToolTipText("Zeigt die Informationen zum ausgewhlten Straensegment im Textfeld an.");
		buttonStrassenSegmentOffsetTest = new Button(compositeUntenRechts, SWT.NONE);
		buttonStrassenSegmentOffsetTest.setText("Straensegment-Offset Test");	
		buttonStrassenSegmentOffsetTest.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				_ortsangabeComposite.setStrassenSegment("strasse.A.52.tmclc.39804",
						"ass.n.11692.n.11691.0", 2000);
			}			
		});
		
		
		buttonBetriebskilometer = new Button(compositeUntenRechts, SWT.NONE);
		buttonBetriebskilometer.setText("Betriebskilometer");
		buttonBetriebskilometertest = new Button(compositeUntenRechts, SWT.NONE);
		buttonBetriebskilometertest.setText("Betriebskilometer Test");
		buttonBetriebskilometertest.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_ortsangabeComposite.setBetriebskilometer(
						"strasse.A.60.tmclc.07162",
						FilterBetriebskilometer.Richtung.SD, 40000, "KeineAngabe");
			}			
		});
		

		buttonBetriebskilometer.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionInformationenBetriebskilometerHolen();
			}			
		});

		/*
		 * Button ASB Ortsangabe
		 */
		buttonASBOrtsangabe = new Button(compositeUntenRechts, SWT.NONE);
		buttonASBOrtsangabe.setText("ASB-Ortsangabe");
		buttonASBOrtsangabe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionInformationenASBHolen();
			}			
		});

		/*
		 * Button ASB Ortsangabe Test
		 */
		buttonAsbOrtsangabeTest = new Button(compositeUntenRechts, SWT.NONE);
		buttonAsbOrtsangabeTest.setText("ASB-Ortsangabe Test");		
		buttonAsbOrtsangabeTest.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
						.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
				mb.setText("Info");
				mb
						.setMessage("Dieser Test luft nur mit der von BitControl gelieferten ASB-Versorgung von BW!!!");
				mb.open();

				String pidStrasse = "strasse.A.5.tmclc.07129";
				String anfangsKnoten = "6517066";
				String endKnoten = "6517057";
				long stationierung = 749l;
				EnumAsbVerkehrsRichtung verkehrsRichtung = EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG;

				_ortsangabeComposite.setAsb(pidStrasse, anfangsKnoten, endKnoten,
						stationierung, verkehrsRichtung);
			}			
		});
		

		buttonStrassensegmentOffset.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionInformationenStrassensegmentHolen();
			}			
		});

		buttonKnotenOffset = new Button(compositeUntenRechts, SWT.NONE);
		buttonKnotenOffset.setText("Knoten-Offset");
		buttonKnotenOffset
				.setToolTipText("Zeigt die Informationen zum ausgewhlten Knoten im Textfeld an.");
		buttonKnotenOffsetSetzen = new Button(compositeUntenRechts, SWT.NONE);
		buttonKnotenOffsetSetzen.setText("Knoten-Offset setzen");
		buttonKnotenOffsetSetzen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_ortsangabeComposite.setKnotenOffset("strasse.A.62.tmclc.07170",
						"strassenKnoten.12039.23459", 123, Richtung.negativ);
			}			
		});
		
		
		buttonUmschalten = new Button(compositeUntenRechts, SWT.NONE);
		buttonUmschalten.setText("Umschalten");
		buttonUmschalten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{try
			{
				// _ortsangabeComposite.selektierePultOrdner(OrtsangabeEnumeratorAuswahl.betriebskilometer);
				// _ortsangabeComposite.setBetriebskilometer("strasse.A.3.tmclc.07077",
				// FilterBetriebskilometer.Richtung.SD, 24400, "102");
				// _ortsangabeComposite.setBetriebskilometer("strasse.A.3.tmclc.07077",
				// FilterBetriebskilometer.Richtung.SD, 40000, "KeineAngabe");
				_ortsangabeComposite.setBetriebskilometer("strasse.A.3.tmclc.07077",
						FilterBetriebskilometer.Richtung.SD, 40000, "102");
			} catch (IllegalArgumentException iae)
			{
				iae.printStackTrace();
			}
			}			
		});
		
		
		buttonKnotenOffset.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionInformationenKnotenHolen();
			}			
		});
	}

} // @jve:decl-index=0:visual-constraint="10,10,505,314"
