/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.tabellen;

import java.util.Date;

import de.bsvrz.buv.rw.basislib.tabellen.*;

public class TabellenTest
{
	/**
	 * Speichert die Objekte zur Reprsentation von SystmeObjekten.
	 */
	private static String[] systemObjekte = new String[] { "System Objekt 0",
			"System Objekt 1", "System Objekt 2", "System Objekt 3",
			"System Objekt 4", "System Objekt 5" };

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		@SuppressWarnings("unused")
		TabellenTest te = new TabellenTest();
	}

	public TabellenTest()
	{

		TabellenDatenStruktur tds = erzeugeTabellenDatenStruktur();
		System.out.println(tds);
		// tds.starteSortierung();
		System.out.println("Nach Sortierung =>\n" + tds);
	}

	public static ZeilenObjekt erzeugeSystemObjekt(int objectId, String attA,
			String attB, String attC)
	{
		ZeilenObjekt zeilenObjekt = null;

		if (objectId < systemObjekte.length)
		{
			Object[] attribute = new Object[] { systemObjekte[objectId], new Date(),
					attA, attB, attC };
			zeilenObjekt = new ZeilenObjekt(systemObjekte[objectId], attribute);
		}

		return zeilenObjekt;
	}

	public static TabellenDatenStruktur erzeugeTabellenDatenStruktur()
	{
		System.out.println("Erzeuge TabellenDatenStruktur...");

		SpaltenElement[] spaltenTitel = new SpaltenElement[] {
				new SpaltenElement("Objekt"), new SpaltenElement("Zeit"),
				new SpaltenElement("Attribut 1"), new SpaltenElement("Attribut 2"),
				new SpaltenElement("Attribut 3") };

		int[] spaltenBreiten = new int[] { 100, 175, 100, 100, 100 };

		ZeilenObjekt a = erzeugeSystemObjekt(0, "A", "B", "C");
		ZeilenObjekt b = erzeugeSystemObjekt(1, "B", "C", "A");
		ZeilenObjekt c = erzeugeSystemObjekt(2, "C", "A", "B");
		ZeilenObjekt d = erzeugeSystemObjekt(3, "A", "B", "C");
		ZeilenObjekt e = erzeugeSystemObjekt(4, "B", "C", "A");
		ZeilenObjekt f = erzeugeSystemObjekt(5, "C", "A", "B");

		TabellenDatenStruktur tds = new TabellenDatenStruktur(5);
		tds.setSpaltenTitel(spaltenTitel);
		tds.setSpaltenBreiten(spaltenBreiten);
		tds.addZeile(a);
		tds.addZeile(b);
		tds.addZeile(c);
		tds.addZeile(d);
		tds.addZeile(e);
		tds.addZeile(f);

		Sortierung[] sortierung = new Sortierung[] { new Sortierung(2),
				new Sortierung(3), new Sortierung(4) };
		tds.setSortierung(sortierung);

		return tds;
	}

}
