/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.tabellen;

import java.util.Iterator;
import java.util.Random;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.tabellen.ContentProviderTabellenDatenStruktur;
import de.bsvrz.buv.rw.basislib.tabellen.DialogFilterVerwaltung;
import de.bsvrz.buv.rw.basislib.tabellen.DialogSpaltenSortierungsReihenfolge;
import de.bsvrz.buv.rw.basislib.tabellen.DragAdapter;
import de.bsvrz.buv.rw.basislib.tabellen.DropAdapter;
import de.bsvrz.buv.rw.basislib.tabellen.ElementFilter;
import de.bsvrz.buv.rw.basislib.tabellen.ElementRegEx;
import de.bsvrz.buv.rw.basislib.tabellen.LabelProviderTabellenDatenStruktur;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur;
import de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.DatenQuelle;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.FilterAusdruck;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.TabellenFilter;

public class TestTabellenView extends ViewPart
{

	/**
	 * ID der View
	 */
	public static final String ID = "de.bsvrz.buv.plugin.pruefrw.tabellen.TestTabellenView";
	private Composite bereichUnten = null;
	private Group gruppeLinks = null;
	private Group gruppeRechts = null;
	private Button buttonSortierung = null;
	private Table _tabelleTest = null;
	private TableViewer _tableViewer = null;

	/**
	 * Test {@link TabellenDatenStruktur}, mit welcher der {@link TableViewer}
	 * dieser View gefttert wird.
	 */
	private TabellenDatenStruktur _tds = TabellenTest
			.erzeugeTabellenDatenStruktur();

	/**
	 * {@link XmlFilterVerwaltung}
	 */
	private XmlFilterVerwaltung _xmlFilter = new XmlFilterVerwaltung();

	/**
	 * Button fr das Fllen der Tabelle
	 */
	private Button _schaltflaecheFuellen = null;

	/**
	 * Checkbox zum Anwenden des Filters
	 */
	private Button _checkBoxFiltern = null;

	/**
	 * Schaltflche Filterverwaltung
	 */
	private Button _schaltflaecheFilterVerwaltung = null;

	/**
	 * Button fr das zurcksetzen.
	 */
	private Button _schaltflaecheRuecksetzen;

	/**
	 * Schaltflche zum Testen der nderungsdarstellung mit festen Daten
	 */
	private Button _schaltflaecheTestDatenEinfuegenFest;

	/**
	 * Schaltflche zum Einfgen von zuflligen Daten fr die nderungsdarstellung
	 */
	private Button _schaltflaecheTestDatenEinfuegenZufaellig;

	/**
	 * Flag. Zeigt an, ob die Tabelle schon mal gefllt wurde.
	 */
	private boolean _tabelleSchonMalGefuellt = false;
	private Composite _top;
	private Combo _comboFilterAuswahl;
	private ComboViewer _comboViewer;

	private Button _checkSortieren;
	private Button _checkAenderung;

	/**
	 * Rotation fr die Indexe der verfgbaren Objekte zum Testen der
	 * nderungsdarstellung
	 */
	private int _rotationIdxAenderungsDarstellung;

	/**
	 * Zufallszahlengenerator zur Generierung von Zufallszahlen zum zuflligen
	 * Einfgen von Daten fr die nderungsdarstellung.
	 */
	private Random ran = new Random();

	@Override
	public void createPartControl(Composite parent)
	{
		_top = new Composite(parent, SWT.NONE);
		_top.setLayout(new GridLayout());

		erzeugeFilterAuswahl();

		bereichUnten = new Composite(_top, SWT.NONE);
		erzeugeGruppeLinks();
		erzeugeGruppeRechts();

		GridData gridDataBereichUnten = new GridData();
		gridDataBereichUnten.horizontalAlignment = GridData.FILL;
		gridDataBereichUnten.grabExcessHorizontalSpace = true;
		gridDataBereichUnten.grabExcessVerticalSpace = true;
		gridDataBereichUnten.verticalAlignment = GridData.FILL;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		bereichUnten.setLayout(gridLayout);
		bereichUnten.setLayoutData(gridDataBereichUnten);

		initFilter();
		initComboViewer();

		// GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 2;
		// bereichUnten = new Composite(parent, SWT.NONE);
		// erzeugeGruppeLinks();
		// bereichUnten.setLayout(gridLayout);
		// createGruppeRechts();
	}

	private void initFilter()
	{
		_xmlFilter = new XmlFilterVerwaltung();

		ElementFilter elementFilter0 = new ElementFilter(_xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Ein Testfilter");

		elementFilter0.addRegEx(new ElementRegEx(elementFilter0
				.bestimmeNaechsteFreieIdRegEx(), "_", ".*"));
		elementFilter0.addRegEx(new ElementRegEx(elementFilter0
				.bestimmeNaechsteFreieIdRegEx(), "_", ".*"));
		elementFilter0.addRegEx(new ElementRegEx(elementFilter0
				.bestimmeNaechsteFreieIdRegEx(), "_", ".*"));
		elementFilter0.addRegEx(new ElementRegEx(elementFilter0
				.bestimmeNaechsteFreieIdRegEx(), "_", ".*"));
		elementFilter0.addRegEx(new ElementRegEx(elementFilter0
				.bestimmeNaechsteFreieIdRegEx(), "_", ".*"));

		_xmlFilter.addElementFilter(elementFilter0);
	}

	private void initComboViewer()
	{
		_comboViewer.setInput(_xmlFilter.getElementFilter());
	}

	@Override
	public void setFocus()
	{
	}

	private void erzeugeFilterAuswahl()
	{
		GridData gridDataGruppeFilter = new GridData();
		gridDataGruppeFilter.horizontalAlignment = GridData.FILL;
		gridDataGruppeFilter.grabExcessHorizontalSpace = true;
		gridDataGruppeFilter.verticalAlignment = GridData.CENTER;
		Group group = new Group(_top, SWT.NONE);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		group.setLayout(gridLayout);
		group.setLayoutData(gridDataGruppeFilter);
		group.setText("Filter:");

		GridData gridDataCombo = new GridData();
		gridDataCombo.horizontalAlignment = GridData.FILL;
		gridDataCombo.grabExcessHorizontalSpace = true;
		gridDataCombo.verticalAlignment = GridData.CENTER;
		_comboFilterAuswahl = new Combo(group, SWT.NONE);
		_comboFilterAuswahl.setLayoutData(gridDataCombo);
		_comboViewer = new ComboViewer(_comboFilterAuswahl);
		_comboViewer.setContentProvider(new ArrayContentProvider());
		_comboViewer.setLabelProvider(new LabelProvider());

		_comboViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof IStructuredSelection)
				{
					IStructuredSelection iss = (IStructuredSelection) selection;

					if (iss.getFirstElement() != null)
					{
						_checkBoxFiltern.setSelection(true);
						aktionFilternDurchfuehren(true);
					} else
					{
						_checkBoxFiltern.setSelection(false);
						aktionFilternDurchfuehren(false);
					}

				}
			}

		});


		_checkBoxFiltern = new Button(group, SWT.CHECK);
		_checkBoxFiltern.setText("Filter anwenden");
		_checkBoxFiltern.addSelectionListener(new SelectionListener()
		{

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (e.getSource() instanceof Button)
				{
					Button b = (Button) e.getSource();

					if (b.getSelection())
					{
						aktionFilternDurchfuehren(true);
					} else
					{
						aktionFilternDurchfuehren(false);
					}
				}
			}
		});
	}

	/**
	 * Erzeugt die Linke Gruppe mit einem {@link TableViewer}.
	 * 
	 */
	private void erzeugeGruppeLinks()
	{
		GridData gridData11 = new GridData();
		gridData11.horizontalAlignment = GridData.FILL;
		gridData11.grabExcessHorizontalSpace = true;
		gridData11.grabExcessVerticalSpace = true;
		gridData11.verticalAlignment = GridData.FILL;
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gruppeLinks = new Group(bereichUnten, SWT.NONE);
		gruppeLinks.setLayout(new GridLayout());
		gruppeLinks.setLayoutData(gridData);
		gruppeLinks.setText("Tabelle");
		_tabelleTest = new Table(gruppeLinks, SWT.FULL_SELECTION);
		_tabelleTest.setHeaderVisible(true);
		_tabelleTest.setLayoutData(gridData11);
		_tabelleTest.setLinesVisible(true);
		_tableViewer = new TableViewer(_tabelleTest);

	}

	/**
	 * Fllt die Tabelle mit Test-Daten. Es werden Kopfzeilen erstellt und
	 * Content- bzw. LabelProvider an den {@link TableViewer} gebunden.
	 */
	private void aktionTabelleFuellen()
	{
		if (!_tabelleSchonMalGefuellt)
		{
			/*
			 * Erzeugen der Kopfzeilen
			 */
			_tds.erzeugeTabellenKopfzeile(_tableViewer, true);

			/*
			 * Anbinden des ContentProviders an den TableViewer
			 */
			ContentProviderTabellenDatenStruktur cp = new ContentProviderTabellenDatenStruktur();
			_tableViewer.setContentProvider(cp);

			/*
			 * Anbinden des LabelProviders an den TableViewer
			 */
			LabelProviderTabellenDatenStruktur lp = new LabelProviderTabellenDatenStruktur();
			_tableViewer.setLabelProvider(lp);

			_tabelleSchonMalGefuellt = true;
		}

		_tableViewer.setInput(_tds);

		int ops = DND.DROP_MOVE;

		Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
		DragAdapter dragAdapter = new DragAdapter(_tableViewer, _tds);
		_tableViewer.addDragSupport(ops, dragTransfers, dragAdapter);

		Transfer[] dropTransfers = new Transfer[] { TextTransfer.getInstance() };
		DropAdapter dropAdapter = new DropAdapter(_tableViewer, _tds);
		_tableViewer.addDropSupport(ops, dropTransfers, dropAdapter);

	}

	/**
	 * ffnet den Sortierdialog zur Auswahl der Spalten, nach denen sortiert
	 * werden soll.
	 */
	private void aktionSortierenDialog()
	{
		DialogSpaltenSortierungsReihenfolge dlg = new DialogSpaltenSortierungsReihenfolge(
				PlatformUI.getWorkbench().getDisplay().getActiveShell());
		if (dlg.open(_tds))
		{
			/*
			 * Vornehmen der durch den Dialog zur Sortierung vorgegebenen Sortierung.
			 */
			_tableViewer.setInput(_tds);
		}
	}

	/**
	 * Aktualisiert die Darstellung und kann die Sortierung aktivieren.
	 * 
	 * @param sortieren
	 *          Wenn <code>true</code>, dann wird sortiert, bei <code>false</code>
	 *          nicht.
	 */
	private void aktionSortierungDurchfuehren(boolean sortieren)
	{
		_tds.setSortieren(sortieren);
		_tableViewer.setInput(_tds);
	}

	/**
	 * Aktualisiert die Darstellung und kann die Filterung aktivieren.
	 * 
	 * @param filtern
	 *          Wenn <code>true</code>, dann wird gefiltert, bei
	 *          <code>false</code> nicht.
	 */
	private void aktionFilternDurchfuehren(boolean filtern)
	{
		if (filtern)
		{
			ISelection selection = _comboViewer.getSelection();
			if (selection instanceof IStructuredSelection)
			{
				IStructuredSelection iss = (IStructuredSelection) selection;
				Object firstElement = iss.getFirstElement();
				if (firstElement != null)
				{
					if (firstElement instanceof ElementFilter)
					{
						ElementFilter elementFilterSelektiert = (ElementFilter) firstElement;
						filterSetzen(elementFilterSelektiert);

						_tds.setGefiltert(filtern);
						_tableViewer.setInput(_tds);

					}
				} else
				{
					System.out.println("Erste Selektion ist NULL");
				}
			}
		} else
		{
			_tds.setGefiltert(false);
			_tableViewer.setInput(_tds);
		}

	}

	// /**
	// * Fhrt die Aktionen hinter der Schaltflche Filtern aus. Es ffnet sich
	// der
	// * Filterdialog.
	// */
	// private void aktionFiltern()
	// {
	// _tds.filtereTabelle(false, true);
	//
	// _tableViewer.setInput(_tds);
	// System.out.println("Filter vor Berarbeitung:\n"
	// + _tds.getTabellenFilter().toString());
	//
	// /*
	// * Filterdialog erzeugen & ffnen
	// */
	// DialogFilterEditor dfe = new DialogFilterEditor(PlatformUI.getWorkbench()
	// .getDisplay().getActiveShell());
	// TabellenFilter filterBearbeitet = dfe.oeffne(_tds.getTabellenFilter());
	// if (filterBearbeitet != null)
	// {
	// /*
	// * Wenn sich etwas am Filter gendert hat, dann die Darstellung der
	// * Tabelle aktualisieren
	// */
	// _tds.setTabellenFilter(filterBearbeitet);
	// _tds.filtereTabelle(false, true);
	// _tableViewer.setInput(_tds);
	// System.out.println("\nFilter nach Bearbeitung:\n"
	// + _tds.getTabellenFilter().toString());
	// }
	// }

	private void aktionFilterVerwaltung()
	{
		DialogFilterVerwaltung dialogFilterVerwaltung = new DialogFilterVerwaltung(
				PlatformUI.getWorkbench().getDisplay().getActiveShell());

		XmlFilterVerwaltung filterVerwaltung = dialogFilterVerwaltung.open(
				_xmlFilter, _tds);
		if (filterVerwaltung != null)
		{
			_xmlFilter = filterVerwaltung;
			_comboViewer.setInput(_xmlFilter.getElementFilter());
			_comboFilterAuswahl.select(0);
			_checkBoxFiltern.setSelection(false);
		}
	}

	@SuppressWarnings("unused")
	private void aktionFilterAnwenden()
	{
		ISelection selection = _comboViewer.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();
			if (firstElement != null)
			{
				if (firstElement instanceof ElementFilter)
				{
					ElementFilter elementFilterSelektiert = (ElementFilter) firstElement;
					System.out.println("Wende folgenden Filter an: "
							+ elementFilterSelektiert);
					filterSetzen(elementFilterSelektiert);
				}
			} else
			{
				System.out.println("Erste Selektion ist NULL");
			}
		}
	}

	/**
	 * Setzt einen {@link ElementFilter} in der {@link TabellenDatenStruktur}.
	 * 
	 * @param elementFilter
	 *          Anzuwendender {@link ElementFilter}.
	 */
	protected void filterSetzen(ElementFilter elementFilter)
	{
		TabellenFilter tabellenFilter = _tds.new TabellenFilter(_tds
				.getAnzahlAttribute(), _tds);
		Iterator<ElementRegEx> it = elementFilter.getRegex().iterator();
		while (it.hasNext())
		{
			ElementRegEx regEx = it.next();
			int id = regEx.getId();
			String stringRegEx = regEx.getRegEx();
			FilterAusdruck filterAusdruck = _tds.new FilterAusdruck(id, stringRegEx,
					_tds);
			tabellenFilter.setFilterAusdruck(id, filterAusdruck);
		}
		_tds.setTabellenFilter(tabellenFilter);
		// _tds.filtereTabelle(false, true);
		// _tableViewer.setInput(_tds);
	}

	/**
	 * Fhrt die Aktionen hinter der Schaltflche Rcksetzen aus. Es werden die
	 * ursprnglichen Daten wieder dargestellt.
	 */
	private void aktionUrsprungsdatenAnzeigen()
	{
		_tds.setDatenQuelle(DatenQuelle.UNGEFILTERT);
		_tableViewer.setInput(_tds);
	}

	/**
	 * Fgt eine Zeile mit festen Werten in die Darstellung ein.
	 */
	private void aktionAenderungsDarstellungFesteDaten()
	{
		// SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		// String wert = sdf.format(new Date());
		// System.out.println("Erzeugt: " + wert);
		// Object[] obj = new Object[] { new Integer(1), new Integer(1),
		// new String(wert), new String("test"), new String("Test
		// nderungsdarstellung") };
		//
		// ZeilenObjekt zo = new ZeilenObjekt(obj);
		//
		// _tds.addZeile(zo);
		//
		// _tableViewer.setInput(_tds);

		ZeilenObjekt systemObjekt = TabellenTest.erzeugeSystemObjekt(
				_rotationIdxAenderungsDarstellung, "Fest A", "Fest B", "Fest C");
		_rotationIdxAenderungsDarstellung = _rotationIdxAenderungsDarstellung + 1;
		if (_rotationIdxAenderungsDarstellung > 1)
		{
			_rotationIdxAenderungsDarstellung = 0;
		}

		_tds.addZeile(systemObjekt);
		_tableViewer.setInput(_tds);
	}

	/**
	 * Fgt eine Zeile mit zuflligen Werten in die Darstellung ein.
	 */
	private void aktionAenderungsDarstellungZufallsDaten()
	{
		int num1 = 1 + Math.abs(ran.nextInt()) % 255;
		int num2 = 1 + Math.abs(ran.nextInt()) % 255;
		int num3 = 1 + Math.abs(ran.nextInt()) % 255;

		ZeilenObjekt systemObjekt = TabellenTest.erzeugeSystemObjekt(
				_rotationIdxAenderungsDarstellung, "Zufall: " + num1, "Zufall: "
						+ (char) num2, "Zufall: " + num3);

		_rotationIdxAenderungsDarstellung = _rotationIdxAenderungsDarstellung + 1;
		if (_rotationIdxAenderungsDarstellung > 1)
		{
			_rotationIdxAenderungsDarstellung = 0;
		}

		_tds.addZeile(systemObjekt);
		_tableViewer.setInput(_tds);

	}

	/**
	 * This method initializes gruppeRechts
	 * 
	 */
	private void erzeugeGruppeRechts()
	{
		GridData gridDataSortieren = new GridData();
		gridDataSortieren.grabExcessHorizontalSpace = false;
		gridDataSortieren.verticalAlignment = GridData.CENTER;
		gridDataSortieren.horizontalAlignment = GridData.FILL;
		GridData gridDataFiltern = new GridData();
		gridDataFiltern.grabExcessHorizontalSpace = false;
		gridDataFiltern.verticalAlignment = GridData.CENTER;
		gridDataFiltern.horizontalAlignment = GridData.FILL;
		GridData gridData1 = new GridData();
		gridData1.grabExcessHorizontalSpace = false;
		gridData1.horizontalAlignment = GridData.CENTER;
		gridData1.verticalAlignment = GridData.FILL;
		gridData1.grabExcessVerticalSpace = true;

		RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
		rowLayout.pack = false;

		gruppeRechts = new Group(bereichUnten, SWT.NONE);
		// gruppeRechts.setLayout(new GridLayout());
		gruppeRechts.setLayout(rowLayout);

		gruppeRechts.setLayoutData(gridData1);
		gruppeRechts.setText("Kontrolle");
		_schaltflaecheFuellen = new Button(gruppeRechts, SWT.NONE);
		_schaltflaecheFuellen.setText("Tabelle Fllen");
		_schaltflaecheFuellen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionTabelleFuellen();
			}			
		});

		/*
		 * Button Sortieren
		 */
		buttonSortierung = new Button(gruppeRechts, SWT.NONE);
		buttonSortierung.setText("Sortierdialog");
		// buttonSortierung.setLayoutData(gridDataSortieren);
		buttonSortierung.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSortierenDialog();
			}			
		});

		_checkSortieren = new Button(gruppeRechts, SWT.CHECK);
		_checkSortieren.setText("Sortieren");
		_checkSortieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (_checkSortieren.getSelection())
				{
					aktionSortierungDurchfuehren(true);
				} else
				{
					aktionSortierungDurchfuehren(false);
				}
			}
		});


		/*
		 * Schaltflche FilterVerwaltung
		 */
		_schaltflaecheFilterVerwaltung = new Button(gruppeRechts, SWT.NONE);
		_schaltflaecheFilterVerwaltung.setText("Filter Verwaltung");
		// _schaltflaecheFilterVerwaltung.setLayoutData(gridDataFiltern);
		_schaltflaecheFilterVerwaltung.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionFilterVerwaltung();
			}			
		});

		/*
		 * Rcksetzen
		 */
		_schaltflaecheRuecksetzen = new Button(gruppeRechts, SWT.NONE);
		_schaltflaecheRuecksetzen.setText("Rcksetzen");
		// _schaltflaecheRuecksetzen.setLayoutData(gridDataFiltern);	
		_schaltflaecheRuecksetzen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionUrsprungsdatenAnzeigen();
			}			
		});

		_checkAenderung = new Button(gruppeRechts, SWT.CHECK);
		_checkAenderung.setText("nderungs Darstellung");
		_checkAenderung.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (_checkAenderung.getSelection())
				{
					_tds.setAenderungsDarstellung(true);
					_tableViewer.setInput(_tds);
				} else
				{
					_tds.setAenderungsDarstellung(false);
					_tableViewer.setInput(_tds);
				}
			}

		});

		/*
		 * nderungsdarstellung-Test mit festen Daten
		 */
		_schaltflaecheTestDatenEinfuegenFest = new Button(gruppeRechts, SWT.NONE);
		_schaltflaecheTestDatenEinfuegenFest
				.setText("nderungsdarstellung Feste Daten");
		_schaltflaecheTestDatenEinfuegenFest.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAenderungsDarstellungFesteDaten();
			}			
		});

		/*
		 * nderungsdarstellung-Test mit zuflligen Daten
		 */
		_schaltflaecheTestDatenEinfuegenZufaellig = new Button(gruppeRechts,
				SWT.NONE);
		_schaltflaecheTestDatenEinfuegenZufaellig
				.setText("nderungsdarstellung Zufllige Daten");
		_schaltflaecheTestDatenEinfuegenZufaellig.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAenderungsDarstellungZufallsDaten();
			}			
		});

		// /*
		// * Filter Speichern
		// */
		// _schaltflaecheFilterSpeichern = new Button(gruppeRechts, SWT.NONE);
		// _schaltflaecheFilterSpeichern.setText("Filter speichern");
		//
		// /*
		// * Filter Laden
		// */

	}

}
