/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.zeitspezifikationselemente;

import java.util.Date;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.kalender.CompositeZeitBereich;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStruktur;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDialog;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.DialogEreignisSystemkalenderUebernahme;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.IEintragBereich;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.IFelderFuellen;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.SchaltflaechenFabrikEreignisSystemKalenderDialog;
import de.bsvrz.sys.funclib.debug.Debug;

public class ViewZeitspezifikationsElemente extends ViewPart
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	public static final String ID = "de.bsvrz.buv.rw.basislib.tests.views.ViewKalenderTest2";

	private Composite _top = null;

	private CompositeZeitBereich _compositeZeitBereich;

	@Override
	public void createPartControl(Composite parent)
	{
		_top = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		_top.setLayout(gridLayout);

		init();
	}

	private void init()
	{
		int stil = SWT.BORDER;

		Label labelDatum = new Label(_top, SWT.NONE);
		labelDatum.setText("Datum:");
		DatumZeit datum = new DatumZeit(_top, stil, DatumZeit.Eingabetyp.datum,
				false, false);
		datum.setToolTipText("Datum mit Auswhler.");

		Label labelDatumZeit = new Label(_top, SWT.NONE);
		labelDatumZeit.setText("Datum und Uhrzeit:");
		DatumZeit datumzeit = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.datumuhr, false, false);
		datumzeit.setToolTipText("Datum + Zeit, mit Auswhler.");

		Label labelZeit = new Label(_top, SWT.NONE);
		labelZeit.setText("Zeit:");
		DatumZeit zeit = new DatumZeit(_top, stil, DatumZeit.Eingabetyp.uhr, false,
				false);
		zeit.setToolTipText("Zeit mit Auswhler.");

		Label labelDatumzeitDigital = new Label(_top, SWT.NONE);
		labelDatumzeitDigital.setText("Datum + Zeit (mit MS):");
		DatumZeit datumzeitDigital = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.datumuhrMS, false, false);
		datumzeitDigital.setToolTipText("Datum + Zeit, mit Auswhler.");

		Label labelZeitDigital = new Label(_top, SWT.NONE);
		labelZeitDigital.setText("Zeit (mit MS):");
		DatumZeit zeitDigital = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.uhrMS, false, false);
		zeitDigital.setToolTipText("Zeit mit Auswhler.");

		Label labelDatumEinfach = new Label(_top, SWT.NONE);
		labelDatumEinfach.setText("Datum (einfach):");
		DatumZeit datumEinfach = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.datumEinfach, false, false);
		datumEinfach.setToolTipText("Datum (einfach)");

		// ////////////
		Label labelDatumZeitEinfach = new Label(_top, SWT.NONE);
		labelDatumZeitEinfach.setText("Datum mit Zeit (einfach):");
		DatumZeit datumzeitEinfach = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.datumuhrEinfach, true, true);
		datumzeitEinfach.setToolTipText("Datum + Zeit (einfach)");

		Label labelZeitEinfach = new Label(_top, SWT.NONE);
		labelZeitEinfach.setText("Zeit (einfach):");
		DatumZeit zeitEinfach = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.uhrEinfach, false, false);
		zeitEinfach.setToolTipText("Zeit (einfach)");

		Label labelDatumzeitEinfachMS = new Label(_top, SWT.NONE);
		labelDatumzeitEinfachMS.setText("Datum mit Zeit (einfach, MS):");
		DatumZeit datumzeitEinfachMS = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.datumuhrEinfachMS, false, false);
		datumzeitEinfachMS
				.setToolTipText("Datum + Zeit (einfach) mit Millisekunden");

		Label labelZeitEinfachMS = new Label(_top, SWT.NONE);
		labelZeitEinfachMS.setText("Zeit (einfach, MS):");
		DatumZeit zeitEinfachMS = new DatumZeit(_top, stil,
				DatumZeit.Eingabetyp.uhrEinfachMS, false, false);
		zeitEinfachMS.setToolTipText("Zeit (einfach) mit Millisekunden");

		Button buttonZeitfilter = new Button(_top, SWT.PUSH);
		buttonZeitfilter.setText("Zeitfiltertest");
		buttonZeitfilter.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				KalenderZeitFilterDialog kzf = new KalenderZeitFilterDialog(PlatformUI
						.getWorkbench().getDisplay().getActiveShell(), SWT.NONE);
				KalenderZeitFilterDatenStruktur struktur = kzf.oeffne(null);
				String string = struktur.erzeugeFilterString();
				_debug.fine(getClass().getName() + "::init \n\t=>FILTER: " + string);
			}
		});

		SelectionListener sl = new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("widgetSelected");
				Object data = e.data;
				if (data instanceof Date)
				{
					Date datum = (Date) data;
					_debug.fine("Datum: " + datum.getTime());
				}

			}

		};

		datumzeit.hinzufuegenSelektionsZuhoerer(sl);

		Button buttonSysErKal = new Button(_top, SWT.PUSH);
		buttonSysErKal.setText("System- und Ereigniskalender");

		buttonSysErKal.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				DialogEreignisSystemkalenderUebernahme dlg = new DialogEreignisSystemkalenderUebernahme(
						PlatformUI.getWorkbench().getDisplay().getActiveShell());

				LinkedList<IEintragBereich> bereiche = dlg.oeffnen();

				if (bereiche != null)
				{
					IEintragBereich bereich = bereiche.getFirst();

					if (bereich != null)
					{
						System.out.println("Bereich (Atomar: " + bereich.istAtomar()
								+ ") - Von: " + bereich.getVonLesbar() + " Bis: "
								+ bereich.getBisLesbar());
					}
				}
			}

		});

		IFelderFuellen felderFuellen = new IFelderFuellen()
		{
			@Override
			public void fuellen(LinkedList<IEintragBereich> bereich)
			{
				if ((bereich != null) && (bereich.size() > 0))
				{
					IEintragBereich first = bereich.getFirst();
					_compositeZeitBereich.setzeStartZeitPunkt(new Date(first.getVon()));
					_compositeZeitBereich.setzeEndZeitPunkt(new Date(first.getBis()));
				}
			}
		};

		SchaltflaechenFabrikEreignisSystemKalenderDialog.erzeugeButton(_top,
				felderFuellen, null, null);
		_compositeZeitBereich = new CompositeZeitBereich(_top, SWT.NONE);
		GridData gridDataZeitbereich = new GridData();
		gridDataZeitbereich.horizontalAlignment = GridData.FILL;
		gridDataZeitbereich.grabExcessHorizontalSpace = true;
		gridDataZeitbereich.verticalAlignment = GridData.CENTER;
		_compositeZeitBereich.setLayoutData(gridDataZeitbereich);

	}

	@Override
	public void setFocus()
	{
	}

}
