/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.zustaende;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.zustaende.Zustaende;

/**
 * ber den Definitionsdialog muss ein Filtereintrag neu angelegt bzw. ein
 * existierender Eintrag gendert werden knnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-10</li>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 14:11:39 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class ZustandDefinitionsdialog extends Dialog
{
  
  /**
   * FilterSpeicher.
   */
  private Zustaende _zustaende = Zustaende.getInstanz();  //  @jve:decl-index=0:
  
//	private EditorRegulaererAusdruck _editorRegulaererAusdruck = null;
  private Shell shell = null;  //  @jve:decl-index=0:visual-constraint="96,108"
  private Composite composite = null;
  private CLabel labelName = null;
  private Text textName = null;
  private String _zustandName = null;
  private String _zustand = null;
  private Group groupFilterDefinitionen = null;
  private Composite compositeFilterDef = null;
  private ScrolledComposite compositeFilterDefScroll = null;
  private Composite compositeBottom = null;
  private Button buttonOk = null;
  private Button buttonAbbrechen = null;
  private Button buttonHilfe = null;
  private Label label = null;
  private Text text = null;
  
  /**
	 * Konstruktor fr eine neue Instanz der Klasse
	 * {@link ZustandDefinitionsdialog}
	 * 
	 * @param parent
	 *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
	 */
	public ZustandDefinitionsdialog(Shell parent)
	{
		super(parent);
	}

	/**
	 * ffnet den Dialog {@link ZustandDefinitionsdialog} zum Anlegen
	 * eines neuen Filters fr Betriebsmeldungen.
	 * 
	 * @return Liefert den erzeugten Filter als Instanz der Klasse
	 *         {@link Filter}.
	 */
	public String oeffne()
	{
	  
	  Shell parent = getParent();
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    shell.setText("Definition ZustandName und Zustand");
    shell.setLayout(new GridLayout());
    createComposite();
    createFilterdefintion();
    createCompositeBottom();
    shell.setSize(new Point(589, 225));

    textName.setEditable(true);

    shell.open();
    
//    
    Display display = parent.getDisplay();
    
    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();
  
	  
		return _zustandName;
	}

	/**
	 * ffnet den Dialog {@link ZustandDefinitionsdialog} zum Bearbeiten
	 * eines bereits erzeugten Filters fr Betriebsmeldungen.
	 * 
	 * @param betriebsMvDefinitionsdialogFilter
	 *          Filter von Betriebsmeldungen, der bearbeitet werden soll.
	 * @return Liefert <code>true</code>, falls der Filter bearbeitet wurde,
	 *         sonst <code>false</code>. Wenn <code>true</code> geliefert
	 *         wird, kann der Verwaltungsdialog Filter Betriebsmeldungen ({@link ZustandVerwaltungsdialog})
	 *         die Darstellung der Liste der Filter aktualisieren.
	 */
	public String oeffne(String name)
	{
	  _zustandName = name;
	  Object zustand = _zustaende.getZustand(_zustandName);
	  _zustand = zustand.toString();
	  

    Shell parent = getParent();
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    shell.setText("Definition Zustand");
    shell.setLayout(new GridLayout());
    createComposite();
    createFilterdefintion();
    createCompositeBottom();
    shell.setSize(new Point(589, 225));
    textName.setEditable(false);
    textName.setText(name);
    text.setText(_zustaende.getZustand(name).toString());

    shell.open();
    
    Display display = parent.getDisplay();
    
    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();
    
	
		return _zustandName;
	}
	
	/**
	 * bernimmt die aktuellen Einstellungen. Dabei wird geprft,
	 * <ul>
	 * <li>ob eine Filterdefinition eingegeben wurde</li>
	 * <li>dass ein Name eingegeben wurde</li>
	 * </ul>
	 * 
	 */
	private void aktionOk()
	{
	  if (!_zustaende.getZustaende().containsKey(_zustandName))
	    _zustaende.hinzufuegenZustand(_zustandName, _zustand);
	  else
	    _zustaende.aendereZustand(_zustandName, _zustand);
	  shell.dispose();
	}

	/**
	 * Schliet den Dialog {@link ZustandDefinitionsdialog} ohne
	 * bernahme der Einstellungen.
	 * 
	 */
	private void aktionAbbrechen()
	{ 
	  shell.dispose();
	}


  /**
   * This method initializes composite	
   *
   */
  private void createComposite()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    composite = new Composite(shell, SWT.NONE);
    composite.setLayoutData(gridData);
    composite.setLayout(gridLayout);
    labelName = new CLabel(composite, SWT.NONE);
    labelName.setText("Zustand Name");
    textName = new Text(composite, SWT.BORDER);
    textName.setLayoutData(gridData1);  
    textName.addModifyListener(new ModifyListener(){
      
      @Override
      public void modifyText(ModifyEvent e)
      {
        _zustandName = textName.getText();
      }});
     
  }

  /**
   * This method initializes Filterdefintion	
   *
   */
  private void createFilterdefintion()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.verticalAlignment = GridData.CENTER;
    groupFilterDefinitionen = new Group(shell, SWT.NONE);
    groupFilterDefinitionen.setLayout(new FillLayout());
    groupFilterDefinitionen.setLayoutData(gridData2);
    createCompositeFilterDef();
    groupFilterDefinitionen.setText("Definition Zustand");
  }

  /**
   * This method initializes compositeFilterDef	
   *
   */
  private void createCompositeFilterDef()
  {
    
    GridData gridData9 = new GridData();
    gridData9.horizontalAlignment = GridData.FILL;
    gridData9.verticalAlignment = GridData.CENTER;
    GridData gridData7 = new GridData();
    gridData7.horizontalAlignment = GridData.FILL;
    gridData7.verticalAlignment = GridData.CENTER;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.FILL;
    gridData6.grabExcessHorizontalSpace = true;
    gridData6.verticalAlignment = GridData.CENTER;
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.grabExcessVerticalSpace = false;
    gridData5.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 3;
    GridData gridData8 = new GridData();
    gridData8.horizontalAlignment = GridData.FILL;
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.verticalAlignment = GridData.CENTER;
   
    compositeFilterDefScroll = new ScrolledComposite(groupFilterDefinitionen, SWT.H_SCROLL | SWT.V_SCROLL);
    
    compositeFilterDef = new Composite(compositeFilterDefScroll, SWT.NONE);
    compositeFilterDef.setLayout(gridLayout1);
    label = new Label(compositeFilterDef, SWT.NONE);
    label.setText("Zustand");
    text = new Text(compositeFilterDef, SWT.BORDER);
    text.setLayoutData(gridData5);
    text.addModifyListener(new ModifyListener(){
    
            @Override
            public void modifyText(ModifyEvent e)
            {
              _zustand = text.getText();
            }
    });
    
    compositeFilterDefScroll.setContent(compositeFilterDef);  
    compositeFilterDef.layout();       
    compositeFilterDefScroll.setLayout(new FillLayout());
    compositeFilterDefScroll.setExpandHorizontal(true);
    compositeFilterDefScroll.setExpandVertical(true);
    
    compositeFilterDefScroll.setMinSize(compositeFilterDef.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    compositeFilterDef.layout();
    
    
    
  }
    

  /**
   * This method initializes compositeBottom	
   *
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(shell, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(compositeBottom, SWT.NONE);
    buttonOk.setText("Ok");    
    buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});
    
    
    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}			
		});
    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
  }

}
