/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import static org.junit.Assert.assertEquals;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * @author HofmannJ
 * 
 */
public class XmlFilterVerwaltungTest
{
	/**
	 * Pfad zur Testdatei. Diese wird, sofern vorhanden, von der tearDown-Methode
	 * wieder entfernt.
	 */
	private static final String _pfadTestDatei = "/C:/xmlFilterVerwaltungTest.xml";

	/**
	 * Nimmt eine XML-Struktur fr Testzwecke auf.
	 */
	private static String _xmlStruktur = "";

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		_xmlStruktur = _xmlStruktur
				+ "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
		_xmlStruktur = _xmlStruktur + "<filterdefinition>\n";
		_xmlStruktur = _xmlStruktur
				+ "\t<filter id=\"0\" bezeichnung=\"Testfilter 0\">\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t<ausdruck id=\"0\" bezeichnung=\"Ausdruck 1\">.*</ausdruck>\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t<ausdruck id=\"1\" bezeichnung=\"Ausdruck 2\">\\d*</ausdruck>\n";
		_xmlStruktur = _xmlStruktur + "\t</filter>\n";
		_xmlStruktur = _xmlStruktur + "</filterdefinition>";
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
		/*
		 * Prfen ob die Testdatei noch besteht und wenn ja, dann wird diese
		 * gelscht.
		 */
		File file = new File(_pfadTestDatei);
		if (file.exists() && file.isFile())
		{
			file.delete();
		}
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#getElementFilter()}.
	 */
	@Test
	public void testGetElementFilter()
	{
		/*
		 * Struktur erzeugen
		 */
		XmlFilterVerwaltung xmlFilter = new XmlFilterVerwaltung();

		ElementFilter elFi0 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 0");

		elFi0.addRegEx(new ElementRegEx(elFi0.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi0.addRegEx(new ElementRegEx(elFi0.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi0);

		ElementFilter elFi1 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 1");

		elFi1.addRegEx(new ElementRegEx(elFi1.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi1.addRegEx(new ElementRegEx(elFi1.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi1);

		assertEquals(true, (xmlFilter.getElementFilter().get(0).getBezeichnung()
				.equals("Testfilter 0"))
				&& (xmlFilter.getElementFilter().get(0).getId() == 0));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#addElementFilter(de.bsvrz.buv.rw.basislib.tabellen.ElementFilter)}.
	 */
	@Test
	public void testAddElementFilter()
	{
		/*
		 * Struktur erzeugen
		 */
		XmlFilterVerwaltung xmlFilter = new XmlFilterVerwaltung();

		ElementFilter elFi0 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 0");

		elFi0.addRegEx(new ElementRegEx(elFi0.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi0.addRegEx(new ElementRegEx(elFi0.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi0);

		ElementFilter elFi1 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 1");

		elFi1.addRegEx(new ElementRegEx(elFi1.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi1.addRegEx(new ElementRegEx(elFi1.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi1);

		assertEquals(true, (xmlFilter.getElementFilter().get(0).getBezeichnung()
				.equals("Testfilter 0"))
				&& (xmlFilter.getElementFilter().get(0).getId() == 0));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#removeElementFilter(de.bsvrz.buv.rw.basislib.tabellen.ElementFilter)}.
	 */
	@Test
	public void testRemoveElementFilter()
	{
		/*
		 * Struktur erzeugen
		 */
		XmlFilterVerwaltung xmlFilter = new XmlFilterVerwaltung();

		ElementFilter elFi0 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 0");
		elFi0.addRegEx(new ElementRegEx(elFi0.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi0.addRegEx(new ElementRegEx(elFi0.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi0);

		ElementFilter elFi1 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 1");
		elFi1.addRegEx(new ElementRegEx(elFi1.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi1.addRegEx(new ElementRegEx(elFi1.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi1);

		ElementFilter elFi2 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 2");

		elFi2.addRegEx(new ElementRegEx(elFi2.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 1", ".*"));
		elFi2.addRegEx(new ElementRegEx(elFi2.bestimmeNaechsteFreieIdRegEx(),
				"Ausdruck 2", "\\d*"));

		xmlFilter.addElementFilter(elFi2);

		boolean nichtVorhanden = true;
		Iterator<ElementFilter> it = xmlFilter.getElementFilter().iterator();
		while (it.hasNext())
		{
			@SuppressWarnings("unused")
			ElementFilter elementFilter = it.next();

		}

		assertEquals(true, nichtVorhanden);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#bestimmeNaechsteFreieIdElementFilter()}.
	 */
	@Test
	public void testBestimmeNaechsteFreieIdElementFilter()
	{
		/*
		 * Struktur erzeugen
		 */
		XmlFilterVerwaltung xmlFilter = new XmlFilterVerwaltung();

		ElementFilter elFi0 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 0");

		xmlFilter.addElementFilter(elFi0);

		ElementFilter elFi1 = new ElementFilter(xmlFilter
				.bestimmeNaechsteFreieIdElementFilter(), "Testfilter 1");

		/*
		 * IDs prfen
		 */
		assertEquals(true, (elFi0.getId() == 0) && (elFi1.getId() == 1));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#einlesenXmlAusDatei(java.lang.String)}.
	 */
	@Test
	public void testEinlesenXmlAusDatei()
	{
		/*
		 * Testdaten schreiben
		 */
		try
		{
			FileWriter fw = new FileWriter(_pfadTestDatei);
			BufferedWriter bw = new BufferedWriter(fw);

			bw.write(_xmlStruktur);

			bw.close();
			fw.close();

		} catch (IOException e)
		{
			e.printStackTrace();
		}

		/*
		 * Filterverwaltung anlegen & Struktur aus Datei einlesen
		 */
		XmlFilterVerwaltung xfv = new XmlFilterVerwaltung();
		xfv.einlesenXmlAusDatei(_pfadTestDatei);

		boolean korrekt = true;
		LinkedList<ElementFilter> elementFilter = xfv.getElementFilter();
		if (elementFilter.size() == 1)
		{
			ElementFilter filter = elementFilter.get(0);
			if ((filter.getId() == 0)
					&& (filter.getBezeichnung().equals("Testfilter 0")))
			{
				LinkedList<ElementRegEx> regex = filter.getRegex();
				if (regex.size() == 2)
				{
					ElementRegEx ausdruck0 = regex.get(0);
					if ((ausdruck0.getId() == 0)
							&& (ausdruck0.getBezeichnung().equals("Ausdruck 1"))
							&& (ausdruck0.getRegEx().equals(".*")))
					{
						ElementRegEx ausdruck1 = regex.get(1);
						if ((ausdruck1.getId() == 1)
								&& (ausdruck1.getBezeichnung().equals("Ausdruck 2"))
								&& (ausdruck1.getRegEx().equals("\\d*")))
						{

						} else
						{
							korrekt = false;
						}
					} else
					{
						korrekt = false;
					}
				}
			} else
			{
				korrekt = false;
			}
		}

		assertEquals(true, korrekt);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#einlesenXmlAusString(java.lang.String)}.
	 */
	@Test
	public void testEinlesenXmlAusString()
	{
		/*
		 * Filterverwaltung anlegen & Struktur aus Datei einlesen
		 */
		XmlFilterVerwaltung xfv = new XmlFilterVerwaltung();
		xfv.einlesenXmlAusString(_xmlStruktur);

		boolean korrekt = true;
		LinkedList<ElementFilter> elementFilter = xfv.getElementFilter();
		if (elementFilter.size() == 1)
		{
			ElementFilter filter = elementFilter.get(0);
			if ((filter.getId() == 0)
					&& (filter.getBezeichnung().equals("Testfilter 0")))
			{
				LinkedList<ElementRegEx> regex = filter.getRegex();
				if (regex.size() == 2)
				{
					ElementRegEx ausdruck0 = regex.get(0);
					if ((ausdruck0.getId() == 0)
							&& (ausdruck0.getBezeichnung().equals("Ausdruck 1"))
							&& (ausdruck0.getRegEx().equals(".*")))
					{
						ElementRegEx ausdruck1 = regex.get(1);
						if ((ausdruck1.getId() == 1)
								&& (ausdruck1.getBezeichnung().equals("Ausdruck 2"))
								&& (ausdruck1.getRegEx().equals("\\d*")))
						{

						} else
						{
							korrekt = false;
						}
					} else
					{
						korrekt = false;
					}
				}
			} else
			{
				korrekt = false;
			}
		}

		assertEquals(true, korrekt);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.XmlFilterVerwaltung#schreibeString(java.lang.String, java.lang.String, boolean)}.
	 */
	@Test
	public void testSchreibeString()
	{
		String testString = "Dies ist der Test-String!";
		XmlFilterVerwaltung xfv = new XmlFilterVerwaltung();
		xfv.schreibeString(_pfadTestDatei, testString, true);

		StringBuffer sb = new StringBuffer();

		try
		{
			BufferedReader br = new BufferedReader(new FileReader(_pfadTestDatei));
			String line;

			try
			{
				while ((line = br.readLine()) != null)
				{
					sb.append(line);
				}
			} catch (IOException e)
			{
				e.printStackTrace();
			}

		} catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}

		assertEquals(testString, sb.toString());
	}

}
