/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * @author HofmannJ
 * 
 */
public class MehrfachZwischenablageDatenTest
{
	private static final String idTest = "de.bsvrz.buv.rw.basislib.mehrfachzwischenablage.MehrfachZwischenablageDatenTestView";

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.mehrfachzwischenablage.MehrfachZwischenablageDaten#selektionHinzufuegen()}.
	 */
	@Test
	public void testSelektionHinzufuegen()
	{
		/**
		 * Prfstrings erzeugen
		 */
		// String[] pruefStrings = new String[] { "1.ster Eintrag", "2.ter Eintrag",
		// "3.ter Eintrag" };
		//		
		LinkedList<String> pruefStrings = new LinkedList<String>();
		pruefStrings.add("1.ster Eintrag");
		pruefStrings.add("2.ter Eintrag");
		pruefStrings.add("3.ter Eintrag");

		/*
		 * Instanz der Mehrfachzwischenablage besorgen
		 */
		MehrfachZwischenablageDaten mzd = MehrfachZwischenablageDaten.getInstanz();

		/*
		 * TestView besorgen
		 */
		MehrfachZwischenablageDatenTestView view = null;

		try
		{
			IViewPart viewPart = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().showView(idTest);

			if (viewPart instanceof MehrfachZwischenablageDatenTestView)
			{
				view = (MehrfachZwischenablageDatenTestView) viewPart;
			}
		} catch (PartInitException e)
		{
			e.printStackTrace();
		}

		if (view != null)
		{
			/*
			 * View mit Testdaten fllen und alle selektieren
			 */
			view.setInput(pruefStrings);
			view.allesSelektieren();

			/*
			 * Die Zwischenablage dazu veranlassen, die letzte Seleketion einzufgen
			 */
			mzd.selektionHinzufuegen();

			List<ZwischenablageDatensatz> liste = mzd.getListe();
			Object[] istWert = liste.toArray();

			LinkedList<String> vergleichsListe = new LinkedList<String>();
			for (Object obj : istWert)
			{
				if (obj instanceof String)
				{
					String wert = (String) obj;
					vergleichsListe.add(wert);
				}
			}

			boolean gleicherInhalt = true;
			if (pruefStrings.size() == vergleichsListe.size())
			{
				for (int i = 0; i < pruefStrings.size(); i++)
				{
					if (!pruefStrings.get(i).equals(vergleichsListe.get(i)))
					{
						gleicherInhalt = false;
						break;
					}
				}
			}

			Warten w = new Warten();
			w.warte(1000);
			try
			{
				PlatformUI
						.getWorkbench()
						.getActiveWorkbenchWindow()
						.getActivePage()
						.showView(
								MehrfachZwischenablageParameter.viewIdSichtMehrfachZwischenablage);
			} catch (PartInitException e)
			{
				e.printStackTrace();
			}

			w.warte(2000);

			assertEquals(true, gleicherInhalt);

		} else
		{
			fail("Die Test-View konnte nicht erzeugt werden!!!");
		}
	}

	@Test
	public void testEinfuegenAusMehrfachZwischenablage()
	{
		MehrfachZwischenablageDaten zwischenablageDaten = MehrfachZwischenablageDaten
				.getInstanz();

		zwischenablageDaten.leereMehrfachZwischenAblage();

		/**
		 * Speichert die Daten, wie sie erwartet werden
		 */
		LinkedList<String> datenSoll = new LinkedList<String>();

		/**
		 * Speichert die Daten, wie sie geliefert werden
		 */
		LinkedList<String> datenIst = new LinkedList<String>();

		/*
		 * Daten erzeugen
		 */
		for (int i = 0; i < 50; i++)
		{
			String wert = "Testeintrag #" + i + " des JUnit-PDE Testes";
			ZwischenablageDatensatz zd = new ZwischenablageDatensatz(wert);
			datenSoll.add(0, wert);
			zwischenablageDaten.hinzufuegenEintrag(zd);
		}

		Warten warten = new Warten();
		warten.warte(1000);

		/*
		 * View, in die eingefgt werden soll, ffnen
		 */
		try
		{
			IViewPart viewPart = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().showView(idTest);

			RoutineKommandoEinfuegen re = new RoutineKommandoEinfuegen();
			
			try
			{
				re.execute(null);

				if (viewPart instanceof MehrfachZwischenablageDatenTestView)
				{
					MehrfachZwischenablageDatenTestView tv = (MehrfachZwischenablageDatenTestView) viewPart;

					Object[] alleTabellenElemente = tv.getAlleTabellenElemente();
					for (Object object : alleTabellenElemente)
					{
						// System.out.println("Element => " + object + " Class: "
						// + object.getClass().getName());
						if (object instanceof String)
						{
							datenIst.add((String) object);
						}
					}
				}

			} catch (ExecutionException e)
			{
				e.printStackTrace();
			}

			warten.warte(1000);
			warten.warte(1000);

			/*
			 * Prfen, ob die Daten in der TestView den erwartenten Daten, welche in
			 * der Zwischenablage eingefgt wurden, entsprechen.
			 */
			boolean datenGleich = true;
			if (datenSoll.size() == datenIst.size())
			{
				for (int i = 0; i < datenSoll.size(); i++)
				{
					if (!datenSoll.get(i).equals(datenIst.get(i)))
					{
						datenGleich = false;
					}
				}
			} else
			{
				datenGleich = false;
			}

			assertEquals(true, datenGleich);
//		assertEquals(true, true);

		} catch (PartInitException e)
		{
			fail("View zum Einfgen von Testdaten konnte nicht geffnet werden.");
		}

	}

	/**
	 * Thread-Klasse, liefert einenb Befehl <code>warten</code>, mit dessen
	 * Hilfe die Ausfhrung des JUnit-Testes pausiert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Warten implements Runnable
	{
		private boolean _laufeThread = false;
		private Long _warteZeit = new Long(1000);

		public void warte(long warteZeit)
		{
			synchronized (_warteZeit)
			{
				_warteZeit = warteZeit;
			}

			startThread();
			synchronized (this)
			{
				try
				{
					wait();
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

		private void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Warten...");
				_laufeThread = true;
				th.start();
			}
		}

		@Override
		public void run()
		{
			if (_laufeThread)
			{
				try
				{
					Thread.sleep(_warteZeit);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				} finally
				{
					synchronized (this)
					{
						notifyAll();
					}
				}
			}

			_laufeThread = false;
		}

	}

}
