/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import static org.junit.Assert.assertEquals;

import java.util.LinkedList;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.FilterAusdruck;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.TabellenFilter;

/**
 * @author HofmannJ
 * 
 */
public class TabellenDatenStrukturTest
{

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur#addZeile(de.bsvrz.buv.rw.basislib.tabellen.ITabellenZeile)}.
	 */
	@Test
	public void testAddZeile()
	{
		TabellenDatenStruktur struktur = new TabellenDatenStruktur(5);

		TestZeile zeile1 = new TestZeile("Zeile1", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile2 = new TestZeile("Zeile2", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile3 = new TestZeile("Zeile3", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile4 = new TestZeile("Zeile4", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile5 = new TestZeile("Zeile5", "Att0", "Att1", "Att2", "Att3",
				"Att4");

		struktur.addZeile(zeile1);
		struktur.addZeile(zeile2);
		struktur.addZeile(zeile3);
		struktur.addZeile(zeile4);
		struktur.addZeile(zeile5);

		LinkedList<ITabellenZeile> zeilen = struktur.getZeilen();

		assertEquals(true, zeilen.get(0) == zeile1 && zeilen.get(1) == zeile2
				&& zeilen.get(2) == zeile3 && zeilen.get(3) == zeile4
				&& zeilen.get(4) == zeile5);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur#getAnzahlZeilen()}.
	 */
	@Test
	public void testGetAnzahlZeilen()
	{
		TabellenDatenStruktur struktur = new TabellenDatenStruktur(5);

		TestZeile zeile1 = new TestZeile("Zeile1", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile2 = new TestZeile("Zeile2", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile3 = new TestZeile("Zeile3", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile4 = new TestZeile("Zeile4", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile5 = new TestZeile("Zeile5", "Att0", "Att1", "Att2", "Att3",
				"Att4");

		struktur.addZeile(zeile1);
		struktur.addZeile(zeile2);
		struktur.addZeile(zeile3);
		struktur.addZeile(zeile4);
		struktur.addZeile(zeile5);

		assertEquals(5, struktur.getAnzahlZeilen());
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur#getAnzahlAttribute()}.
	 */
	@Test
	public void testGetAnzahlAttribute()
	{
		TabellenDatenStruktur struktur = new TabellenDatenStruktur(5);

		TestZeile zeile1 = new TestZeile("Zeile1", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile2 = new TestZeile("Zeile2", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile3 = new TestZeile("Zeile3", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile4 = new TestZeile("Zeile4", "Att0", "Att1", "Att2", "Att3",
				"Att4");
		TestZeile zeile5 = new TestZeile("Zeile5", "Att0", "Att1", "Att2", "Att3",
				"Att4");

		struktur.addZeile(zeile1);
		struktur.addZeile(zeile2);
		struktur.addZeile(zeile3);
		struktur.addZeile(zeile4);
		struktur.addZeile(zeile5);

		assertEquals(5, struktur.getAnzahlAttribute());
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur#sortiereListe(java.util.LinkedList, de.bsvrz.buv.rw.basislib.tabellen.Sortierung[])}.
	 */
	@Test
	public void testSortiereListe()
	{
		TabellenDatenStruktur struktur = new TabellenDatenStruktur(5);

		TestZeile zeile1 = new TestZeile("Zeile1", "Att0Z1", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile2 = new TestZeile("Zeile2", "Att0Z2", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile3 = new TestZeile("Zeile3", "Att0Z3", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile4 = new TestZeile("Zeile4", "Att0Z4", "Att1b", "Att2",
				"Att3", "Att4");
		TestZeile zeile5 = new TestZeile("Zeile5", "Att0Z4", "Att1a", "Att2",
				"Att3", "Att4");

		struktur.addZeile(zeile1);
		struktur.addZeile(zeile2);
		struktur.addZeile(zeile3);
		struktur.addZeile(zeile4);
		struktur.addZeile(zeile5);

		SpaltenElement se0 = new SpaltenElement("Spalte 0");
		SpaltenElement se1 = new SpaltenElement("Spalte 1");
		SpaltenElement se2 = new SpaltenElement("Spalte 2");
		SpaltenElement se3 = new SpaltenElement("Spalte 3");
		SpaltenElement se4 = new SpaltenElement("Spalte 4");

		SpaltenElement[] titel = new SpaltenElement[] { se0, se1, se2, se3, se4 };
		struktur.setSpaltenTitel(titel);

		se0.setAufsteigend(false);
		se1.setAufsteigend(false);

		LinkedList<SpaltenElement> sel = new LinkedList<SpaltenElement>();
		sel.add(se0);
		sel.add(se1);
		struktur.setSortierung(sel);

		struktur.setSortieren(true);

		LinkedList<ITabellenZeile> zeilenSortiert = struktur.getZeilen();

		// Iterator<ITabellenZeile> it = zeilenSortiert.iterator();
		// while (it.hasNext())
		// {
		// ITabellenZeile tabellenZeile = it.next();
		// System.out.println("Obj: " + tabellenZeile.getZeilenObject() + " A0: "
		// + tabellenZeile.getAttribut(0) + " A1: "
		// + tabellenZeile.getAttribut(1));
		// }

		assertEquals(true, zeilenSortiert.get(0).equals(zeile4)
				&& zeilenSortiert.get(1).equals(zeile5)
				&& zeilenSortiert.get(2).equals(zeile3)
				&& zeilenSortiert.get(3).equals(zeile2)
				&& zeilenSortiert.get(4).equals(zeile1));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur#filtereTabelle(java.util.LinkedList)}.
	 */
	@Test
	public void testFiltereTabelle()
	{
		/*
		 * Struktur erzeugen
		 */
		TabellenDatenStruktur struktur = new TabellenDatenStruktur(5);

		TestZeile zeile1 = new TestZeile("Zeile1", "Att0Z1", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile2 = new TestZeile("Zeile2", "Att0Z2", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile3 = new TestZeile("Zeile3", "Att0Z3", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile4 = new TestZeile("Zeile4", "Att0Z4", "Att1b", "Att2",
				"Att3", "Att4");
		TestZeile zeile5 = new TestZeile("Zeile5", "Att0Z4", "Att1a", "Att2",
				"Att3", "Att4");

		struktur.addZeile(zeile1);
		struktur.addZeile(zeile2);
		struktur.addZeile(zeile3);
		struktur.addZeile(zeile4);
		struktur.addZeile(zeile5);

		SpaltenElement se0 = new SpaltenElement("Spalte 0");
		SpaltenElement se1 = new SpaltenElement("Spalte 1");
		SpaltenElement se2 = new SpaltenElement("Spalte 2");
		SpaltenElement se3 = new SpaltenElement("Spalte 3");
		SpaltenElement se4 = new SpaltenElement("Spalte 4");

		SpaltenElement[] titel = new SpaltenElement[] { se0, se1, se2, se3, se4 };
		struktur.setSpaltenTitel(titel);

		/*
		 * Filter erzeugen
		 */
		TabellenFilter fi = struktur.new TabellenFilter(struktur
				.getAnzahlAttribute(), struktur);
		FilterAusdruck filterAusdruck = fi.getFilterAusdruck(0);
		filterAusdruck.setFilter("Att0Z4");

		struktur.setTabellenFilter(fi);

		struktur.setGefiltert(true);

		/*
		 * Gefilterte Zeilen holen und prfen
		 */
		LinkedList<ITabellenZeile> zeilen = struktur.getZeilen();
		// Iterator<ITabellenZeile> it = zeilen.iterator();
		// while (it.hasNext())
		// {
		// ITabellenZeile next = it.next();
		// System.out.println("Obj: " + next.getZeilenObject() + " A0: "
		// + next.getAttribut(0));
		// }

		assertEquals(true, zeilen.get(0).equals(zeile4)
				&& zeilen.get(1).equals(zeile5));
	}

	/**
	 * Testet die nderungsdarstellung
	 */
	@Test
	public void testAenderungsDarstellung()
	{
		/*
		 * Struktur erzeugen
		 */
		TabellenDatenStruktur struktur = new TabellenDatenStruktur(5);

		TestZeile zeile1 = new TestZeile("ZeilenObjekt1", "Att0T1", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile2 = new TestZeile("ZeilenObjekt1", "Att0T1", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile3 = new TestZeile("ZeilenObjekt1", "Att0T2", "Att1", "Att2",
				"Att3", "Att4");
		TestZeile zeile4 = new TestZeile("ZeilenObjekt2", "Att0Z1", "Att1b",
				"Att2", "Att3", "Att4");
		TestZeile zeile5 = new TestZeile("ZeilenObjekt2", "Att0Z1", "Att1b",
				"Att2", "Att3", "Att4");
		TestZeile zeile6 = new TestZeile("ZeilenObjekt2", "Att0Z2", "Att1b",
				"Att2", "Att3", "Att4");

		struktur.addZeile(zeile1);
		struktur.addZeile(zeile2);
		struktur.addZeile(zeile3);
		struktur.addZeile(zeile4);
		struktur.addZeile(zeile5);
		struktur.addZeile(zeile6);

		SpaltenElement se0 = new SpaltenElement("Spalte 0");
		SpaltenElement se1 = new SpaltenElement("Spalte 1");
		SpaltenElement se2 = new SpaltenElement("Spalte 2");
		SpaltenElement se3 = new SpaltenElement("Spalte 3");
		SpaltenElement se4 = new SpaltenElement("Spalte 4");

		SpaltenElement[] titel = new SpaltenElement[] { se0, se1, se2, se3, se4 };
		struktur.setSpaltenTitel(titel);

		/*
		 * nderungsdarstellugn holen und auswerten
		 */
		struktur.setAenderungsDarstellung(true);

		LinkedList<ITabellenZeile> zeilen = struktur.getZeilen();

//		System.out.println("SIZE: " + zeilen.size());
//		Iterator<ITabellenZeile> it = zeilen.iterator();
//		while (it.hasNext())
//		{
//			ITabellenZeile tabellenZeile = it.next();
//			System.out.println("=> Obj: " + tabellenZeile.getZeilenObject()
//					+ " Attribut1: " + tabellenZeile.getAttribut(0));
//		}

		assertEquals(true, (zeilen.get(0) == zeile1)
				&& (zeilen.get(1).equals(zeile3)) && (zeilen.get(2).equals(zeile4))
				&& (zeilen.get(3).equals(zeile6)));
	}

	/**
	 * Test-Implementierung einer {@link ITabellenZeile}. Ist fest mit 5
	 * Attributen vorbelegt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class TestZeile implements ITabellenZeile
	{
		/**
		 * Objekt der Zeile
		 */
		private String _objekt = null;

		/**
		 * Attribute in Form von Strings
		 */
		private String[] _attribute = new String[5];

		/**
		 * Zur bercksichtigende Attribute
		 */
		private int[] _zuBeruecksichtigendeAttribute = null;

		/**
		 * bernimmt ZeilenObjekt und Strings der 5 Beispielattribute
		 * 
		 * @param objekt
		 *          Objekt der Zeile
		 * @param attribut0
		 *          0. Attribut
		 * @param attribut1
		 *          1. Attribut
		 * @param attribut2
		 *          2. Attribut
		 * @param attribut3
		 *          3. Attribut
		 * @param attribut4
		 *          4. Attribut
		 */
		public TestZeile(String objekt, String attribut0, String attribut1,
				String attribut2, String attribut3, String attribut4)
		{
			_objekt = objekt;

			_attribute[0] = attribut0;
			_attribute[1] = attribut1;
			_attribute[2] = attribut2;
			_attribute[3] = attribut3;
			_attribute[4] = attribut4;
		}

		@Override
		public int getAnzahlAttribute()
		{
			return _attribute.length;
		}

		@Override
		public Object getAttribut(int idx)
		{
			if ((idx >= 0) && (idx < getAnzahlAttribute()))
			{
				return _attribute[idx];
			} else
			{
				return null;
			}
		}

		@Override
		public Object getZeilenObject()
		{
			return _objekt;
		}

		@Override
		public int[] getZuBeruecksichtigendeAttribute()
		{
			/*
			 * Bercksichtigen aller Attribute fr die nderungsdarstellung
			 */
			_zuBeruecksichtigendeAttribute = new int[] { 0, 1, 2, 3, 4 };

			return _zuBeruecksichtigendeAttribute;
		}

		@Override
		public int compareTo(ITabellenZeile o)
		{
			int re = 0;

			boolean ungleich = false;
			if (this.getAnzahlAttribute() == o.getAnzahlAttribute())
			{
				for (int i = 0; i < this.getAnzahlAttribute(); i++)
				{
					if (!o.getAttribut(i).equals(this.getAttribut(i)))
					{
						ungleich = true;
						break;
					}
				}

				if (ungleich)
				{
					re = 1;
				}
			}

			return re;
		}

	}

}
