/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.datenidentifikation;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Klasse Datenidentifikation ermglicht die Auswahl von Objekten ber
 * folgende Auswahllisten: <br>
 * - Objekt-Liste<br>
 * - Objekttyp-Liste<br>
 * - Attributgruppen-Liste<br>
 * - Aspekt-Liste<br>
 * - Simulations-Variante<br>
 * 
 * <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-96 und 97
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 13:15:29 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 */
public class Datenidentifikation extends Dialog
{

	// Links
	private Composite _compositeLinks = null;

	private Composite _compositeLinksObjekttypTitel = null;

	private Composite _compositeLinksObjekttyp = null;

	private Composite _compositeLinksAttributgruppeTitel = null;

	private Composite _compositeLinksAttributgruppe = null;

	private Composite _compositeLinksAspektTitel = null;

	private Composite _compositeLinksAspekt = null;

	private Composite _compositeLinksSimulationsvariante = null;

	private Button _schaltflaecheLoescheObjekttyp = null;

	private Text _textAnzahlObjekttyp = null;

	private Button _schaltflaecheLoescheAttributgruppe = null;

	private Text _textAnzahlAttributgruppe = null;

	private Button _schaltflaecheLoescheAspekt = null;

	private Text _textAnzahlAspekte = null;

	private Spinner _spinnerSimulationsvariante = null;

	// Rechts
	private Composite _compositeRechts = null;

	private Composite _compositeRechtsOben = null;

	private Composite _compositeRechtsMitte = null;

	private Composite _compositeRechtsUnten = null;

	private Text _textAnzahlObjekte = null;

	private Button _schaltflaecheLoescheSelektierteObjekte = null;

	/**
	 * OK Schaltflche
	 */
	private Button _schaltflaecheOk = null;

	/**
	 * Abbrechen Schaltflche
	 */
	private Button _schaltflaecheAbbrechen = null;

	/**
	 * Hilfe Schaltflche
	 */
	private Button _schaltflaecheHilfe = null;

	// Rechts InfoBereich
	private Composite _compositeRechtsInfoBereich = null;

	private Group _gruppeInfo = null;

	private Text _infoText = null;

	// SWT-Listen
	/**
	 * Enthlt die Darstellung der ObjektTypen-Liste
	 */
	private List _listeSWTObjekttypListe = null;

	/**
	 * Enthlt die Darstellung der Attribut-Liste
	 */
	private List _listeSWTAttributListe = null;

	/**
	 * Enthlt die Darstellung der Aspekt-Liste
	 */
	private List _listeSWTAspektListe = null;

	/**
	 * Enthlt die Darstellung der Obhekt-Liste
	 */
	private List _listeSWTObjektListe = null;

	/*
	 * TreeMaps fr die Objektdatenhaltung
	 */

	/**
	 * TreeMap fr die Objekte
	 */
	private TreeMap<String, ListenElementObjekt> _treeMapObjekt = null;

	/**
	 * TreeMap fr die aktuelle Ansicht der Objekte
	 */
	@SuppressWarnings("unused")
	private TreeMap<String, ListenElementObjekt> _treeMapObjektAktuell = null;

	/**
	 * TreeMap fr die ObjektTypen
	 */
	private TreeMap<String, ListenElementObjektTyp> _treeMapObjektTyp = null;

	/**
	 * TreeMap der Attributgruppen
	 */
	private TreeMap<String, ListenElementAttributGruppe> _treeMapAttributgruppe = null;

	/**
	 * TreeMap der Aspekte
	 */
	private TreeMap<String, ListenElementAspekt> _treeMapAspekt = null;

	/*
	 * Die Referenzierung bezieht sich an dieser Stelle immer auf das
	 * list-Interface
	 */

	// /**
	// * Array der Selektionen in der Liste Objekttypen
	// */
	// private int[] _selektionObjekttyp = null;
	//
	// /**
	// * Array der Selektionen in der Liste Attributgruppe
	// */
	// private int[] _selektionAttributgruppe = null;
	//
	// /**
	// * Array der Selektionen in der Liste Aspekte
	// */
	// private int[] _selektionAspekt = null;
	//
	// /**
	// * Array der Selektionen in der Liste Objekte
	// */
	// private int[] _selektionObjekte = null;

	/**
	 * In diesem Array werden sie selektierten SystemObjekte abgelegt und vom
	 * Dialog zurckgegeben.
	 */
	private SystemObject[] _systemObjectAuswahl = null;

	private Shell _shell = null;

	/**
	 * Verbindung zum Datenverteiler
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Breite des Dialogs
	 */
	private static final int _breite = 700;

	/**
	 * Hhe des Dialogs
	 */
	private static final int _hoehe = 900;

	/**
	 * Font fr die Lschen-Buttons (X)
	 */
	private Font _buttonFont = new Font(Display.getCurrent(), new FontData(
			"Arial", 8, SWT.BOLD));

	/**
	 * Enumerator, gibt an was dargestellt werden soll.
	 */
	// private Darstellung _darstellung = null;

	/**
	 * Standard SWT-Dialog Konstruktor
	 * 
	 * @param parent
	 *          Parent Shell, von der ausgehend die Darstellung generiert werden
	 *          soll.
	 */
	public Datenidentifikation(Shell parent, ClientDavInterface con)
	{
		super(parent);
		this._connection = con;
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation.
	 * 
	 * @param darstellung
	 *          Gibt an, welche Bestandteile der Datenidentifikation angezeigt
	 *          werden sollen.
	 * @return SystemObject-Array mit den selektierten Objekten.
	 */
	public SystemObject[] oeffne(Darstellung darstellung)
	{
		// this._darstellung = darstellung;

		Shell parent = getParent();
		_shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL
				| SWT.RESIZE | SWT.MAX);
		_shell.setText("Datenidentifikations-Eingabe");
		GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 2;
		_shell.setLayout(gridLayout);

		_shell.setSize(_breite, _hoehe);
		erzeugeGui();
		initialisiere();

		// _shell.pack();
		_shell.open();

		Display display = parent.getDisplay();

		// Dialog zentrieren
		Monitor primary = display.getPrimaryMonitor();
		Rectangle bounds = primary.getBounds();
		Rectangle rect = _shell.getBounds();
		int x = bounds.x + (bounds.width - rect.width) / 2;
		int y = bounds.y + (bounds.height - rect.height) / 2;
		_shell.setLocation(x, y);

		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		// return true;

		/*
		 * Hier wird die Auswahl der selektierten Objekte in einem Array von
		 * SystemObjekten zurckgegeben.
		 */
		return _systemObjectAuswahl;
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation, jedoch werden nur die
	 * ObjektTypen nagezeigt, die vom ObjektTyp des bergebenen Filters sind.
	 * 
	 * @param sObjektTypFilter
	 *          Gibt an, welche Objekte von welchem Typ angezeigt werden sollen.
	 * @param darstellung
	 *          Gibt an, welche Bestandteile der Datenidentifikation angezeigt
	 *          werden sollen.
	 * @return SystemObject-Array mit den selektierten Objekten.
	 */
	public SystemObject[] oeffne(String sObjektTypFilter[],
			Darstellung darstellung)
	{
		// this._darstellung = darstellung;

		Shell parent = getParent();
		_shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL
				| SWT.RESIZE | SWT.MAX);
		_shell.setText("Datenidentifikations-Eingabe");
		GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 2;
		_shell.setLayout(gridLayout);

		_shell.setSize(_breite, _hoehe);

		erzeugeGui();
		initialisiere();

		// Lschen aller nicht bentigter ObjektTypen aus TreeMapObjektTyp

		entferneObjektTypen(sObjektTypFilter);

		// Aufruf von fuellelisteSWTObjekteFiltered2
		_listeSWTObjekttypListe.removeAll();
		fuellelisteSWTObjektTyp();

		fuellelisteSWTObjekteGefiltert();

		/*
		 * Initiale Darstellung der Objekte zu den vorhandenen Objekttypen.
		 */
		fuellelisteSWTObjekteFiltered(_listeSWTObjekttypListe.getItems());

		// _shell.pack();
		_shell.open();

		Display display = parent.getDisplay();
		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		// return true;
		return _systemObjectAuswahl;
	}

	/**
	 * Erstellt die Basis des SWT-Dialogs
	 * 
	 */
	private void erzeugeGui()
	{
		Composite flaeche = new Composite(_shell, SWT.NONE);
		flaeche.setLayout(new FillLayout());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		flaeche.setLayoutData(gridData);

		SashForm formBasis = new SashForm(flaeche, SWT.HORIZONTAL);

		erzeugeCompositeLinks(formBasis);
		erzeugeCompositeRechts(formBasis);

		formBasis.setWeights(new int[] { 50, 50 });

		erzeugeCompositeRechtsUnten(_shell);

	}

	/**
	 * Initialisiert die bentigten TreeMaps und startet die Basisfllung der
	 * Datenstrukturen.
	 * 
	 */
	private void initialisiere()
	{
		// TreeMaps initialisieren
		_treeMapObjekt = new TreeMap<String, ListenElementObjekt>();
		_treeMapObjektAktuell = new TreeMap<String, ListenElementObjekt>();
		_treeMapObjektTyp = new TreeMap<String, ListenElementObjektTyp>();
		_treeMapAttributgruppe = new TreeMap<String, ListenElementAttributGruppe>();
		_treeMapAspekt = new TreeMap<String, ListenElementAspekt>();

		if (_connection != null)
		{
			fuelleAlles();
		}

	}

	/**
	 * Generiert das linke Composite
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinks(Composite parent)
	{
		_compositeLinks = new Composite(parent, SWT.BORDER);

		// GridLayout gridLayout = new GridLayout();
		// _compositeLinks.setLayout(gridLayout);
		// GridData gridData = new GridData();
		// gridData.horizontalAlignment = GridData.CENTER;
		// gridData.verticalAlignment = GridData.BEGINNING;
		// _compositeLinks.setLayoutData(gridData);

		_compositeLinks.setLayout(new FillLayout());

		SashForm formLinks = new SashForm(_compositeLinks, SWT.VERTICAL);
		Composite obj = new Composite(formLinks, SWT.NONE);
		Composite atg = new Composite(formLinks, SWT.NONE);
		Composite asp = new Composite(formLinks, SWT.NONE);
		Composite simvar = new Composite(formLinks, SWT.NONE);
		formLinks.setWeights(new int[] { 30, 30, 30, 10 });

		FillLayout fillLayout = new FillLayout();
		obj.setLayout(fillLayout);
		atg.setLayout(fillLayout);
		asp.setLayout(fillLayout);
		simvar.setLayout(fillLayout);

		Composite objComplex = new Composite(obj, SWT.NONE);
		objComplex.setLayout(new GridLayout());
		erzeugeCompositeLinksObjekttypTitel(objComplex);
		erzeugeCompositeLinksObjekttyp(objComplex);

		Composite atgComplex = new Composite(atg, SWT.NONE);
		atgComplex.setLayout(new GridLayout());
		erzeugeCompositeLinksAttributgruppeTitel(atgComplex);
		erzeugeCompositeLinksAttributgruppe(atgComplex);

		Composite aspComplex = new Composite(asp, SWT.NONE);
		aspComplex.setLayout(new GridLayout());
		erzeugeCompositeLinksAspektTitel(aspComplex);
		erzeugeCompositeLinksAspekt(aspComplex);

		Composite simvarCompex = new Composite(simvar, SWT.NONE);
		simvarCompex.setLayout(new GridLayout());
		erzeugeCompositeLinksSimulationsVariante(simvarCompex);

		// GridData gridDataOben = new GridData();
		// gridDataOben.horizontalAlignment = GridData.FILL;
		// gridDataOben.grabExcessHorizontalSpace = true;
		// gridDataOben.verticalAlignment = GridData.CENTER;
		// compositeTop.setLayout(new RowLayout(SWT.HORIZONTAL));
		// compositeTop.setLayoutData(gridDataOben);
		// Label label = new Label(compositeTop, SWT.NONE);
		// label.setText("Infotext");
		// Button button = new Button(compositeTop, SWT.PUSH);
		// button.setText("Testbutton");

		// GridData gridDataUnten = new GridData();
		// gridDataUnten.horizontalAlignment = GridData.FILL;
		// gridDataUnten.grabExcessHorizontalSpace = true;
		// gridDataUnten.grabExcessVerticalSpace = true;
		// gridDataUnten.verticalAlignment = GridData.FILL;
		// //compositeBottom.setLayout(new GridLayout());
		// compositeBottom.setLayout(new FillLayout());
		// compositeBottom.setLayoutData(gridDataUnten);
		// fillListGenerator(compositeBottom);

		// Composite compAtgOben = new Composite(atg, SWT.BORDER);
		// compAtgOben.setLayout(new GridLayout());
		// erzeugeCompositeLinksAttributgruppeTitel(compAtgOben);

		// Composite compAtgUnten = new Composite(atg, SWT.BORDER);
		// compAtgUnten.setLayout(new GridLayu());
		// fillListGenerator(compAtgUnten);

		// erzeugeCompositeLinksObjekttypTitel(_compositeLinks);
		// erzeugeCompositeLinksObjekttyp(_compositeLinks);
		// erzeugeCompositeLinksAttributgruppeTitel(_compositeLinks);
		// erzeugeCompositeLinksAttributgruppe(_compositeLinks);
		// erzeugeCompositeLinksAspektTitel(_compositeLinks);
		// erzeugeCompositeLinksAspekt(_compositeLinks);
		// erzeugeCompositeLinksSimulationsVariante(_compositeLinks);
		//		
		// GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 1;
		// _compositeLinks.setLayout(gridLayout);
		//		
		// GridData gridData = new GridData();
		// gridData.horizontalAlignment = GridData.CENTER;
		// gridData.verticalAlignment = GridData.BEGINNING;
		// _compositeLinks.setLayoutData(gridData);

	}

	/**
	 * Generiert den Objekttyp-Titel im linken Composite
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinksObjekttypTitel(Composite parent)
	{

		_compositeLinksObjekttypTitel = new Composite(parent, SWT.NONE);

		Composite compositeLinksObjekttypTitelLinks = new Composite(
				_compositeLinksObjekttypTitel, SWT.NONE);
		Composite compositeLinksObjekttypTitelRechts = new Composite(
				_compositeLinksObjekttypTitel, SWT.NONE);

		Label labelObjekttyp = new Label(compositeLinksObjekttypTitelLinks,
				SWT.NONE);
		labelObjekttyp.setText("Objekttyp");

		_textAnzahlObjekttyp = new Text(compositeLinksObjekttypTitelRechts,
				SWT.BORDER);
		_textAnzahlObjekttyp.setText("0 / 0");
		_textAnzahlObjekttyp.setEditable(false);
		_schaltflaecheLoescheObjekttyp = new Button(
				compositeLinksObjekttypTitelRechts, SWT.PUSH);

		_schaltflaecheLoescheObjekttyp.setText("X");
		_schaltflaecheLoescheObjekttyp.setFont(_buttonFont);

		// Image imageDelete = new Image(_compositeLinksObjekttypTitel.getDisplay(),
		// Activator.class
		// .getResourceAsStream("/icons/actions/editdelete16x16.png"));
		// _buttonLoescheObjekttyp.setImage(imageDelete);

		_schaltflaecheLoescheObjekttyp.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{

				_listeSWTObjekttypListe.deselectAll();
				_listeSWTObjektListe.setVisible(false);
				_listeSWTObjektListe.removeAll();
				fuellelisteSWTObjekte();
				// listeSWTObjekttypListe.selectAll();
				// fuellelisteSWTObjekteFiltered2();
				// listeSWTObjekttypListe.deselectAll();
				_listeSWTObjektListe.setVisible(true);
				fuelleTreeMapAttributgruppeFiltered(_listeSWTObjekttypListe
						.getSelection());
			}
		});

		GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 3;
		gridLayout.numColumns = 2;
		_compositeLinksObjekttypTitel.setLayout(gridLayout);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_compositeLinksObjekttypTitel.setLayoutData(gridData);

		compositeLinksObjekttypTitelLinks.setLayout(new GridLayout());
		// compositeLinksObjekttypTitelRechts

		GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = true;
		rowLayoutRechts.justify = false;
		compositeLinksObjekttypTitelRechts.setLayout(rowLayoutRechts);
		compositeLinksObjekttypTitelRechts.setLayoutData(gridDataRechts);
	}

	/**
	 * Generiert das linke Composite, dass die SWT-Liste mit der Darstellung der
	 * vorhandenen ObjektTypen enthlt.
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinksObjekttyp(Composite parent)
	{
		_compositeLinksObjekttyp = new Composite(parent, SWT.NONE);

		_listeSWTObjekttypListe = new List(_compositeLinksObjekttyp, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL);
		_listeSWTObjekttypListe.setBounds(0, 0, 300, 200);

		_listeSWTObjekttypListe.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				if (_listeSWTObjekttypListe.getSelectionIndex() > -1)
				{
					// _selektionObjekttyp =
					// _listeSWTObjekttypListe.getSelectionIndices();

					berechneAnzahlObjekttypen();

					_listeSWTAttributListe.removeAll();
					_listeSWTAspektListe.removeAll();

					fuellelisteSWTObjekteFiltered(_listeSWTObjekttypListe.getSelection());
					fuelleTreeMapAttributgruppeFiltered(_listeSWTObjekttypListe
							.getSelection());

				}
			}
		});

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_compositeLinksObjekttyp.setLayout(new FillLayout());
		_compositeLinksObjekttyp.setLayoutData(gridData);

	}

	/**
	 * Generiert das linke Composite, dass den Attributgruppen-Titel enthlt
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinksAttributgruppeTitel(Composite parent)
	{

		_compositeLinksAttributgruppeTitel = new Composite(parent, SWT.NONE);

		Composite compositeLinksAttributgruppeTitelLinks = new Composite(
				_compositeLinksAttributgruppeTitel, SWT.NONE);
		Composite compositeLinksAttributgruppeTitelRechts = new Composite(
				_compositeLinksAttributgruppeTitel, SWT.NONE);

		Label labelAttributgruppe = new Label(
				compositeLinksAttributgruppeTitelLinks, SWT.NONE);
		labelAttributgruppe.setText("Attributgruppe");

		_textAnzahlAttributgruppe = new Text(
				compositeLinksAttributgruppeTitelRechts, SWT.BORDER);
		_textAnzahlAttributgruppe.setEditable(false);
		_textAnzahlAttributgruppe.setText("0 / 0");

		_schaltflaecheLoescheAttributgruppe = new Button(
				compositeLinksAttributgruppeTitelRechts, SWT.PUSH);
		_schaltflaecheLoescheAttributgruppe.setText("X");
		_schaltflaecheLoescheAttributgruppe.setFont(_buttonFont);

		// Image imageDelete = new Image(_compositeLinksObjekttypTitel.getDisplay(),
		// Activator.class
		// .getResourceAsStream("/icons/actions/editdelete16x16.png"));
		// _buttonLoescheAttributgruppe.setImage(imageDelete);

		_schaltflaecheLoescheAttributgruppe
				.addSelectionListener(new SelectionAdapter()
				{
					@Override
					public void widgetSelected(SelectionEvent e)
					{
						_listeSWTAttributListe.deselectAll();
					}
				});

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		_compositeLinksAttributgruppeTitel.setLayout(gridLayout);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_compositeLinksAttributgruppeTitel.setLayoutData(gridData);

		compositeLinksAttributgruppeTitelLinks.setLayout(new GridLayout());

		GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = true;
		rowLayoutRechts.justify = false;

		compositeLinksAttributgruppeTitelRechts.setLayout(rowLayoutRechts);
		compositeLinksAttributgruppeTitelRechts.setLayoutData(gridDataRechts);

	}

	/**
	 * Generiert das linke Composite, dass die Darstellung der Liste der
	 * Attributgruppen enthlt.
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinksAttributgruppe(Composite parent)
	{
		_compositeLinksAttributgruppe = new Composite(parent, SWT.NONE);
		// compositeObjekttyp.setLayout(gridLayout);
		_listeSWTAttributListe = new List(_compositeLinksAttributgruppe, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL);
		_listeSWTAttributListe.setBounds(0, 0, 300, 200);

		_listeSWTAttributListe.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				if (_listeSWTAttributListe.getSelectionIndex() > -1)
				{
					// _selektionAttributgruppe = _listeSWTAttributListe
					// .getSelectionIndices();

					berechneAnzahlAttribute();

					fuelleTreeMapAspekteFiltered(_listeSWTAttributListe.getSelection());
					fuellelisteSWTObjekteGefiltert();
				}

			}
		});

		GridData gridDataUnten = new GridData();
		gridDataUnten.horizontalAlignment = GridData.FILL;
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.grabExcessVerticalSpace = true;
		gridDataUnten.verticalAlignment = GridData.FILL;
		_compositeLinksAttributgruppe.setLayout(new FillLayout());
		_compositeLinksAttributgruppe.setLayoutData(gridDataUnten);

	}

	/**
	 * Generiert den Titel des linken Aspekt-Composites
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinksAspektTitel(Composite parent)
	{

		_compositeLinksAspektTitel = new Composite(parent, SWT.NONE);

		Composite compositeLinksAspektTitelLinks = new Composite(
				_compositeLinksAspektTitel, SWT.NONE);
		Composite compositeLinksAspektTitelRechts = new Composite(
				_compositeLinksAspektTitel, SWT.NONE);

		Label labelAspektTitel = new Label(compositeLinksAspektTitelLinks, SWT.NONE);
		labelAspektTitel.setText("Aspekt");

		_textAnzahlAspekte = new Text(compositeLinksAspektTitelRechts, SWT.BORDER);
		_textAnzahlAspekte.setText("0 / 0");
		_textAnzahlAspekte.setEditable(false);

		_schaltflaecheLoescheAspekt = new Button(compositeLinksAspektTitelRechts,
				SWT.PUSH);
		_schaltflaecheLoescheAspekt.setText("X");
		_schaltflaecheLoescheAspekt.setFont(_buttonFont);

		// Image imageDelete = new Image(_compositeLinksObjekttypTitel.getDisplay(),
		// Activator.class
		// .getResourceAsStream("/icons/actions/editdelete16x16.png"));
		// _buttonLoescheAspekt.setImage(imageDelete);

		_schaltflaecheLoescheAspekt.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_listeSWTAspektListe.deselectAll();
			}
		});

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		_compositeLinksAspektTitel.setLayout(gridLayout);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_compositeLinksAspektTitel.setLayoutData(gridData);

		compositeLinksAspektTitelLinks.setLayout(new GridLayout());

		GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = false;
		rowLayoutRechts.justify = false;
		compositeLinksAspektTitelRechts.setLayout(rowLayoutRechts);
		compositeLinksAspektTitelRechts.setLayoutData(gridDataRechts);

	}

	/**
	 * Generiert das linke Composite, dass die Darstellung der Liste der Aspekte
	 * enthlt.
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeLinksAspekt(Composite parent)
	{
		_compositeLinksAspekt = new Composite(parent, SWT.NONE);
		// compositeObjekttyp.setLayout(gridLayout);
		_listeSWTAspektListe = new List(_compositeLinksAspekt, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL);
		_listeSWTAspektListe.setBounds(0, 0, 300, 200);

		_listeSWTAspektListe.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{

				if (_listeSWTAspektListe.getSelectionIndex() > -1)
				{
					// _selektionAspekt = _listeSWTAspektListe.getSelectionIndices();

					berechneAnzahlAspekte();
					fuellelisteSWTObjekteGefiltert();
				}

			}
		});

		GridData gridDataUnten = new GridData();
		gridDataUnten.horizontalAlignment = GridData.FILL;
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.grabExcessVerticalSpace = true;
		gridDataUnten.verticalAlignment = GridData.FILL;
		_compositeLinksAspekt.setLayout(new FillLayout());
		_compositeLinksAspekt.setLayoutData(gridDataUnten);

	}

	/**
	 * Generiert das linke Composite, dass die Darstellung des
	 * Simulationsvarianten-Spinners enthlt.
	 * 
	 * @param parent
	 */
	private void erzeugeCompositeLinksSimulationsVariante(Composite parent)
	{

		_compositeLinksSimulationsvariante = new Composite(parent, SWT.NONE);
		Label labelSimulationsVariante = new Label(
				_compositeLinksSimulationsvariante, SWT.NONE);
		labelSimulationsVariante.setText("Simulationsvariante:");

		_spinnerSimulationsvariante = new Spinner(
				_compositeLinksSimulationsvariante, SWT.BORDER);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		_compositeLinksSimulationsvariante.setLayout(gridLayout);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;

		_compositeLinksSimulationsvariante.setLayoutData(gridData);

		GridData gridDataSpinnerSimulationsvariante = new GridData();
		gridDataSpinnerSimulationsvariante.horizontalAlignment = GridData.FILL;
		gridDataSpinnerSimulationsvariante.grabExcessHorizontalSpace = true;
		gridDataSpinnerSimulationsvariante.verticalAlignment = GridData.CENTER;

		_spinnerSimulationsvariante
				.setLayoutData(gridDataSpinnerSimulationsvariante);
	}

	private void erzeugeCompositeRechts(Composite parent)
	{

		_compositeRechts = new Composite(parent, SWT.BORDER);
		_compositeRechts.setLayout(new FillLayout());

		SashForm formRechts = new SashForm(_compositeRechts, SWT.VERTICAL);
		Composite obj = new Composite(formRechts, SWT.NONE);
		Composite info = new Composite(formRechts, SWT.NONE);
		// Composite kontrolle = new Composite(formRechts, SWT.NONE);
		formRechts.setWeights(new int[] { 70, 30 });

		FillLayout fillLayout = new FillLayout();
		obj.setLayout(fillLayout);
		info.setLayout(fillLayout);
		// kontrolle.setLayout(fillLayout);

		Composite objComplex = new Composite(obj, SWT.NONE);
		objComplex.setLayout(new GridLayout());
		erzeugeCompositeRechtsOben(objComplex);
		erzeugeCompositeRechtsMitte(objComplex);

		Composite infoComplex = new Composite(info, SWT.NONE);
		infoComplex.setLayout(new GridLayout());
		erzeugeCompositeRechtsInfoBereich(infoComplex);

		// Composite kontrolleComplex = new Composite(kontrolle, SWT.NONE);
		// kontrolleComplex.setLayout(new GridLayout());
		// erzeugeCompositeRechtsUnten(kontrolleComplex);

		// erzeugeCompositeRechtsOben(_compositeRechts);
		// erzeugeCompositeRechtsMitte(_compositeRechts);
		// erzeugeCompositeRechtsUnten(_compositeRechts);
		// erzeugeCompositeRechtsInfoBereich(_compositeRechts);
		//
		// GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 1;
		// _compositeRechts.setLayout(gridLayout);
		//
		// GridData gridData = new GridData();
		// gridData.horizontalAlignment = GridData.CENTER;
		// gridData.verticalAlignment = GridData.BEGINNING;
		// _compositeRechts.setLayoutData(gridData);

	}

	/**
	 * Generiert das obere, rechte Composite, dass den Titel der Objekte enthlt.
	 * 
	 * @param parent
	 *          Parent-Composite
	 */
	private void erzeugeCompositeRechtsOben(Composite parent)
	{
		_compositeRechtsOben = new Composite(parent, SWT.NONE);

		Composite compositeRechtsObenLinks = new Composite(_compositeRechtsOben,
				SWT.NONE);
		Composite compositeRechtsObenRechts = new Composite(_compositeRechtsOben,
				SWT.NONE);

		Label labelObjekte = new Label(compositeRechtsObenLinks, SWT.NONE);
		labelObjekte.setText("Objekte");

		_textAnzahlObjekte = new Text(compositeRechtsObenRechts, SWT.BORDER);
		_textAnzahlObjekte.setEditable(false);
		_textAnzahlObjekte.setText("0 / 0");

		_schaltflaecheLoescheSelektierteObjekte = new Button(
				compositeRechtsObenRechts, SWT.PUSH);
		_schaltflaecheLoescheSelektierteObjekte.setText("X");
		_schaltflaecheLoescheSelektierteObjekte.setFont(_buttonFont);

		// Image imageDelete = new Image(_compositeLinksObjekttypTitel.getDisplay(),
		// Activator.class
		// .getResourceAsStream("/icons/actions/editdelete16x16.png"));
		// _buttonLoescheSelektierteObjekte.setImage(imageDelete);

		_schaltflaecheLoescheSelektierteObjekte
				.addSelectionListener(new SelectionAdapter()
				{
					@Override
					public void widgetSelected(SelectionEvent e)
					{
						_listeSWTObjektListe.deselectAll();
					}
				});

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		_compositeRechtsOben.setLayout(gridLayout);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_compositeRechtsOben.setLayoutData(gridData);

		compositeRechtsObenLinks.setLayout(new GridLayout());

		GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = false;
		rowLayoutRechts.justify = false;

		compositeRechtsObenRechts.setLayout(rowLayoutRechts);
		compositeRechtsObenRechts.setLayoutData(gridDataRechts);

	}

	/**
	 * Erzeugt ein {@link Composite} rechts unten. Dieses beinhaltet die Buttons
	 * OK und Abbreche.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 */
	private void erzeugeCompositeRechtsUnten(Composite parent)
	{
		_compositeRechtsUnten = new Composite(parent, SWT.NONE);

		_schaltflaecheOk = new Button(_compositeRechtsUnten, SWT.PUSH);
		_schaltflaecheOk.setText("OK");

		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_shell.dispose();
			}
		});

		_schaltflaecheAbbrechen = new Button(_compositeRechtsUnten, SWT.PUSH);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_shell.dispose();
			}
		});

		_schaltflaecheHilfe = new Button(_compositeRechtsUnten, SWT.PUSH);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				String hilfeKontextId = Activator.PLUGIN_ID + ".objektzusammenstellung";
				PlatformUI.getWorkbench().getHelpSystem().displayHelp(hilfeKontextId);
			}
		});

		/*
		 * Erzeugen von Buttons gleicher Gre...
		 */
		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.pack = false;
		rowLayout.fill = false;

		/*
		 * ...und am Ende (rechts) mittels GridData in einem GridLayout platzieren
		 */
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;

		_compositeRechtsUnten.setLayoutData(gridData);
		_compositeRechtsUnten.setLayout(rowLayout);

	}

	/**
	 * Generiert das rechte, mittlere Composite, dass die Darstellung der Liste
	 * der Objekte enthlt. Diese Darstellung ist essentiell fr die Auswahl der
	 * Objekte.
	 * 
	 * @param parent
	 */
	private void erzeugeCompositeRechtsMitte(Composite parent)
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout.fill = true;
		_compositeRechtsMitte = new Composite(parent, SWT.NONE);
		// compositeRechtsMitte.setLayout(rowLayout);
		_listeSWTObjektListe = new List(_compositeRechtsMitte, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		_listeSWTObjektListe.setBounds(0, 0, 300, 200);

		_listeSWTObjektListe.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				if (_listeSWTObjektListe.getSelectionIndex() > -1)
				{
					// Selektierte Elemente in selektionObjekttyp ablegen
					// _selektionObjekte = _listeSWTObjektListe.getSelectionIndices();

					berechneAnzahlObjekte();
					agregiereSystemObjectAuswahl();
				}

				String tempSelect = _listeSWTObjektListe.getSelection()[0];

				SystemObject so = _treeMapObjekt.get(tempSelect).getSystemObjekt();

				fuelleInfoBereich(so.getInfo().getShortInfo());

			}
		});

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_compositeRechtsMitte.setLayout(new FillLayout());
		_compositeRechtsMitte.setLayoutData(gridData);
	}

	/**
	 * Generiert das Composite Rechts mit Infobereich
	 * 
	 * @param parent
	 *          Parent Composite
	 */
	private void erzeugeCompositeRechtsInfoBereich(Composite parent)
	{
		_compositeRechtsInfoBereich = new Composite(parent, SWT.NONE);
		_gruppeInfo = new Group(_compositeRechtsInfoBereich, SWT.NONE);
		_gruppeInfo.setText("Information");

		_infoText = new Text(_gruppeInfo, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL
				| SWT.H_SCROLL);

		_infoText.setEditable(false);

		_compositeRechtsInfoBereich.pack();

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_compositeRechtsInfoBereich.setLayout(new FillLayout());
		_compositeRechtsInfoBereich.setLayoutData(gridData);

		_gruppeInfo.setLayout(new FillLayout());
	}

	/**
	 * Anzeigen des Datenidentifikations-Dialog. Nach dem Schliessen des Dialogs
	 * gibt diese Methode die angewhlten System-Objekte zurck.
	 * 
	 * @return angewhlte System-Objekte oder <code>null</code> bei keiner Auswahl
	 *         bzw. Abbrechen.
	 */
	// public SystemObject[] anzeigen()
	// {
	// return null;
	// }
	/**
	 * Fllt smtliche, bentigte Listen
	 * 
	 */
	private void fuelleAlles()
	{
		if (_connection != null)
		{
			DataModel dataModel = _connection.getDataModel();

			try
			{
				SystemObjectType type = dataModel.getType("typ.konfigurationsBereich");

				// Liste aller Systemobjekte
				java.util.List<SystemObject> objects = type.getObjects();

				Iterator<SystemObject> name = objects.iterator();

				while (name.hasNext())
				{
					ConfigurationArea element = (ConfigurationArea) name.next();

					Collection<SystemObject> currentObjects = element.getCurrentObjects();

					Iterator<SystemObject> name2 = currentObjects.iterator();

					while (name2.hasNext())
					{
						SystemObject element1 = (SystemObject) name2.next();
						ListenElementObjekt elemObjekttyp = new ListenElementObjekt(
								element1);

						// Objekte
						fuelleObjektListe(elemObjekttyp);

						// ObjektTypen
						ListenElementObjektTyp elemObjTyp = new ListenElementObjektTyp(
								element1);
						fuelleObjektTypListe(elemObjTyp);

					}
				}
			} catch (RuntimeException e)
			{
				e.printStackTrace();
			}

			// Darstellung
			fuellelisteSWTObjekte();
			fuellelisteSWTObjektTyp();

		}

	}

	/**
	 * Fllt die Liste ObjekttypListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 * 
	 */
	private void fuelleObjektListe(ListenElementObjekt elementObjekt)
	{
		_treeMapObjekt.put(elementObjekt.getName(), elementObjekt);
	}

	/**
	 * Fllt die Liste AttributListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 */
	private void fuelleAttributListe(ListenElementAttributGruppe lAtt)
	{
		if (_treeMapAttributgruppe.containsKey(lAtt.getName()))
		{
			// Nichts hinzufgen
		} else
		{
			_treeMapAttributgruppe.put(lAtt.getName(), lAtt);
		}

	}

	/**
	 * Fllt die Liste AspektListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 */
	private void fuelleAspektListe(ListenElementAspekt lAsp)
	{
		if (_treeMapAspekt.containsKey(lAsp.getName()))
		{
			// Nichts hinzufgen
		} else
		{
			_treeMapAspekt.put(lAsp.getName(), lAsp);
		}

	}

	/**
	 * Fllt die Liste ObjektListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 */
	private void fuelleObjektTypListe(ListenElementObjektTyp lTyp)
	{

		if (_treeMapObjektTyp.containsKey(lTyp.getName()))
		{
		} else
		{
			_treeMapObjektTyp.put(lTyp.getName(), lTyp);
		}

	}

	/**
	 * Fllt den TreeMapAttributgruppe mit gefilterten Attributen, die den in der
	 * listeSWTObjektTyp selektierten Elementen vorkommenden Attributgruppen
	 * entsprechen.
	 * 
	 */
	private void fuelleTreeMapAttributgruppeFiltered(String filterObjektTypen[])
	{
		// Nur die Elemente aus der Filterselektion anzeigen
		if (filterObjektTypen.length > 0)
		{
			// Filter enthlt die Strings von Objekttypen
			_treeMapAttributgruppe.clear();
			for (int i = 0; i < filterObjektTypen.length; i++)
			{
				SystemObject so = _treeMapObjektTyp.get(filterObjektTypen[i])
						.getSystemObject();
				java.util.List<AttributeGroup> attributeGroups = so.getType()
						.getAttributeGroups();
				Iterator<AttributeGroup> iterator = attributeGroups.iterator();
				while (iterator.hasNext())
				{
					AttributeGroup tempAtg = iterator.next();
					ListenElementAttributGruppe elementAttribut = new ListenElementAttributGruppe(
							tempAtg);
					fuelleAttributListe(elementAttribut);
				}
			}

		} else
		{
			Iterator<Entry<String, ListenElementObjektTyp>> it = _treeMapObjektTyp
					.entrySet().iterator();
			_treeMapAttributgruppe.clear();
			while (it.hasNext())
			{
				Entry<String, ListenElementObjektTyp> entry = it.next();
				// Map.Entry entry = (Map.Entry) it.next();
				ListenElementObjektTyp tempObjektTyp = ((ListenElementObjektTyp) entry
						.getValue());
				SystemObject so = tempObjektTyp.getSystemObject();
				java.util.List<AttributeGroup> attributeGroups = so.getType()
						.getAttributeGroups();
				Iterator<AttributeGroup> iterator = attributeGroups.iterator();
				while (iterator.hasNext())
				{
					AttributeGroup tempAtg = iterator.next();
					ListenElementAttributGruppe elementAttribut = new ListenElementAttributGruppe(
							tempAtg);
					fuelleAttributListe(elementAttribut);
				}

			}

		}

		_listeSWTAttributListe.setVisible(false);
		_listeSWTAttributListe.removeAll();
		fuellelisteSWTAttribute();
		_listeSWTAttributListe.setVisible(true);

		int itemCount = _listeSWTAttributListe.getItemCount();
		_textAnzahlAttributgruppe.setText("0 / " + itemCount);
		_textAnzahlAttributgruppe.getParent().pack();

		fuelleTreeMapAspekteFiltered(_listeSWTAttributListe.getSelection());

	}

	private void fuelleTreeMapAspekteFiltered(String filterAttributgruppen[])
	{
		_treeMapAspekt.clear();

		if (filterAttributgruppen.length > 0)
		{
			for (int i = 0; i < filterAttributgruppen.length; i++)
			{
				ListenElementAttributGruppe listenElementAttributGruppe = _treeMapAttributgruppe
						.get(filterAttributgruppen[i]);

				Collection<Aspect> aspects = listenElementAttributGruppe
						.getAttributGruppe().getAspects();

				Iterator<Aspect> aspectIterator = aspects.iterator();
				while (aspectIterator.hasNext())
				{
					Aspect tempAspect = aspectIterator.next();
					ListenElementAspekt elementAspekt = new ListenElementAspekt(
							tempAspect);
					fuelleAspektListe(elementAspekt);
				}
			}
		} else
		{
			Iterator<Entry<String, ListenElementAttributGruppe>> it = _treeMapAttributgruppe
					.entrySet().iterator();
			// Iterator it = _treeMapAttributgruppe.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, ListenElementAttributGruppe> entry = it.next();
				// Map.Entry entry = (Map.Entry) it.next();
				ListenElementAttributGruppe tempAttributGruppe = ((ListenElementAttributGruppe) entry
						.getValue());
				Collection<Aspect> aspects = tempAttributGruppe.getAttributGruppe()
						.getAspects();
				Iterator<Aspect> aspectIterator = aspects.iterator();
				while (aspectIterator.hasNext())
				{
					Aspect tempAspect = aspectIterator.next();
					ListenElementAspekt elementAspekt = new ListenElementAspekt(
							tempAspect);
					fuelleAspektListe(elementAspekt);
				}
			}

		}

		_listeSWTAspektListe.removeAll();
		fuellelisteSWTAspekte();
	}

	/*
	 * Fllen der SWT-Listen im GUI
	 */

	/**
	 * Fllt die Liste listeSWTOAttributLise
	 */
	private void fuellelisteSWTAttribute()
	{
		Iterator<Entry<String, ListenElementAttributGruppe>> it = _treeMapAttributgruppe
				.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, ListenElementAttributGruppe> entry = it.next();
			_listeSWTAttributListe.add((String) entry.getKey());
		}

		berechneAnzahlAttribute();
	}

	/**
	 * Fllt die Liste listeSWTAspektListe
	 * 
	 */
	private void fuellelisteSWTAspekte()
	{
		Iterator<Entry<String, ListenElementAspekt>> it = _treeMapAspekt.entrySet()
				.iterator();
		while (it.hasNext())
		{
			Entry<String, ListenElementAspekt> entry = it.next();
			_listeSWTAspektListe.add((String) entry.getKey());
		}

		berechneAnzahlAspekte();
	}

	/**
	 * Fllt die Liste listeSWTObjektListe
	 */
	private void fuellelisteSWTObjekte()
	{
		Iterator<Entry<String, ListenElementObjekt>> it = _treeMapObjekt.entrySet()
				.iterator();
		while (it.hasNext())
		{
			Entry<String, ListenElementObjekt> entry = it.next();
			_listeSWTObjektListe.add((String) entry.getKey());
		}

		berechneAnzahlObjekte();
	}

	/**
	 * Fllt die Objektliste mit Objekten vom Typ der in der Liste Objekttyp
	 * selektierten Objekttypen.
	 * 
	 * @param filterObjektTypen
	 */
	private void fuellelisteSWTObjekteFiltered(String filterObjektTypen[])
	{
		// temporre TreeMap anlegen, in der die anzuzeigenden Werte enthalten sind
		TreeMap<String, ListenElementObjekt> tempObjektMap = new TreeMap<String, ListenElementObjekt>();

		for (int i = 0; i < filterObjektTypen.length; i++)
		{
			Iterator<Entry<String, ListenElementObjekt>> it = _treeMapObjekt
					.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, ListenElementObjekt> entry = it.next();
				ListenElementObjekt tempObjektElement = (ListenElementObjekt) entry
						.getValue();
				SystemObject so = tempObjektElement.getSystemObjekt();

				// ListenElementObjektTyp tempObjektTypElement =
				// TreeMapObjektTyp.get(filterObjektTypen[i]);

				if ((so.getType().toString()).equals(filterObjektTypen[i]))
				{

					if (!tempObjektMap.containsKey(tempObjektElement.getName()))
					{
						tempObjektMap.put(tempObjektElement.getName(), tempObjektElement);
					}

				} else
				{
				}

			}

		}

		_listeSWTObjektListe.setVisible(false);
		_listeSWTObjektListe.removeAll();

		Iterator<Entry<String, ListenElementObjekt>> it = tempObjektMap.entrySet()
				.iterator();
		while (it.hasNext())
		{
			Entry<String, ListenElementObjekt> entry = it.next();
			_listeSWTObjektListe.add((String) entry.getKey());
		}

		_listeSWTObjektListe.setVisible(true);

		int itemCount = _listeSWTObjektListe.getItemCount();
		_textAnzahlObjekte.setText("0 / " + itemCount);
		_textAnzahlObjekte.getParent().pack();

	}

	private void fuellelisteSWTObjekteGefiltert()
	{

		String filterObjektTypen[] = _listeSWTObjekttypListe.getSelection();
		String filterAttributgruppen[] = _listeSWTAttributListe.getSelection();
		String filterAspekt[] = _listeSWTAspektListe.getSelection();

		// temporre TreeMap anlegen, in der die anzuzeigenden Werte enthalten sind
		TreeMap<String, ListenElementObjekt> tempObjektTypMap = new TreeMap<String, ListenElementObjekt>();
		TreeMap<String, ListenElementObjekt> tempAspektMap = new TreeMap<String, ListenElementObjekt>();
		TreeMap<String, ListenElementObjekt> tempAttributGruppeMap = new TreeMap<String, ListenElementObjekt>();
		TreeMap<String, ListenElementObjekt> tempResultMap = new TreeMap<String, ListenElementObjekt>();

		for (int i = 0; i < filterObjektTypen.length; i++)
		{
			Iterator<Entry<String, ListenElementObjekt>> it = _treeMapObjekt
					.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, ListenElementObjekt> entry = it.next();
				ListenElementObjekt tempObjektElement = (ListenElementObjekt) entry
						.getValue();
				SystemObject so = tempObjektElement.getSystemObjekt();

				// ListenElementObjektTyp tempObjektTypElement =
				// TreeMapObjektTyp.get(filterObjektTypen[i]);

				// Prfung auf Objekt vom gleichen ObjektTyp
				if ((so.getType().toString()).equals(filterObjektTypen[i]))
				{

					// Prfung auf gleiche Attributgruppe

					if (!tempObjektTypMap.containsKey(tempObjektElement.getName()))
					{
						tempObjektTypMap
								.put(tempObjektElement.getName(), tempObjektElement);
					}

				} else
				{
					// fuellelisteSWTObjekteFiltered -> KEIN gleicher Typ
				}

			}

		}

		// ////
		// Filter ber die Attributgruppen tempAttributGruppeMap

		// Iterator ber alle Objekte
		Iterator<Entry<String, ListenElementObjekt>> it = _treeMapObjekt.entrySet()
				.iterator();
		while (it.hasNext())
		{
			// System Objekte aus TreeMap holen
			Entry<String, ListenElementObjekt> entry = it.next();
			ListenElementObjekt tempObjektElement = (ListenElementObjekt) entry
					.getValue();
			SystemObject so = tempObjektElement.getSystemObjekt();

			// Iterator zu den Attributen der SystemObjekte bilden
			java.util.List<AttributeGroup> listAttributeGroups = so.getType()
					.getAttributeGroups();
			Iterator<AttributeGroup> itAttributeGroups = listAttributeGroups
					.iterator();

			while (itAttributeGroups.hasNext())
			{
				AttributeGroup tempAttributgruppeSystemObjekt = itAttributeGroups
						.next();
				for (int i = 0; i < filterAttributgruppen.length; i++)
				{
					// AttributGruppe der selektierten Attributgruppe in der SWT-Liste
					ListenElementAttributGruppe tempAttributGruppe = _treeMapAttributgruppe
							.get(filterAttributgruppen[i]);

					if (tempAttributGruppe.getAttributGruppe().getName().equals(
							tempAttributgruppeSystemObjekt.getName()))
					{
						if (!tempAttributGruppeMap.containsKey(tempObjektElement.getName()))
						{
							tempAttributGruppeMap.put(tempObjektElement.getName(),
									tempObjektElement);
						}
					}
				}
			}

		}

		// ////
		// Filter ber Aspekte tempAspektMap
		// Iterator ber alle Objekte

		it = _treeMapObjekt.entrySet().iterator();

		while (it.hasNext())
		{

			// System Objekte aus TreeMap holen
			Entry<String, ListenElementObjekt> entry = it.next();
			ListenElementObjekt tempObjektElement = (ListenElementObjekt) entry
					.getValue();
			SystemObject so = tempObjektElement.getSystemObjekt();

			// Iterator zu den Attributen der SystemObjekte bilden
			java.util.List<AttributeGroup> listAttributeGroups = so.getType()
					.getAttributeGroups();
			Iterator<AttributeGroup> itAttributeGroups = listAttributeGroups
					.iterator();

			while (itAttributeGroups.hasNext())
			{
				AttributeGroup tempAttributgruppeSystemObjekt = itAttributeGroups
						.next();
				Collection<Aspect> aspects = tempAttributgruppeSystemObjekt
						.getAspects();
				Iterator<Aspect> itAspects = aspects.iterator();
				while (itAspects.hasNext())
				{
					Aspect tempAspect = itAspects.next();
					for (int i = 0; i < filterAspekt.length; i++)
					{
						ListenElementAspekt asp = (ListenElementAspekt) _treeMapAspekt
								.get(filterAspekt[i]);

						if (asp.getAspekt().equals(tempAspect))
						{
							if (!tempAspektMap.containsKey(tempObjektElement.getName()))
							{
								tempAspektMap.put(tempObjektElement.getName(),
										tempObjektElement);
							}
						}

					}
				}

			}

		}

		// ////
		// Ergebnisse in den Maps zusammenfgen
		// TreeMap<String, ListenElementObjekt> tempObjektTypMap = new
		// TreeMap<String, ListenElementObjekt>();
		// TreeMap<String, ListenElementObjekt> tempAspektMap = new TreeMap<String,
		// ListenElementObjekt>();
		// TreeMap<String, ListenElementObjekt> tempAttributGruppeMap = new
		// TreeMap<String, ListenElementObjekt>();
		// TreeMap<String, ListenElementObjekt> tempResultMap = new TreeMap<String,
		// ListenElementObjekt>();

		// Ergebnisse aus tempObjektTypMap
		Iterator<Entry<String, ListenElementObjekt>> mergeIt = tempObjektTypMap
				.entrySet().iterator();
		while (mergeIt.hasNext())
		{
			boolean vorhandenInAnderenMaps = true;
			ListenElementObjekt tempListenElementObjekt = null;
			Entry<String, ListenElementObjekt> entry = mergeIt.next();
			tempListenElementObjekt = (ListenElementObjekt) entry.getValue();

			if (tempAspektMap.size() > 0)
			{
				if (!tempAspektMap.containsKey(tempListenElementObjekt.getName()))
				{
					vorhandenInAnderenMaps = false;
				}
			}

			if (tempAttributGruppeMap.size() > 0)
			{
				if (!tempAttributGruppeMap.containsKey(tempListenElementObjekt
						.getName()))
				{
					vorhandenInAnderenMaps = false;
				}
			}

			if ((!tempResultMap.containsKey(tempListenElementObjekt.getName()))
					&& vorhandenInAnderenMaps)
			{
				tempResultMap.put(tempListenElementObjekt.getName(),
						tempListenElementObjekt);
			}
		}

		// Ergebnisse aus tempAttributGruppeMap
		mergeIt = tempAttributGruppeMap.entrySet().iterator();
		while (mergeIt.hasNext())
		{
			boolean vorhandenInAnderenMaps = true;
			ListenElementObjekt tempListenElementObjekt = null;
			Entry<String, ListenElementObjekt> entry = mergeIt.next();
			tempListenElementObjekt = (ListenElementObjekt) entry.getValue();

			if (tempObjektTypMap.size() > 0)
			{
				if (!tempObjektTypMap.containsKey(tempListenElementObjekt.getName()))
				{
					vorhandenInAnderenMaps = false;
				}
			}

			if (tempAspektMap.size() > 0)
			{
				if (!tempAspektMap.containsKey(tempListenElementObjekt.getName()))
				{
					vorhandenInAnderenMaps = false;
				}
			}

			if ((!tempResultMap.containsKey(tempListenElementObjekt.getName()))
					&& vorhandenInAnderenMaps)
			{
				tempResultMap.put(tempListenElementObjekt.getName(),
						tempListenElementObjekt);
			}
		}

		// Ergebnisse aus tempAspektMap
		mergeIt = tempAspektMap.entrySet().iterator();
		while (mergeIt.hasNext())
		{
			boolean vorhandenInAnderenMaps = true;
			ListenElementObjekt tempListenElementObjekt = null;
			Entry<String, ListenElementObjekt> entry = mergeIt.next();
			tempListenElementObjekt = (ListenElementObjekt) entry.getValue();

			if (tempObjektTypMap.size() > 0)
			{
				if (!tempObjektTypMap.containsKey(tempListenElementObjekt.getName()))
				{
					vorhandenInAnderenMaps = false;
				}
			}

			if (tempAttributGruppeMap.size() > 0)
			{
				if (!tempAttributGruppeMap.containsKey(tempListenElementObjekt
						.getName()))
				{
					vorhandenInAnderenMaps = false;
				}
			}

			if ((!tempResultMap.containsKey(tempListenElementObjekt.getName()))
					&& vorhandenInAnderenMaps)
			{
				tempResultMap.put(tempListenElementObjekt.getName(),
						tempListenElementObjekt);
			}
		}

		_listeSWTObjektListe.setVisible(false);
		_listeSWTObjektListe.removeAll();

		Iterator<Entry<String, ListenElementObjekt>> outIt = tempResultMap
				.entrySet().iterator();
		while (outIt.hasNext())
		{
			// System Objekte aus TreeMap holen
			Entry<String, ListenElementObjekt> entry = outIt.next();
			ListenElementObjekt tempObjektElement = entry.getValue();
			_listeSWTObjektListe.add(tempObjektElement.getName());
		}

		_listeSWTObjektListe.setVisible(true);

		// int itemCount = _listeSWTObjektListe.getItemCount();
		// _textAnzahlObjekte.setText("0 / " + itemCount);
		// _textAnzahlObjekte.getParent().pack();

		berechneAnzahlObjekte();

	}

	/**
	 * Fllt die Liste listeSWTObjektTyp
	 */
	private void fuellelisteSWTObjektTyp()
	{
		Iterator<Entry<String, ListenElementObjektTyp>> it = _treeMapObjektTyp
				.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, ListenElementObjektTyp> entry = it.next();
			_listeSWTObjekttypListe.add((String) entry.getKey());
		}

		fuelleTreeMapAttributgruppeFiltered(_listeSWTObjekttypListe.getSelection());

		berechneAnzahlObjekttypen();
	}

	/**
	 * Berechnet die Anzahlt fr selektierte Objekte, Objekttypen, Attributgruppen
	 * und Aspekte in einem Rutsch
	 * 
	 */
	@SuppressWarnings("unused")
	private void berechneAnzahlFuerAlle()
	{
		berechneAnzahlObjekte();
		berechneAnzahlObjekttypen();
		// berechneAnzahlAttribute();
		// berechneAnzahlAspekte();
	}

	/**
	 * Entfernt aus TreeMapObjektTyp alle ObjektTypen, die nicht denjenigen aus
	 * sFilter entsprechen.
	 * 
	 * @param sFilter
	 *          String-Arrays, enthlt die Pids der bentigten ObjektTypen
	 */
	private void entferneObjektTypen(String sFilter[])
	{
		// Enthlt die Keys der zu lschenden ObjektTypen
		LinkedList<String> loeschListe = new LinkedList<String>();

		// Prfen, welche Eintrge in TreeMapObjektTyp gelscht werden sollen
		Iterator<Entry<String, ListenElementObjektTyp>> itMapObjTyp = _treeMapObjektTyp
				.entrySet().iterator();
		while (itMapObjTyp.hasNext())
		{
			Entry<String, ListenElementObjektTyp> entry = itMapObjTyp.next();
			ListenElementObjektTyp tempLEObjTyp = entry.getValue();

			boolean loeschen = true;
			for (int i = 0; i < sFilter.length; i++)
			{
				if (tempLEObjTyp.getSystemObject().getType().getPid()
						.equals(sFilter[i]))
				{
					// nicht lschen
					loeschen = false;
				}

			}

			if (loeschen)
			{
				loeschListe.add(entry.getKey());
			}

		}

		// Lschen der Entsprechenden Eintrge
		Iterator<String> it = loeschListe.iterator();
		while (it.hasNext())
		{
			_treeMapObjektTyp.remove(it.next());
		}
	}

	/**
	 * Zeigt die Anzahl der selektierten und der insgesamt vorhandenen Objekttypen
	 * im Feld _textAnzahlObjekttyp an.
	 */
	private void berechneAnzahlObjekttypen()
	{
		int selektiert = _listeSWTObjekttypListe.getSelectionCount();
		int anzahlGesamt = _listeSWTObjekttypListe.getItemCount();

		String printString = "" + selektiert + " / " + anzahlGesamt;
		_textAnzahlObjekttyp.setText(printString);
		_textAnzahlObjekttyp.getParent().pack();
	}

	/**
	 * Zeigt die Anzahl der selektierten und der insgesamt vorhandenen
	 * Attributgruppen im Feld _textAnzahlAttributgruppe an.
	 */
	private void berechneAnzahlAttribute()
	{
		int selektiert = _listeSWTAttributListe.getSelectionCount();
		int anzahlGesamt = _listeSWTAttributListe.getItemCount();

		String printString = "" + selektiert + " / " + anzahlGesamt;
		_textAnzahlAttributgruppe.setText(printString);
		_textAnzahlAttributgruppe.getParent().pack();
	}

	/**
	 * Zeigt die Anzahl der selektierten und der insgesamt vorhandenen Aspekte im
	 * Feld _textAnzahlAspekte an.
	 */
	private void berechneAnzahlAspekte()
	{
		int selektiert = _listeSWTAspektListe.getSelectionCount();
		int anzahlGesamt = _listeSWTAspektListe.getItemCount();

		String printString = "" + selektiert + " / " + anzahlGesamt;
		_textAnzahlAspekte.setText(printString);
		_textAnzahlAspekte.getParent().pack();
	}

	/**
	 * Zeigt die anzahl der selektierten und der insgesamt vorhandenen Objekte im
	 * Feld _textAnzahlObjekte an.
	 */
	private void berechneAnzahlObjekte()
	{
		int selektiert = _listeSWTObjektListe.getSelectionCount();
		int anzahlGesamt = _listeSWTObjektListe.getItemCount();

		String printString = "" + selektiert + " / " + anzahlGesamt;
		_textAnzahlObjekte.setText(printString);
		_textAnzahlObjekte.getParent().pack();
	}

	/**
	 * Setzt die Simulationsvariante.
	 * 
	 * @param variante
	 *          Simulationsvariante, die gesetzt werden soll.
	 */

	@SuppressWarnings("unused")
	private void setzeSimulationsvariante(int variante)
	{

	}

	/*
	 * Hilfsmethoden
	 */
	/**
	 * Fllt den Infobereich
	 * 
	 * @param Text
	 *          , der im Infobereich erscheinen soll
	 */
	private void fuelleInfoBereich(String text)
	{
		_infoText.setText(text);
		_shell.layout();
	}

	@SuppressWarnings("unused")
	private void fuelleTextAnzahlObjekte(String wert)
	{
		_textAnzahlObjekte.setText(wert);
		_textAnzahlObjekte.getParent().pack();
	}

	/**
	 * Legt in der Variable systemObjectAuswahl alle in der Liste SystemObjekte
	 * selektierten Elemente an.
	 */
	private void agregiereSystemObjectAuswahl()
	{
		String[] auswahl = _listeSWTObjektListe.getSelection();
		SystemObject[] returnSystemObject = new SystemObject[auswahl.length];
		for (int i = 0; i < auswahl.length; i++)
		{
			returnSystemObject[i] = ((ListenElementObjekt) _treeMapObjekt
					.get(auswahl[i])).getSystemObjekt();
		}

		_systemObjectAuswahl = returnSystemObject;
	}

	/**
	 * Enummerator fr die Darstellung
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Darstellung
	{
		/**
		 * Folgende Auswahllisten werden angezeigt:<br>
		 * - Objekt-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		OBJEKTELISTE,

		/**
		 * Folgende Auswahllisten werden angezeigt:<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		OBJEKTELISTE_OBJEKTTYP,

		/**
		 * Folgende Auswahllisten werden angezeigt:<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Attributgruppen-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		OBJEKTELISTE_OBJEKTTYP_ATG,

		/**
		 * Folgende Auswahllisten werden angezeigt:<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Attributgruppen-Liste<br>
		 * - Aspekt-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		// OBJEKTELISTE_OBJEKTTYP_ATG_ASP,
		OBJEKTELISTE_OBJEKTTYP_ATG_ASP

		/**
		 * Folgende Auswahllisten werden angezeigt:<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Attributgruppen-Liste<br>
		 * - Aspekt-Liste<br>
		 * - Simulations-Variante<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		// OBJEKTELISTE_OBJEKTTYP_ATG_ASP_SIMUVAR,
	};

	/*
	 * Reprsentation der jeweiligen Elemente in den Listen
	 */

	/**
	 * Reprsentiert die Listenelemente vom Typ Objekttyp
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ListenElementObjekt
	{
		private String name = null;

		private SystemObject systemObject = null;

		// public ListenElementObjekt()
		// {
		// }

		public ListenElementObjekt(SystemObject sysObj)
		{
			setSystemObjekt(sysObj);
		}

		public String getName()
		{
			return name;
		}

		private void setName(String name)
		{
			this.name = name;
		}

		public SystemObject getSystemObjekt()
		{
			return systemObject;
		}

		private void setSystemObjekt(SystemObject systemObject)
		{
			this.systemObject = systemObject;
			// this.setName(systemObject.getName());
			// Name, pid, id
			String tempName = "";
			if (systemObject.getName() != null && systemObject.getName() != "")
				tempName = tempName + systemObject.getName() + " ";
			if (systemObject.getPid() != null && systemObject.getPid() != "")
				tempName = tempName + "(" + systemObject.getPid() + ") ";
			if (systemObject.getId() > 0)
				tempName = tempName + "[" + String.valueOf(systemObject.getId()) + "]";

			this.setName(tempName);

		}
	}

	/**
	 * Reprsentiert die Listenelemente vom Typ Attributgruppe
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ListenElementAttributGruppe
	{
		/**
		 * Name dieses Elements
		 */
		private String name = null;

		/**
		 * Attribute dieses Elements
		 */
		private AttributeGroup attributeGroup;

		public ListenElementAttributGruppe(AttributeGroup wert)
		{
			setAttributGruppe(wert);
		}

		public String getName()
		{
			return name;
		}

		private void setName(String name)
		{
			this.name = name;
		}

		/**
		 * Liefert das Attribut
		 * 
		 * @return Attribut
		 */
		public AttributeGroup getAttributGruppe()
		{
			return attributeGroup;
		}

		/**
		 * Setzt das Attribut
		 * 
		 * @param attributeGroup
		 */
		private void setAttributGruppe(AttributeGroup wert)
		{
			this.attributeGroup = wert;
			String tempName = "";
			if (attributeGroup.getName() != null && attributeGroup.getName() != "")
				tempName = tempName + attributeGroup.getName() + " ";
			if (attributeGroup.getPid() != null && attributeGroup.getPid() != "")
				tempName = tempName + "(" + attributeGroup.getPid() + ") ";
			// if(attributeGroup.getId() > 0)
			// tempName = tempName + "[" + String.valueOf(attributeGroup.getId()) +
			// "]";
			this.setName(tempName);
		}
	}

	/**
	 * Reprsentiert die Listenelemente vom Typ Aspekt
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ListenElementAspekt
	{
		/**
		 * Name dieses Elements
		 */
		private String name = null;

		/**
		 * Aspect dieses Elements
		 */
		private Aspect aspect = null;

		public ListenElementAspekt(Aspect aspekt)
		{
			setAspekt(aspekt);
		}

		public String getName()
		{
			return name;
		}

		private void setName(String name)
		{
			this.name = name;
		}

		public Aspect getAspekt()
		{
			return aspect;
		}

		private void setAspekt(Aspect aspekt)
		{
			this.aspect = aspekt;
			String tempName = "";
			if (aspect.getName() != null && aspect.getName() != "")
				tempName = tempName + aspect.getName();
			if (aspect.getPid() != null)
				tempName = tempName + " (" + aspekt.getPid() + ")";
			if (aspect.getId() > 0)
				tempName = tempName + " " + "[" + String.valueOf(aspect.getId()) + "]";
			setName(tempName);
		}
	}

	/**
	 * Reprsentiert die Listenelemente vom Typ ObjektTyp
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ListenElementObjektTyp
	{
		/**
		 * Liefert den Namen des ObjektTyps
		 */
		private String name = null;

		/**
		 * Hlt ein SystemObjekt zwecks Auswertung und Anzeige der Attributgruppen
		 * zur Verfgung.
		 */
		private SystemObject systemObject = null;

		public ListenElementObjektTyp(SystemObject sysObj)
		{
			setName(sysObj.getType().toString());
			this.systemObject = sysObj;
		}

		/**
		 * Liefert den Namen des ObjektTyps
		 * 
		 * @return Name des Objekttyps
		 */
		public String getName()
		{
			return name;
		}

		private void setName(String name)
		{
			this.name = name;
		}

		/**
		 * Liefert das SystemObject
		 * 
		 * @return SystemObject
		 */
		public SystemObject getSystemObject()
		{
			return systemObject;
		}

	}

}
