/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import org.apache.commons.codec.binary.Base64InputStream;
import org.apache.commons.codec.binary.Base64OutputStream;

//import com.sun.xml.internal.messaging.saaj.packaging.mime.util.BASE64DecoderStream;
//import com.sun.xml.internal.messaging.saaj.packaging.mime.util.BASE64EncoderStream;

/**
 * Test des {@link EinstellungenSpeicher}s. Sollen noch mehr Daten untergebracht
 * werden, so muss der JVM wesentlich mehr Speicher zugeordnet werden.
 * 
 * @author HofmannJ
 * 
 */
public class EinstellungenSpeicherTestA {
	private static final String _inputFilePath = "C:\\kurzinfo.pdf";
	private static final String _outputFilePath = "C:\\kurzinfo_rebuild.pdf";
	// private static final String _inputFilePath = "C:\\test.png";
	// private static final String _outputFilePath = "C:\\test_rebuild.png";

	private static final boolean _ausgabe = false;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Einfache Daten im Einstellungenspeicher ablegen
		 */
		String muster = "aaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaBaaaB";
		StringBuffer stringBuffer = new StringBuffer();
		for (int i = 0; i < 321; i++) {
			stringBuffer.append(muster);
		}
		String testString = stringBuffer.toString();

		System.out.println("Teststring (" + testString.length() + " Zeichen): "
				+ testString);

		EinstellungenSpeicher esp = new EinstellungenSpeicher();

		esp.editiereParameter("1", testString);

		if (_ausgabe)
			System.out.println(esp.ausgabeXmlFormatiert(true));

		String vergleichsParameter = esp.getParameter("1");

		if (_ausgabe) {
			System.out.println("Teststring          (" + testString.length()
					+ " Zeichen): " + testString);
			System.out.println("Vergleichsparameter ("
					+ vergleichsParameter.length() + " Zeichen): "
					+ vergleichsParameter);
		}

		if (testString.equals(vergleichsParameter)) {
			System.out.println("Strings sind identisch.");
		} else {
			System.out.println("Strings sind NICHT identisch!!!");
		}

		String uue = "";

		/*
		 * Weitere Daten einlesen aus Datei und Base64 kodieren und im
		 * Einstellungenspeicher ablegen
		 */
		FileInputStream fis;
		try {
			fis = new FileInputStream(_inputFilePath);
			BufferedInputStream bis = new BufferedInputStream(fis);

			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			// Durch Commons ersetzt.
			// BASE64EncoderStream bes = new BASE64EncoderStream(baos);
			Base64OutputStream bes = new Base64OutputStream(baos);

			int len;

			byte[] buf = new byte[4096];
			try {
				while ((len = bis.read(buf)) > 0) {
					bes.write(buf, 0, len);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}

			StringBuffer sb = new StringBuffer();
			byte[] uueBA = baos.toByteArray();
			for (byte c : uueBA) {
				sb.append((char) (c & 0xff));
			}

			uue = sb.toString();

			if (_ausgabe) {
				System.out.println("SB:\n" + uue);
			}

		} catch (FileNotFoundException e1) {

			e1.printStackTrace();
		}

		/*
		 * Base64 kodierte Daten zum Einstellungenspeicher hinzufgen
		 */
		esp.editiereParameter("UUE", uue);

		/*
		 * Belastungstest mit Dateien in Base64
		 */
		// for(int i = 0; i < 5; i++)
		// {
		// System.out.println("==>I: " + i);
		// esp.editiereParameter("UUE" + i, uue);
		// }
		/*
		 * Ausgabe XML-Struktur des Einstellungenspeichers
		 */
		if (_ausgabe) {
			System.out.println("XML:\n" + esp.ausgabeXmlFormatiert(true));
		}

		/*
		 * Auslesen der Base64 kodierten Daten aus dem Einstellungen Speicher
		 */
		String uueVergleich = esp.getParameter("UUE");

		/*
		 * Prfen, ob die Daten aus dem Einstellungenspeicher noch mit den
		 * Eingangsdaten bereinstimmen
		 */
		if (uue.equals(uueVergleich)) {
			System.out.println("Daten stimmen berein.");
		} else {
			System.out.println("Daten stimmen NICHT berein!!!");
		}

		/*
		 * Base64 kodierte Daten aus dem Einstellungenspeicher dekodieren und
		 * wieder in eine Datei schreiben
		 */
		ByteArrayInputStream bis = new ByteArrayInputStream(uueVergleich
				.getBytes());

		try {
			FileOutputStream fos = new FileOutputStream(_outputFilePath);

			// Durch commons ersetzt.
			// BASE64DecoderStream baseDec = new BASE64DecoderStream(bis);
			Base64InputStream baseDec = new Base64InputStream(bis);

			byte[] buf = new byte[4096];
			int len;
			try {
				while ((len = baseDec.read(buf)) > 0) {
					fos.write(buf, 0, len);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		/*
		 * Belastungstest
		 */
		for (int i = 0; i < 250; i++) {
			esp.editiereParameter("belastung" + i, testString);
		}

		/*
		 * Alle Parameter holen
		 */
		HashMap<String, String> alleParameter = esp.getAlleParameter();
		System.out.println("ANZ: " + alleParameter.size());
		Iterator<Entry<String, String>> itAlleParameter = alleParameter
				.entrySet().iterator();
		while (itAlleParameter.hasNext()) {
			Entry<String, String> entry = itAlleParameter.next();

			if (_ausgabe) {
				System.out.println("IDX: " + entry.getKey() + " Value: "
						+ entry.getValue());
			}

		}

		esp.schreibeString("C:\\Ausgabe.xml", esp.ausgabeXmlFormatiert(true),
				true);

	}

}
