/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.basislib.login.UrlasserInfo;
import de.bsvrz.buv.rw.basislib.login.UrlasserObjekt;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Mit Hilfe dieser Klasse lassen sich die Globalen Einstellungen lesen bzw.
 * schreiben. Die dahinterstehende XML-Struktur wird als String geliefert bzw.
 * geschrieben.
 * 
 * @author HofmannJ
 * 
 */
public class GlobaleEinstellungenNetzweit implements ClientSenderInterface
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _verbindung = null;

	/**
	 * Datenmodell des DaV
	 */
	private DataModel _dataModell = null;

	/**
	 * SystemObjekt
	 */
	private SystemObject _systemObject = null;

	/**
	 * Bezeichnung der Attributgruppe
	 */
	private static final String _atgBezeichnung = "atg.globaleEinstellungenOberflcheNetzweit";

	/**
	 * Bezeichnung des Aspektes soll (zum Empfangen)
	 */
	private static final String _aspSoll = "asp.parameterSoll";

	/**
	 * Bezeichnung des Aspektes vorgabe (zum Senden)
	 */
	private static final String _aspVorgabe = "asp.parameterVorgabe";

	/**
	 * Attributgruppe
	 */
	private AttributeGroup _atg = null;

	/**
	 * Aspekt
	 */
	private Aspect _asp = null;

	/**
	 * Datenbeschreibung
	 */
	private DataDescription _dataDescription = null;

	private static final String _itemBenutzereinstellungenGlobal = "BenutzereinstellungenGlobal";

	private static final String _itemVoreinstellungen = "Voreinstellungen";

	/**
	 * Synchronisation der Sende Steuerung
	 */
	private Boolean _sendeSteuerung = new Boolean(true);

	/**
	 * Flag, zeigt an ob schon mal eine Meldung zu der Angabe eines falschen
	 * Oberflchenobjekte ber den Parameter <code>-oberflaechenObjekt</code>
	 * ausgegeben wurde.
	 */
	private boolean _meldungFalschesOberflaechenObjektSchonGezeigt = false;

	/**
	 * Wenn <code>true</code>, erfolgt das senden ohne den {@link UrlasserDialog}.
	 */
	private boolean _sendenOhneUrlasserDlg = false;

	/**
	 * Konstruktor, bernimmt die Verbindung zum DaV.
	 * 
	 * @param verbindung
	 *          Verbindung zum DaV
	 */
	public GlobaleEinstellungenNetzweit(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		this._dataModell = _verbindung.getDataModel();
	}

	/**
	 * Sendet die XML-Struktur.
	 * 
	 * @param xmlStruktur
	 *          Zu sendende XML-Struktur innerhalb eines Strings.
	 */
	public void sendeXmlStruktur(String xmlStruktur)
	{
		if (_verbindung != null)
		{
			// anmeldenSenden();
			// synchronized (_sendeSteuerung)
			// {
			// try
			// {
			// _sendeSteuerung.wait();
			// }
			// catch (InterruptedException e)
			// {
			// System.err.println(getClass().getName() + " Fehler beim Warten auf die
			// Sendesteuerung\n\t=> " + e.toString());
			// }
			// }
			String[] split = xmlStruktur.split("\n");
			sendeZeilen(split);
			// abmeldenSenden();
		}
	}

	/**
	 * Anmelden zum Senden von Daten
	 */
	@SuppressWarnings("unused")
	private void anmeldenSenden()
	{
		_dataModell = _verbindung.getDataModel();

		/*
		 * SystemObjekt bestimmen
		 */
		String objekt = bestimmePidOberflaechenObjekt();
		System.out.println("Objekt Netzweit");

		if (objekt != null)
		{
			_systemObject = _dataModell.getObject(objekt);
			_atg = _dataModell.getAttributeGroup(_atgBezeichnung);

			_asp = _dataModell.getAspect(_aspVorgabe);
			_dataDescription = new DataDescription(_atg, _asp);

			SenderRole senderRole = SenderRole.sender();

			try
			{
				_verbindung.subscribeSender(this, _systemObject, _dataDescription,
						senderRole);
			} catch (OneSubscriptionPerSendData e)
			{
				e.printStackTrace();
			}
		}
	}

	/**
	 * Sendet die Werte aus einem String-Array.
	 * 
	 * @param wert
	 *          String-Array mit den Werten als String.
	 */
	private void sendeZeilen(String[] wert)
	{
		_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
		Data data = _verbindung.createData(_atg);
		Data item = data.getItem(_itemBenutzereinstellungenGlobal);
		TextArray textArray = item.getTextArray(_itemVoreinstellungen);
		textArray.setLength(wert.length);

		for (int i = 0; i < wert.length; i++)
		{
			textArray.getTextValue(i).setText(wert[i]);
		}

		/*
		 * Aus der alten DataDescription eine neue unter dem Parameter
		 * asp.parameterVorgabe erzeugen.
		 */
		DataDescription dataDescriptionNeu = new DataDescription(_dataDescription
				.getAttributeGroup(), _verbindung.getDataModel().getAspect(_aspVorgabe));

		/*
		 * SystemObjekt bestimmen
		 */
		String objekt = bestimmePidOberflaechenObjekt();

		if (objekt != null)
		{
			_systemObject = _verbindung.getDataModel().getObject(objekt);

			ResultData datenTelegramm = new ResultData(_systemObject,
					dataDescriptionNeu, new Date().getTime(), data);

			final LinkedList<ResultData> liste = new LinkedList<ResultData>();
			liste.add(datenTelegramm);

			if (!_sendenOhneUrlasserDlg)
			{
				Runnable run = new Runnable()
				{
					@Override
					public void run()
					{
						UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell());
						uld.anzeigen(liste);
					}
				};

				PlatformUI.getWorkbench().getDisplay().asyncExec(run);
			} else
			{
				BenutzerAngemeldet ba = BenutzerAngemeldet.getInstanz();
				UrlasserInfo ui = new UrlasserInfo(ba.getName(), ba.getPasswort(),
						"Senden ohne UrlasserDialog", ba.getName());
				UrlasserObjekt uo = new UrlasserObjekt(ui);
				uo.baueDaVVerbindungAuf();

				LinkedList<ResultData> sendeDaten = uo.sendeDaten(liste);
				if (sendeDaten.size() > 0)
				{
					String meldung = getClass().getName() + "::aktionOk " + "Von "
							+ liste.size() + " Datenstzen konnten " + sendeDaten.size()
							+ " Datenstze nicht versendet werden.";

					String meldungLog = meldung + " Datenstze:\n";
					for (ResultData resultData : sendeDaten)
					{
						meldung = meldung + "\n=> " + resultData;
					}

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID, meldungLog));

					MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					mb.setText("Datenstze nicht versendbar");
					mb.setMessage(meldung);
					mb.open();
				}
			}
		}

	}

	/**
	 * Abmeldung
	 */
	@SuppressWarnings("unused")
	private void abmeldenSenden()
	{
		_verbindung.unsubscribeSender(this, _systemObject, _dataDescription);

	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de)
	 * 
	 * @param objPid
	 *          Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	@SuppressWarnings("unused")
	private List<SystemObject> bestimmeObjekte(String objPid)
	{
		List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = _verbindung.getDataModel().getType(objPid);

		if (typeSysObj == null)
			return objekte;

		if (typeSysObj.isConfigurating())
		{
			typeSysObj = (ConfigurationObjectType) typeSysObj;
		} else
		{
			typeSysObj = (DynamicObjectType) typeSysObj;
		}

		// Elemente dieses Systemobjekts einlesen

		List<SystemObject> typobjekte = typeSysObj.getElements();

		Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

		while (iterator_Typen.hasNext())
		{
			SystemObject sysObj = (SystemObject) iterator_Typen.next();

			objekte.add(sysObj);

			// System.out.println(sysObj.getPid());
		}

		return objekte;
	}

	@Override
	public void dataRequest(SystemObject object, DataDescription dataDescription,
			byte state)
	{
		if (state == 0)
		{
			synchronized (_sendeSteuerung)
			{
				_sendeSteuerung.notify();
			}
		}
	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription)
	{
		return true;
	}

	/*
	 * Empfangsteil
	 */

	/**
	 * Ruft aus dem DaV die XML-Strukturen der allgemeinen, netzweiten
	 * Einstellungen ab.
	 */
	public String empfangeXmlStruktur()
	{
		String re = null;
		String pidSysObj = bestimmePidOberflaechenObjekt();

		if (pidSysObj != null)
		{
			_systemObject = _dataModell.getObject(pidSysObj);

			_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
			_asp = _dataModell.getAspect(_aspSoll);
			_dataDescription = new DataDescription(_atg, _asp);

			ResultData resultData = _verbindung.getData(_systemObject,
					_dataDescription, 10000l);
			re = verarbeiteResultData(resultData);
		}

		return re;
	}

	public String verarbeiteResultData(ResultData resultData)
	{
		String re = null;

		if (resultData.hasData())
		{

			if (resultData.getDataDescription().getAttributeGroup().getPid()
					.equalsIgnoreCase(_atgBezeichnung))
			{
				StringBuffer buf = new StringBuffer();

				Data data = resultData.getData();
				Data item = data.getItem(_itemBenutzereinstellungenGlobal);

				TextArray textArray = item.getTextArray(_itemVoreinstellungen);

				for (int i = 0; i < textArray.getLength(); i++)
				{
					String wert = textArray.getTextValue(i).getText();
					buf.append(wert);
				}

				re = buf.toString();
			}
		}

		return re;
	}

	/**
	 * Setzt die DaV-Verbindung.
	 * 
	 * @param verbindung
	 *          DaV-Verbindung
	 */
	public void setDaVVerbindung(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		if (_verbindung != null)
		{
			_dataModell = _verbindung.getDataModel();
		}
	}

	/**
	 * Bestimmt die PID des Oberflchenobjektes an Hand des
	 * Kommandozeilenparameters <code>-oberflaechenObjekt</code>. Wird dieser
	 * nicht gefunden wird der AOE verwendet.
	 * 
	 * @return Pid des Oberflchenobjektes oder <code>null</code>, wenn diese Pid
	 *         nicht bestimmt werden konnte.
	 */
	private String bestimmePidOberflaechenObjekt()
	{
		String[] kommandozeilenArgumente = Platform.getCommandLineArgs();

		String pidSysObj = null;

		/*
		 * Suchen des bentigten Parameters -oberflaechenObjekt
		 */
		for (int i = 0; i < kommandozeilenArgumente.length; i++)
		{
			if (kommandozeilenArgumente[i].contains("-oberflaechenObjekt"))
			{
				String[] split = kommandozeilenArgumente[i].split("=");
				if (split.length == 2)
				{
					pidSysObj = split[1];
				}

				/*
				 * Prfen ob ein Objekt zu dieser PID existiert. Ist dies nicht der
				 * Fall, wird pidSysObj auf Null gesetzt und der AOE wird verwendet.
				 */
				_systemObject = _dataModell.getObject(pidSysObj);
				if (_systemObject == null)
				{
					final String meldung = "Der Parameter -oberflaechenObjekt ist fehlerhaft. "
							+ "Es existiert kein OberflchenObjekt mit der PID "
							+ pidSysObj
							+ ". Bestimme und verwende die PID der AutarkenOrganisationsEinheit "
							+ "zur Bestimmung der globalen, netzweiten Einstellungen (Klasse:"
							+ this.getClass().getName() + ").";

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
									meldung, null));

					if (!_meldungFalschesOberflaechenObjektSchonGezeigt)
					{
						Runnable run = new Runnable()
						{

							@Override
							public void run()
							{
								MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
										.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_WARNING);
								mb.setText("Parameter -oberflaechenObjekt fehlerhaft");
								mb.setMessage(meldung);
								mb.open();
							}

						};

						PlatformUI.getWorkbench().getDisplay().syncExec(run);
						_meldungFalschesOberflaechenObjektSchonGezeigt = true;
					}

					pidSysObj = null;
				}

				break;
			}
		}

		/*
		 * Wenn dieser nicht gefunden wurde, AOE verwenden!
		 */
		if (pidSysObj == null)
		{
			pidSysObj = _verbindung.getLocalConfigurationAuthority().getPid();
		}

		return pidSysObj;
	}

	/**
	 * Legt fest, ob mit oder ohne {@link UrlasserDialog} gesendet werden soll.
	 * 
	 * @param sendenOhneUrlasserDialog
	 *          Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *          gesendet, sonst nicht.
	 */
	public void setSendenOhneUrlasserDialog(boolean sendenOhneUrlasserDialog)
	{
		this._sendenOhneUrlasserDlg = sendenOhneUrlasserDialog;
	}

	/**
	 * Prft, ob mit oder ohne {@link UrlasserDialog} gesendet wird.
	 * 
	 * @return Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *         gesendet, sonst nicht.
	 */
	public boolean isSendenOhneUrlasserDialog()
	{
		return _sendenOhneUrlasserDlg;
	}

}
