/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Date;
import java.util.LinkedList;

import org.eclipse.nebula.widgets.cdatetime.CDT;
import org.eclipse.nebula.widgets.cdatetime.CDateTime;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;

import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.IEintragBereich;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.IFelderFuellen;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.SchaltflaechenFabrikEreignisSystemKalenderDialog;

/**
 * Eingabefeld fr Datum + Zeit. <br>
 * <br>
 * In der Klasse KalenderDatum wird ein Eingabefeld des Datums zur Verfgung
 * gestellt. Auer durch die Tastatureingabe gibt es noch die Mglichkeit der
 * Eingabe ber einen Kalenderauswahldialog oder ber einen
 * Ereignissauswahldialog. Die Eingabemglichkeiten knnen per Konstuktor
 * eingestellt werden. Ebenso kann die Granularitt der Eingabe eingesetllt
 * werden, d.h. dass die kleinste einstellbare Zeiteinheit vorgegeben werden
 * kann. Wird als Eingabemglichkeit die Ereignisauswahl ermglicht, so wird
 * eine Verbindung zum Datenverteiler bentigt, da die Ereignisse aus dem
 * Ereignisskalender kommen. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-36
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/05/28 15:17:54 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class DatumZeit extends Composite
{
	/**
	 * Spezifiziert den Eingabetyp
	 */
	private Eingabetyp _eingabetyp = null;

	/**
	 * VorgabeDatum
	 */
	private Date _vorgabeDatum = null;

	/**
	 * CDateTime widget
	 */
	private CDateTime _cdt = null;

	/**
	 * Flag Anzeigen Ereigniskalender
	 */
	private boolean _ereigniskalender = false;

	/**
	 * Flag Anzeigen Systemkalender
	 */
	private boolean _systemkalender = false;

	/**
	 * Gibt an, welches Feld eines Bereichs bercksichtigt werden soll (von oder
	 * bis)
	 */
	private BereichsFeld _bereichsFeld = null;

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
	 * @param style
	 *          SWT-Stil.
	 * @param eingabetyp
	 *          Eingabetyp, bestimmt die Gestalt des KalenderZeit-Elementes an
	 *          Hand des Enumerators {@link Eingabetyp}.
	 * @param ereigniskalender
	 *          Wenn <code>true</code>, dann werden die Ereigniskalendereintrge
	 *          fr den aktuellen Monat dargestellt, bei <code>false</code> nicht.
	 *          Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der die
	 *          Datumseingabe beinhaltet.
	 * @param systemkalender
	 *          Wenn <code>true</code>, dann werden die Systemkalendereintrge fr
	 *          den aktuellen Monat dargestellt, bei <code>false</code> nicht.
	 *          Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der die
	 *          Datumseingabe beinhaltet.
	 */
	public DatumZeit(Composite parent, int style, Eingabetyp eingabetyp,
			boolean ereigniskalender, boolean systemkalender)
	{
		super(parent, style);
		this._eingabetyp = eingabetyp;
		this._ereigniskalender = ereigniskalender;
		this._systemkalender = systemkalender;
		init();
	}

	public DatumZeit(Composite parent, int style, Eingabetyp eingabetyp,
			boolean ereigniskalender, boolean systemkalender,
			BereichsFeld bereichsFeld)
	{
		super(parent, style);
		this._eingabetyp = eingabetyp;
		this._ereigniskalender = ereigniskalender;
		this._systemkalender = systemkalender;
		this._bereichsFeld = bereichsFeld;
		init();
	}

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
	 * @param style
	 *          SWT-Stil
	 * @param eingabetyp
	 *          Eingabetyp, bestimmt die Gestalt des KalenderZeit-Elementes an
	 *          Hand des Enumerators {@link Eingabetyp}.
	 * @param vorgabeDatum
	 *          Vorgabe Datum/Uhrzeit als Instanz der Klasse {@link Date}. Wenn
	 *          <code>null</code>, dann wird vom aktuellen Datum ausgegangen.
	 * @param ereigniskalender
	 *          Wenn <code>true</code>, dann werden die Ereigniskalendereintrge
	 *          fr den aktuellen Monat dargestellt, bei <code>false</code> nicht.
	 *          Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der die
	 *          Datumseingabe beinhaltet.
	 * @param systemkalender
	 *          Wenn <code>true</code>, dann werden die Systemkalendereintrge fr
	 *          den aktuellen Monat dargestellt, bei <code>false</code> nicht.
	 *          Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der die
	 *          Datumseingabe beinhaltet.
	 */
	public DatumZeit(Composite parent, int style, Eingabetyp eingabetyp,
			Date vorgabeDatum, boolean ereigniskalender, boolean systemkalender)
	{
		super(parent, style);
		this._eingabetyp = eingabetyp;
		this._vorgabeDatum = vorgabeDatum;
		this._ereigniskalender = ereigniskalender;
		this._systemkalender = systemkalender;
		init();
	}

	public DatumZeit(Composite parent, int style, Eingabetyp eingabetyp,
			Date vorgabeDatum, boolean ereigniskalender, boolean systemkalender,
			BereichsFeld bereichsFeld)
	{
		super(parent, style);
		this._eingabetyp = eingabetyp;
		this._vorgabeDatum = vorgabeDatum;
		this._ereigniskalender = ereigniskalender;
		this._systemkalender = systemkalender;
		this._bereichsFeld = bereichsFeld;
		init();
	}

	private void init()
	{
		erzeugeDarstellung();

		if (_vorgabeDatum != null)
		{
			_cdt.setSelection(_vorgabeDatum);
		}
	}

	/**
	 * Setzt das Datum (mit Uhrzeit)
	 * 
	 * @param datum
	 *          Datum
	 */
	public void setDatum(Date datum)
	{
		_cdt.setSelection(datum);
	}

	/**
	 * Liefert das selektierte Datum (mit Uhrzeit).
	 * 
	 * @return Datum.
	 */
	public Date getDatum()
	{
		return _cdt.getSelection();
	}

	/**
	 * Hinzufuegen eines {@link SelectionListener}. Im Data-Feld des Events wird
	 * das aktuelle, selektierte Datum bertragen!
	 * 
	 * @param zuhoerer
	 *          {@link SelectionListener}.
	 */
	public void hinzufuegenSelektionsZuhoerer(SelectionListener zuhoerer)
	{
		if (_cdt != null)
		{
			_cdt.addSelectionListener(zuhoerer);
		}
	}

	public void entferneSelektionsZuhoerer(SelectionListener zuhoerer)
	{
		if (_cdt != null)
		{
			_cdt.removeSelectionListener(zuhoerer);
		}
	}

	private void erzeugeDarstellung()
	{
		switch (_eingabetyp)
		{
		case datum:
			 erzeugeDatum();
			break;
		case datumuhr:
			erzeugeDatumUhr();
			break;
		case uhr:
			erzeugeUhr();
			break;
		case datumuhrMS:
			erzeugeDatumUhrMS();
			break;
		case datumOhneJahr:
			erzeugeDatumOhneJahr();
			break;
		case uhrMS:
			erzeugeUhrMS();
			break;
		case datumEinfach:
			erzeugeDatumEinfach();
			break;
		case datumuhrEinfach:
			erzeugeDatumUhrEinfach();
			break;
		case uhrEinfach:
			erzeugeUhrEinfach();
			break;
		case datumuhrEinfachMS:
			erzeugeDatumUhrEinfachMitMS();
			break;
		case uhrEinfachMS:
			erzeugeUhrEinfachMitMS();
			break;
		}

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_cdt.setLayoutData(gridData);

		if (_ereigniskalender || _systemkalender)
		{
			erzeugeEreignisSystemkalender();
		}

		GridLayout gridLayout = new GridLayout();

		if (_ereigniskalender || _systemkalender)
			gridLayout.numColumns = 2;
		else
			gridLayout.numColumns = 1;

		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginLeft = 0;
		gridLayout.marginRight = 0;
		gridLayout.marginTop = 0;
		gridLayout.marginBottom = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;

		this.setLayout(gridLayout);

		this.update();
	}

	private void erzeugeEreignisSystemkalender()
	{

		IFelderFuellen felderFuellen = new IFelderFuellen()
		{

			@Override
			public void fuellen(LinkedList<IEintragBereich> bereiche)
			{
				aktionFuellen(bereiche.getFirst());
			}

		};
		Button button = SchaltflaechenFabrikEreignisSystemKalenderDialog
				.erzeugeButton(this, felderFuellen, null, null);
		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = false;
		gd.grabExcessVerticalSpace = false;
		gd.horizontalAlignment = GridData.BEGINNING;
		gd.verticalAlignment = GridData.CENTER;
		button.setLayoutData(gd);

	}

	private void aktionFuellen(IEintragBereich bereich)
	{
		if (bereich != null)
		{
			if (_bereichsFeld != null)
			{
				if (_bereichsFeld.equals(BereichsFeld.vonFeld))
				{
					setDatum(new Date(bereich.getVon()));
				} else if (_bereichsFeld.equals(BereichsFeld.bisFeld))
				{
					setDatum(new Date(bereich.getBis()));
				}
			} else
			{
				setDatum(new Date(bereich.getVon()));
			}
		}
	}

	private void erzeugeDatum()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.DATE_MEDIUM);
	}

	private void erzeugeDatumUhr()
	{
		_cdt = new CDateTime(this, CDT.DROP_DOWN | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM | CDT.CLOCK_24_HOUR);

	}

	private void erzeugeUhr()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.TIME_MEDIUM
				| CDT.CLOCK_24_HOUR);
	}

	private void erzeugeDatumUhrMS()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM | CDT.CLOCK_24_HOUR);

		/*
		 * Pattern
		 */
		_cdt.setPattern("dd.MM.yyyy HH:mm:ss:SSS");

	}

	private void erzeugeUhrMS()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.CLOCK_24_HOUR);

		/*
		 * Pattern
		 */
		_cdt.setPattern("HH:mm:ss:SSS");
	}

	private void erzeugeDatumEinfach()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM);
	}

	private void erzeugeDatumUhrEinfach()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM | CDT.TIME_MEDIUM);
	}

	private void erzeugeUhrEinfach()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.TIME_MEDIUM);
	}

	/**
	 * Erzeugt ein Datum+Uhrzeitelement, mit dessen Hilfe auch Millisekunden
	 * erfasst werden knnen.
	 * 
	 */
	private void erzeugeDatumUhrEinfachMitMS()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM | CDT.TIME_MEDIUM);

		/*
		 * Pattern
		 */
		_cdt.setPattern("dd.MM.yyyy HH:mm:ss:SSS");

		/*
		 * Elementdimension neu bestimmen.
		 */
		Point size = _cdt.getSize();
		int breite = size.x;
		int hoehe = size.y;
		_cdt.setSize(breite + 15, hoehe);

	}

	/**
	 * Erzeugt ein Datum+Uhrzeitelement, mit dessen Hilfe auch Millisekunden
	 * erfasst werden knnen.
	 * 
	 */
	private void erzeugeDatumOhneJahr()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM | CDT.TIME_MEDIUM);

		/*
		 * Pattern
		 */
		_cdt.setPattern("dd.MM.");

		/*
		 * Elementdimension neu bestimmen.
		 */
		Point size = _cdt.getSize();
		int breite = size.x;
		int hoehe = size.y;
		_cdt.setSize(breite + 15, hoehe);

	}

	/**
	 * Erzeugt ein Uhrzeitelement, mit dessen Hife auch Millisekunden erfasst
	 * werden knnen.
	 * 
	 */
	private void erzeugeUhrEinfachMitMS()
	{
		_cdt = new CDateTime(this, CDT.SPINNER | CDT.TIME_MEDIUM);

		/*
		 * Pattern
		 */
		_cdt.setPattern("HH:mm:ss:SSS");
		Point size = _cdt.getSize();
		int breite = size.x;
		int hoehe = size.y;
		_cdt.setSize(breite + 15, hoehe);
	}

	/**
	 * Enumerator zur Angabe welches Feld des {@link IEintragBereich}es
	 * bercksichtigt werden soll.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum BereichsFeld
	{
		/**
		 * Von Feld des {@link IEintragBereich}es
		 */
		vonFeld,

		/**
		 * Bis Feld des {@link IEintragBereich}es
		 */
		bisFeld
	}

	/**
	 * Enumerator fr die Auswahl von Datum und Uhrzeit
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Eingabetyp
	{
		/**
		 * Nur Datum
		 */
		datum,
		/**
		 * Datum und Uhr
		 */
		datumuhr,
		/**
		 * Nur Uhr
		 */
		uhr,
		/**
		 * Datum und Uhr
		 */
		datumuhrMS,
		/**
		 * Nur Uhr
		 */
		uhrMS,
		/**
		 * Datum einfach
		 */
		datumEinfach,
		/**
		 * Datum + Uhr einfach
		 */
		datumuhrEinfach,
		/**
		 * Uhr einfach
		 */
		uhrEinfach,
		/**
		 * Datum + Uhr einfach, mit Millisekunden
		 */
		datumuhrEinfachMS,
		/**
		 * Datum ohne Jahr
		 */
		datumOhneJahr,
		/**
		 * Datum + Uhr einfach mit Millisekunden
		 */
		uhrEinfachMS
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Widget#notifyListeners(int, org.eclipse.swt.widgets.Event)
	 */
	@Override
	public void notifyListeners(int eventType, Event event) {
		// TODO Auto-generated method stub
		_cdt.notifyListeners(eventType, event);
	}

}
