/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Ereigniskalenderverwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface
{
	/**
	 * Das Verbindungsobjekt
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Liste der Systemobjekte, auf die sich fr den Empfang angemeldet wurde.
	 * Wird fr die Abmeldung bentigt.
	 */
	private LinkedList<SystemObject> _anmeldungAufSOFuerEmpfang = new LinkedList<SystemObject>();

	/**
	 * Liste der Ereigniskalendereintraege
	 */
	private List<EintragEreigniskalender> _eintraege = new LinkedList<EintragEreigniskalender>();

	/**
	 * Objekt zur Synchronisation des Empfangs durch die Update-Methode
	 */
	private Object _syncEmpfang = new Object();

	/**
	 * Konstruktor, setzt die Verbindung zum DaV.
	 * 
	 * @param connection
	 *          Verbindung zum DaV.
	 */
	public DaVKommunikation()
	{
		/*
		 * Bestimmen der DaV-Verbindung
		 */
		this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

	}

	/**
	 * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der
	 * Attibutgruppe atgPid unter dem Aspekt aspPid.
	 * 
	 * @param objId
	 *          Id des Objekts
	 * @param atgPid
	 *          Attributgruppe die abgemeldet werden soll
	 * @param aspPid
	 *          Apekt der abgemeldet werden soll
	 */
	@SuppressWarnings("unused")
	private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
	{
		if (objId == null || atgPid == null || aspPid == null)
			throw new IllegalArgumentException();

		SystemObject obj = _connection.getDataModel().getObject(objId);
		AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
		Aspect asp = _connection.getDataModel().getAspect(aspPid);
		DataDescription dd = new DataDescription(atg, asp);

		_connection.unsubscribeReceiver(this, obj, dd);
	}

	/**
	 * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der
	 * Attibutgruppe atgPid unter dem Aspekt aspPid.
	 * 
	 * @param objId
	 *          Id des Objekts
	 * @param atgPid
	 *          Attributgruppe die abgemeldet werden soll
	 * @param aspPid
	 *          Apekt der abgemeldet werden soll
	 */
	@SuppressWarnings("unchecked")
	private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
	{
		if (objs == null || atgPid == null || aspPid == null)
			throw new IllegalArgumentException();

		AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
		Aspect asp = _connection.getDataModel().getAspect(aspPid);
		DataDescription dd = new DataDescription(atg, asp);

		_connection.unsubscribeReceiver(this, objs, dd);
	}

	/**
	 * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum
	 * Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
	 * 
	 * @param objPid
	 *          Pid des Objekts
	 * @param atgPid
	 *          Attributgruppe die angemeldet werden soll
	 * @param aspPid
	 *          Apekt der angemeldet werden soll
	 * @param role
	 *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
	 */
	@SuppressWarnings("unused")
	private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid,
			ReceiverRole role)
	{
		if (objPid == null || atgPid == null || aspPid == null || role == null)
			throw new IllegalArgumentException();

		SystemObject obj = _connection.getDataModel().getObject(objPid);
		AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
		Aspect asp = _connection.getDataModel().getAspect(aspPid);
		DataDescription dd = new DataDescription(atg, asp);

		_connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

	}

	/**
	 * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum
	 * Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
	 * 
	 * @param objPid
	 *          Pid des Objekts
	 * @param atgPid
	 *          Attributgruppe die angemeldet werden soll
	 * @param aspPid
	 *          Apekt der angemeldet werden soll
	 * @param role
	 *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
	 */
	@SuppressWarnings("unchecked")
	private void anmeldeLeseDaV(List objs, String atgPid, String aspPid,
			ReceiverRole role)
	{

		if (objs == null || atgPid == null || aspPid == null || role == null)
			throw new IllegalArgumentException();

		AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
		Aspect asp = _connection.getDataModel().getAspect(aspPid);
		DataDescription dd = new DataDescription(atg, asp);

		_connection
				.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

		@SuppressWarnings("unused")
		ConfigurationObject konfigurationsObjekt = (ConfigurationObject) _connection
				.getLocalConfigurationAuthority();

	}

	/**
	 * Verarbeitet smtliche Updates
	 */
	public void update(ResultData[] results)
	{
		List<EintragEreigniskalender> listLoesch = new ArrayList<EintragEreigniskalender>();
		List<EintragEreigniskalender> listHinzu = new ArrayList<EintragEreigniskalender>();

		for (int i = 0; i < results.length; i++)
		{

			final ResultData result = results[i];
			final Data data = result.getData();
			final SystemObject object = result.getObject();

			if (data != null)
			{

				EintragEreigniskalender eintrag = new EintragEreigniskalender();
				eintrag.setPid(object.getPid());
				eintrag.setName(object.getName());
				AttributeGroup atg = _connection.getDataModel().getAttributeGroup(
						"atg.ereignisEigenschaften");
				Data configData = object.getConfigurationData(atg);
				String beschreibung = configData.getItem("Ereignisbeschreibung")
						.asTextValue().getText();
				eintrag.setBeschreibung(beschreibung);
				SystemObject typObject = configData.getItem("EreignisTypReferenz")
						.asReferenceValue().getSystemObject();
				eintrag.setEreignisTyp(typObject);
				eintrag.setQuelle(data.getItem("Quelle").asTextValue().getText());
				SystemObject systemObject = data.getItem(
						"SystemKalenderEintragReferenz").asReferenceValue()
						.getSystemObject();

				long anfang = data.getItem("BeginnZeitlicheGltigkeit").asTimeValue()
						.getMillis();
				long ende = data.getItem("EndeZeitlicheGltigkeit").asTimeValue()
						.getMillis();

				if (anfang != ende)
				{

					eintrag
							.setArtDerZeitspezifikation(EintragEreigniskalender._artDerZeitspezifikation.Zeitbereichsangabe);
				} else
				{

					eintrag
							.setArtDerZeitspezifikation(EintragEreigniskalender._artDerZeitspezifikation.Systemkalendereintrag);
				}

				eintrag.setAnfangsZeitpunkt(anfang);
				eintrag.setEndZeitpunkt(ende);
				eintrag.setSystemKalenderEintrag(systemObject);

				List<EintragRaeumlicheGueltigkeit> list1 = new ArrayList<EintragRaeumlicheGueltigkeit>();
				ReferenceArray array1 = data.getItem("RumlicheGltigkeit")
						.asReferenceArray();
				for (int j = 0; j < array1.getLength(); j++)
				{
					SystemObject so = array1.getReferenceValue(j).getSystemObject();
					ConfigurationObject co = (ConfigurationObject) so;
					EintragRaeumlicheGueltigkeit e = new EintragRaeumlicheGueltigkeit(so
							.getName(), co.getInfo().getShortInfo(), so);
					list1.add(e);

				}
				eintrag.setListeEintragRaeumlicheGueltigkeit(list1);

				List<EintragVerkehrlicheGueltigkeit> list2 = new ArrayList<EintragVerkehrlicheGueltigkeit>();
				Array array2 = data.getItem("VerkehrlicheGltigkeit").asArray();
				for (int j = 0; j < array2.getLength(); j++)
				{
					EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt anfBezug = null;
					EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt endBezug = null;

					long anfZeit = array2.getItem(j).getItem("ZeitDauerAnfangIntervall")
							.asTimeValue().getMillis();
					String s1 = array2.getItem(j).getItem("ZeitBezugAnfangIntervall")
							.asTextValue().getText();
					long endZeit = array2.getItem(j).getItem("ZeitDauerEndeIntervall")
							.asTimeValue().getMillis();
					String s2 = array2.getItem(j).getItem("ZeitBezugEndeIntervall")
							.asTextValue().getText();

					if (s1
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang
									.toString()))
						anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang;
					else if (s1
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang
									.toString()))
						anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang;
					else if (s1
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde
									.toString()))
						anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde;
					else if (s1
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde
									.toString()))
						anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde;

					if (s2
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang
									.toString()))
						endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang;
					else if (s2
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang
									.toString()))
						endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang;
					else if (s2
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde
									.toString()))
						endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde;
					else if (s2
							.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde
									.toString()))
						endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde;

					EintragVerkehrlicheGueltigkeit e = new EintragVerkehrlicheGueltigkeit(
							anfZeit, anfBezug, endZeit, endBezug);
					list2.add(e);

				}
				eintrag.setListeEintragVerkehrlicheGueltigkeit(list2);

				String vorhanden = null;

				for (EintragEreigniskalender e : _eintraege)
				{
					if (e.getPid().equals(eintrag.getPid()))
						vorhanden = e.getPid();
				}

				if (vorhanden == null)
					_eintraege.add(eintrag);
				else
				{
					for (EintragEreigniskalender e : _eintraege)
					{
						if (e.getPid().equals(eintrag.getPid()))
						{
							listLoesch.add(e);
							listHinzu.add(eintrag);

						}
					}
				}
			}
		}

		_eintraege.removeAll(listLoesch);
		_eintraege.addAll(listHinzu);

		/*
		 * Prfen, ob zu den Objekten auf die sich angemeldet wurde, alle Daten
		 * empfangen worden sind.
		 */
		boolean alleSysObjDa = true;
		Iterator<SystemObject> itVgl = _anmeldungAufSOFuerEmpfang.iterator();
		while (itVgl.hasNext())
		{
			boolean tempVgl = false;
			SystemObject vgl = itVgl.next();
			Iterator<EintragEreigniskalender> itEin = _eintraege.iterator();
			while (itEin.hasNext())
			{
				EintragEreigniskalender ein = itEin.next();
				if (vgl.getPid().equals(ein.getPid()))
				{
					tempVgl = true;
				}
				if (tempVgl)
				{
					break;
				}
			}

			if (!tempVgl)
			{
				alleSysObjDa = false;
				break;
			}
		}

		/*
		 * Wenn alle Daten empfangen wurden, dann ber das SyncEmpfang Objekt die
		 * Synchronisation durchfhren.
		 */
		if (alleSysObjDa)
		{
			synchronized (_syncEmpfang)
			{
				_syncEmpfang.notify();
			}
		}

	}

	public List<EintragEreigniskalender> getEreignisKalenderEintraege()
	{

		return _eintraege;

	}

	/**
	 * Erzeugt ein Ereignis anhand des bergegeben eintags
	 * 
	 * @param eintrag
	 */
	public void erzeugeEreignisKalenderEintrag(EintragEreigniskalender eintrag)
	{

		DataModel dataModel = _connection.getDataModel();

		// ConfigurationArea configArea =
		// dataModel.getConfigurationArea("kb.default.testKonfiguration");
		// ConfigurationObject configObj =
		// (ConfigurationObject)dataModel.getObject("kv.testKonfiguration");

		ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
		ConfigurationAuthority configObj = _connection
				.getLocalConfigurationAuthority();

		DynamicObjectType dynObjTyp = (DynamicObjectType) dataModel
				.getType("typ.ereignis");
		AttributeGroup attributgruppe = dataModel
				.getAttributeGroup("atg.ereignisEigenschaften");
		Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
		VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection,
				dataModel, configArea, dynObjTyp, configObj, attributgruppe, aspekt);

		if (dataModel.getObject(eintrag.getPid()) == null)
		{

			Data data = _connection.createData(attributgruppe);
			data.getItem("Ereignisbeschreibung").asTextValue().setText(
					eintrag.getBeschreibung());
			data.getItem("EreignisTypReferenz").asReferenceValue().setSystemObject(
					eintrag.getEreignisTyp());
			Data[] datas = new Data[1];
			datas[0] = data;
			vewEigenschaften.erzeuge(eintrag.getPid(), eintrag.getName(),
					"Ereignisse", datas);

		}

	}

	/**
	 * Loescht das Ereignis mit der angegebenen Pid
	 * 
	 * @param pid
	 */
	public void loescheEreignisKalenderEintrag(String pid)
	{

		DataModel dataModel = _connection.getDataModel();

		AttributeGroup atg = dataModel.getAttributeGroup("atg.ereignisParameter");
		Aspect asp = dataModel.getAspect("asp.parameterSoll");
		DataDescription dd = new DataDescription(atg, asp);

		_connection.unsubscribeReceiver(this, dataModel.getObject(pid), dd);

		// ConfigurationArea configArea =
		// dataModel.getConfigurationArea("kb.default.testKonfiguration");
		// ConfigurationObject confObject =
		// (ConfigurationObject)dataModel.getObject("kv.testKonfiguration");

		ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
		ConfigurationAuthority confObject = _connection
				.getLocalConfigurationAuthority();

		DynamicObjectType dynObjType = (DynamicObjectType) dataModel
				.getType("typ.ereignis");

		AttributeGroup attributgruppe = dataModel
				.getAttributeGroup("atg.ereignisEigenschaften");
		Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
		VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection,
				dataModel, configArea, dynObjType, confObject, attributgruppe, aspekt);
		Data data = _connection.createData(attributgruppe);
		Data[] datas = new Data[1];
		datas[0] = data;
		vewEigenschaften.setDynamicObject((DynamicObject) dataModel.getObject(pid));
		vewEigenschaften.loesche(pid, "Ereignisse");

	}

	/**
	 * Fhrt eine Neuanmldung der Ereignisse durch
	 */
	public void anmelden()
	{

		if (_connection != null)
		{
			ConfigurationObject konfigurationsObjekt = (ConfigurationObject) _connection
					.getLocalConfigurationAuthority();

			ObjectSet objekte = konfigurationsObjekt.getObjectSet("Ereignisse");

			List<SystemObject> list = objekte.getElements();

			List<SystemObject> listTmp = new ArrayList<SystemObject>();
			for (SystemObject systemObject : list)
			{
				if (systemObject.isValid())
				{
					listTmp.add(systemObject);

				}
			}

			_anmeldungAufSOFuerEmpfang.clear();

			_anmeldungAufSOFuerEmpfang.addAll(listTmp);

			anmeldeLeseDaV(listTmp, "atg.ereignisParameter", "asp.parameterSoll",
					ReceiverRole.receiver());
		}
	}

	public void abmelden()
	{
		abmeldeLeseDaV(_anmeldungAufSOFuerEmpfang, "atg.ereignisParameter",
				"asp.parameterSoll");
	}

	public LinkedList<EintragEreigniskalender> getEreigniskalenderEintraegeAlle()
	{
		LinkedList<EintragEreigniskalender> re = new LinkedList<EintragEreigniskalender>();

		synchronized (_syncEmpfang)
		{
			anmelden();
			try
			{
				_syncEmpfang.wait();
				re.addAll(_eintraege);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			} finally
			{
				abmelden();
			}
		}

		if (re.size() > 1)
		{
			Collections.sort(re);
		}

		return re;
	}

	/**
	 * Liefert eine gefilterte Liste aller Ereigniskalendereintrge in einem bestimmten Bereich.
	 * @param von Anfangszeitpunkt
	 * @param bis Endzeitpunkt
	 * @return
	 */
	public LinkedList<EintragEreigniskalender> getEreigniskalenderEintraegeGefiltert(
			Long von, Long bis)
	{
		LinkedList<EintragEreigniskalender> ereignisse = getEreigniskalenderEintraegeAlle();
		LinkedList<EintragEreigniskalender> re = new LinkedList<EintragEreigniskalender>();

		Iterator<EintragEreigniskalender> it = ereignisse.iterator();
		while (it.hasNext())
		{
			EintragEreigniskalender ereigniskalender = it.next();
			Long anfang = ereigniskalender.getAnfangsZeitpunkt();
			Long ende = ereigniskalender.getEndZeitpunkt();

			if ((von <= anfang) && (bis >= ende))
			{
				re.add(ereigniskalender);
			}

		}

		return re;
	}

	/**
	 * Bestimmt den Default-Konfigurationsbereich des DaV
	 * 
	 * @param connection
	 * @return
	 */
	public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
	{
		Data data = connection.getDataModel().getConfigurationAuthority()
				.getConfigurationData(
						connection.getDataModel().getAttributeGroup(
								"atg.konfigurationsVerantwortlicherEigenschaften"));

		if (data != null)
		{
			TextArray array = data.getTextArray("defaultBereich");

			if (array.getLength() == 1)
			{
				return (ConfigurationArea) connection.getDataModel().getObject(
						array.getText(0));
			}
		}

		return null;
	}

}
