/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse {@link EintragRaeumlicheGueltigkeit} reprsentiert die Datenstze zur rumlichen Gltigkeit, die im Dialog
 * {@link Definitionsdialog} in der Liste der rumlichen Gltigkeiten dargestellt werden. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/02/04 06:12:25 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class EintragRaeumlicheGueltigkeit implements Cloneable
{
  /**
   * Name
   */
  private String _Name = null;

  /**
   * Info
   */
  private String _Info = null;

  /**
   * SystemObject
   */
  private SystemObject _SystemObjekt = null;

  /**
   * Konstruktor zur Erezugung eines Datensatzes der rumlichen Gltigkeit.
   * 
   * @param name
   *          Name der rumlichen Gltigkeit.
   * @param info
   *          Info zur rumlichen Gltigkeit.
   */
  public EintragRaeumlicheGueltigkeit(String name, String info, SystemObject systemObjekt)
  {
    this._Name = name;
    this._Info = info;
    this._SystemObjekt = systemObjekt;
  }

  /**
   * Liefert den Namen der rumlichen Gltigkeit.
   * 
   * @return Name der rumlichen Gltigkeit.
   */
  public String getName()
  {
    return _Name;
  }

  /**
   * Setzt den Namen der rumlichen Gltigkeit.
   * 
   * @param name
   *          Name der rumlichen Gltigkeit.
   */
  public void setName(String name)
  {
    _Name = name;
  }

  /**
   * Liefert die Info zur rumlichen Gltigkeit.
   * 
   * @return Info zur rumlichen Gltigkeit.
   */
  public String getInfo()
  {
    return _Info;
  }

  /**
   * Setzt die Info zur rumlichen Gltigkeit.
   * 
   * @param info
   *          Info zur rumlichen Gltigkeit.
   */
  public void setInfo(String info)
  {
    _Info = info;
  }

  /**
   * Liefert einen Clone der Instanz dieser Klasse.
   * 
   * @return Clone der Instanz dieser Klasse.
   */
  public EintragRaeumlicheGueltigkeit clone()
  {
    EintragRaeumlicheGueltigkeit raeumlicheGueltigkeit = new EintragRaeumlicheGueltigkeit(_Name, _Info, _SystemObjekt);
    return raeumlicheGueltigkeit;
  }

  public SystemObject getSystemObjekt()
  {
    return _SystemObjekt;
  }

  public void setSystemObjekt(SystemObject systemObjekt)
  {
    _SystemObjekt = systemObjekt;
  }

  @Override
  public boolean equals(Object obj)
  {
    boolean flagge = false;
    EintragRaeumlicheGueltigkeit eintrag = (EintragRaeumlicheGueltigkeit)obj;
    if (_SystemObjekt.equals(eintrag.getSystemObjekt()))
      flagge = true;
    return flagge;

  }

}
