/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.LinkedList;

import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

public interface IEintragKapsel
{

	/**
	 * Liefert die Pid des Systemkalendereintrags
	 * 
	 * @return
	 */
	public abstract String getPid();

	/**
	 * Fgt einen neuen {@link IEintragBereich}, fr den dieser
	 * {@link SystemkalenderEintrag} Gltigkeit hat, zur Liste der Bereiche hinzu.
	 * Der {@link IEintragBereich} wird aus den Von- und Bis-Zeitpunkten des Bereiches
	 * erstellt.
	 * 
	 * @param von
	 *          Von-Zeitpunkt
	 * @param bis
	 *          Bis-Zeitpunkt
	 */
	public abstract void addBereich(Long von, Long bis);

	/**
	 * Fgt einen {@link IEintragBereich} zur Liste der Bereiche hinzu.
	 * 
	 * @param bereich
	 *          Hinzuzufgender {@link IEintragBereich}.
	 */
	public abstract void addBereich(IEintragBereich bereich);

	/**
	 * Liefert eine Liste von {@link IEintragBereich}en, fr den dieser
	 * Systemkalendereintrag Gltigkeit hat.
	 * 
	 * @return Liste gltiger {@link IEintragBereich}e.
	 */
	public abstract LinkedList<IEintragBereich> getBereiche();

	/**
	 * Liefert die Anzahl der Bereiche zu einem Eintrag.
	 * 
	 * @return Anzahl der Bereiche.
	 */
	public abstract int getBereicheAnzahl();

	public abstract String toString();

	public abstract String toStringLong();

	public abstract int compareTo(IEintragKapsel o);

}