/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GMBH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225 / 64-03
 * Fax: +49 (0)7225 / 64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.rw.basislib.legende;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.Activator;

/**
 * Diese View realisiert die Darstellung der Legende. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * -TBuV 20
 * 
 * @author HofmannJ
 * 
 */
public class LegendeSicht extends ViewPart
{

	/**
	 * Der Debugger
	 */
	private static final boolean _debug = false;

	/**
	 * In dieser HashMap werden die einzelnen LegendenAbschnitte abgelegt.
	 */
	private TreeMap<String, ILegendenAbschnitt> _seiten = new TreeMap<String, ILegendenAbschnitt>();

	// private List<ILegendenAbschnitt> _lSeiten = new
	// LinkedList<ILegendenAbschnitt>();

	/**
	 * Scrolled Composite der Legende.
	 */
	private ScrolledComposite _legendenComposite;

	/**
	 * Eigentlicher Inhalt der Legende.
	 */
	private Composite _legendeContent;

	private Composite _compositeBasis;

	private Composite _compositeOben;

	private Composite _compositeWappen;

	private Composite _compositeZeit;

	/**
	 * {@link Canvas}, in dem das Wappen angezeigt wird.
	 */
	private Canvas _canvasWappen = null;

	/**
	 * GridData fr das Wappen
	 */
	private GridData _gridDataWappenCanvas = null;

	/**
	 * Listener zur Reaktion auf genderte Zeitpunkte.
	 */
	private ILegendenZeitpunkt _zeitpunktListener = null;

	private Text _zeitText;

	private IPartListener _partListener;

	private IWorkbenchPart _firstPart = null;

	@Override
	public void init(IViewSite site) throws PartInitException
	{
		super.init(site);

	}

	@Override
	public void createPartControl(Composite parent)
	{
		/*
		 * Parent
		 */
		parent.setLayout(new GridLayout(1, false));

		/*
		 * Basis
		 */
		_compositeBasis = new Composite(parent, SWT.NONE);
		_compositeBasis.setLayout(new GridLayout(1, false));

		GridData gridDataCompositeBasis = new GridData();
		gridDataCompositeBasis.horizontalAlignment = GridData.FILL;
		gridDataCompositeBasis.grabExcessHorizontalSpace = true;
		gridDataCompositeBasis.verticalAlignment = GridData.CENTER;
		_compositeBasis.setLayoutData(gridDataCompositeBasis);

		/*
		 * Oben
		 */
		_compositeOben = new Composite(_compositeBasis, SWT.NONE);
		_compositeOben.setLayout(new GridLayout());

		GridData gridDataCompositeOben = new GridData();
		gridDataCompositeOben.horizontalAlignment = GridData.FILL;
		gridDataCompositeOben.grabExcessHorizontalSpace = true;
		gridDataCompositeOben.verticalAlignment = GridData.CENTER;
		_compositeOben.setLayoutData(gridDataCompositeOben);

		/*
		 * Wappen
		 */
		GridData gridDataCompositeWappen = new GridData();
		gridDataCompositeWappen.horizontalAlignment = GridData.FILL;
		gridDataCompositeWappen.grabExcessHorizontalSpace = true;
		gridDataCompositeWappen.verticalAlignment = GridData.CENTER;
		_compositeWappen = new Composite(_compositeOben, SWT.NONE);
		_compositeWappen.setLayout(new GridLayout());

		// setWappenIntern("icons/sonstiges/logo.gif");
		setWappenIntern("icons/wappen_bw/landeswappen_bw_mittel.gif");
		_compositeWappen.setLayoutData(gridDataCompositeWappen);

		/*
		 * Composite Zeit
		 */
		_compositeZeit = new Composite(_compositeOben, SWT.NONE);

		GridData gridDataCompositeZeit = new GridData();
		gridDataCompositeZeit.horizontalAlignment = GridData.FILL;
		gridDataCompositeZeit.grabExcessHorizontalSpace = true;
		gridDataCompositeZeit.verticalAlignment = GridData.CENTER;

		_compositeZeit.setLayout(new GridLayout(1, false));
		_compositeZeit.setLayoutData(gridDataCompositeZeit);

		/*
		 * Zeit-Feld
		 */
		_zeitText = new Text(_compositeZeit, SWT.NONE);
		_zeitText.setEditable(false);
		_zeitText.setFont(new Font(Display.getCurrent(), "Arial", 12, SWT.BOLD));
		_zeitText.setText("                             ");

		GridData gridDataZeitFeld = new GridData();
		gridDataZeitFeld.horizontalAlignment = GridData.CENTER;
		gridDataZeitFeld.verticalAlignment = GridData.CENTER;
		gridDataZeitFeld.grabExcessHorizontalSpace = true;
		_zeitText.setLayoutData(gridDataZeitFeld);

		registriereLegendenZeitpunktListener();

		_legendenComposite = new ScrolledComposite(parent, SWT.NONE | SWT.H_SCROLL
				| SWT.V_SCROLL);
		GridData gdCompLeggende = new GridData();
		gdCompLeggende.horizontalAlignment = GridData.FILL;
		gdCompLeggende.grabExcessHorizontalSpace = true;
		gdCompLeggende.grabExcessVerticalSpace = true;
		gdCompLeggende.verticalAlignment = GridData.FILL;

		_legendenComposite.setLayoutData(gdCompLeggende);

		_partListener = new IPartListener()
		{

			@Override
			public void partOpened(IWorkbenchPart part)
			{
				if (_debug)
				{
					System.out.println("partOpened: " + getPartInfo(part));
				}

				Object adapter = part.getAdapter(ILegendenAbschnitt.class);
				if (adapter instanceof ILegendenAbschnitt)
				{
					ILegendenAbschnitt lPage = (ILegendenAbschnitt) adapter;

					hinzufuegenLegendenAbschnitt(lPage);
				}
			}

			@Override
			public void partDeactivated(IWorkbenchPart part)
			{
				if (_debug)
				{
					System.out.println("partDeactivated: " + getPartInfo(part));
				}
			}

			@Override
			public void partClosed(IWorkbenchPart part)
			{
				if (_debug)
				{
					System.out.println("partClosed: " + getPartInfo(part));
				}
				Object adapter = part.getAdapter(ILegendenAbschnitt.class);
				if (adapter instanceof ILegendenAbschnitt)
				{
					ILegendenAbschnitt lPage = (ILegendenAbschnitt) adapter;

					entferneLegendenAbschnitt(lPage);
				}
			}

			@Override
			public void partBroughtToTop(IWorkbenchPart part)
			{
				if (_debug)
				{
					System.out.println("partBroughtToTop: " + getPartInfo(part));
				}
			}

			@Override
			public void partActivated(IWorkbenchPart part)
			{
				if (_debug)
				{
					System.out.println("partActivated: " + getPartInfo(part));
				}

				Object adapter = part.getAdapter(ILegendenAbschnitt.class);
				if (adapter instanceof ILegendenAbschnitt)
				{
					ILegendenAbschnitt lPage = (ILegendenAbschnitt) adapter;

					hinzufuegenLegendenAbschnitt(lPage, false);

					reGenerieren(part);
				}
			}
		};

		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService()
				.addPartListener(_partListener);

		IWorkbenchWindow wbw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		IWorkbenchPage[] pages = wbw.getPages();
		for (IWorkbenchPage page : pages)
		{
			if (page != null)
			{
				IViewReference[] viewReferences = page.getViewReferences();
				for (IViewReference iViewReference : viewReferences)
				{
					IViewPart view = iViewReference.getView(false);
					if (view instanceof ILegendenAbschnitt)
					{
						ILegendenAbschnitt abs = (ILegendenAbschnitt) view;
						hinzufuegenLegendenAbschnitt(abs, false);
					}
				}

				IEditorReference[] editorReferences = page.getEditorReferences();
				for (IEditorReference iEditorReference : editorReferences)
				{
					IEditorPart editor = iEditorReference.getEditor(false);
					if (editor instanceof ILegendenAbschnitt)
					{
						ILegendenAbschnitt abs = (ILegendenAbschnitt) editor;
						hinzufuegenLegendenAbschnitt(abs, false);
					}
				}
			}

		}
		reGenerieren(null);

		/*
		 * Aktiven Part merken
		 */
		IWorkbench workbench = PlatformUI.getWorkbench();
		if (workbench != null)
		{
			IWorkbenchWindow activeWorkbenchWindow = workbench
					.getActiveWorkbenchWindow();
			if (activeWorkbenchWindow != null)
			{
				IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();
				if (activePage != null)
				{
					IWorkbenchPart activePart = activePage.getActivePart();
					if ((activePart != null)
							&& (activePart instanceof ILegendenAbschnitt))
					{
						ILegendenAbschnitt abs = (ILegendenAbschnitt) activePart;

						hinzufuegenLegendenAbschnitt(abs, false);

						reGenerieren(activePart);
					}
				}
			}
		}

		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService()
				.addPartListener(_partListener);

		PlatformUI.getWorkbench().getHelpSystem().setHelp(_compositeBasis,
				"de.bsvrz.buv.rw.basislib.legende");

	}

	/**
	 * Aktuallisiert die Darstellung der Legendenabschnitte. Dies ist z.B. dann
	 * notwendig, wenn ein Legendenabschnit entfernt, hinzugefgt oder
	 * aktuallisiert werden soll.
	 * 
	 * @param firstPart
	 *          Part des obersten Legendenabschnittes.
	 */
	private void reGenerieren(IWorkbenchPart firstPart)
	{
		if (firstPart != null)
		{
			_firstPart = firstPart;
		}

		if (_legendeContent == null)
		{
			_legendeContent = new Composite(_legendenComposite, SWT.NONE);
			_legendenComposite.setContent(_legendeContent);
		} else
		{
			for (Control child : _legendeContent.getChildren())
			{
				if (!child.isDisposed())
				{
					child.dispose();
				}
			}
		}

		_legendeContent.setLayout(new GridLayout(1, false));
		GridData gridDataParent = new GridData();
		gridDataParent.horizontalAlignment = GridData.FILL;
		gridDataParent.grabExcessHorizontalSpace = true;
		gridDataParent.verticalAlignment = GridData.CENTER;
		_legendeContent.setLayoutData(gridDataParent);

		LinkedList<ILegendenAbschnitt> lAbschnitte = new LinkedList<ILegendenAbschnitt>();
		synchronized (_seiten)
		{
			Iterator<Entry<String, ILegendenAbschnitt>> it = _seiten.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, ILegendenAbschnitt> entry = it.next();
				if (entry.getValue() != null)
				{
					lAbschnitte.add(entry.getValue());
				}
			}
		}

		if (_firstPart != null)
		{
			if (_firstPart instanceof ILegendenAbschnitt)
			{
				ILegendenAbschnitt first = (ILegendenAbschnitt) _firstPart;
				if (lAbschnitte.contains(first))
				{
					lAbschnitte.remove(first);
					lAbschnitte.addFirst(first);
				}
			}
		}

		for (ILegendenAbschnitt iLegendenAbschnitt : lAbschnitte)
		{
			ILegendenAbschnitt element = iLegendenAbschnitt;
			Group pageParent = new Group(_legendeContent, SWT.NONE);
			pageParent.setText(element.getTitel());
			element.erzeugeAbschnittsInhalt(pageParent);
		}

		_legendeContent.layout();
		_legendeContent.pack();

		if (_debug)
		{
			System.out.println("Gre Legende Content=>" + _legendeContent.getSize()

			);
		}

//		System.out.println("HERE");
	}

	/**
	 * Fgt einen LegendenAbschnitt hinzu.
	 * 
	 * @param abschnitt
	 *          Abschnitt, der hinzugefgt werden soll.
	 */
	public void hinzufuegenLegendenAbschnitt(ILegendenAbschnitt abschnitt)
	{
		hinzufuegenLegendenAbschnitt(abschnitt, true);
	}

	/**
	 * Fgt einen LegendenAbschnitt hinzu.
	 * 
	 * @param abschnitt
	 *          Abschnitt, der hinzugefgt werden soll.
	 * @param regenerieren
	 *          Wenn <code>true</code>, wird die Darstellung nach der
	 *          Sortierreihenfolge der TreeMap neu erstellt. Wenn
	 *          <code>false</code>, dann nicht.
	 * @return Liefert <code>true</code> wenn der Abschnitt erfolgreich
	 *         hinzugefgt werden konnte, sonst <code>false</code>.
	 */
	private boolean hinzufuegenLegendenAbschnitt(ILegendenAbschnitt abschnitt,
			boolean regenerieren)
	{
		boolean erfolgreichHinzugefuegt = false;

		try
		{
			if (!((abschnitt.getId() == null) || (abschnitt.getId().length() == 0)))
			{
				if (!((abschnitt.getTitel() == null) || (abschnitt.getTitel().length() == 0)))
				{
					synchronized (_seiten)
					{
						if (_seiten.containsKey(abschnitt.getId()))
						{
							_seiten.remove(abschnitt.getId());
						}

						_seiten.put(abschnitt.getId(), abschnitt);

						erfolgreichHinzugefuegt = true;
					}

					if (erfolgreichHinzugefuegt && regenerieren)
					{
						reGenerieren(null);
					}
				} else
				{
					String msg = "Der vom ILegendenAbschnit ("
							+ abschnitt.getClass().getName()
							+ ") erhaltene Titel ist null oder hat die Lnge 0. Der Legendenabschnitt kann somit nicht dargestellt werden.";
					ILog log = Activator.getDefault().getLog();
					log.log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
				}
			} else
			{
				String msg = "Die vom ILegendenAbschnit ("
						+ abschnitt.getClass().getName()
						+ ") erhaltene ID ist null oder hat die Lnge 0.  Der Legendenabschnitt kann somit nicht dargestellt werden.";
				ILog log = Activator.getDefault().getLog();
				log.log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
			}
		} catch (NullPointerException npe)
		{
			String msg = "Ein bergebener ILegendenAbschnitt ist null und kann somit nicht verarbeitet und dargestellt werden.";
			ILog log = Activator.getDefault().getLog();
			log.log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg, npe));
		}

		return erfolgreichHinzugefuegt;
	}

	/**
	 * Entfernt einen Legendenabschnitt.
	 * 
	 * @param abschnitt
	 *          Abschnitt, der entfernt werden soll.
	 */
	public void entferneLegendenAbschnitt(ILegendenAbschnitt abschnitt)
	{
		if (_legendeContent != null)
		{
			for (Control child : _legendeContent.getChildren())
			{
				child.dispose();
			}
		}

		boolean seiteGeloescht = false;
		synchronized (_seiten)
		{
			if (_seiten.containsKey(abschnitt.getId()))
			{
				_seiten.remove(abschnitt.getId());
				seiteGeloescht = true;
			}
		}

		if (seiteGeloescht)
		{
			reGenerieren(null);
		}

	}

	/**
	 * Setzt das Wappen.
	 * 
	 * @param pfad
	 *          Pfad zum zu setzenden Wappen
	 */
	public void setWappen(String pfad)
	{
		if (_gridDataWappenCanvas == null)
		{
			_gridDataWappenCanvas = new GridData();
			_gridDataWappenCanvas.horizontalAlignment = GridData.CENTER;
			_gridDataWappenCanvas.verticalAlignment = GridData.CENTER;
			_gridDataWappenCanvas.grabExcessHorizontalSpace = true;
		}

		if (_canvasWappen == null)
		{
			_canvasWappen = new Canvas(_compositeWappen, SWT.NONE);
			_canvasWappen.setLayoutData(_gridDataWappenCanvas);
		}

		Image img = null;
		File pruefe = new File(pfad);
		if (pruefe.exists())
		{
			// img = Activator.getImageDescriptor(pfad).createImage();

			img = new Image(Display.getCurrent(), pfad);
			_gridDataWappenCanvas.widthHint = img.getBounds().width;
			_gridDataWappenCanvas.heightHint = img.getBounds().height;
			_canvasWappen.setBackgroundImage(img);
		}

	}

	/**
	 * Setzt das Wappen aus internen Sourcen.
	 * 
	 * @param pfad
	 *          Pfad zum zu setzenden Wappen
	 */
	public void setWappenIntern(String pfad)
	{
		if (_gridDataWappenCanvas == null)
		{
			_gridDataWappenCanvas = new GridData();
			_gridDataWappenCanvas.horizontalAlignment = GridData.CENTER;
			_gridDataWappenCanvas.verticalAlignment = GridData.CENTER;
			_gridDataWappenCanvas.grabExcessHorizontalSpace = true;
		}

		if (_canvasWappen == null)
		{
			_canvasWappen = new Canvas(_compositeWappen, SWT.NONE);
			_canvasWappen.setLayoutData(_gridDataWappenCanvas);
		}

		Image img = Activator.getImageDescriptor(pfad).createImage();

		_gridDataWappenCanvas.widthHint = img.getBounds().width;
		_gridDataWappenCanvas.heightHint = img.getBounds().height;
		_canvasWappen.setBackgroundImage(img);

	}

	/**
	 * Registriert einen {@link ILegendenZeitpunkt} Listener und setzt damit die
	 * i.d. Legende anzuzeigende Uhrzeit.
	 */
	private void registriereLegendenZeitpunktListener()
	{
		if (_zeitpunktListener == null)
		{
			_zeitpunktListener = new ILegendenZeitpunkt()
			{

				@Override
				public void getZeit(long zeit)
				{
					DateFormat dfmt = new SimpleDateFormat("dd.MM.yyyy - HH:mm:ss");

					if (zeit > -1)
					{
						_zeitText.setText(dfmt.format(new Date(zeit)));
					} else
					{
						_zeitText.setText(dfmt.format(new Date()));
					}

				}
			};

			LegendeDarzustellendeZeit.getInstanz().addZeitpunktListener(
					_zeitpunktListener);
		}
	}

	@Override
	public void setFocus()
	{
		if (_compositeBasis != null)
			_compositeBasis.setFocus();
	}

	private static String getPartInfo(IWorkbenchPart part)
	{
		String re = "";

		if (part != null)
		{
			boolean isLegende = part instanceof ILegendenAbschnitt;
			re = re + "(IstLegende=" + isLegende + ")"
					+ part.getClass().getSimpleName() + "(hash=" + part + ")";
		}

		return re;
	}

	@Override
	public void dispose()
	{
		if (_partListener != null)
		{
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService()
					.removePartListener(_partListener);
		}
		super.dispose();
	}

}
