/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.FaltStatusSpalten;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableCellResizeListener;
import de.kupzog.ktable.KTableCellSelectionListener;

/**
 * Dialog fr die Listendarstellung.
 * 
 * @author HofmannJ
 * 
 */
public class ListenDarstellungDialog
{

	private Shell _shell = null;

	private Group groupOben = null;

	private Composite compositeUnten = null;

	private Composite compositeUntenRechts = null;

	private Button buttonOk = null;

	private Button buttonAbbrechen = null;

	private DavEmpfangenSendenAenderungsDarstellung _davEmpfangAenderung = null;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _davVerbindung = null;

	/**
	 * ObjektTyp
	 */
	private String _typ = "typ.autarkeOrganisationsEinheit";

	/**
	 * Pid des SystemObjektes, auf welches die Anmeldung erfolgt
	 */
	private String _pidSystemObject = "kv.testKonfiguration";

	/**
	 * Attributgruppe, unter welcher die Anmeldung erfolgt
	 */
	private String _atg = "atg.oberflchenBerechtigung";

	/**
	 * Aspekt, unter dem die Anmeldung erfolgt.
	 */
	private String _asp = "asp.parameterIst";

	/**
	 * Tabelle, auf deren Basis die Darstellung erfolgt.
	 */
	private KTable _kTable = null;

	/**
	 * Das Tabellenmodell, mit dem die Tabelle versorgt wird.
	 */
	protected TabellenModell _tabellenModell;

	/**
	 * Art, wie die Darstellung erfolgen soll.
	 */
	private EnumDarstellungsArt _darstellungsArt = null;

	/**
	 * Gibt die Breite fr die Faltung an
	 */
	private int _breiteFaltung = 10;

	/**
	 * Konstruktor.
	 * 
	 * @param davVerbindung
	 *          Verbindung zum DaV.
	 */
	public ListenDarstellungDialog(ClientDavConnection davVerbindung)
	{
		this._davVerbindung = davVerbindung;
	}

	/**
	 * This method initializes groupOben
	 * 
	 */
	private void createGroupOben()
	{
		GridData gridDataObenInhalt = new GridData();
		gridDataObenInhalt.horizontalAlignment = GridData.FILL;
		gridDataObenInhalt.grabExcessHorizontalSpace = true;
		gridDataObenInhalt.grabExcessVerticalSpace = true;
		gridDataObenInhalt.verticalAlignment = GridData.FILL;
		GridData gridDataOben = new GridData();
		gridDataOben.grabExcessHorizontalSpace = true;
		gridDataOben.verticalAlignment = GridData.FILL;
		gridDataOben.grabExcessVerticalSpace = true;
		gridDataOben.horizontalAlignment = GridData.FILL;
		groupOben = new Group(_shell, SWT.NONE);
		groupOben.setLayout(new GridLayout());
		groupOben.setLayoutData(gridDataOben);
		groupOben.setText("Tabelle:");

		_kTable = new KTable(groupOben, SWT.V_SCROLL | SWT.H_SCROLL
				| SWT.HIDE_SELECTION);
		_kTable.setLayoutData(gridDataObenInhalt);

		LinkedList<DataTableObjectErweitert> dtos = new LinkedList<DataTableObjectErweitert>();

		_tabellenModell = new TabellenModell(_davVerbindung, _typ, _atg, _asp, dtos);

		// _tabellenModell
		// .setDarstellungsArt(EnumDarstellungsArt.aenderungsDarstellung);

		_tabellenModell.setDarstellungsArt(_darstellungsArt);

		_kTable.setModel(_tabellenModell);

		/*
		 * Spalten der Tabelle auf die optimale Breite bringen
		 */
		int anzSpalten = _kTable.getModel().getColumnCount();
		for (int i = 0; i < anzSpalten; i++)
		{
			_kTable.resizeColumnOptimal(i);
		}

		_kTable.addCellSelectionListener(new KTableCellSelectionListener()
		{
			public void cellSelected(int col, int row, int statemask)
			{
				System.out.println("Cell [" + col + ";" + row + "] selected.");
			}

			public void fixedCellSelected(int col, int row, int statemask)
			{
				System.out.println("Header [" + col + ";" + row + "] selected.");

				falten(col, row);

			}

		});

		_kTable.addCellResizeListener(new KTableCellResizeListener()
		{
			public void columnResized(int col, int newWidth)
			{
				System.out.println("Column: " + col + " resized to " + newWidth);
				// _kTable.getModel().getFaltStatusSpalten().setZelleGefaltet(col, 1,
				// newWidth, false);

				if (_kTable.getModel() instanceof TabellenModell)
				{
					TabellenModell tabellenModell = (TabellenModell) _kTable.getModel();
					tabellenModell.getFaltStatusSpalten().setZelleGefaltet(col, 1,
							newWidth, false);
				}

				_kTable.redraw();
			}

			public void rowResized(int row, int newHeight)
			{
				System.out.println("Row " + row + " resized to " + newHeight);
			}
		});

	}

	/**
	 * Fhrt die Faltung durch
	 * 
	 * @param x
	 *          Spalte
	 * @param y
	 *          Zeile
	 */
	protected void falten(int x, int y)
	{
		if (_kTable.getModel() instanceof TabellenModell)
		{
			TabellenModell modell = (TabellenModell) _kTable.getModel();

			int xNeu = x - modell.getFixedColumnCount();
			System.out.println("XNEU: " + xNeu);

			Knoten wurzelKnoten = modell.getWurzelKopfzeile();
			LinkedList<Knoten> knotenAufEbene = wurzelKnoten
					.bestimmeKnotenAufEbene(y);
			Iterator<Knoten> it = knotenAufEbene.iterator();
			Knoten knotenImBereich = null;

			while (it.hasNext())
			{
				Knoten knoten = it.next();
				if (knoten.istInBereich(y, xNeu))
				{
					knotenImBereich = knoten;
					System.out.println("=> " + knoten.getWert());
					break;
				}
			}

			if (knotenImBereich != null)
			{
				LinkedList<Knoten> faltListe = new LinkedList<Knoten>();
				faltListe.add(knotenImBereich);
				if (knotenImBereich.hatNachfolger())
				{
					faltListe.addAll(knotenImBereich
							.getListeAllerKnotenDfsRekursiv(knotenImBereich));
				}

				if (faltListe.size() > 0)
				{

					FaltStatusSpalten faltStatusSpalten = modell.getFaltStatusSpalten();
					Knoten ersterKnoten = faltListe.get(0);
					int vonX = ersterKnoten.getVonX() + modell.getFixedColumnCount();
					int vonY = ersterKnoten.getVonY();
					boolean falten = !faltStatusSpalten.isZelleGefaltet(vonX, vonY);

					Iterator<Knoten> itFalten = faltListe.iterator();
					while (itFalten.hasNext())
					{
						Knoten faltKnoten = itFalten.next();
						vonX = faltKnoten.getVonX() + modell.getFixedColumnCount();
						vonY = faltKnoten.getVonY();
						System.out.println(faltKnoten.getSpaltenBezeichnung()
								+ " => VONX: " + vonX + " VONY: " + vonY + " Breite: "
								+ modell.getColumnWidth(vonX));

						faltStatusSpalten.setZelleGefaltet(vonX, vonY, modell
								.getColumnWidth(vonX), falten);

						if (falten)
						{
							System.out.println("X WIDTH: " + vonX);

							modell.setColumnWidth(vonX, _breiteFaltung);
						} else
						{
							modell.setColumnWidth(vonX, faltStatusSpalten.getAusgangsBreite(
									vonX, vonY));
							System.out.println("X WIDTH: " + vonX);
						}
					}

					_kTable.redraw();
				}

			}

		}
	}

	/**
	 * This method initializes compositeUnten
	 * 
	 */
	private void createCompositeUnten()
	{
		GridData gridDataUnten = new GridData();
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.verticalAlignment = GridData.CENTER;
		gridDataUnten.horizontalAlignment = GridData.FILL;
		compositeUnten = new Composite(_shell, SWT.NONE);
		compositeUnten.setLayout(new GridLayout());
		createCompositeUntenRechts();
		compositeUnten.setLayoutData(gridDataUnten);
	}

	/**
	 * Schliet die Shell.
	 */
	protected void aktionSchliessen()
	{
		if (_davEmpfangAenderung != null)
		{
			_davEmpfangAenderung.abmeldenEmpfang(_pidSystemObject, _atg, _asp);
		}

		_shell.dispose();
	}

	private void aktionAktualisieren()
	{
		_kTable.redraw();
	}

	/**
	 * This method initializes compositeUntenRechts
	 * 
	 */
	private void createCompositeUntenRechts()
	{
		GridData gridDataUntenRechts = new GridData();
		gridDataUntenRechts.horizontalAlignment = GridData.END;
		gridDataUntenRechts.grabExcessHorizontalSpace = true;
		gridDataUntenRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		compositeUntenRechts = new Composite(compositeUnten, SWT.NONE);
		compositeUntenRechts.setLayout(rowLayout);
		compositeUntenRechts.setLayoutData(gridDataUntenRechts);
		buttonOk = new Button(compositeUntenRechts, SWT.NONE);
		buttonOk.setText("OK");		
		buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});
		
		buttonAbbrechen = new Button(compositeUntenRechts, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				// aktionSchliessen();
				aktionAktualisieren();
			}			
		});
	}

	protected void anmeldenTabelleZumEmpfang()
	{
		if (_davEmpfangAenderung == null)
		{

			_davEmpfangAenderung = new DavEmpfangenSendenAenderungsDarstellung(
					_davVerbindung);

			_davEmpfangAenderung.setDisplay(Display.getDefault());

		}

		if (_davEmpfangAenderung != null)
		{
			_davEmpfangAenderung.setKTable(_kTable);
			_davEmpfangAenderung.setTabellenModell(_tabellenModell);

			_davEmpfangAenderung.anmeldenEmpfang(_pidSystemObject, _atg, _asp);
		}
	}

	/**
	 * ffnet den Dialog zur ListenDarstellung mit notwendigen Parametern. Der
	 * Entwickler hat dafr sorge zu tragen, dass die genannten Parameter korrekt
	 * sind.
	 * 
	 * @param pidSystemObject
	 *          SystemObject, auf welches die Anmeldung erfolgen soll.
	 * @param typ
	 *          Objekttyp
	 * @param atg
	 *          AttributGruppe der Anmeldung
	 * @param asp
	 *          Aspekt, unter dem die Anmeldung erfolgen soll.
	 * @param darstellungsArt
	 *          Art, wie die Darstellung erfolgen soll.
	 * @return
	 */
	public boolean open(String pidSystemObject, String typ, String atg,
			String asp, EnumDarstellungsArt darstellungsArt)
	{
		this._pidSystemObject = pidSystemObject;
		this._typ = typ;
		this._atg = atg;
		this._asp = asp;
		this._darstellungsArt = darstellungsArt;

		boolean re = false;

		Display display = Display.getDefault();

		erzeugeShell();
		_shell.open();

		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		// display.dispose();

		re = true;
		return re;

	}

	/**
	 * This method initializes sShell
	 */
	private void erzeugeShell()
	{
		_shell = new Shell();
		_shell.setText("Listendarstellung");
		createGroupOben();
		createCompositeUnten();
		_shell.setLayout(new GridLayout());
		_shell.setSize(800, 600);

		/*
		 * Anmeldung zum Empfang
		 */
		anmeldenTabelleZumEmpfang();
	}

}
