/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen;

import java.io.InputStream;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.rw.Activator;
import de.kupzog.ktable.KTableModel;
import de.kupzog.ktable.KTableSortComparator;
import de.kupzog.ktable.KTableSortedModel;
import de.kupzog.ktable.renderers.FixedCellRenderer;

/**
 * Erweiterter {@link FixedCellRenderer} zur Darstellung von gefalteten Zellen.
 * 
 * @author HofmannJ
 * 
 */
public class FixedCellRendererFaltbar extends FixedCellRenderer
{
	/**
	 * Pfeil nach rechts
	 */
	public static final Image _imagePfeilRechts = imageLaden(
			Display.getCurrent(),
			"/de/bsvrz/buv/rw/basislib/listendarstellung/ktableerweiterungen/icons/rechts_10.png");

	/**
	 * Pfeil nach links
	 */
	public static final Image _imagePfeilLinks = imageLaden(
			Display.getCurrent(),
			"/de/bsvrz/buv/rw/basislib/listendarstellung/ktableerweiterungen/icons/links_10.png");

	/**
	 * Konstruktor entsprechend {@link FixedCellRenderer#FixedCellRenderer(int)}.
	 * 
	 * @param Argument
	 *          entsprechend {@link FixedCellRenderer#FixedCellRenderer(int)}.
	 */
	public FixedCellRendererFaltbar(int style)
	{
		super(style);
	}

	@Override
	protected void drawCellContent(GC gc, Rectangle rect, int col, int row,
			Object content, KTableModel model, Color bgColor, Color fgColor)
	{
		drawCellContent(gc, rect, col, row, content, model, bgColor, fgColor, true);
	}

	/**
	 * Check for sort indicator and delegate content drawing to drawCellContent()
	 */
	protected void drawCellContent(GC gc, Rectangle rect, int col, int row,
			Object content, KTableModel model, Color bgColor, Color fgColor,
			boolean fixed)
	{
		int oldWidth = rect.width;
		Image indicator = null;
		Image pfeil = null;
		int x = 0, y = 0;
		int xPfeil = 0, yPfeil = 0;

		if ((m_Style & INDICATION_SORT) != 0 && model instanceof KTableSortedModel
				&& ((KTableSortedModel) model).getSortColumn() == col
				&& model.getFixedHeaderRowCount() > row && row == 0)
		{

			int sort = ((KTableSortedModel) model).getSortState();
			if (sort == KTableSortComparator.SORT_UP)
				indicator = IMAGE_ARROWDOWN;
			else if (sort == KTableSortComparator.SORT_DOWN)
				indicator = IMAGE_ARROWUP;

			if (indicator != null)
			{
				int contentLength = rect.x + 11 + gc.stringExtent(content.toString()).x;
				x = rect.x + rect.width - 8;
				if (contentLength < x)
					x = contentLength;
				else
					rect.width -= indicator.getBounds().width + 8;
				y = rect.y + rect.height / 2 - indicator.getBounds().height / 2;

				// do not draw if there is not enough space for the image:
				if (rect.width + 4 < indicator.getBounds().width)
				{
					rect.width += indicator.getBounds().width + 8;
					indicator = null;
				}
			}
		}

		/*
		 * Pfeilbild setzen
		 */
		if (fixed)
		{
			int fixedHeaderRowCount = model.getFixedHeaderRowCount();
			int fixedHeaderColumnCount = model.getFixedHeaderColumnCount();

			if ((row < fixedHeaderRowCount) && (col >= fixedHeaderColumnCount))
			{
//				System.out.println("COL: " + col + " ROW: " + row + " FIXED: " + fixed);

				if (model instanceof IFaltStautsSpalten)
				{
					IFaltStautsSpalten faltStautsSpalten = (IFaltStautsSpalten) model;
					FaltStatusSpalten faltStatusSpalten = faltStautsSpalten
							.getFaltStatusSpalten();
					if (faltStatusSpalten.isZelleGefaltet(col, row))
					{
						pfeil = _imagePfeilLinks;
					} else
					{
						pfeil = _imagePfeilRechts;
					}
				}

				int contentLength = rect.x + 11 + gc.stringExtent(content.toString()).x;
				xPfeil = rect.x + rect.width - 8;
				if (contentLength < xPfeil)
					xPfeil = contentLength;
				else
					rect.width -= pfeil.getBounds().width + 8;
				yPfeil = rect.y + rect.height / 2 - pfeil.getBounds().height / 2;

				// do not draw if there is not enough space for the image:
				if (rect.width + 4 < pfeil.getBounds().width)
				{
					rect.width += pfeil.getBounds().width + 8;
					pfeil = null;
				}
			}

		}

		drawCellContent(gc, rect, content.toString(), null, fgColor, bgColor);

		// draw sort indicator:
		if (indicator != null)
		{
			gc.fillRectangle(rect.x + rect.width, rect.y, Math.min(indicator
					.getBounds().width + 8, oldWidth - rect.width - 1), rect.height);
			gc.drawImage(indicator, x, y);
		}

		/*
		 * Faltungspfeil
		 */
		if (pfeil != null)
		{
			gc.fillRectangle(rect.x + rect.width, rect.y, Math.min(
					pfeil.getBounds().width + 8, oldWidth - rect.width - 1), rect.height);
			gc.drawImage(pfeil, xPfeil, yPfeil);
		}
	}

	/**
	 * Ldt eine Bilddatei aus den Ressourcen.
	 * 
	 * @param display
	 *          {@link Display} zur Darstellung.
	 * @param pfad
	 *          Pfad zum Bild.
	 * @return Liefert da Bild oder <code>null</code>, wenn das Bild nicht
	 *         gefunden werden konnte.
	 */
	private static Image imageLaden(Display display, String pfad)
	{
		try
		{
			Image ret = null;
			InputStream is = FixedCellRendererFaltbar.class.getResourceAsStream(pfad);
			if (is != null)
			{
				ret = new Image(display, is);
				is.close();
			}
			return ret;
		} catch (Exception e1)
		{
			Activator.getDefault().getLog().log(
					new Status(Status.ERROR, Activator.PLUGIN_ID, "Image \"" + pfad
							+ "\" nicht vefgbar", e1));

			return null;
		}
	}
}
