/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import org.eclipse.ui.IWorkbenchPart;

/**
 * Diese Klasse realisiert die Kapselung fr Objekte, die in der
 * Mehrfachzwischenablage gehalten und in der Sicht
 * {@link SichtMehrfachZwischenablage} dargestellt werden. Erfasst werden:
 * <ul>
 * <li>das Objekt,</li>
 * <li>der {@link IWorkbenchPart}, von dem das Objekt geliefert wurde
 * (optional, nur notwendig bei Ausschneideaktionen),</li>
 * <li>ein Flag das anzeigt, ob das Objekt des Datensatzes bereits irgendwo
 * eingefgt wurde.</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class ZwischenablageDatensatz
{
	/**
	 * Wert des Datensatzes der Mehrfachzwischenablage
	 */
	private Object _wert = null;

	/**
	 * Herkunft der Mehrfachzwischenablage Datensatzes. Damit kann dieser z.B.
	 * beim Ausschneiden aus der Herkunft View ode dem Herkunft Editor gelscht
	 * werden.
	 */
	private IWorkbenchPart _herkunft = null;

	/**
	 * Gibt an, ob dieses Objekt ausgeschnitten wurde.
	 */
	private boolean _ausgeschnitten = false;

	/**
	 * Flag, zeigt an ob der Eintrag aus der Mehrfachzwischenablage schon mal
	 * irgendwo eingefgt wurde.
	 */
	private boolean _bereitsEingefuegt = false;

	/**
	 * Konstruktor, bernimmt als Wert ein beliebiges Objekt.
	 * 
	 * @param wert
	 *          Wert des Datensatzes
	 */
	public ZwischenablageDatensatz(Object wert)
	{
		this._wert = wert;
	}

	/**
	 * Konstruktor, ber nimmt als Wert ein beliebiges Objekt sowie die Herkunft
	 * des Objektes.
	 * 
	 * @param wert
	 *          Wert des Datensatzes
	 * @param herkunft
	 *          Herkunft des Objektes
	 */
	public ZwischenablageDatensatz(Object wert, IWorkbenchPart herkunft)
	{
		this._wert = wert;
		this._herkunft = herkunft;
	}

	/**
	 * Liefert den Wert des Datensatzes zurck.
	 * 
	 * @return Wert des Datensatzes
	 */
	public Object getWert()
	{
		return _wert;
	}

	/**
	 * Liefert die Herkunft des Objektes des Datensatzes
	 * 
	 * @return Herkunft des Objektes
	 */
	public IWorkbenchPart getHerkunft()
	{
		return _herkunft;
	}

	/**
	 * Setzt das Flag, das anzeigt, ob das Objekt ausgeschnitten wurde oder nicht.
	 * 
	 * @param ausgeschnitten
	 *          Ist das Objekt ausgeschnitten, muss hier <code>true</code>
	 *          gesetzt werden, sonst <code>false</code>.
	 */
	public void setAuschgeschnitten(boolean ausgeschnitten)
	{
		this._ausgeschnitten = ausgeschnitten;
	}

	/**
	 * Prft, ob das Objekt ausgeschnitten wurde oder nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn das Objekt ausgeschnitten wurde,
	 *         sonst <code>false</code>.
	 */
	public boolean isAusgeschnitten()
	{
		return _ausgeschnitten;
	}

	/**
	 * Setzt das Flag, das anzeigt, ob der Datensatz der Mehrfachzwischenablage
	 * schon mal eingefgt wurde.
	 * 
	 * @param bereitsEingefuegt
	 *          Wenn <code>true</code>, dann wurde der Datensatz schon mal
	 *          eingefgt, bei <code>false</code> nicht.
	 */
	public void setBereitsEingefuegt(boolean bereitsEingefuegt)
	{
		_bereitsEingefuegt = bereitsEingefuegt;
	}

	/**
	 * Prft, ob der Datensatz schon mal eingefgt wurde.
	 * 
	 * @return Liefert <code>true</code>, wenn der Datensatz schon mal
	 *         eingefgt wurde, sonst <code>false</code>.
	 */
	public boolean isBereitsEingefuegt()
	{
		return _bereitsEingefuegt;
	}

}
