/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorPlugin;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorStart;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorWurzel;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.EnumDeskriptorTyp;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.IDeskriptorElementUndAktion;

/**
 * Baumdarstellung der registrierten Elemente von Men- und Symbolleiste. <br>
 * 
 * @version $Revision: 1.3 $ / $Date: 2012/08/21 08:25:03 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class CompositeDarstellungRegistrierterElemente extends Composite
{
	/**
	 * TreeViwer fr die Baumdarstellung.
	 */
	public TreeViewer _treeViewer = null;

	/**
	 * Hier wird der benutzte Descriptortyp hinterlegt.
	 */
	private EnumDeskriptorTyp _enumDescriptorTyp = null;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
	 * @param style
	 *          SWT-Stil.
	 * @param typ
	 *          Beschreibt, welche Deskriptoren verarbeitet werden.
	 */
	public CompositeDarstellungRegistrierterElemente(Composite parent, int style,
			EnumDeskriptorTyp typ)
	{
		super(parent, style);
		this._enumDescriptorTyp = typ;
		init();
	}

	/**
	 * Fhrt die Initialisierung durch.
	 * 
	 */
	private void init()
	{
		erzeugeTreeViewer();
		setLayout(new FillLayout());

	}

	/**
	 * Erezugt den TreeViewer fgt Drag-Support sowie die notwendigen Transfers
	 * hinzu.
	 * 
	 */
	private void erzeugeTreeViewer()
	{
		_treeViewer = new TreeViewer(this, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		_treeViewer.setContentProvider(new TreeViewerInhaltsVersorger());
		_treeViewer.setLabelProvider(new TreeViewerDarstellungsVersorger());
		_treeViewer.setSorter(new Vsort());

		// int ops = DND.DROP_COPY | DND.DROP_MOVE;
		int ops = DND.DROP_MOVE;

		// An dieser Stelle knnen mehrere Transferelemente untergebracht werden.
		Transfer[] transfers = null;
		if (_enumDescriptorTyp == EnumDeskriptorTyp.deskriptorAktion)
		{
			transfers = new Transfer[] { TransferDeskriptorAktion.getInstanz() };
		}
		if (_enumDescriptorTyp == EnumDeskriptorTyp.deskriptorGlobal)
		{
			transfers = new Transfer[] { TransferDeskriptorAktion.getInstanz() };
		} else if (_enumDescriptorTyp == EnumDeskriptorTyp.deskriptorElement)
		{
			transfers = new Transfer[] { TransferDeskriptorElement.getInstanz() };
		} else if (_enumDescriptorTyp == EnumDeskriptorTyp.deskriptorGemischt)
		{
			transfers = new Transfer[] { TransferDeskriptorElement.getInstanz(),
					TransferDeskriptorAktion.getInstanz() };
		}

		// Transfer[] transfers = new
		// Transfer[]{TransferDescriptorElement.getInstanz(),
		// TransferDescriptorAction.getInstanz()};
		_treeViewer.addDragSupport(ops, transfers, new ZugZuhoererDeskriptor(
				_treeViewer));

		// _treeViewer.addDropSupport(ops, transfers, new
		// TreeDropAdapterDescriptorElement(_treeViewer));

		// _treeViewer.setSorter(ViewerSorter sorter);

	}

	/**
	 * {@link ViewerSorter} erweiterung fr die Anforderungen des Meneditors.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Vsort extends ViewerSorter
	{

		@Override
		public int compare(Viewer viewer, Object e1, Object e2)
		{
			int re = 0;

			if ((e1 != null) && (e2 != null))
			{
				String str1 = substitute(e1.toString());
				String str2 = substitute(e2.toString());

				re = str1.compareTo(str2);
			}

			return re;
		}

		/**
		 * Startet ein {@link String} mit einem <code>&</code>, wird dieses
		 * entfernt.
		 * 
		 * @param str
		 *          {@link String}.
		 * @return {@link String} mit entferntem <code>&</code> am Anfang. Ist kein
		 *         <code>&</code> an Position 0 vorhanden, wird der Eingangswert
		 *         zurckgeliefert.
		 */
		private String substitute(String str)
		{
			String re = null;

			if (str != null)
			{
				if ((str.startsWith("&")) && (str.length() > 1))
				{
					re = str.substring(1, str.length());
				} else
				{
					re = str;
				}
			}

			return re;
		}

	}

	/**
	 * Setzt den Inhalt des TreeViewer. Der {@link DeskriptorStart} beinhaltet
	 * alle hierzu notwendigen Informationen.
	 * 
	 * @param descriptorStart
	 *          {@link DeskriptorStart}, bildet die Wurzel des Baumes.
	 */
	public void setzeInhalt(DeskriptorStart descriptorStart)
	{
		_treeViewer.setInput(descriptorStart);
		_treeViewer.expandAll();
	}

	/**
	 * Bereitet die Inhalte des TreeViewers auf. {@link ITreeContentProvider}
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class TreeViewerInhaltsVersorger implements ITreeContentProvider
	{

		public Object[] getChildren(Object parentElement)
		{
			Object[] objects = null;

			/*
			 * DescriptorStart
			 */
			if (parentElement instanceof DeskriptorStart)
			{
				DeskriptorStart descriptorStart = (DeskriptorStart) parentElement;
				objects = new Object[1];
				objects[0] = descriptorStart.getKind();
				return objects;
			}
			/*
			 * DescriptorWurzel
			 */
			else if (parentElement instanceof DeskriptorWurzel)
			{
				DeskriptorWurzel descriptorWurzel = (DeskriptorWurzel) parentElement;
				List<DeskriptorPlugin> plugins = descriptorWurzel.getPlugins();
				if (plugins.size() > 0)
				{
					objects = new Object[plugins.size()];
					int i = -1;
					Iterator<DeskriptorPlugin> it = plugins.iterator();
					while (it.hasNext())
					{
						i++;
						objects[i] = it.next();
					}
				}

				return objects;
			}

			/*
			 * DescriptorPlugin
			 */
			else if (parentElement instanceof DeskriptorPlugin)
			{
				DeskriptorPlugin descriptorPlugin = (DeskriptorPlugin) parentElement;
				List<Object> elemente = descriptorPlugin.getElemente();
				if (elemente.size() > 0)
				{
					objects = new Object[elemente.size()];
					int i = -1;
					Iterator<Object> it = elemente.iterator();
					while (it.hasNext())
					{
						i++;
						objects[i] = it.next();
					}
					return objects;
				} else
				{
					return null;
				}
			} else
			{
				return null;
			}
		}

		public Object getParent(Object element)
		{
			return null;
		}

		public boolean hasChildren(Object element)
		{
			/*
			 * DescriptorWurzel
			 */
			if (element instanceof DeskriptorWurzel)
			{
				DeskriptorWurzel descriptorWurzel = (DeskriptorWurzel) element;
				if (descriptorWurzel.getPlugins().size() > 0)
				{
					return true;
				} else
				{
					return false;
				}
			}
			/*
			 * DescriptorPlugin
			 */
			else if (element instanceof DeskriptorPlugin)
			{
				DeskriptorPlugin descriptorPlugin = (DeskriptorPlugin) element;
				if (descriptorPlugin.getElemente().size() > 0)
				{
					return true;
				} else
				{
					return false;
				}
			}
			// Sonst...
			else
			{
				return false;
			}
		}

		public Object[] getElements(Object inputElement)
		{
			return getChildren(inputElement);
		}

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	/**
	 * Erezeugt die Label der Elemente des TreeViewers, bestehend aus Text und
	 * Grafik.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class TreeViewerDarstellungsVersorger extends LabelProvider
	{
		private ImageRegistry _ir = new ImageRegistry();

		public String getText(Object element)
		{
			if (element instanceof DeskriptorWurzel)
			{
				DeskriptorWurzel descriptorWurzel = (DeskriptorWurzel) element;
				return descriptorWurzel.getName();
			} else if (element instanceof DeskriptorPlugin)
			{
				DeskriptorPlugin descriptorPlugin = (DeskriptorPlugin) element;
				return descriptorPlugin.getPluginID();
			} else if (element instanceof DeskriptorAktion)
			{
				String returnText = "";
				IDeskriptorElementUndAktion descriptorAction = (IDeskriptorElementUndAktion) element;
				returnText = returnText + descriptorAction.getText() + "("
						+ descriptorAction.getAktionID() + ")";

				return returnText;
			} else if (element instanceof DeskriptorElement)
			{
				String returnText = "";
				DeskriptorElement descriptorElement = (DeskriptorElement) element;
				returnText = returnText + descriptorElement.getText() + " ("
						+ descriptorElement.getAktionID() + ")";
				return returnText;
			} else
			{
				return null;
			}

		}

		public Image getImage(Object element)
		{
			Image returnImage = null;

			if (element instanceof DeskriptorPlugin)
			{
				getImageDescriptor("icons/menubar/plug.gif");
				returnImage = getIrImage("icons/menubar/plug.gif");
			}

			if (element instanceof DeskriptorAktion)
			{
				getImageDescriptor("icons/menubar/menuitem.gif");
				returnImage = getIrImage("icons/menubar/menuitem.gif");
			}

			if (element instanceof DeskriptorElement)
			{
				getImageDescriptor("icons/menubar/menuitem_red_16x16.png");
				returnImage = getIrImage("icons/menubar/menuitem_red_16x16.png");
			}

			if (element instanceof DeskriptorWurzel)
			{
				getImageDescriptor("icons/menubar/menu.gif");
				returnImage = getIrImage("icons/menubar/menu.gif");
			}

			return returnImage;
		}

		/**
		 * Liefert einen {@link ImageDescriptor} aus der {@link ImageRegistry}. Ist
		 * dort der entsprechende {@link ImageDescriptor} noch nicht vorhanden wird
		 * aus dem bergebenen Pfad versucht, diesen aufzubauene.
		 * 
		 * @param imagePath
		 *          Pfad zum Image.
		 * @return {@link ImageDescriptor} oder <code>null</code>, wenn der Pfad
		 *         nicht existiert und die {@link ImageRegistry} den
		 *         {@link ImageDescriptor} nicht erzeugen konnte.
		 */
		private ImageDescriptor getImageDescriptor(String imagePath)
		{
			ImageDescriptor re = null;

			if ((imagePath != null) && (_ir != null) && (imagePath.length() > 0))
			{
				if (_ir.getDescriptor(imagePath) == null)
				{
					_ir.put(imagePath, Activator.getImageDescriptor(imagePath));
				}

				re = _ir.getDescriptor(imagePath);
			}

			return re;
		}

		private Image getIrImage(String imagePath)
		{
			return _ir.get(imagePath);
		}

	}

}
