/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.editoren;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.TransferData;
import org.jdom.Element;

import de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff.TransferDeskriptorAktion;
import de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff.TransferDeskriptorElement;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlBezeichnerMenuSymbol;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlElementItem;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert den {@link ViewerDropAdapter} mit dem es mglich wird, in den TreeViewer des
 * {@link EditorMenuSymbolLeisteEditor} {@link DeskriptorAktion}s und {@link DeskriptorElement}s per Drag and Drop
 * fallen zu lassen. <br>
 * <br>
 * 
 * <b>Achtung:</b> Es ist darauf zu achten, dass mittels der Methode setMenuSymbolLeistedaten immer auf die aktuell
 * passende {@link MenuSymbolLeisteDaten}-Struktur durch die Klasse {@link EditorMenuSymbolLeiste} verwiesen wird!
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:19:30 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class BaumFallAdapterElementSichtMenuLeisteEditor extends ViewerDropAdapter
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Verweis auf die Hintergrunddatenstruktur.
   */
  private MenuSymbolLeisteDaten _msld = null;

  private Element _dragElement;

  protected BaumFallAdapterElementSichtMenuLeisteEditor(TreeViewer viewer, MenuSymbolLeisteDaten msld)
  {
    super(viewer);
    this._msld = msld;
  }

  /**
   * Setzt die Struktur {@link MenuSymbolLeisteDaten}.
   * 
   * @param msld
   *          Zu setzende {@link MenuSymbolLeisteDaten}-Struktur
   */
  protected void setMenuSymbolLeistedaten(MenuSymbolLeisteDaten msld)
  {
    this._msld = msld;
  }

  @Override
  public boolean performDrop(Object data)
  {
    boolean actionFlag = false;

    _debug.fine(getClass().getName() + "::performDrop => Eingehende Daten Klasse: " + data.getClass());

    // ZielElement
    Object target = getCurrentTarget();

    if (target == null)
      target = (Element)getViewer().getInput();

    Element zielElement = (Element)target;

    _debug.fine(getClass().getName() + "::performDrop => Target Name: " + zielElement.getName());

    List<Element> elemente = new LinkedList<Element>();

    /*
     * DeskriptorElement[] ?
     */
    if (data instanceof DeskriptorElement[])
    {
      actionFlag = true;

      DeskriptorElement[] toDrop = (DeskriptorElement[])data;

      for (int i = 0; i < toDrop.length; i++)
      {
        String id = toDrop[i].getPluginID();
        String actionid = toDrop[i].getAktionID();
        String name = toDrop[i].getText();
        boolean bAusklappbar = toDrop[i].istAusklappAktion();

        XmlElementItem elementItem = new XmlElementItem(id, actionid, name, bAusklappbar);

        Element element = elementItem.getElementItem();

        elemente.add(element);

        _msld.setSchmutzig(true);
      }
    }

    /*
     * DeskriptorAktion[] ?
     */
    if (data instanceof DeskriptorAktion[])
    {
      actionFlag = true;

      DeskriptorAktion[] toDrop = (DeskriptorAktion[])data;
      for (int i = 0; i < toDrop.length; i++)
      {
        String actionid = toDrop[i].getAktionID();

        String name = toDrop[i].getText();

        XmlElementItem elementItem = new XmlElementItem(actionid, actionid, name);

        Element element = elementItem.getElementItem();
        elemente.add(element);

        _msld.setSchmutzig(true);
      }
    }

    /*
     * String? Intern gespeichertes Element
     */
    if (data instanceof String)
    {
      actionFlag = false;

      _dragElement = ZugElementSpeicher.getInstanz().getZugElement();
      // if ((dragElement != null) && (dragElement != zielElement))
      if (_dragElement != null)
      {
        /*
         * Wenn das Element im DragElementSpeicher von der ID mit der ID aus dem TextTransfer bereinstimmt, dann
         * handelt es sich um das richtige Element.
         */
        if (_dragElement.getAttributeValue(XmlBezeichnerMenuSymbol.attId).equals((String)data))
        {
          // Bestimmen der Position
          int loc = getCurrentLocation();

          // Item
          if (_dragElement.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
          {
            _debug.fine("TreeDropAdapterElementViewMenuBarEditor.performDrop(): Bearbeite ITEM");

            // Element cloneElement = (Element) dragElement.clone();
            if ((zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
                || (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator)))
            {
              int position = _msld.bestimmeElementPositionInElternElement(zielElement);

              _debug.fine("TreeDropAdapterElementViewMenuBarEditor.performDrop(): Zielelement ist Item mit ID: "
                  + zielElement.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
              _debug
                  .fine("TreeDropAdapterElementViewMenuBarEditor.performDrop(): Position Zielelement in Elternelement: "
                      + position);

              if ((loc == LOCATION_BEFORE) || (loc == LOCATION_ON))
              {
                // System.out.println("VOR");
                // _mbd.verschiebeItem(dragElement
                // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
                // .getParentElement().getAttributeValue(
                // XmlBezeichnerMenu.attId), position);

                _msld.verschiebeItem(_dragElement, zielElement.getParentElement(), position);

              }
              else if (loc == LOCATION_AFTER)
              {
                // System.out.println("DANACH");
                // _mbd.verschiebeItem(dragElement
                // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
                // .getParentElement().getAttributeValue(
                // XmlBezeichnerMenu.attId), position++);

                _msld.verschiebeItem(_dragElement, zielElement.getParentElement(), position++);

              }
              else
              {
                // Nichts
              }
            }
            if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
            {
              // dragElement.detach();
              // zielElement.addContent(cloneElement);
              _msld.verschiebeItem(_dragElement, zielElement);
            }

          }

          /*
           * Separator
           */
          else if (_dragElement.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
          {

            if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementItem)
                || zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
            {
              if (loc == LOCATION_BEFORE || loc == LOCATION_ON)
              {
                Element cloneElement = (Element)_dragElement.clone();
                _dragElement.detach();

                _msld.hinzufuegenSeparatorInMenu(cloneElement, zielElement);
              }
              else if (loc == LOCATION_AFTER)
              {
                Element cloneElement = (Element)_dragElement.clone();
                _dragElement.detach();

                int zielPosition = _msld.bestimmeElementPositionInElternElement(zielElement);

                zielPosition = zielPosition + 1;

                _msld.setzeElementBei(zielElement.getParentElement(), cloneElement, zielPosition);
              }

            }

            if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
            {
              _msld.hinzufuegenSeparatorInMenu(_dragElement, zielElement.getParentElement(), _msld
                  .bestimmeElementPositionInElternElement(zielElement));
            }

            if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
            {
              _msld.hinzufuegenSeparatorAnMenu(_dragElement, zielElement);
            }

            // Element cloneElement = (Element) dragElement.clone();
            // dragElement.detach();
            // zielElement.addContent(cloneElement);
          }

          /*
           * Men
           */
          else if (_dragElement.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
          {
            // if
            // (zielElement.getName().equals(XmlBezeichnerMenu.elementSubMenu))
            // {
            // if (zielElement.getChildren().size() < 1)
            // {
            // Element cloneElement = (Element) dragElement.clone();
            // dragElement.detach();
            // zielElement.addContent(cloneElement);
            // }
            //							
            // }
            _msld.verschiebeMenu(_dragElement, zielElement);
          }

          /*
           * Submen
           */
          else if (_dragElement.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
          {
            // Element cloneElement = (Element) dragElement.clone();
            if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementItem)
                || zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu)
                || zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
            {
              int position = _msld.bestimmeElementPositionInElternElement(zielElement);
              if ((loc == LOCATION_BEFORE))
              {
                // System.out.println("VOR");
                // _mbd.verschiebeSubMenu(dragElement
                // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
                // .getParentElement().getAttributeValue(
                // XmlBezeichnerMenu.attId), position);
                _msld.verschiebeSubMenu(_dragElement, zielElement.getParentElement(), position);

              }
              else if (loc == LOCATION_ON)
              {
                if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
                {
                  String ausklappbar = zielElement.getAttributeValue(XmlBezeichnerMenuSymbol.attItemAusklappbar);
                  if ((ausklappbar != null) && (!ausklappbar.equals("")))
                  {
                    _msld.verschiebeSubMenu(_dragElement, zielElement);
                  }
                  else
                  {
                    _msld.verschiebeSubMenu(_dragElement, zielElement.getParentElement(), position);
                  }

                }
                else
                {
                  _msld.verschiebeSubMenu(_dragElement, zielElement.getParentElement(), position);
                }
              }
              else if (loc == LOCATION_AFTER)
              {
                // _mbd.verschiebeSubMenu(dragElement
                // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
                // .getParentElement().getAttributeValue(
                // XmlBezeichnerMenu.attId), position++);
                _msld.verschiebeSubMenu(_dragElement, zielElement.getParentElement(), position++);

              }
              else
              {
                // Nichts
              }
            }
            if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
            {
              // dragElement.detach();
              // zielElement.addContent(cloneElement);
              _msld.verschiebeSubMenu(_dragElement, zielElement);
            }

            // ////Code zum allgemeinen anfgen an beliebige Elemente
            // Element cloneElement = (Element) dragElement.clone();
            // dragElement.detach();
            // zielElement.addContent(cloneElement);
          }

          TreeViewer viewer = (TreeViewer)getViewer();
          viewer.refresh();
          viewer.expandAll();

        }

      }

    }

    /*
     * actionFlag?
     */
    if (actionFlag)
    {
      _debug.fine(getClass().getName() + "::performDrop => Listsize einzufgender Elemente: " + elemente.size());

      TreeViewer viewer = (TreeViewer)getViewer();
      Iterator<Element> it = elemente.iterator();
      int j = 0;

      while (it.hasNext())
      {
        j++;

        Element addElement = it.next();

        boolean ziel = false;
        if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
        {
          ziel = true;
        }
        else if (zielElement.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
        {
          ziel = true;
        }

        if (target != null && ziel)
        {
          int loc = getCurrentLocation();

          int position = _msld.bestimmeElementPositionInElternElement(zielElement);

          zielElement = zielElement.getParentElement();

          if (loc == LOCATION_BEFORE)
          {
          }
          else if (loc == LOCATION_AFTER)
          {
            position++;
          }
          else
          {
          }

          _msld.setzeElementBei(zielElement, addElement, position);

        }
        else
        {
          zielElement.addContent(addElement);

        }

        // viewer.add(target, addElement);

        viewer.refresh();
        viewer.expandAll();

      }
    }
    else
    {
      _debug.fine(getClass().getName() + "::perform Drop => Texttransfer");

    }

    return true;

  }

  @Override
  public boolean validateDrop(Object target, int operation, TransferData transferType)
  {

    if (target instanceof Element)
    {

      Element em = (Element)target;
      _debug.fine(getClass().getName() + "::validateDrop => Target ist Gltig: " + em.getName());

      if (TransferDeskriptorElement.getInstanz().isSupportedType(transferType)
          || (TransferDeskriptorAktion.getInstanz().isSupportedType(transferType)))
      {
        Element element = (Element)target;
        boolean elementTyp = false;

        if (element.getName().equals(XmlBezeichnerMenuSymbol.elementMenu))
        {
          elementTyp = true;
        }
        else if (element.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
        {
          elementTyp = true;
        }
        else if (element.getName().equals(XmlBezeichnerMenuSymbol.elementSeparator))
        {
          elementTyp = true;
        }

        // if (element.getName().equals(XmlBezeichner.elementMenu)
        // || element.getName().equals(XmlBezeichner.elementItem))
        if (elementTyp)
        {
          return true;
        }
        else
        {
          return false;
        }
      }
      else if (TextTransfer.getInstance().isSupportedType(transferType))
      {
        return true;
      }
      else
      {
        return false;
      }

    }
    else
    {
      return false;
    }

  }
}
