/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.generatoren;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.CoolBarManager;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.jdom.Element;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.EreignisMenuSymbolLeisteDatenGeandert;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.IZuhoererMenuSymbolLeisteDatenAenderung;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.XmlBezeichnerMenuSymbol;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse generiert das Men zur Laufzeit des Rahmenwerks.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 07:00:44 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class SymbolleisteGenerator
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * {@link CoolBarManager}, welcher die Hauptsymbolleiste bildet.
	 */
	private ICoolBarManager _cbm = null;

	/**
	 * MenBarDaten-Klasse, die den Aufbau des Mens innerhalb einer XML-Struktur
	 * beinhaltet.
	 */
	private MenuSymbolLeisteDaten _mbd = null;

	/**
	 * Liste der {@link ToolBarManager}, die in der Hauptsymbolleiste
	 * {@link CoolBarManager} die verschiedenen Abschnitte und damit Separatoren
	 * bilden. <br>
	 * <br>
	 * Diese einzelnen Abschnitte werden der Reihe nach eingefgt und mssen bei
	 * spterer Verarbeitung der Reiche nach bearbeitet werden.
	 */
	private LinkedList<ToolBarManager> _listeToolBarManager = new LinkedList<ToolBarManager>();

	/**
	 * Konstruktor.
	 * 
	 * @param coolBarManager
	 *          {@link CoolBarManager}, auf dessen Basis die Darstellung erfolgen
	 *          soll.
	 * @param msld
	 *          {@link MenuSymbolLeisteDaten}, die die Hintergrunddatenstruktur
	 *          bilden.
	 */
	public SymbolleisteGenerator(ICoolBarManager coolBarManager,
			MenuSymbolLeisteDaten msld)
	{
		this._cbm = coolBarManager;
		this._mbd = msld;
		init();
	}

	private void init()
	{
		// registrieren der Zurhrer bei MenuBarDaten
		IZuhoererMenuSymbolLeisteDatenAenderung zuhoerer = new IZuhoererMenuSymbolLeisteDatenAenderung()
		{

			public void ausfuehren(EreignisMenuSymbolLeisteDatenGeandert e)
			{
				generiereSymbolLeiste();
			}

		};
		_mbd.hinzufuegenZuhoerer(zuhoerer);

	}

	public void generiereSymbolLeiste()
	{
		ElementRegistrierung elementRegistrierung = ElementRegistrierung
				.getInstanz();

		// Iterator<Object> it =
		// elementRegistrierung.getPluginDeskriptorenKombiniert("de.bsvrz.buv.rw.rw.ui").iterator();
		// while(it.hasNext())
		// {
		// Object obj = it.next();
		//			
		// }

		_debug.fine("GRE: "
				+ elementRegistrierung.getPluginAktionDeskriptorenGlobal(
						"de.bsvrz.buv.rw.rw.ui").size());
		Iterator<DeskriptorAktion> itC = elementRegistrierung
				.getPluginAktionDeskriptorenGlobal("de.bsvrz.buv.rw.rw.ui").iterator();
		while (itC.hasNext())
		{
			@SuppressWarnings("unused")
			DeskriptorAktion da = itC.next();
		}

		DeskriptorAktion ds = elementRegistrierung
				.getDeskriptorAktionGlobal("save");
		if (ds != null)
		{
			_debug.fine("PRIMA");
		} else
		{
			_debug.fine("FEHLER");
		}

		leereCoolBarUndToolBars();

		/*
		 * Erzeugen des ersten ToolBarManager
		 */
		ToolBarManager tbm = new ToolBarManager();
		_cbm.add(tbm);
		_listeToolBarManager.add(tbm);

		Element wurzel = _mbd.getWurzel().getChild(
				XmlBezeichnerMenuSymbol.elementMenu);
		if (wurzel.getChildren().size() > 0)
		{
			@SuppressWarnings("unchecked")
			Iterator it = wurzel.getChildren().iterator();
			int i = 0;

			while (it.hasNext())
			{
				i++;
				_debug.fine("\n====== Durchlauf #" + i + " ======");
				Element element = (Element) it.next();

				// Item?
				if (element.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
				{
					_debug.fine("ITEM: " + element.toString());

					String value = element
							.getAttributeValue(XmlBezeichnerMenuSymbol.attItemAusklappbar);

					_debug.fine(value);

					if (value == null)
					{
						_debug.fine("Normales Item");
						_debug.fine("ITEM ID: "
								+ element.getAttributeValue(XmlBezeichnerMenuSymbol.attId));

						String id = element
								.getAttributeValue(XmlBezeichnerMenuSymbol.attId);

						/*
						 * Normal
						 */
						DeskriptorAktion descriptorAction = elementRegistrierung
								.getDeskriptorAktion(id);

						if (descriptorAction != null)
						{
							tbm.add(descriptorAction.getAktion());
							_debug.fine("DeskriptorAktion " + id + " hinzugefgt.");
						} else
						{
							_debug.fine("DeskriptorAktion " + id + " NICHT hinzugefgt.");
						}

						/*
						 * Global
						 */
						DeskriptorAktion deskriptorAktionGlobal = elementRegistrierung
								.getDeskriptorAktionGlobal(id);
						if (deskriptorAktionGlobal != null)
						{
							tbm.add(deskriptorAktionGlobal.getAktion());
							_debug.fine("DeskriptorAktionGlobal " + id + " hinzugefgt");
						} else
						{
							_debug
									.fine("DeskriptorAktionGlobal " + id + " NICHT hinzugefgt");
						}

					} else if (element.getAttributeValue(
							XmlBezeichnerMenuSymbol.attItemAusklappbar).equals("ja"))
					{
						_debug.fine("Klapp Item" + " ITEM ID: "
								+ element.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
						DeskriptorAktion descriptorAction = elementRegistrierung
								.getDeskriptorAktion(element
										.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
						Action action = (Action) descriptorAction.getAktion();

						// Wurde auch eine echte DropDown-Action registriert?
						if (action.getStyle() == SWT.DROP_DOWN)
						{
							tbm.add(action);

							// Hat DropDown-Item ein Submen?
							Element itemSubmenu = element
									.getChild(XmlBezeichnerMenuSymbol.elementSubMenu);
							if (itemSubmenu != null)
							{
								// InternerMenuGenerator img = new InternerMenuGenerator(
								// itemSubmenu);
								// action.setMenuCreator(img);

								action.setMenuCreator(new MenGen(itemSubmenu));

							} else
							{
								_debug
										.error(getClass().getName()
												+ "::generiereCoolBar \n\t=> Obwohl es sich bei der Action mit der ID"
												+ element
														.getAttributeValue(XmlBezeichnerMenuSymbol.attId
																+ " um eine DropDown-Action handelt, ist in der XML-Struktur keine Men fr diese Action definiert. Abbruch! \n"));
							}

						} else
						{
							_debug
									.error(getClass().getName()
											+ "::generiereCoolBar \n\t=> Entgegen der Definition innerhalb der XML-Struktur handelt es sich bei der Action mit der ID "
											+ element
													.getAttributeValue(XmlBezeichnerMenuSymbol.attId)
											+ " nicht um eine DropDown-Action! \n");
							tbm.add(action);
						}
					}

				}

				// Separator?
				else if (element.getName().equals(
						XmlBezeichnerMenuSymbol.elementSeparator))
				{
					// Separatoren werden fr die CoolBar noch nicht verarbeitet.

					tbm = new ToolBarManager();
					_cbm.add(tbm);
					_listeToolBarManager.add(tbm);

				} else
				{
					_debug.fine("Element 2 Strint: " + element.toString());
				}

				// Normales Item?
				// if
				// (element.getAttributeValue(XmlBezeichnerMenu.attItemRunterKlappen)
				// .equals("nein"))

				// if (element.getAttributeValue(XmlBezeichnerMenu.attItemRunterKlappen)
				// .equals("nein"))
				// {
				// System.out.println("Normales Item");
				// DescriptorAction descriptorAction = ElementRegistrierung
				// .getInstanz().getDescriptorAction(
				// element.getAttributeValue(XmlBezeichnerMenu.attId));
				// IAction action = descriptorAction.getAction();
				// _cbm.add(action);
				// }
				//
				// // DropDownItem?
				// else if (element.getAttributeValue(
				// XmlBezeichnerMenu.attItemRunterKlappen).equals("ja"))
				// {
				// System.out.println("DropDownItem");
				// System.out.println("Klappen.");
				// DescriptorAction descriptorAction = ElementRegistrierung
				// .getInstanz().getDescriptorAction(
				// element.getAttributeValue(XmlBezeichnerMenu.attId));
				// IAction action = descriptorAction.getAction();
				//
				// // Wurde auch eine echte DropDown-Action registriert?
				// if (action.getStyle() == SWT.DROP_DOWN)
				// {
				// _cbm.add(action);
				//
				// // Hat DropDown-Item ein Submen?
				// Element itemSubmenu = element
				// .getChild(XmlBezeichnerMenu.elementSubMenu);
				// if (itemSubmenu != null)
				// {
				// InternerMenuGenerator img = new InternerMenuGenerator(itemSubmenu);
				// action.setMenuCreator(img);
				// } else
				// {
				// System.err
				// .println(getClass().getName()
				// + "::generiereCoolBar \n\t=> Obwohl es sich bei der Action mit der ID
				// "
				// + element.getAttributeValue(XmlBezeichnerMenu.attId)
				// + " um eine DropDown-Action handelt, ist in der XML-Struktur keine
				// Men fr diese Action definiert. Abbruch! \n");
				// }
				//
				// } else
				// {
				// System.err
				// .println(getClass().getName()
				// + "::generiereCoolBar \n\t=> Entgegen der Definition innerhalb der
				// XML-Struktur handelt es sich bei der Action mit der ID "
				// + element.getAttributeValue(XmlBezeichnerMenu.attId)
				// + " nicht um eine DropDown-Action! \n");
				// _cbm.add(action);
				// }
				// }

			}

		} else
		{
			_debug
					.error(getClass().getName()
							+ "::generiereCoolBar \n\t=>Es existieren keine Elemente, die in die CoolBar eingebracht werden knnen. Abbruch. \n");
		}

		tbm.update(true);

		// _cbm.add(generiereEditToolBarManager());

		_cbm.update(true);

		/*
		 * Listener darber informieren, dass die Symbolleiste aktualisiert wurde.
		 */
		MenueLadeInformation.getInstance()
				.benachrichtigeListenerUeberAktualisierungMenueleiste();
	}

	/**
	 * Entfernt alle Elemente aus der Hauptsymbolleiste (also dem
	 * {@link CoolBarManager}) und leert die Liste der {@link ToolBarManager},
	 * welche die separierten Abschnitte in der Hauptsymbolleiste bilden.
	 */
	private void leereCoolBarUndToolBars()
	{
		/*
		 * Entfernen aller Elemente aus der Hauptsymbolleiste
		 */
		if (_cbm != null)
		{
			_cbm.removeAll();
		}

		/*
		 * Entfernen aller gespeicherten ToolBarManager aus der Liste der
		 * ToolBarManager
		 */
		if (_listeToolBarManager != null)
		{
			if (_listeToolBarManager.size() > 0)
			{
				_listeToolBarManager.clear();
			}
		}
	}

	/**
	 * Klasse {@link MenGen} implementiert einen {@link IMenuCreator}, mit dessen
	 * Hilfe die XML-Struktur des Mens fr einen PullDown-Button geliefert wird.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class MenGen implements IMenuCreator
	{
		private Element _submenu = null;

		public MenGen(Element submenu)
		{
			super();
			this._submenu = submenu;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public Menu getMenu(Control parent)
		{
			Menu testMenu = new Menu(parent);

			if (_submenu != null)
			{
				Action test1 = new Action("Test1")
				{
					public void run()
					{
						_debug.fine("Test1");
					}
				};

				Action test2 = new Action("Test2")
				{
					public void run()
					{
						_debug.fine("Test2");
					}
				};

				addActionToMenu(testMenu, test1);
				addActionToMenu(testMenu, test2);

				Action subaction = new Action("SubAction1", SWT.DROP_DOWN)
				{
					public void run()
					{
						_debug.fine("SubAction");
					}
				};

				// Neues Submen anlegen

				MenuItem tempSubMenuItem = new MenuItem(testMenu, SWT.CASCADE);
				tempSubMenuItem.setText("Submenname");

				// Menu tempSubmenu = new Menu(tempSubMenuItem);
				Menu tempSubmenu = new Menu(testMenu);
				tempSubMenuItem.setMenu(tempSubmenu);

				// Aktion zum tempSubmen hinzufgen
				addActionToMenu(tempSubmenu, subaction);
			} else
			{
				_debug.fine(getClass().getName()
						+ "::FEHLER => Es wurde kein Element bergeben");
			}

			baueMenuStruktur(testMenu);

			return testMenu;
		}

		/**
		 * Baut die Menstruktur in einem Men auf.
		 * 
		 * @param menu
		 */
		private void baueMenuStruktur(Menu menu)
		{
			if (_submenu.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
			{
				_debug.fine("Hofhfh");
				baueSubMenuRekursiv(menu, _submenu);
			} else
			{
				_debug
						.error(getClass().getName()
								+ "::baueMenuStruktur \n\t=>Das bergebene Element ist nicht vom Typ Submen. Abbruch.\n");
			}
		}

		/**
		 * Baut rekursiv die SubMenstruktur auf.
		 * 
		 * @param menu
		 *          Men, auf dessen Basis die SubMenstruktur aufgebaut werden
		 *          soll.
		 * @param subElement
		 *          Element, welches die Struktur des Submens enthlt.
		 */
		private void baueSubMenuRekursiv(Menu menu, Element subElement)
		{
			if (subElement.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
			{
				/*
				 * Neues Submen anlegen
				 */
				MenuItem tempSubMenuItem = new MenuItem(menu, SWT.CASCADE);
				tempSubMenuItem.setText(subElement
						.getAttributeValue(XmlBezeichnerMenuSymbol.attName));

				Menu tempSubmenu = new Menu(tempSubMenuItem);
				tempSubMenuItem.setMenu(tempSubmenu);

				/*
				 * Kindelemente des Submen-Elementes auslesen und die Struktur weiter
				 * aufbauen.
				 */
				Element elementMenu = subElement
						.getChild(XmlBezeichnerMenuSymbol.elementMenu);
				if (elementMenu != null)
				{
					@SuppressWarnings("unchecked")
					List children = elementMenu.getChildren();

					@SuppressWarnings("unchecked")
					Iterator it = children.iterator();
					while (it.hasNext())
					{
						Element child = (Element) it.next();

						/*
						 * item
						 */
						if (child.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
						{
							Action action = null;

							DeskriptorAktion descriptorAction = ElementRegistrierung
									.getInstanz().getDeskriptorAktion(
											child.getAttributeValue(XmlBezeichnerMenuSymbol.attId));

							if (descriptorAction != null)
							{
								action = (Action) descriptorAction.getAktion();
							} else
							{
								DeskriptorAktion deskriptorAktionGlobal = ElementRegistrierung
										.getInstanz().getDeskriptorAktionGlobal(
												child.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
								if (deskriptorAktionGlobal != null)
								{
									action = (Action) deskriptorAktionGlobal.getAktion();
								}
							}

							if (action != null)
							{
								addActionToMenu(tempSubmenu, action);
							}

						}
						//
						// // separator
						// if (child.getName()
						// .equals(XmlBezeichnerMenuSymbol.elementSeparator))
						// {
						// MenuItem separator = new MenuItem(tempSubmenu, SWT.SEPARATOR);
						// }
						//

						/*
						 * submenu
						 */
						if (child.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
						{
							baueSubMenuRekursiv(tempSubmenu, child);
						}

					}
				} else
				{
					_debug
							.error(getClass().getName()
									+ "::baueSubMenuRekursiv \n\t=>keine Men-Element im SubMen (id:= name:= ) vorhanden.  \n");
				}

			} else
			{
				_debug
						.error(getClass().getName()
								+ "::baueSubMenuRekursiv \n\t=>Das bergebene Element ist nicht vom Typ Submen. Abbruch.\n");
			}
		}

		/**
		 * Fgt eine {@link Action} zu einem Men hinzu.
		 * 
		 * @param parent
		 *          Men, in dass die {@link Action} eingefgt werden soll.
		 * @param action
		 *          Einzufgende {@link Action}.
		 */
		public void addActionToMenu(Menu parent, Action action)
		{
			ActionContributionItem aci = new ActionContributionItem(action);
			aci.fill(parent, -1);
		}

		@Override
		public Menu getMenu(Menu parent)
		{
			return null;
		}

	}

	@SuppressWarnings("unused")
	private class InternerMenuGenerator implements IMenuCreator
	{
		/**
		 * Men, dass erzeugt wird.
		 */
		Menu _menu = null;

		/**
		 * Submenu, das zum Erzeugen des Mens ausgelesen wird.
		 */
		Element _submenu = null;

		public InternerMenuGenerator(Element submenu)
		{
			super();
			this._submenu = submenu;
		}

		public void dispose()
		{
		}

		public Menu getMenu(Control parent)
		{
			_debug.fine("GET MENU!");

			if (_submenu == null)
			{
				_debug.fine("NULL");
				_debug
						.error(getClass().getName()
								+ "::getMenu \n\t=>Es wurde keine Submen-Element im Konstruktor bergeben. Abbruch. \n");
				return null;
			} else
			{
				_debug.fine("NOT NULL");
				_menu = new Menu(parent);
				baueMenuStruktur(_menu);
				return _menu;
			}
		}

		private void baueMenuStruktur(Menu menu)
		{
			if (_submenu.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
			{
				baueSubMenuRekursiv(menu, _submenu);
			} else
			{
				_debug
						.error(getClass().getName()
								+ "::baueMenuStruktur \n\t=>Das bergebene Element ist nicht vom Typ Submen. Abbruch.\n");
			}
		}

		private void baueSubMenuRekursiv(Menu menu, Element subElement)
		{
			if (subElement.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
			{
				// Neues Submen anlegen
				MenuItem tempSubMenuItem = new MenuItem(menu, SWT.CASCADE);
				tempSubMenuItem.setText(subElement
						.getAttributeValue(XmlBezeichnerMenuSymbol.attName));

				// Menu tempSubmenu = new Menu(tempSubMenuItem);
				Menu tempSubmenu = new Menu(menu);
				tempSubMenuItem.setMenu(tempSubmenu);

				Element elementMenu = subElement
						.getChild(XmlBezeichnerMenuSymbol.elementMenu);
				if (elementMenu != null)
				{
					@SuppressWarnings("unchecked")
					List children = elementMenu.getChildren();
					@SuppressWarnings("unchecked")
					Iterator it = children.iterator();
					while (it.hasNext())
					{
						Element child = (Element) it.next();

						// item
						if (child.getName().equals(XmlBezeichnerMenuSymbol.elementItem))
						{
							DeskriptorAktion descriptorAction = ElementRegistrierung
									.getInstanz().getDeskriptorAktion(
											child.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
							Action action = (Action) descriptorAction.getAktion();
							einfuegenActionInMenu(tempSubmenu, action);
						}

						// separator
						if (child.getName()
								.equals(XmlBezeichnerMenuSymbol.elementSeparator))
						{
							MenuItem separator = new MenuItem(tempSubmenu, SWT.SEPARATOR);
						}

						// submen
						if (child.getName().equals(XmlBezeichnerMenuSymbol.elementSubMenu))
						{
							baueSubMenuRekursiv(tempSubmenu, child);
						}

					}

				} else
				{
					_debug
							.error(getClass().getName()
									+ "::baueSubMenuRekursiv \n\t=>keine Men-Element im SubMen (id:= name:= ) vorhanden.  \n");
				}

			} else
			{
				_debug
						.error(getClass().getName()
								+ "::baueSubMenuRekursiv \n\t=>Das bergebene Element ist nicht vom Typ Submen. Abbruch.\n");
			}
		}

		public void hinzufuegenAction(IAction action)
		{

		}

		public void hinzufuegenSubMenu(Menu submenu)
		{

		}

		public Menu getMenu(Menu parent)
		{
			_debug.fine("SHIT HAPPENS!");
			return null;
		}

		/**
		 * Fgt eien Action als {@link ActionContributionItem} in ein {@link Menu}
		 * ein.
		 * 
		 * @param elternMenu
		 *          Elternmen, in dass die Action eingefgt werden soll.
		 * @param action
		 *          Action, die eingefgt werden soll.
		 */
		private void einfuegenActionInMenu(Menu elternMenu, Action action)
		{
			ActionContributionItem aci = new ActionContributionItem(action);
			aci.fill(elternMenu, -1);
		}

	}

}
