/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.basislib.datenidentifikation.Datenidentifikation;
import de.bsvrz.buv.rw.basislib.datenidentifikation.Datenidentifikation.Darstellung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse ermglicht die Objektzusammenstellung. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-96 und 97
 * 
 * Einschrnkungen: In einer Eclipse RCP Anwendung sind Dialoge modal, d.h. sie
 * berlagern alle anderen Dialoge und damit das gesamte Rahmenwerk, wenn sie
 * aktiv sind. Dadurch ist es nicht mglich, per Drag & Drop SystemObjekte aus
 * der Komponente zur Datenidentifikation hinzuzufgen.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 13:15:29 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 * 
 */
public class Objektzusammenstellung extends Dialog
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private ClientDavInterface _connection = null;

	/*
	 * Standard GUI-Elemente
	 */
	private Shell _shell = null;

	private Group _groupObjekte = null;

	private Composite _compositeMitte = null;

	private Button _checkBoxObjekttypEinschraenken = null;

	private Label _labelObjekttypenEinschraenken = null;

	private Group _groupBeschreibung = null;

	private Text _textAreaBeschreibung = null;

	private Button _buttonHinzufuegen = null;

	private Button _buttonLoeschen = null;

	private Button _buttonHoch = null;

	private Button _buttonRunter = null;

	private Button _buttonOk = null;

	private Button _buttonAbbrechen = null;

	private Button _buttonHilfe = null;

	/**
	 * SWT-Liste, die im Dialog sichtbar ist
	 */
	private List _ListeSWT = null;

	/**
	 * Enspricht den Indices der in der Liste ListeSWT selektierten Elementen.
	 */
	private int[] _selektion = null;

	/**
	 * ber die Check-Box "Objekttypen einschrnken" wird vorgegeben, dass nur
	 * Datenidentifikationen in die Objektzusammenstellung bernommen werden
	 * knnen, die von einem Objekttyp sind, der bereits in der
	 * Objektzusammenstellung vorkommt. Wenn z.B. nur Objekte vom Typ MQ in der
	 * Objektzusammenstellung enthalten sind und die Check-Box aktiviert ist, dann
	 * knnen auch nur weitere Objekte vom Typ MQ hinzugefht werden. Dieser
	 * Umstand muss sich direkt fr die Datenidentifikations-Eingabe auswirken.
	 * D.h. wenn in diesem Fall "Hinzufgen" bettigt wird, muss die
	 * Datenidentifikations-Eingabe bereits auf Objekte vom Typ "MQ" filtern. Drag
	 * und Drop wird nur fr Objekte der entsprechenden Objekttypen zugelassen Die
	 * Check-Box ist optional. Fr die Zusammenstellung eines Streckenprofils ist
	 * die Check-Box nicht erforderlich. In diesen Fllen bestimmt die Anwendung
	 * bereits welche Objekttypen zulssig sind. Checkbox ja/nein
	 * 
	 * Gibt an, ob nach ObjektTyp eingeschrnkt werden soll oder nicht.
	 */
	private boolean _einschraenkenObjektTyp = false;

	/**
	 * Liste von Listenobjekte, die der Objektreprsentation der in der Liste
	 * listObjektListe enthaltenen Elementen entspricht.
	 */
	private java.util.List<ListenObjekt> _ListeDaten = null;

	/**
	 * Hier werden die Erfassten SystemObjekte an die Klasse
	 * Objektzusammenstellung zurckgegeben.
	 */
	private java.util.List<SystemObject> _listeRueckgabeSystemObjekte = null;

	/**
	 * Konstruktor der Klasse {@link Objektzusammenstellung}. <br>
	 * Erezeugt eine neue Instanz dieser Klasse, die Darstellung erfolgt auf der
	 * Basis der bergebenen Eltern-Shell. <br>
	 * 
	 * @param parent
	 *          Eltern-Shell, auf deren Basis die Darstellung erfolgen soll.
	 * @param connection
	 *          Verbindung zum DaV. Kann auch <code>null</code> sein, dann kann
	 *          jedoch kein {@link SystemObject} ber den Dialog zur
	 *          Datenidentifikation hinzugefgt werden.
	 * @param rueckListe
	 *          Liste von Systemobjekten, in der die Objektzusammenstellung
	 *          erfolgen soll. Sind in dieser Liste schon Systemobjekte vorhanden,
	 *          werden diese im Dialog angezeigt.
	 */
	public Objektzusammenstellung(Shell parent, ClientDavInterface connection,
			java.util.List<SystemObject> rueckListe)
	{
		super(parent);
		this._connection = connection;
		_listeRueckgabeSystemObjekte = rueckListe;
	}

	/**
	 * Konstruktor der Klasse {@link Objektzusammenstellung}. Erezeugt eine neue
	 * Instanz dieser Klasse, die Darstellung erfolgt auf der Basis der
	 * bergebenen Eltern-Shell. <br>
	 * 
	 * @param parent
	 *          Eltern-Shell, auf deren Basis die Darstellung erfolgen soll.
	 * @param connection
	 *          Verbindung zum DaV. Kann auch <code>null</code> sein, dann kann
	 *          jedoch kein {@link SystemObject} ber den Dialog zur
	 *          Datenidentifikation hinzugefgt werden.
	 * @param rueckListe
	 *          Liste von Systemobjekten, in der die Objektzusammenstellung
	 *          erfolgen soll. Sind in dieser Liste schon Systemobjekte vorhanden,
	 *          werden diese im Dialog angezeigt.
	 * @param filtern
	 *          Wenn <code>true</code>, dann wird nach den Systemobjekttypen
	 *          gefiltert, bei <code>false</code> nicht.
	 */
	public Objektzusammenstellung(Shell parent, ClientDavInterface connection,
			java.util.List<SystemObject> rueckListe, boolean filtern)
	{
		super(parent);
		this._connection = connection;
		_listeRueckgabeSystemObjekte = rueckListe;
		_einschraenkenObjektTyp = filtern;
	}

	// Hier mssen noch die RckgabeWerte als Array von SystemObject
	// eingebaut werden.
	// Prfung auf bereits in der Liste vorhandene SystemObjekte hinzufgen.

	/**
	 * ffnet den Dialog zur Objektzusammenstellung.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn der erfolgreich geffnet
	 *         wurde.
	 */
	public boolean open()
	{
		Shell parent = getParent();
		_shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL);

		// 1. Erezugen der grafischen Darstellung.
		erzeugeShell();

		// 2. Initialisieren der Datenstrukturen.
		initDatenstrukturen();

		// 3. ffnen der Shell.
		_shell.open();

		Display display = parent.getDisplay();

		// Dialog zentrieren
		Monitor primary = display.getPrimaryMonitor();
		Rectangle bounds = primary.getBounds();
		Rectangle rect = _shell.getBounds();
		int x = bounds.x + (bounds.width - rect.width) / 2;
		int y = bounds.y + (bounds.height - rect.height) / 2;
		_shell.setLocation(x, y);

		// Displayloop.
		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return true;
	}

	/**
	 * Initialisiert die hier bentigten Datenstrukturen
	 * 
	 */
	private void initDatenstrukturen()
	{
		System.out
				.println("ObjektzusammenstellungDialog::Init => Datenstrukturen werden initialisiert.");
		_ListeDaten = new LinkedList<ListenObjekt>();

		// Fllen der Liste mit den bentigten Elementen
		fuelleObjektListe();

		// Fllten der Listen mit den bergebenen Elementen.
		uebergabeListeAngleichen();

		_checkBoxObjekttypEinschraenken.setEnabled(true);

		_ListeSWT.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				if (_ListeSWT.getSelectionIndex() > -1)
				{
					// Beschreibung des jeweiligen Objektes in
					// textAreaBeschreibung anzeigen
					_textAreaBeschreibung.setText("Beschreibung: "
							+ getListenObjektBeschreibung(_ListeSWT.getSelectionIndex()));

					// (Mehrfach-)Selektion als Index der jeweils selektierten
					// Elemente
					// im Array Selektion ablegen.
					_selektion = _ListeSWT.getSelectionIndices();

					_debug.fine("Anzahl selektierter Elemente: " + _selektion.length);
				}

			}
		});

	}

	/**
	 * Initialisierung von shell
	 */
	private void erzeugeShell()
	{
		_shell.setText("Objektzusammenstellung");
		_shell.setLayout(null);
		erzeugeGruppeObjekte();
		erzeugeCompositeMitte();
		erzeugeGruppeBeschreibung();
		_shell.setSize(new Point(481, 483));
		_buttonOk = new Button(_shell, SWT.NONE);
		_buttonOk.setBounds(new Rectangle(60, 420, 101, 23));
		_buttonOk.setText("OK");
		_buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (_selektion != null && _selektion.length > 0)
				{
					String ElementUebersicht = "";
					for (int i = 0; i < _selektion.length; i++)
					{
						ElementUebersicht = ElementUebersicht + "E:" + _selektion[i] + ", ";
					}
					_debug.fine("Selektierte Elemente: " + ElementUebersicht);
				} else
				{
					// Nix tun
				}

				_shell.dispose();
			}
		});

		_buttonAbbrechen = new Button(_shell, SWT.NONE);
		_buttonAbbrechen.setBounds(new Rectangle(180, 420, 111, 23));
		_buttonAbbrechen.setText("Abbrechen");
		_buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_shell.dispose();
			}
		});

		_buttonHilfe = new Button(_shell, SWT.NONE);
		_buttonHilfe.setBounds(new Rectangle(310, 420, 101, 23));
		_buttonHilfe.setText("Hilfe");
		_buttonHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Hilfe wurde geklickt.");
			}
		});
	}

	/**
	 * Initialisierung von groupObjekte
	 * 
	 */
	private void erzeugeGruppeObjekte()
	{
		_groupObjekte = new Group(_shell, SWT.NONE);
		_groupObjekte.setLayout(null);
		_groupObjekte.setText("Objekte");
		_groupObjekte.setBounds(new Rectangle(10, 10, 451, 191));
		_ListeSWT = new List(_groupObjekte, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL
				| SWT.H_SCROLL);
		_ListeSWT.setBounds(new Rectangle(13, 23, 291, 151));

		/*
		 * Hinzufgen
		 */
		_buttonHinzufuegen = new Button(_groupObjekte, SWT.NONE);
		_buttonHinzufuegen.setBounds(new Rectangle(323, 23, 121, 31));
		_buttonHinzufuegen.setText("Hinzufgen");
		_buttonHinzufuegen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				hinzufuegenObjekt();
			}
		});

		/*
		 * Lschen
		 */
		_buttonLoeschen = new Button(_groupObjekte, SWT.NONE);
		_buttonLoeschen.setBounds(new Rectangle(323, 73, 121, 31));
		_buttonLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				entferneSelektierteObjekteAusListe();
			}
		});

		/*
		 * Hoch
		 */
		_buttonHoch = new Button(_groupObjekte, SWT.ARROW | SWT.UP);
		_buttonHoch.setBounds(new Rectangle(339, 133, 25, 25));
		_buttonHoch.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Hoch wurde geklickt.");
				// ListeSWT.getSelectionIndex()
				if (_ListeSWT.getSelectionIndex() > -1)
				{
					verschiebeNachOben(_ListeSWT.getSelectionIndex());
				}
			}
		});

		/*
		 * Runter
		 */
		_buttonRunter = new Button(_groupObjekte, SWT.ARROW | SWT.DOWN);
		_buttonRunter.setBounds(new Rectangle(393, 133, 25, 25));
		_buttonRunter.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Runter wurde geklickt.");
				if (_ListeSWT.getSelectionIndex() > -1)
				{
					verschiebeNachUnten(_ListeSWT.getSelectionIndex());
				}
			}
		});
	}

	/**
	 * Initialisierung von compositeMitte
	 * 
	 */
	private void erzeugeCompositeMitte()
	{
		_compositeMitte = new Composite(_shell, SWT.NONE);
		_compositeMitte.setBounds(160, 212, 200, 30);

		_checkBoxObjekttypEinschraenken = new Button(_compositeMitte, SWT.CHECK);
		_checkBoxObjekttypEinschraenken.setBounds(new Rectangle(10, 10, 13, 16));
		_checkBoxObjekttypEinschraenken.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (e.getSource() instanceof Button)
				{
					Button b = (Button) e.getSource();
					if (b.getSelection())
					{
						_debug.fine("Objekttypeinschrnken wurde aktiviert");
						_einschraenkenObjektTyp = true;
						String tempString[] = bestimmeVorhandeneObjektTypen();
						if (tempString.length > 0)
						{
							for (int i = 0; i < tempString.length; i++)
							{
								_debug.fine("ObjektTyp #" + i + ": " + tempString[i]);
							}
						}
					} else
					{
						_debug.fine("Objekttypeinschrnken wurde deaktiviert");
						_einschraenkenObjektTyp = false;
					}
				}
			}
		});

		// _checkBoxObjekttypEinschraenken.setEnabled(false);

		// Checkbox ObjekttypEinschraenken abhaken, wen eingeschrnkt werden soll.
		if (_einschraenkenObjektTyp)
		{
			_checkBoxObjekttypEinschraenken.setSelection(true);
		}
		// Sonst nicht.
		else
		{
			_checkBoxObjekttypEinschraenken.setSelection(false);
		}

		_labelObjekttypenEinschraenken = new Label(_compositeMitte, SWT.NONE);
		_labelObjekttypenEinschraenken.setText("Objekttyp einschrnken");
		_labelObjekttypenEinschraenken.setBounds(new Rectangle(30, 10, 114, 13));

		_compositeMitte.setSize(new Point(161, 34));
		_compositeMitte.setLayout(null);

	}

	/**
	 * Initialisierung von groupBeschreibung
	 * 
	 */
	private void erzeugeGruppeBeschreibung()
	{
		_groupBeschreibung = new Group(_shell, SWT.NONE);
		_groupBeschreibung.setLayout(null);
		_groupBeschreibung.setText("Beschreibung");
		_groupBeschreibung.setBounds(new Rectangle(10, 250, 451, 151));
		_textAreaBeschreibung = new Text(_groupBeschreibung, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		_textAreaBeschreibung.setEditable(false);
		_textAreaBeschreibung.setBounds(5, 15, 440, 132);
	}

	/**
	 * Fllt/Aktuallisiert listObjektListe mit den entsprechenden Werten aus Liste
	 * li der ListenObjekte
	 * 
	 */
	private void fuelleObjektListe()
	{
		ListIterator<ListenObjekt> it = _ListeDaten.listIterator();
		while (it.hasNext())
		{
			ListenObjekt obj = it.next();
			_ListeSWT.add(obj.getName());
		}
	}

	/**
	 * Liefert die Beschreibung fr ein selektiertes ListenObjekt.
	 * 
	 * @return
	 */
	private String getListenObjektBeschreibung(int index)
	{
		return (_ListeDaten.get(index).getBeschreibung());
	}

	/**
	 * Entfernt die in listObjektListe selektierten Elemente
	 * 
	 */
	private void entferneSelektierteObjekteAusListe()
	{
		if (_selektion != null && _selektion.length > 0)
		{
			String Elemente = "";
			int[] loeschen = _ListeSWT.getSelectionIndices();

			for (int i = 0; i < loeschen.length; i++)
			{
				Elemente = Elemente + loeschen[i] + ", ";
				_ListeSWT.remove(loeschen[i] - i); // -i, weill schon i Elemente aus
				// der Liste entfernt wurden
				_ListeDaten.remove(loeschen[i] - i); // -i, weill schon i Elemente aus
				// der Liste entfernt wurden
			}

			_selektion = null;

			_debug.fine("Die Elemente " + Elemente
					+ " wurden aus der Liste entfernt.");
		} else
		{
			System.out
					.println("Es sind keine Objekte selektiert worden. Diese knnen nicht aus der Liste entfernt werden.");
		}

		// Wenn die Liste kein Element mehr enthltz, dann
		// checkBoxObjekttypEinschraenken deaktiviern
		if (_ListeDaten.size() == 0 && _ListeDaten.size() == 0)
		{
			if (_checkBoxObjekttypEinschraenken.getSelection())
			{
				_checkBoxObjekttypEinschraenken.setSelection(false);
			}
			_checkBoxObjekttypEinschraenken.setEnabled(false);
		}

		rueckgabeListeAngleichen();
	}

	/**
	 * Fgt ein Objekt zur Liste hinzu.
	 * 
	 */
	private void hinzufuegenObjekt()
	{
		if (_connection != null)
		{
			ListenObjekt listenObjekt = null;

			// Wenn ein Verbindungsobjekt zum DaV vorhanden ist
			if (_connection != null)
			{
				// An dieser Stelle muss die Datenidentifikationseingabe geffnet werden
				// und das entsprechend zurckgegebene Objekt eingefgt werden.
				Shell shell = new Shell(Display.getDefault());
				Datenidentifikation dialogDatenIdent = new Datenidentifikation(shell,
						_connection);
				// dialogDatenIdent.setConnection(_connection);

				SystemObject[] objects = null;
				if (_einschraenkenObjektTyp)
				{
					// objects = dialogDatenIdent
					// .oeffne(
					// bestimmeVorhandeneObjektTypen(),
					// Datenidentifikation.Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP_SIMUVAR);
					objects = dialogDatenIdent.oeffne(bestimmeVorhandeneObjektTypen(),
							Datenidentifikation.Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP);
					_debug.fine("ffnete eingeschrnkte ObjektTypen");

				} else
				{
					// objects = dialogDatenIdent
					// .oeffne(Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP_SIMUVAR);
					objects = dialogDatenIdent
							.oeffne(Darstellung.OBJEKTELISTE_OBJEKTTYP_ATG_ASP);
					_debug.fine("ffnete alle ObjektTypen");
				}

				_debug.fine("Anzahl SystemObjekte: " + objects.length);

				_ListeSWT.setVisible(false);
				for (int i = 0; i < objects.length; i++)
				{
					listenObjekt = new ListenObjekt(objects[i]);
					_ListeDaten.add(listenObjekt);
					_ListeSWT.add(listenObjekt.getName());
				}
				_ListeSWT.setVisible(true);

			} else
			{
				listenObjekt = new ListenObjekt();
				// Neues ListenObjekt aus der Rckgabe der Datenidentifikationseingabe
				// erzeugen
				listenObjekt.setName("Objekt aus Datenidentifikationseingabe");
				listenObjekt
						.setBeschreibung("Dies ist ein Testobjekt aus der Datenidentifikationseingabe!");
				_ListeDaten.add(listenObjekt);
				_ListeSWT.add(listenObjekt.getName());
			}

			rueckgabeListeAngleichen();
		} else
		{
			MessageBox box = new MessageBox(_shell, SWT.OK | SWT.ICON_ERROR);
			box.setText("Keine Verbindung zum DaV");
			box
					.setMessage("Es wurde keine Verbindung zum DaV bergeben. \nDer Dialog zur Datenidentifikation wird nicht angezeigt.");
			box.open();
		}

	}

	// /**
	// * Fgt ein Array von Systemobjekten zur Liste hinzu
	// *
	// * @param objekt
	// * Array von Systemobjekten, dass zur Liste hinzugefgt werden soll
	// */
	// public void hinzufuegenSystemObjekte(SystemObject[] objekt)
	// {
	// ListenObjekt listenObjekt = null;
	// for (int i = 0; i < objekt.length; i++)
	// {
	// listenObjekt = new ListenObjekt();
	// listenObjekt.setSystemObject(objekt[i]);
	// _ListeDaten.add(listenObjekt);
	// _ListeSWT.add(listenObjekt.getName());
	// }
	// }

	// /**
	// * Setzt den Haken ObjekttypEinschrnken
	// *
	// * @param einschraenken
	// * true oder false
	// */
	// public void setObjekttypEinschraenken(boolean einschraenken)
	// {
	// if (einschraenken)
	// {
	// _checkBoxObjekttypEinschraenken.setEnabled(true);
	// } else
	// {
	// _checkBoxObjekttypEinschraenken.setEnabled(false);
	// }
	// }

	/**
	 * Verschiebt ein Element in der ListeObjekt/ListeSWT um eine Position nach
	 * oben.
	 * 
	 * @param index
	 *          Index des Elements, dass nach oben verschoben werden soll.
	 */
	private void verschiebeNachOben(int index)
	{
		if (index == 0)
		{
			@SuppressWarnings("unused")
			boolean ok = false;
			MessageBox box = new MessageBox(_shell, SWT.OK | SWT.ICON_ERROR);
			box.setText("Fehler");
			box
					.setMessage("Das Element kann nicht nach oben verschoben werden, \ndenn es ist schon das letzte Element in der Liste.");
			if (box.open() == SWT.OK)
			{
				ok = true;
			}
		} else
		{
			// Verschieben
			// 1. In der ListeDaten
			ListenObjekt tempListenObjekt = _ListeDaten.get(index);
			_ListeDaten.remove(index);
			_ListeDaten.add(index - 1, tempListenObjekt);

			// 2. In der ListeSWT
			String tempListeSWTElement = _ListeSWT.getItem(index);
			_ListeSWT.remove(index);
			_ListeSWT.add(tempListeSWTElement, index - 1);

			// 3. Erneutes selektieren des verschobenen Elementes in ListeSWT
			_ListeSWT.select(index - 1);

		}

		rueckgabeListeAngleichen();
	}

	/**
	 * Verschiebt ein Element in der ListeObjekt/ListeSWT um eine Position nach
	 * unten.
	 * 
	 * @param index
	 *          Index des Elements, dass nach unten verschoben werden soll.
	 */
	private void verschiebeNachUnten(int index)
	{
		if (index == _ListeDaten.size() - 1)
		{
			@SuppressWarnings("unused")
			boolean ok = false;
			MessageBox box = new MessageBox(_shell, SWT.OK | SWT.ICON_ERROR);
			box.setText("Fehler");
			box
					.setMessage("Das Element kann nicht nach unten verschoben werden, \ndenn es ist schon das letzte Element in der Liste.");
			if (box.open() == SWT.OK)
			{
				ok = true;
			}
		} else
		{
			// Verschieben
			// 1. In der ListeDaten
			ListenObjekt tempListenObjekt = _ListeDaten.get(index);
			_ListeDaten.remove(index);
			_ListeDaten.add(index + 1, tempListenObjekt);

			// 2. In der ListeSWT
			String tempListeSWTElement = _ListeSWT.getItem(index);
			_ListeSWT.remove(index);
			_ListeSWT.add(tempListeSWTElement, index + 1);

			// 3. Erneutes selektieren des verschobenen Elementes in ListeSWT
			_ListeSWT.select(index + 1);
		}

		rueckgabeListeAngleichen();
	}

	/*
	 * Hilfsfunktionen
	 */

	/**
	 * Bestimmt die in der Liste vorhandenen ObjektTypen und gibt deren Namen in
	 * einem Array von Strings zurck. Die wird bentigt wenn der der Haken fr
	 * die Einschrnkung gesetzt wird, dass nur Objekte vom gleichen Typ ber den
	 * Datenidentifiaktions- Dialog aufgenommen werden sollen.
	 */
	private String[] bestimmeVorhandeneObjektTypen()
	{
		// Rckgabe String
		String tempString[] = null;

		TreeMap<String, String> tempMap = new TreeMap<String, String>();
		Iterator<ListenObjekt> it = _ListeDaten.iterator();
		while (it.hasNext())
		{
			ListenObjekt listenObjekt = it.next();
			SystemObject tempSO = listenObjekt.getSystemObject();
			String insertKey = tempSO.getType().getPid();
			String insertValue = tempSO.getType().getName();
			if (!tempMap.containsKey(insertKey))
			{
				tempMap.put(insertKey, insertValue);
			}
		}

		if (tempMap.size() > 0)
		{
			int i = 0;
			tempString = new String[tempMap.size()];
			Iterator<Entry<String, String>> itTempMap = tempMap.entrySet().iterator();
			while (itTempMap.hasNext())
			{
				tempString[i] = itTempMap.next().getKey();
				i++;
			}
		}

		return tempString;
	}

	/**
	 * Fgt in die Darstellung die bergebenen SystemObjekte ein.
	 * 
	 */
	private void uebergabeListeAngleichen()
	{
		if (_listeRueckgabeSystemObjekte != null)
		{
			Iterator<SystemObject> it = _listeRueckgabeSystemObjekte.iterator();
			while (it.hasNext())
			{
				SystemObject systemObject = it.next();
				ListenObjekt neuesListenObjekt = new ListenObjekt(systemObject);
				_ListeDaten.add(neuesListenObjekt);
				_ListeSWT.add(neuesListenObjekt.getName());
			}
		}

	}

	/**
	 * Gleicht die RckgabeListe mit der hier gefhrten _ListeDaten an.
	 * 
	 */
	private void rueckgabeListeAngleichen()
	{
		if (_ListeDaten.size() > 0)
		{
			_listeRueckgabeSystemObjekte.clear();
			Iterator<ListenObjekt> it = _ListeDaten.iterator();
			while (it.hasNext())
			{
				ListenObjekt objekt = it.next();
				SystemObject systemObject = objekt.getSystemObject();
				_listeRueckgabeSystemObjekte.add(systemObject);
			}
		}

	}

	/**
	 * Reprsentiert die Objekte, die in listObjektListe angezeigt werden sollen,
	 * dient somit als Datencontainer.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ListenObjekt
	{
		/**
		 * Name des Objektes, taucht damit auch i.d. Liste auf
		 */
		private String name;

		/**
		 * Beschreibung des Objekts
		 */
		private String beschreibung;

		/**
		 * Das SystemObject
		 */
		private SystemObject systemObject;

		/*
		 * An dieser Stelle nicht vergessen, das eigentliche Objekt ber eine
		 * entsprechende Instanz zu "verlinken".
		 */

		/**
		 * Std-Konstruktor
		 */
		public ListenObjekt()
		{
		}

		public ListenObjekt(SystemObject sysObj)
		{
			this.setSystemObject(sysObj);
		}

		/**
		 * Liefert den Namen des Objekts
		 * 
		 * @return Objektname
		 */
		public String getName()
		{
			return name;
		}

		/**
		 * Setzt den Namen des Objekts
		 * 
		 * @param name
		 *          Name des Objekts
		 */
		public void setName(String name)
		{
			this.name = name;
		}

		/**
		 * Liefert die Beschreibung des Objekts
		 * 
		 * @return
		 */
		public String getBeschreibung()
		{
			return beschreibung;
		}

		/**
		 * Setzt die Beschreibung des Objekts
		 * 
		 * @param beschreibung
		 */
		public void setBeschreibung(String beschreibung)
		{
			this.beschreibung = beschreibung;
		}

		/**
		 * Liefert das SystemObject dieser Klasse
		 * 
		 * @return SystemObject
		 */
		public SystemObject getSystemObject()
		{
			return systemObject;
		}

		/**
		 * Setzt das Systemobject dieser Klasse. Gleichzeitig wird Name und
		 * Beschreibung fr die Darstellung in der Liste gesetzt.
		 * 
		 * @param systemObject
		 *          Zu setzendes Systemobjekt
		 */
		public void setSystemObject(SystemObject systemObject)
		{
			this.systemObject = systemObject;
			// this.setName(systemObject.getName());
			String tempName = "";
			if (systemObject.getName() != null && systemObject.getName() != "")
				tempName = tempName + systemObject.getName() + " ";
			if (systemObject.getPid() != null && systemObject.getPid() != "")
				tempName = tempName + "(" + systemObject.getPid() + ") ";
			if (systemObject.getId() > 0)
				tempName = tempName + "[" + String.valueOf(systemObject.getId()) + "]";

			this.setName(tempName);
			this.setBeschreibung((systemObject.getInfo()).getShortInfo());
		}

	}

}
