/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenTeilSegment.BetriebsKilometerEintrag;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.sys.funclib.debug.Debug;

public class Betriebskilometer
{
	/**
	 * Flag fr Shellausgaben
	 */
	private boolean _shellAusgaben = false;

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _verbindung = null;

	/**
	 * Enthlt die Instanz der Klasse {@link KonfigurationNeu}, welche die hier
	 * bentigten Datenstrukturen zur Verfgung stellt.
	 */
	private KonfigurationNeu _konf = null;

	/**
	 * Diese {@link Struktur} enthlt die der Reihenfolge nach geordneten
	 * uerenStraenSegmente einer Strae, deren Richtung <b>positiv</b> ist.
	 */
	private Struktur _strukturPositiv = null;

	/**
	 * Diese {@link Struktur} enthlt die der Reihenfolge nach geordneten
	 * uerenStraenSegmente einer Strae, deren Richtung <b>negativ</b> ist.
	 */
	private Struktur _strukturNegativ = null;

	/**
	 * Enthlt die Liste der Reihenfolge nach geordneten ueren StraenSegmente
	 * in <b>positiver</b> Richtung.
	 */
	private LinkedList<StrukturElement> _listePosAeussereStrassenSegmente = null;

	/**
	 * Enthlt die Liste der Reihenfolge nach geordneten ueren StraenSegmente
	 * in <b>negativer</b> Richtung.
	 */
	private LinkedList<StrukturElement> _listeNegAeussereStrassenSegmente = null;

	/**
	 * In diesem String wird die PID der aktuellen Strae hinterlegt. <br>
	 * Id die Pid der alten Strae mit der Pid der aktuellen Strae identisch, so
	 * muss nicht neu nach den uerenStraenSegmenten der Strae gefiltert
	 * werden.
	 */
	private String _pidAktuelleStrasse = null;

	/**
	 * In diesem String wird die PID der alten Strae hinterlegt. <br>
	 * Id die Pid der alten Strae mit der Pid der aktuellen Strae identisch, so
	 * muss nicht neu nach den uerenStraenSegmenten der Strae gefiltert
	 * werden.
	 */
	private String _pidAlteStrasse = null;

	/**
	 * Konstruktor. bernimmt die Verbindung zum DaV und fhrt die
	 * Basis-Initialisierung durch.
	 * 
	 * @param verbindung
	 *          {@link ClientDavConnection}
	 */
	public Betriebskilometer(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		einlesenBasisStruktur();
	}

	/**
	 * Testmethode
	 * 
	 * @deprecated Dient <b>nur</b> zum Testen und sollte <b><i>nicht</i></b>
	 *             andersweitig verwendet werden und ist deshalb als
	 *             <code>deprecated</code> markiert.
	 */
	@SuppressWarnings("unused")
	private void TestAuffaedeln()
	{
		_konf = KonfigurationNeu.getInstanz();
		_konf.bestimmeObjekte(_verbindung);

		// String pidStrasse = "strasse.A.3.tmclc.07077";
		String pidStrasse = "strasse.A.45.tmclc.07115";

		_konf.filtereNachStrasse(pidStrasse);

		// Struktur strukturPositiv = new Struktur();
		// Struktur strukturNegativ = new Struktur();

		LinkedList<AeusseresStrassenSegment> _strassenSegmentePositiv = new LinkedList<AeusseresStrassenSegment>();
		LinkedList<AeusseresStrassenSegment> _strassenSegmenteNegativ = new LinkedList<AeusseresStrassenSegment>();
		Iterator<Entry<String, AeusseresStrassenSegment>> it = _konf
				.getAeussereStrassenSegmenteGefiltert().entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, AeusseresStrassenSegment> entry = it.next();
			AeusseresStrassenSegment aeusseresStrassenSegment = entry.getValue();
			if (aeusseresStrassenSegment.getRichtungStrasse().equals("positiv"))
			{
				_strassenSegmentePositiv.add(aeusseresStrassenSegment);
			} else
			{
				_strassenSegmenteNegativ.add(aeusseresStrassenSegment);
			}
		}

		Struktur strukturPositiv = erzeugeStruktur(_strassenSegmentePositiv);
		// _debug.fine("STRUKTUR:\n" + strukturPositiv.toString());
		StrukturElement element = strukturPositiv
				.bestimmeStrukturElementOhneNachfolger();
		LinkedList<StrukturElement> listePos = strukturPositiv
				.bestimmeNachfolger(element);

		Struktur strukturNegativ = erzeugeStruktur(_strassenSegmenteNegativ);
		// _debug.fine("STRUKTUR:\n" + strukturNegativ.toString());
		StrukturElement elementNeg = strukturNegativ
				.bestimmeStrukturElementOhneNachfolger();
		LinkedList<StrukturElement> listeNeg = strukturNegativ
				.bestimmeNachfolger(elementNeg);

		System.out
				.println("===>Strae PID: " + pidStrasse + " (positive Richtung)");

		System.out
				.println("===>Strae PID: " + pidStrasse + " (negative Richtung)");

		// A45
		pruefeBetriebsKilometerVorkommenInListe(listePos, 119702);

		// A45
		pruefeBetriebsKilometerVorkommenInListe(listePos, 119702);

		// A3
		// Pruefung(listePos, 24400); //102
		// Pruefung(listePos, 78461); //-1
		// Pruefung(listePos, 138120); //101
		// Pruefung(listeNeg, 76460); //-1
		// Pruefung(listeNeg, 135547); //101
		// Pruefung(listeNeg, 97973); //-1

	}

	/**
	 * Prft ob ein Betriebskilometer vorkommt und liefert eine Liste von
	 * BlockNummern zurck.
	 * 
	 * @param pidStrasse
	 *          PID der Strae von der geprft werden soll, ob der angegebene
	 *          Betriebskilometer in Ihr vorkommt.
	 * @param betriebsKilometer
	 *          Anzugebender Betriebskilometer, von dem das Vorkommen geprft
	 *          wird.
	 * @return Liste von BlockNummern, innerhalb deren Bereich der
	 *         Betriebskilometer liegt.<br>
	 *         Enthlt die Liste den Wert <code>-1</code>, dann existiert zwar
	 *         ein Bereich in dem der Betriebskilometer liegt, jedoch wird keine
	 *         eindeutige Zuordnung zu einem Block getroffen.<br>
	 *         Ist die Liste leer, liegt der Betriebkilometer <b><u>nicht</u></b>
	 *         im LCL-Segment.
	 */
	public LinkedList<String> pruefeBetriebsKilometerVorkommen(String pidStrasse,
			long betriebsKilometer)
	{
		LinkedList<String> zusammengefuehrt = null;

		if (_konf != null)
		{

			_pidAktuelleStrasse = pidStrasse;

			if (!((_pidAlteStrasse != null) && (_pidAktuelleStrasse
					.equals(_pidAlteStrasse))))
			{
				/*
				 * AlteStrae auf die Pid der aktuell bearbeiteten Strae setzen. Auf
				 * diese Weise wird verhindert, dass eine bereits in die Struktur
				 * berfhrte Strae erneut eingelesen und verarbeitet werden muss.
				 */
				_pidAlteStrasse = pidStrasse;

				_konf.filtereNachStrasse(pidStrasse);

				/*
				 * Durchlaufen aller uerenStraenSegmente, sortieren nach positiver
				 * und negativer Richtung.
				 */
				LinkedList<AeusseresStrassenSegment> _strassenSegmentePositiv = new LinkedList<AeusseresStrassenSegment>();
				LinkedList<AeusseresStrassenSegment> _strassenSegmenteNegativ = new LinkedList<AeusseresStrassenSegment>();
				Iterator<Entry<String, AeusseresStrassenSegment>> it = _konf
						.getAeussereStrassenSegmenteGefiltert().entrySet().iterator();
				while (it.hasNext())
				{
					Entry<String, AeusseresStrassenSegment> entry = it.next();
					AeusseresStrassenSegment aeusseresStrassenSegment = entry.getValue();
					if (aeusseresStrassenSegment.getRichtungStrasse().equals("positiv"))
					{
						_strassenSegmentePositiv.add(aeusseresStrassenSegment);
					} else
					{
						_strassenSegmenteNegativ.add(aeusseresStrassenSegment);
					}
				}

				_strukturPositiv = erzeugeStruktur(_strassenSegmentePositiv);
				StrukturElement element = _strukturPositiv
						.bestimmeStrukturElementOhneNachfolger();
				_listePosAeussereStrassenSegmente = _strukturPositiv
						.bestimmeNachfolger(element);

				_strukturNegativ = erzeugeStruktur(_strassenSegmenteNegativ);
				StrukturElement elementNeg = _strukturNegativ
						.bestimmeStrukturElementOhneNachfolger();
				_listeNegAeussereStrassenSegmente = _strukturNegativ
						.bestimmeNachfolger(elementNeg);

				if (_shellAusgaben)
				{
					System.out
							.println(aufbereitenMitSts(_listePosAeussereStrassenSegmente));
					System.out
							.println(aufbereitenMitSts(_listeNegAeussereStrassenSegmente));
				}

			} else
			{
				_debug.fine(getClass().getName()
						+ "::pruefeBetriebsKilometerVorkommen \n\t=> NICHT EINLESEN");

			}

			/*
			 * Prfung in positiver und negativer Richtung
			 */
			LinkedList<String> pruefeBetriebsKilometerVorkommenInListePos = pruefeBetriebsKilometerVorkommenInListe(
					_listePosAeussereStrassenSegmente, betriebsKilometer);
			LinkedList<String> pruefeBetriebsKilometerVorkommenInListeNeg = pruefeBetriebsKilometerVorkommenInListe(
					_listeNegAeussereStrassenSegmente, betriebsKilometer);

			/*
			 * Zusammenfhren der Ergebnisse in der nachfolgenden Liste
			 */
			zusammengefuehrt = new LinkedList<String>();

			// Hinzufgen BetriebsKilometer in positiver Richtung
			Iterator<String> itPos = pruefeBetriebsKilometerVorkommenInListePos
					.iterator();
			while (itPos.hasNext())
			{
				String stringPos = itPos.next();
				if (!zusammengefuehrt.contains(stringPos))
				{
					zusammengefuehrt.add(stringPos);
				}
			}

			// Hinzufgen Betriebskilometer in negativer Richtung
			Iterator<String> itNeg = pruefeBetriebsKilometerVorkommenInListeNeg
					.iterator();
			while (itNeg.hasNext())
			{
				String stringNeg = itNeg.next();
				if (!zusammengefuehrt.contains(stringNeg))
				{
					zusammengefuehrt.add(stringNeg);
				}
			}

		}

		return zusammengefuehrt;
	}

	/**
	 * Liest die Basisstruktur ein.
	 */
	private void einlesenBasisStruktur()
	{
		if (_verbindung != null)
		{
			_konf = KonfigurationNeu.getInstanz();
			_konf.bestimmeObjekte(_verbindung);
		} else
		{
			_debug
					.error(getClass().getName()
							+ "::einlesenBasisStruktur \n\t=>Es wurde keine Verbindung zum DaV bergeben. Kann die Datenstrukturen nicht aus dem DaV einlesen.");
		}

	}

	/**
	 * Mit Hilfe dieser Methode lt sich erfragen, zu welcher Blocknummer ein
	 * Betriebskilometer innerhalb einer nach Reihenfolge sortierten Liste von
	 * ueren Straensegmenten gehrt.
	 * 
	 * @param strukturElemente
	 *          Liste der {@link StrukturElement}e, innerhalb derer die Prfung
	 *          vorgenommen werden soll.
	 * @param betriebsKilometer
	 *          Betriebskilometer in Metern zu dem bestimmt werden soll, zu
	 *          welcher Blocknummer er zuzuordnen ist.
	 * @return Liste von BlockNummern, in deren Bereich der bergebende
	 *         Betriebskilometer liegt. Es kann vorkommen, dass eine konkrete
	 *         Zuordnung zu einem Block nicht mglich ist, wenn keine BlockNummer
	 *         bei den Betriebskilometern angegeben wurde. In diesem Fall wird
	 *         <code>-1</code> als Wert in der Liste erscheinen. <br>
	 *         Mehrfach vorkommende Blocknummern werden nur <b>1x</b> in der
	 *         Liste aufgenommen.
	 */
	private LinkedList<String> pruefeBetriebsKilometerVorkommenInListe(
			LinkedList<StrukturElement> strukturElemente, long betriebsKilometer)
	{
		/**
		 * Betriebskilometer aufsteigend oder fallend?
		 */
		boolean aufsteigend = true;

		/**
		 * Liste der BlockNummern, fr welche die Betriebskilometerangabe gltig
		 * ist.
		 */
		LinkedList<String> blockNummern = new LinkedList<String>();

		/**
		 * Wert des BetriebsKilometers des aktuellen Blocks
		 */
		long bkWertAktuell = -1;

		/**
		 * Wert des BetriebsKilometers des nachfolgenden Blocks
		 */
		long bkWertNachfolger = -1;

		/*
		 * StraenSegmente
		 */
		for (int i = 0; i < strukturElemente.size(); i++)
		{
			AeusseresStrassenSegment aeusseresStrassenSegment = strukturElemente.get(
					i).getElement();

			Vector<StrassenTeilSegment> strassenTeilSegmente = aeusseresStrassenSegment
					.getStrassenTeilSegmente();

			/*
			 * StraenTeilSegmente
			 */
			if (strassenTeilSegmente.size() > 0)
			{
				for (int j = 0; j < strassenTeilSegmente.size(); j++)
				{
					/*
					 * Zurcksetzen
					 */
					bkWertAktuell = -1;
					bkWertNachfolger = -1;

					/*
					 * Temporre BlockNummer zum Speichern einer erfassten Nummer
					 */
					String tempBlockNummer = "KeineAngabe";

					/*
					 * Bestimmung des Wertes des aktuellen Betriebskilometers
					 */
					LinkedList<BetriebsKilometerEintrag> bkAktuell = strassenTeilSegmente
							.get(j).getBetriebsKilometer();
					if ((bkAktuell != null) && (bkAktuell.size() > 0))
					{
						bkWertAktuell = bkAktuell.get(0).getWert();
						tempBlockNummer = bkAktuell.get(0).getBlockNummer();
					} else
					{
						bkWertAktuell = -1;
					}

					/*
					 * Existiert ein zweiter Betriebskilometereintrag im
					 * StraenTeilSegment, dessen Offset grer 0 ist, so definiert dieser
					 * das Ende.
					 */
					if ((bkAktuell != null) && (bkAktuell.size() > 1))
					{
						if (bkAktuell.get(1).getOffset() > 0)
						{
							bkWertNachfolger = bkAktuell.get(1).getWert();
							if ((bkAktuell.get(1).wurdeBerechnet()))
							{
								_debug
										.fine(getClass().getName()
												+ "::pruefeBetriebsKilometerVorkommenInListe \n\t=> BERECHNETEN GEFUNDEN");
							}
						}
					}

					/*
					 * Suche des Wertes des nachfolgenden Betriebskilometers, wenn nicht
					 * ein zweiter Betriebskilometereintrag im StraenTeilSegment
					 * vorhanden ist.
					 */
					if (bkWertNachfolger < 0)
					{

						if (j + 1 < strassenTeilSegmente.size())
						{
							/*
							 * Innerhalb der StraenTeilSegmente des gleichen StraenSegments
							 */
							LinkedList<BetriebsKilometerEintrag> bkNachfolger = strassenTeilSegmente
									.get(j + 1).getBetriebsKilometer();
							if ((bkNachfolger != null) && (bkNachfolger.size() > 0))
							{
								bkWertNachfolger = bkNachfolger.get(0).getWert();
							}

						} else if (i + 1 < strukturElemente.size())
						{
							/*
							 * Keine weiteren StraenTeilSegmente im gleichen Segment? Dann
							 * wird im ersten StraenTeilSegment des nachfolgenden
							 * StraenSegments gesucht.
							 */

							LinkedList<StrukturElement> nachfolger = strukturElemente.get(
									i + 1).getNachfolger(); // Nachfolgendes
							// StraenSegment
							if ((nachfolger != null) && (nachfolger.size() > 0))
							{
								StrukturElement strukturElement = nachfolger.get(0);
								Vector<StrassenTeilSegment> sst = strukturElement.getElement()
										.getStrassenTeilSegmente();
								if ((sst != null) && (sst.size() > 0))
								{
									/*
									 * Suche im nachfolgenden StraenTeilSegment
									 */
									LinkedList<BetriebsKilometerEintrag> betriebsKilometerNachfolgend = sst
											.get(0).getBetriebsKilometer();
									if ((betriebsKilometerNachfolgend != null)
											&& (betriebsKilometerNachfolgend.size() > 0))
									{
										bkWertNachfolger = betriebsKilometerNachfolgend.get(0)
												.getWert();
									} else
									{
										bkWertNachfolger = -1;
									}

								} else
								{
									bkWertNachfolger = -1;
								}
							}
						} else
						{
							bkWertNachfolger = -1;
						}

						/*
						 * BetriebsKilometerEintrag mit berechntetem Betriebskilometer
						 * generieren und zum StraenTeilSegment hinzufgen.
						 */
						if (bkWertNachfolger > 0)
						{
							StrassenTeilSegment strassenTeilSegment = strassenTeilSegmente
									.get(j);

							BetriebsKilometerEintrag bke = strassenTeilSegment.new BetriebsKilometerEintrag(
									strassenTeilSegment.getLaenge(), bkWertNachfolger,
									tempBlockNummer, true);

							strassenTeilSegment.hinzufuegenBetriebsKilometerEintrag(bke);
						}

					}

					/*
					 * Bestimmung ob AUF- oder ABSTEIGEND
					 */
					if (bkWertNachfolger > -1)
					{
						if (bkWertNachfolger > bkWertAktuell)
						{
							aufsteigend = true;
						} else if (bkWertAktuell > bkWertNachfolger)
						{
							aufsteigend = false;
						}
					}

					/*
					 * Wenn Aufsteigend, dann wird die Lnge des StraenTeilSegments zum
					 * Wert des Betriebskilometers hinzuaddiert. Der zu prfende
					 * Betriebskilometer muss zwischen Betriebskilometerwert und
					 * Betriebskilometerwert + Lnge StraenTeilSegment liegen.
					 */
					if (aufsteigend)
					{
						if (betriebsKilometer >= bkWertAktuell)
						{
							if (betriebsKilometer <= bkWertAktuell
									+ strassenTeilSegmente.get(j).getLaenge())
							{
								if (!blockNummern.contains(tempBlockNummer))
								{
									blockNummern.add(tempBlockNummer);
									_debug.fine(":::=>PASST AUFSTEIGEND");

								}
							}
						}
					}

					/*
					 * Wenn Absteigend, dann wird die Lnge des StraenTeilSegments vom
					 * Wert des Betriebskilometers abgezogen. Der zu prfende
					 * Betriebskilometer muss zwischen Betriebskilometerwert und
					 * Betriebskilometerwert - Lnge StraenTeilSegment liefen.
					 */
					if (!aufsteigend)
					{
						if (betriebsKilometer <= bkWertAktuell)
						{
							if (betriebsKilometer >= bkWertAktuell
									- strassenTeilSegmente.get(j).getLaenge())
							{
								if (!blockNummern.contains(tempBlockNummer))
								{
									blockNummern.add(tempBlockNummer);
									_debug.fine(":::=>PASST FALLEND");

								}
							}
						}
					}

					String ausg = "bkWertAktuell: " + bkWertAktuell
							+ " bkWertNachfolger: " + bkWertNachfolger + "\n";
					if (aufsteigend)
					{
						ausg = ausg + "Richtung: Aufsteigend" + "\n";
					} else
					{
						ausg = ausg + "Richtung: Fallend" + "\n";
					}
					ausg = ausg + "Lnge StraenTeilSegment: "
							+ strassenTeilSegmente.get(j).getLaenge() + " m.\n";
					_debug.fine("=>\n" + ausg + "\n");

				}
			}

		}

		/*
		 * Blocknummern ausgeben
		 */
		Iterator<String> it = blockNummern.iterator();
		while (it.hasNext())
		{
			String blockNummer = it.next();
			_debug.fine(getClass().getName()
					+ "::pruefeBetriebsKilometerVorkommenInListe \n\t=> BLOCKNUMMER: "
					+ blockNummer);

		}

		return blockNummern;

	}

	/**
	 * Erzeugt eine aufbereitete Ausgabe mit den Elementen:<br>
	 * <ul>
	 * <li>ueres Straensegment</li>
	 * <li>Zugehrige StraenTeilSegmente</li>
	 * <li>Betriebskilometern</li>
	 * </ul>
	 * 
	 * @param strukturElemente
	 *          Liste von {@link StrukturElement}en, fr welche die Ausgabe
	 *          aufbereitet werden soll.
	 * @return String zur Ausgabe
	 */
	private String aufbereitenMitSts(LinkedList<StrukturElement> strukturElemente)
	{
		String re = "";

		Iterator<StrukturElement> it = strukturElemente.iterator();
		while (it.hasNext())
		{
			StrukturElement element = it.next();
			AeusseresStrassenSegment segment = element.getElement();
			Iterator<StrassenTeilSegment> itSST = segment.getStrassenTeilSegmente()
					.iterator();

			re = re + "ueresStraenSegment PID: " + segment.getPid() + "\n";

			while (itSST.hasNext())
			{
				StrassenTeilSegment strassenTeilSegment = itSST.next();
				re = re + "\t" + strassenTeilSegment.toString();
			}
			re = re + "\n";
		}

		return re;
	}

	/**
	 * Erzeugt aus einer Liste von {@link AeusseresStrassenSegment} eine
	 * {@link Struktur}.<br>
	 * In dieser Struktur knnen Vorgnger und Nachfolger leicht gefunden werden.
	 * 
	 * @param strassenSegmente
	 * @return
	 */
	private Struktur erzeugeStruktur(
			LinkedList<AeusseresStrassenSegment> strassenSegmente)
	{
		Struktur struktur = new Struktur();

		Iterator<AeusseresStrassenSegment> it = strassenSegmente.iterator();
		while (it.hasNext())
		{
			AeusseresStrassenSegment aeusseresStrassenSegment = it.next();

			// _debug.fine("AES:" + aeusseresStrassenSegment.getPid());
			AeusseresStrassenSegment vorgaenger = bestimmeVorgaenger(
					aeusseresStrassenSegment, aeusseresStrassenSegment.getTmcRichtung());
			AeusseresStrassenSegment nachfolger = bestimmeNachfolger(
					aeusseresStrassenSegment, aeusseresStrassenSegment.getTmcRichtung());

			StrukturElement strukturElement = struktur
					.erzeugeStrukturElement(aeusseresStrassenSegment);

			if (nachfolger != null)
			{
				StrukturElement strukturNachfolger = struktur
						.erzeugeStrukturElement(nachfolger);
				strukturElement.addNachfolger(strukturNachfolger);
				strukturNachfolger.addVorgaenger(strukturElement);
			}

			if (vorgaenger != null)
			{
				StrukturElement strukturVorgaenger = struktur
						.erzeugeStrukturElement(vorgaenger);
				strukturElement.addVorgaenger(strukturVorgaenger);
				strukturVorgaenger.addNachfolger(strukturElement);
			}

		}

		_debug.fine(getClass().getName() + "::erzeugeStruktur \n\t=> Gre: "
				+ struktur.size());

		return struktur;
	}

	/**
	 * Bestimmt den Vorgnger eines {@link AeusseresStrassenSegment}
	 * 
	 * @param ass
	 *          {@link AeusseresStrassenSegment}, dessen Vorgnger
	 *          {@link AeusseresStrassenSegment} bestimmt werden soll.
	 * @param richtung
	 *          Richtung (<code>positiv</code> oder <code>negativ</code>)
	 *          der {@link AeusseresStrassenSegment}e.
	 * @return Vorgnger des {@link AeusseresStrassenSegment}es.
	 */
	private AeusseresStrassenSegment bestimmeVorgaenger(
			AeusseresStrassenSegment ass, String richtung)
	{
		String vorg = "\t=>Vorgnger: ";
		String vorgAES = "";
		AeusseresStrassenSegment re = null;

		Iterator<StrassenSegment> itSSVorgaenger = ass.getVorgaengerAufStrae()
				.iterator();

		while (itSSVorgaenger.hasNext())
		{
			StrassenSegment issVorgaenger = itSSVorgaenger.next();
			vorg = vorg + " PID: " + issVorgaenger.getPid() + " ANZ: "
					+ issVorgaenger.getVorgaengerAufStrae().size();

			Iterator<StrassenSegment> itAesVorg = issVorgaenger
					.getVorgaengerAufStrae().iterator();
			while (itAesVorg.hasNext())
			{
				StrassenSegment aesVorg = itAesVorg.next();
				if (aesVorg instanceof AeusseresStrassenSegment)
				{
					AeusseresStrassenSegment vorgaengerAss = (AeusseresStrassenSegment) aesVorg;
					if (vorgaengerAss.getTmcRichtung().equals(richtung))
					{
						// _debug.fine("POS: " + vorgaengerAss.getPid());
						vorgAES = vorgAES + "POS: " + vorgaengerAss.getPid() + "\n";
						re = vorgaengerAss;
					}
				}
			}
		}
		// _debug.fine(vorg + "\n" + vorgAES);
		return re;
	}

	/**
	 * Bestimmt den Nachfolger eines ueren StraenSegments.
	 * 
	 * @param ass
	 *          {@link AeusseresStrassenSegment}, dessen Nachfolger bestimmt
	 *          werden soll.
	 * @param richtung
	 *          Richtung des {@link AeusseresStrassenSegment}, in der der
	 *          Nachfolger bestimmt werden soll (sollte <code>positiv</code>
	 *          oder <code>negativ</code> sein).
	 * @return Nachfolger des {@link AeusseresStrassenSegment}es.
	 */
	private AeusseresStrassenSegment bestimmeNachfolger(
			AeusseresStrassenSegment ass, String richtung)
	{
		AeusseresStrassenSegment re = null;

		Iterator<StrassenSegment> itISSVorgaenger = ass.getNachfolgerAufStrasse()
				.iterator();
		while (itISSVorgaenger.hasNext())
		{
			StrassenSegment strassenSegment = itISSVorgaenger.next();
			Iterator<StrassenSegment> itAESVorgaenger = strassenSegment
					.getNachfolgerAufStrasse().iterator();
			while (itAESVorgaenger.hasNext())
			{
				StrassenSegment aus = itAESVorgaenger.next();
				if (aus instanceof AeusseresStrassenSegment)
				{
					AeusseresStrassenSegment aeusseresStrassenSegmentNachfolger = (AeusseresStrassenSegment) aus;
					if (aeusseresStrassenSegmentNachfolger.getTmcRichtung().equals(
							richtung))
					{
						re = aeusseresStrassenSegmentNachfolger;
						// _debug.fine("PID NACHFOLGER: " + re.getPid());
					}
				}
			}
		}

		return re;
	}

	/**
	 * Liefert die Elemente einer Struktur der {@link AeusseresStrassenSegment}e
	 * einer Strae. <br>
	 * Die {@link AeusseresStrassenSegment}e dieser Strae knnen sortiert in
	 * Reihenfolge ausgelesen werden, sofern ein Zusammenhang besteht.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Struktur
	{
		/**
		 * Liste der Elemente dieser Struktur
		 */
		private HashMap<String, StrukturElement> elemente = new HashMap<String, StrukturElement>();

		private int zaehler = 0;

		/**
		 * Konstruktor
		 */
		public Struktur()
		{
			_debug.fine(getClass().getName() + "::Struktur \n\t=> Struktur erzeugt.");

		}

		/**
		 * Erzeugt ein neues {@link StrukturElement} und liefert dieses zurck. Ist
		 * bereits eiens mit dem gleichen {@link AeusseresStrassenSegment}
		 * vorhanden, so wird dieses zurckgeliefert.
		 * 
		 * @param aeusseresStrassenSegment
		 * @return
		 */
		public StrukturElement erzeugeStrukturElement(
				AeusseresStrassenSegment aeusseresStrassenSegment)
		{
			/*
			 * StrukturElement besorgen
			 */
			StrukturElement strukturElement = null;
			if (containsKey(aeusseresStrassenSegment.getPid()))
			{
				strukturElement = this.getStrukturElement(aeusseresStrassenSegment
						.getPid());
			} else
			{
				strukturElement = new StrukturElement(aeusseresStrassenSegment);
				this.addStrukturElement(strukturElement);
			}

			return strukturElement;
		}

		/**
		 * Fgt ein {@link StrukturElement} in die {@link HashMap} ein, sofern
		 * dieses noch nicht enthalten ist.
		 * 
		 * @param element
		 *          Hinzuzufgendes {@link StrukturElement}
		 */
		public void addStrukturElement(StrukturElement element)
		{
			if (!elemente.containsKey(element.getPid()))
			{
				elemente.put(element.getPid(), element);
			}
		}

		/**
		 * Liefert ein {@link StrukturElement} an Hand ihres Keys aus der
		 * {@link HashMap} zurck.
		 * 
		 * @param pid
		 *          Key des StrukturElements, entsprechend der PID des
		 *          {@link AeusseresStrassenSegment}.
		 * @return {@link StrukturElement}, zugehrig zum Key.
		 */
		public StrukturElement getStrukturElement(String pid)
		{
			if (elemente.containsKey(pid))
			{
				return elemente.get(pid);
			} else
			{
				return null;
			}
		}

		/**
		 * Prft, ob ein {@link StrukturElement} mit einem bestimmten Key in der
		 * {@link HashMap} der {@link StrukturElement}e vorhanden ist.
		 * 
		 * @param key
		 *          Schlssel des auf Existenz zu prfenden {@link StrukturElement}s.
		 * @return Liefert <code>true</code>, wenn das Element in der
		 *         {@link HashMap} vorhanden ist, sonst <code>false</code>.
		 */
		public boolean containsKey(String key)
		{
			return elemente.containsKey(key);
		}

		/**
		 * Prft, ob die {@link HashMap} ein bestimmtes Objekt enthlt.
		 * 
		 * @param object
		 *          Objekt, auf dessen Existenz geprft werden soll.
		 * @return Liefert <code>true</code>, wenn das Objekt in der
		 *         {@link HashMap} vorhanden ist, sonst <code>false</code>.
		 */
		@SuppressWarnings("unused")
		public boolean containsValue(Object object)
		{
			boolean re = false;
			Iterator<Entry<String, StrukturElement>> it = elemente.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, StrukturElement> entry = it.next();
				if (entry.getValue().getElement().equals(object))
				{
					re = true;
				}
			}

			return re;
		}

		/**
		 * Liefert die Gre der Struktur.
		 * 
		 * @return Gre der Struktur
		 */
		public int size()
		{
			return elemente.size();
		}

		/**
		 * Bestimmt das {@link StrukturElement}, welches keinen Nachfolger hat.
		 * 
		 * @return {@link StrukturElement} ohne Nachfolger, sonst <code>null</code>.
		 */
		public StrukturElement bestimmeStrukturElementOhneNachfolger()
		{
			StrukturElement element = null;

			Iterator<Entry<String, StrukturElement>> it = elemente.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, StrukturElement> entry = it.next();
				int size = entry.getValue().getVorgaenger().size();
				if (size == 0)
				{
					element = entry.getValue();
					break;
				}
			}

			_debug
					.fine(getClass().getName()
							+ "::bestimmeStrukturElementOhneNachfolger \n\t=>Element ohne Vorgaenger: "
							+ element.getPid());

			return element;
		}

		/**
		 * Diese Funktion liefert eine geordnete Liste der
		 * {@link AeusseresStrassenSegment}e.
		 * 
		 * @param element
		 *          {@link StrukturElement}, an Hand dessen Nachfolgern (und damit
		 *          auch Vorgngern) die {@link AeusseresStrassenSegment} in
		 *          Reihenfolge bestimmt werden sollen.
		 * @return Liste mit in Reihenfolgen gebrachten {@link StrukturElement}en.
		 */
		public LinkedList<StrukturElement> bestimmeNachfolger(
				StrukturElement element)
		{
			_debug
					.fine(getClass().getName()
							+ "::bestimmeNachfolger \n\tNachfolger werden bestimmt, siehe nachfolgende Liste.");

			LinkedList<StrukturElement> liste = new LinkedList<StrukturElement>();

			LinkedList<StrukturElement> nach = bestimmeNachfolgerRekursiv(element);
			if (nach != null)
			{
				liste.addAll(nach);
			}

			return liste;
		}

		/**
		 * Bestimmt <i>rekursiv</i> die Nachfolger eines Elementes <br>
		 * <b>Nicht fr ffentlichen Gebrauch bestimmt.</b>
		 * 
		 * @param element
		 * @return
		 */
		private LinkedList<StrukturElement> bestimmeNachfolgerRekursiv(
				StrukturElement element)
		{
			LinkedList<StrukturElement> liste = new LinkedList<StrukturElement>();
			liste.add(element);

			_debug
					.fine("\t" + zaehler + " Element in Nachfolge: " + element.getPid());
			zaehler++;

			if (element.getNachfolger().size() > 0)
			{
				LinkedList<StrukturElement> nach = bestimmeNachfolgerRekursiv(element
						.getNachfolger().get(0));
				if (nach != null)
				{
					liste.addAll(nach);
				}
			}

			return liste;
		}

		public String toString()
		{
			String re = "";
			Iterator<Entry<String, StrukturElement>> it = elemente.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, StrukturElement> entry = it.next();
				re = re + "PID: " + entry.getKey() + " Anz. Nachfolger: "
						+ entry.getValue().getNachfolger().size() + " Anz. Vorgaenger: "
						+ entry.getValue().getVorgaenger().size() + "\n";

			}

			return re;
		}

	}

	/**
	 * Reprsentiert ein Element in der Struktur.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class StrukturElement
	{
		private AeusseresStrassenSegment _element = null;

		private LinkedList<StrukturElement> _vorgaenger = new LinkedList<StrukturElement>();

		private LinkedList<StrukturElement> _nachfolger = new LinkedList<StrukturElement>();

		/**
		 * Konstruktor, bernimmt ein {@link AeusseresStrassenSegment} als Element
		 * der Klasse {@link StrukturElement}.
		 * 
		 * @param element
		 *          {@link AeusseresStrassenSegment}
		 */
		public StrukturElement(AeusseresStrassenSegment element)
		{
			this._element = element;
		}

		/**
		 * Liefert das {@link AeusseresStrassenSegment} als Element dieses
		 * {@link StrukturElement}s zurck.
		 * 
		 * @return {@link AeusseresStrassenSegment}
		 */
		public AeusseresStrassenSegment getElement()
		{
			return _element;
		}

		/**
		 * Liefert die Pid des {@link AeusseresStrassenSegment}.
		 * 
		 * @return Pid
		 */
		public String getPid()
		{
			return _element.getPid();
		}

		/**
		 * Fgt einen Vorgnger hinzu, falls dieser noch nicht i.d. Liste der
		 * Vorgnger vorhanden ist.
		 * 
		 * @param vorgaenger
		 *          Vorgnger
		 */
		public void addVorgaenger(StrukturElement vorgaenger)
		{
			if (!_vorgaenger.contains(vorgaenger))
			{
				_vorgaenger.add(vorgaenger);
			}
		}

		/**
		 * Liefert die Liste der Vorgnger.
		 * 
		 * @return Liste der Vorgnger
		 */
		public LinkedList<StrukturElement> getVorgaenger()
		{
			return _vorgaenger;
		}

		/**
		 * Fgt einen Nachfolger vom Typ {@link StrukturElement} zur Liste der
		 * Nachfolger hinzu.
		 * 
		 * @param nachfolger
		 *          {@link StrukturElement} als Nachfolger
		 */
		public void addNachfolger(StrukturElement nachfolger)
		{
			if (!_nachfolger.contains(nachfolger))
			{
				_nachfolger.add(nachfolger);
			}
		}

		/**
		 * Liefert die Liste der Nachfolger.
		 * 
		 * @return Liste der Nachfolger.
		 */
		public LinkedList<StrukturElement> getNachfolger()
		{
			return _nachfolger;
		}

	}

	/**
	 * Liefert eine dem Namen nach sortierte Liste der {@link Strasse}n zurck.
	 * 
	 * @return Sortierte Liste der {@link Strasse}n.
	 */
	public LinkedList<Strasse> getStrassen()
	{

		HashMap<String, Strasse> strassen = _konf.getStrassen();

		TreeMap<String, Strasse> strassenSortiert = new TreeMap<String, Strasse>();

		strassenSortiert.putAll(strassen);

		LinkedList<Strasse> listeStrassen = new LinkedList<Strasse>();
		Iterator<Entry<String, Strasse>> it = strassenSortiert.entrySet()
				.iterator();
		while (it.hasNext())
		{
			Entry<String, Strasse> entry = it.next();

			if (!entry.getValue().getName().equals("null"))
			{
				listeStrassen.add(entry.getValue());
			}

		}

		return listeStrassen;
	}

}
