/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;

/**
 * "Editor" fr regulre Ausdrcke. Liefert ein "groes" Textfeld, in dem der
 * regulre Ausdruck bearbeitet werden kann. <br>
 * 
 * Die Darstellung besteht aus:
 * <ul>
 * <li>Einem Textfeld, in dem der regulre Ausdruck bearbeitet werden kann.</li>
 * <li>Einem OK-Button, der die nderungen am regulren Ausdruck bernimmt.</li>
 * <li>Einem Abbrechen-Button, der die nderungen am regulren Ausruck verwirft.
 * </li>
 * </ul>
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:12:25 $ / ($Author: HofmannJ
 *          $)
 * @author HofmannJ
 * 
 */
public class EditorRegulaererAusdruck extends Dialog
{
	/**
	 * Speichert den regulren Ausdruck.
	 */
	private String _regAusdruck = null;

	private Shell _shell = null; // @jve:decl-index=0:visual-constraint="204,120"

	private Composite _compositeTop = null;

	private Composite _compositeBottom = null;

	private Button _schaltflaecheOk = null;

	private Button _schaltflaecheAbbrechen = null;

	private Button _schaltflaecheHilfe = null;

	private Group _gruppeBearbeiten = null;

	private Text _textEingabe = null;

	private boolean _modified = false;

	/**
	 * Konstruktor, liefert eine neue Instanz der Klasse
	 * {@link EditorRegulaererAusdruck}.
	 * 
	 * @param parent
	 *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
	 */
	public EditorRegulaererAusdruck(Shell parent)
	{
		super(parent);
	}

	/**
	 * ffnet den Dialog {@link EditorRegulaererAusdruck} zum Editieren/Anlegen
	 * eines neuen regulren Ausdrucks.
	 * 
	 * @return Regulrer Ausdruck als String.
	 */
	public String oeffne()
	{
		// shell = new Shell();
		_shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN
				| SWT.MAX | SWT.CLOSE);
		_shell.setText("Editor regulrer Ausdruck");
		_shell.setImage(Activator.getImageDescriptor(
				"icons/sonstiges/text_editor_16x16.png").createImage());

		_shell.setLayout(new GridLayout());
		erzeugeBereichOben();
		erzeugeBereichUnten();
		_shell.setSize(new Point(436, 259));

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		_shell.open();

		Display display = getParent().getDisplay();

		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return _regAusdruck;

	}

	/**
	 * ffnet den Dialog {@link EditorRegulaererAusdruck} zum Editieren eines
	 * bereits vorhandenen regulren Ausdrucks.
	 * 
	 * @param regAusdruck
	 *          Regulrer Ausdruck, der bearbeitet werden soll.
	 * 
	 * @return Liefert den bearbeiteten, regulren Ausduck. Der Dialog
	 *         {@link FilterDefinitionsdialog} kann die Darstellung des regulren
	 *         Ausdrucks des jeweiligen Filterelements hiermit aktuallisieren.
	 * 
	 */
	public String oeffne(String regAusdruck)
	{

		_regAusdruck = regAusdruck;

		// shell = new Shell();
		_shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN
				| SWT.MAX | SWT.CLOSE);
		_shell.setText("Editor regulrer Ausdruck");
		_shell.setImage(Activator.getImageDescriptor(
				"icons/sonstiges/text_editor_16x16.png").createImage());

		_shell.setLayout(new GridLayout());
		erzeugeBereichOben();
		erzeugeBereichUnten();
		_shell.setSize(new Point(436, 259));

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		_shell.open();
		_textEingabe.setText(_regAusdruck);

		Display display = getParent().getDisplay();

		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return _regAusdruck;
	}

	/**
	 * bernimmt die nderungen am regulren Ausdruck und schliet den Dialog.
	 * 
	 */
	private void aktionOk()
	{
		if (_modified)
		{
			_regAusdruck = _textEingabe.getText();
			_modified = false;
		}
		_shell.dispose();
	}

	/**
	 * Verwirft die nderungen am regulren Ausdruck und schliet den Dialog.
	 * 
	 */
	private void aktionAbbrechen()
	{
		_regAusdruck = null;
		_shell.dispose();
	}

	/**
	 * ffnet die Hilfe zu diesem Editor.
	 */
	private void aktionHilfe()
	{
		String helpContextId = Activator.PLUGIN_ID + ".filterDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * This method initializes compositeTop
	 * 
	 */
	private void erzeugeBereichOben()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_compositeTop = new Composite(_shell, SWT.NONE);
		_compositeTop.setLayout(new GridLayout());
		erzeugeGruppe();
		_compositeTop.setLayoutData(gridData);
	}

	/**
	 * This method initializes compositeBottom
	 * 
	 */
	private void erzeugeBereichUnten()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.wrap = true;
		rowLayout.pack = false;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.CENTER;
		gridData1.grabExcessHorizontalSpace = false;
		gridData1.verticalAlignment = GridData.CENTER;
		_compositeBottom = new Composite(_shell, SWT.NONE);
		_compositeBottom.setLayoutData(gridData1);
		_compositeBottom.setLayout(rowLayout);
		_schaltflaecheOk = new Button(_compositeBottom, SWT.NONE);
		_schaltflaecheOk.setText("Ok");
		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}
		});

		_schaltflaecheAbbrechen = new Button(_compositeBottom, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}
		});

		_schaltflaecheHilfe = new Button(_compositeBottom, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}
		});
	}

	/**
	 * This method initializes group
	 * 
	 */
	private void erzeugeGruppe()
	{
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		_gruppeBearbeiten = new Group(_compositeTop, SWT.NONE);
		_gruppeBearbeiten.setLayout(new GridLayout());
		_gruppeBearbeiten.setText("Regulrer Ausdruck");
		_gruppeBearbeiten.setLayoutData(gridData2);
		_textEingabe = new Text(_gruppeBearbeiten, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		_textEingabe.setLayoutData(gridData3);
		_textEingabe.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				_modified = true;

			}
		});

	}

}
