/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.ui.internal;

import org.eclipse.core.runtime.Platform;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;

// import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenBezeichner;

/**
 * This class controls all aspects of the application's execution
 */
public class Application implements IApplication
{

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.equinox.app.IApplication#start(org.eclipse.equinox.app.IApplicationContext)
	 */
	public Object start(IApplicationContext context)
	{
		Display display = PlatformUI.createDisplay();

		/*
		 * Zugriff auf den PreferenceStore der Platform
		 */
		IPreferenceStore _praeferenzen = PlatformUI.getPreferenceStore();

		Location installLocation = Platform.getInstallLocation();


		/*
		 * Installationspfad des Rahmenwerkes ermitteln
		 */
		String path = installLocation.getURL().getPath();

		/*
		 * Verzeichnis fr Voreinstellungen setzen
		 */
		String einstellungenPfad = path + "einstellungen" + "/";

//		System.out.println("Pfad zu den Einstellungen: " + einstellungenPfad);

		_praeferenzen.setDefault("PfadEinstellungen", einstellungenPfad);


		String[] kommandozeilenArgumente = Platform.getCommandLineArgs();

		boolean online = true;

		/*
		 * Suchen des bentigten Parameters
		 */
		for (int i = 0; i < kommandozeilenArgumente.length; i++)
		{
			if (kommandozeilenArgumente[i].equalsIgnoreCase("-betriebsart"))
			{
				i++;
				if (kommandozeilenArgumente[i].equalsIgnoreCase("online"))
				{
					online = true;
				} else
				{
					online = false;
				}
			} else
			{
				online = true;
			}
		}

		if (online)
		{
			ApplicationActionBarAdvisor._aktionAnmelden.run();
		} else
		{
			ApplicationActionBarAdvisor._aktionKonfigurationsDateiAuswaehlen.run();
		}

		// if (betriebsArt.equals(BetriebsArt.online)) {
		// /*
		// * Anmelden via Login-Dialog
		// */
		// ApplicationActionBarAdvisor._aktionAnmelden.run();
		//
		// } else if (betriebsArt.equals(BetriebsArt.offline)) {
		// /*
		// * Konfigurationsdatei auswhlen
		// */
		// ApplicationActionBarAdvisor._aktionKonfigurationsDateiAuswaehlen
		// .run();
		//
		// /*
		// * Deaktivieren des Anmeldens am DaV.
		// */
		// ApplicationActionBarAdvisor._aktionAnmelden.setEnabled(false);
		//
		// }

		Kommandozeilenparameter kzp = Kommandozeilenparameter.getInstanz();
		KmdParameter kp = kzp.getKommandoZeilenParameter("-berechtigungsKonzept");
		if (kp != null)
		{
			if (kp.getWert().equals("true"))
			{
				@SuppressWarnings("unused")
				OberflaechenBerechtigung instanz = OberflaechenBerechtigung
						.getInstanz();
			}
		}

		try
		{
			int returnCode = PlatformUI.createAndRunWorkbench(display,
					new ApplicationWorkbenchAdvisor());

			if (returnCode == PlatformUI.RETURN_RESTART)
			{
				return IApplication.EXIT_RESTART;
			}
			return IApplication.EXIT_OK;
		} finally
		{
			display.dispose();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.equinox.app.IApplication#stop()
	 */
	public void stop()
	{
		final IWorkbench workbench = PlatformUI.getWorkbench();
		if (workbench == null)
			return;
		final Display display = workbench.getDisplay();
		display.syncExec(new Runnable()
		{
			public void run()
			{
				if (!display.isDisposed())
					workbench.close();
			}
		});
	}
}
