/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.ui.internal;

import org.eclipse.ui.application.IWorkbenchConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchAdvisor;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.ui.internal.ende.BeendenDlg;
import de.bsvrz.buv.rw.rw.ui.internal.ende.Ende;

public class ApplicationWorkbenchAdvisor extends WorkbenchAdvisor
{

	@SuppressWarnings("unused")
	private BeendenDlg _dlg = null;

	public WorkbenchWindowAdvisor createWorkbenchWindowAdvisor(
			IWorkbenchWindowConfigurer configurer)
	{

		// BetriebsArt betriebsArt = BetriebsArtAuswerten.getInstanz()
		// .getBetriebsArt();
		// if (betriebsArt.equals(BetriebsArt.online)) {
		// /*
		// * Anmelden via Login-Dialog
		// */
		// ApplicationActionBarAdvisor._aktionAnmelden.run();
		//
		// } else if (betriebsArt.equals(BetriebsArt.offline)) {
		// /*
		// * Konfigurationsdatei auswhlen
		// */
		// ApplicationActionBarAdvisor._aktionKonfigurationsDateiAuswaehlen
		// .run();
		//
		// /*
		// * Deaktivieren des Anmeldens am DaV.
		// */
		// ApplicationActionBarAdvisor._aktionAnmelden.setEnabled(false);
		//
		// }

		return new ApplicationWorkbenchWindowAdvisor(configurer);
	}

	/**
	 * Initialisiert das Speichern der dargestellten Fenster beim beenden der
	 * Umgebung
	 */
	public void initialize(IWorkbenchConfigurer configurer)
	{
		super.initialize(configurer);
		configurer.setSaveAndRestore(true);
	}

	public String getInitialWindowPerspectiveId()
	{
		return Perspective.class.getName();
	}

	public boolean preShutdown()
	{
		// System.out.println("PRE-SHUTDOWN");

		_dlg = new BeendenDlg();
		@SuppressWarnings("unused")
		Ende instanz = Ende.getInstanz();

		if (System.getProperty("os.name").equals("Linux"))
		{
			Warten warten = new Warten();
			warten.warte(5000);
		}

		return true;
	}

	public void postShutdown()
	{
		// System.out.println("POST-SHUTDOWN");
		/*
		 * DaV-Verbindung trennen
		 */
		DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
		if (daVVerbindung.istDaVVerbindungAktiv())
		{
			daVVerbindung.trenneDaVVerbindung();
		}
	}

	/**
	 * Thread-Klasse, liefert einen Befehl <code>warten</code>, mit dessen
	 * Hilfe das Beenden pausiert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Warten implements Runnable
	{
		private boolean _laufeThread = false;
		private Long _warteZeit = new Long(1000);

		public Warten()
		{
		}

		/**
		 * Veranlasst das Programm dazu, eine bestimmte Zeit lang zu warten.
		 * 
		 * @param warteZeit
		 *          Wartezeit in Millisekunden.
		 */
		public void warte(long warteZeit)
		{
			synchronized (_warteZeit)
			{
				_warteZeit = warteZeit;
			}

			startThread();
			synchronized (this)
			{
				try
				{
					wait();
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

		private void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Warten...");
				_laufeThread = true;
				th.start();
			}
		}

		@Override
		public void run()
		{
			if (_laufeThread)
			{
				try
				{
					Thread.sleep(_warteZeit);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				finally
				{
					synchronized (this)
					{
						notifyAll();
					}
				}
			}

			_laufeThread = false;
		}

	}

}
