/*
 * This file is part of Rahmenwerk.
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * 
 * Copyright (c) 2012 Swarco Traffic Systems GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * Swarco Traffic Systems GmbH - initial API and implementation
 * 
 * 
 * 
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau 
 * T. +49 (0)7225 / 64-03
 * F. +49 (0)7225 / 64-300 
 * E. office.sts@swarco.de
 */
package de.bsvrz.buv.rw.rw.ui.internal.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.ui.internal.Activator;

/**
 * Realisiert eine {@link Action} zum Abmelden vom aktuellen DatenVerteiler. Die
 * Abmeldung erfolgt erst nach einer besttigten Sicherheitsabfrage.
 * 
 * @author HofmannJ
 * 
 */
public class AktionAbmelden extends Action implements IDaVVerbindungZuhoerer
{

	/**
	 * Konstruktor. Login-Dialog wird mit statischem Layout aufgerufen.
	 */
	public AktionAbmelden()
	{
		initialisiere();
	}

	/**
	 * Fhrt die Initialisierung der Action durch.
	 */
	private void initialisiere()
	{
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(this);
		setText("Abmelden");
		setEnabled(true);
		this.setId(Activator.PLUGIN_ID + ".aktionAbmelden");
		this.setText("Abmelden");
		this.setToolTipText("Ermglicht die Abmeldung vom Datenverteiler..");

		validiereVerbindung(DaVVerbindung.getInstanz().istDaVVerbindungAktiv());
	}

	@Override
	public void run()
	{
		boolean antwortDaVVerbindungSchliessen = MessageDialog.openQuestion(
				PlatformUI.getWorkbench().getDisplay().getActiveShell(),
				"Verbindung zum DaV schlieen?",
				"Mchten Sie wirklich die aktive Verbindung zum DaV \""
						+ BenutzerAngemeldet.getInstanz().getIp() + "\" schlieen?");

		if (antwortDaVVerbindungSchliessen)
		{
			if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
			{
				DaVVerbindung.getInstanz().trenneDaVVerbindung();
			}
		}

	}

	@Override
	public void verbindungAbgebaut(DaVVerbindungEreignis e)
	{
		validiereVerbindung(false);
	}

	@Override
	public void verbindungAufgebaut(DaVVerbindungEreignis e)
	{
		validiereVerbindung(true);
	}

	/**
	 * Prft ob die Verbindung zum DaV aktiv. Abhngig hiervon wird diese Aktion
	 * aktiviert bzw. deaktiviert.
	 * 
	 * @param davVerbindungAktiv
	 *          Flag das anzeigt, ob die Verbindung zum DaV aktiv ist (
	 *          <code>true</code>) oder nicht (<code>false</code>).
	 */
	private void validiereVerbindung(boolean davVerbindungAktiv)
	{
		this.setEnabled(davVerbindungAktiv);
	}

}
