/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.ui.internal.ende;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;

import de.bsvrz.buv.rw.rw.ui.internal.ApplicationWorkbenchAdvisor;
import de.bsvrz.buv.rw.rw.ui.internal.ZyklischesSpeichern.ZyklischesSpeichern;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Fhrt Aktionen zur Beendigung des Rahmenwerkes durch, u. a. <br>
 * <ul>
 * <li>Beenden des Threads zum zyklischen Speichern</li>
 * <li>Senden des Speichern-Signals</li>
 * </ul>
 * 
 * Erfllt BuV-7
 * 
 * @author HofmannJ
 * 
 */
public class Ende
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Privater Konstruktor. Das Beenden der DaV-Verbindung erfolgt ab sofort in
	 * der Methode {@link ApplicationWorkbenchAdvisor#postShutdown()}.
	 */
	private Ende()
	{
		/*
		 * Signal zum zyklischen Speichern deaktivieren
		 */
		ZyklischesSpeichern.getInstanz().stopThread();

		/*
		 * Signal zum Speichern senden
		 */
		try
		{
			ResourcesPlugin.getWorkspace().save(true, null);
		}
		catch (CoreException e)
		{
			_debug.error(getClass().getName()
					+ "::Ende Fehler beim Senden des Speichersignals\n\t=> "
					+ e.toString(), e);
		}

		// /*
		// * DaV-Verbindung trennen
		// */
		// DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
		// if (daVVerbindung.istDaVVerbindungAktiv())
		// {
		// daVVerbindung.trenneDaVVerbindung();
		// }

	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link Ende}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen
	{
		private static Ende INSTANZ = new Ende();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link Ende}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link Ende}
	 */
	public static Ende getInstanz()
	{
		return Innen.INSTANZ;
	}
}
