package de.bsvrz.buv.rw.rw.dav;

/**
 * Speichert den Benutzer, der sich im Offlinemodus angemeldet hat.
 * 
 * @author HofmannJ
 * 
 */
public class BenutzerOffline
{
	private static BenutzerOffline _instance = null;

	/**
	 * Benutzername
	 */
	private String _name = null;

	/**
	 * Passwort
	 */
	private String _passwort = null;

	/**
	 * Adresse des jeweiligen DaVs
	 */
	private String _ip = null;

	/**
	 * Port
	 */
	private int _tcpPort;

	private BenutzerOffline()
	{
	}

	public static BenutzerOffline getInstance()
	{
		if (_instance == null)
		{
			synchronized (BenutzerOffline.class)
			{
				if (_instance == null)
				{
					_instance = new BenutzerOffline();
				}
			}
		}

		return _instance;
	}

	/**
	 * Setzt den offline Benutzer
	 * 
	 * @param name
	 *          Name des Benutzers
	 * @param passwort
	 *          Passwort des Benutzers
	 * @param ip
	 *          IP-Adresse des DaV
	 * @param port
	 *          Port-Nummer des DaV
	 */
	public void setOfflineBenutzer(String name, String passwort, String ip,
			int port)
	{
		this._name = name;
		this._passwort = passwort;
		this._ip = ip;
		this._tcpPort = port;
	}

	/**
	 * Liefert den Namen
	 * 
	 * @return Name.
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Liefert das Passwort.
	 * 
	 * @return Passwort.
	 */
	public String getPasswort()
	{
		return _passwort;
	}

	/**
	 * Liefert die IP bzw. die Domain.
	 * 
	 * @return IP bzw. die Domain.
	 */
	public String getIp()
	{
		return _ip;
	}

	/**
	 * Liefert den Port.
	 * 
	 * @return port.
	 */
	public int getTcpPort()
	{
		return _tcpPort;
	}

	/**
	 * Setzt die Daten zurck, nur von {@link BenutzerAngemeldet} aus zugnglich.
	 */
	protected void zuruecksetzen()
	{
		this._name = null;
		this._passwort = null;
		this._ip = null;
		this._tcpPort = -1;
	}
}
