/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.deskriptoren;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Descriptor beschreibt ein Plug-In.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 08:25:32 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class DeskriptorPlugin
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * ID des Plug-Ins
   */
  private String _pluginID = null;

  /**
   * Liste der Kindelemente
   */
  private List<Object> _kindElemente = null;

  /**
   * Descriptor Typ
   */
  private EnumDeskriptorTyp _enumDeskriptorTyp = null;

  /**
   * Instanz der {@link ElementRegistrierung}.
   */
  private ElementRegistrierung registrierung = ElementRegistrierung.getInstanz();

  /**
   * Konstruktor dieser Klasse.
   * 
   * @param pluginID
   *          ID des Plugins.
   * @param enumDescriptorTyp
   *          Gibt den Descriptortyp an.
   * 
   * 
   */
  public DeskriptorPlugin(String pluginID, EnumDeskriptorTyp enumDescriptorTyp)
  {
    _debug.fine(getClass().getName() + "::DescriptorPlugin => Plug-In ID: " + pluginID);

    this._pluginID = pluginID;
    this._enumDeskriptorTyp = enumDescriptorTyp;
    init();
  }

  /**
   * Fhrt die Initialisierung durch.
   */
  private void init()
  {
    _kindElemente = new LinkedList<Object>();

    // Action
    if (_enumDeskriptorTyp == EnumDeskriptorTyp.deskriptorAktion)
    {
      Iterator<DeskriptorAktion> it = registrierung.getPluginAktionDeskriptoren(this._pluginID).iterator();
      while (it.hasNext())
      {
        IDeskriptorElementUndAktion action = it.next();
        _kindElemente.add(action);
      }
    }

    // Globale Action
    if (_enumDeskriptorTyp == EnumDeskriptorTyp.deskriptorGlobal)
    {
      Iterator<DeskriptorAktion> it = registrierung.getPluginAktionDeskriptorenGlobal(this._pluginID).iterator();
      while (it.hasNext())
      {
        IDeskriptorElementUndAktion action = it.next();
        _kindElemente.add(action);
      }
    }

    // Element
    if (_enumDeskriptorTyp == EnumDeskriptorTyp.deskriptorElement)
    {
      Iterator<DeskriptorElement> it = registrierung.getPluginElementDeskriptoren(this._pluginID).iterator();
      while (it.hasNext())
      {
        DeskriptorElement element = it.next();
        _kindElemente.add(element);
      }
    }

    // Gemischt
    if (_enumDeskriptorTyp == EnumDeskriptorTyp.deskriptorGemischt)
    {
      Iterator<Object> it = registrierung.getPluginDeskriptorenKombiniert(this._pluginID).iterator();
      while (it.hasNext())
      {
        Object gemischtesElement = it.next();
        _kindElemente.add(gemischtesElement);
      }
    }
  }

  /**
   * Liefert eine Liste aller Kindelemente. Mit <code>instanceof</code> lt sich prfen, ob die Elemente der Liste
   * vom Typ {@link DeskriptorAktion} oder {@link DeskriptorElement} sind.
   * 
   * @return Liste aller Kindelemente.
   */
  public List<Object> getElemente()
  {
    return this._kindElemente;
  }

  /**
   * Liefert die ID des Plug-Ins.
   * 
   * @return ID des Plug-Ins.
   */
  public String getPluginID()
  {
    return this._pluginID;
  }

	@Override
	public String toString()
	{
		return getPluginID();
	}
  
  
}
