/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.deskriptoren;

import org.eclipse.jface.action.IAction;

/**
 * Interface zur Beschreibung der Basisfunktionalitten von Descriptoren.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:06 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public interface IDeskriptorElementUndAktion
{

  /**
   * Liefert die Action.
   * 
   * @param Action.
   */
  public abstract IAction getAktion();

  /**
   * @return PluginID.
   */
  public abstract String getPluginID();

  /**
   * Liefert die ID der reprsentierten Action.
   */
  public abstract String getAktionID();

  /**
   * Liefert den Text der Action.
   * 
   * @return Text der Action.
   */
  public abstract String getText();

  /**
   * Liefert den ToolTipText der Action.
   * 
   * @return ToolTipText de Action.
   */
  public abstract String getToolTipText();

  /**
   * Liefert die Beschreibung der Action.
   * 
   * @return Beschreibung der Action.
   */
  public abstract String getAktionBeschreibung();

  /**
   * Liefert das Icon der Action.
   * 
   * @return Icon der Action.
   */
  public abstract String getIconPfad();

  /**
   * Liefert das HoverIcon der Action.
   * 
   * @return HoverIcon.
   */
  public abstract String getHoverIconPfad();

  /**
   * Liefert das DisabledIcon der Action.
   * 
   * @return DisabledIcon.
   */
  public abstract String getDeaktiviertesIconPfad();

  /**
   * Liefert fas Flag fr die GlobaleAction
   * 
   * @return Liefert <code>true</code> wenn es sich um eine globale Action handelt, sonst <code>false</code>.
   */
  public boolean istGlobaleAktion();

}