/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.xml.datenstruktur;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Reprsentation eines "GroupMarkers". Die Definition eines GroupMarkers richtet sich nach
 * <code>menubar.dtd</code>.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:05 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 * @deprecated
 * 
 */
public class XmlElementGruppenMarker
{
  private String _id = null;

  private String _name = null;

  private Element _elGroupMarker = null;

  private String _elGroupMarkerName = XmlBezeichnerMenuSymbol.elementGroupMarker;

  private String _elGroupMarkerAttributId = XmlBezeichnerMenuSymbol.attId;

  private String _elGroupMarkerAttributName = XmlBezeichnerMenuSymbol.attName;

  /**
   * Parameterloser Konstruktor
   * 
   */
  public XmlElementGruppenMarker()
  {
  }

  /**
   * Konstruktor, erzeugt direkt einen GroupMarker mit id und name.
   * 
   * @param id
   *          Id des zu erzeugenden GroupMarkers.
   * @param name
   *          Name des zu erzeugenden GroupMarkers.
   */
  public XmlElementGruppenMarker(String id, String name)
  {
    this._id = id;
    this._name = name;
    generiereElement();
  }

  /**
   * Generiert das <code>Element</code> fr den <code>JDOM</code>-Baum
   * 
   */
  private void generiereElement()
  {
    _elGroupMarker = new Element(_elGroupMarkerName);
    Attribute elItemId = new Attribute(_elGroupMarkerAttributId, _id);
    Attribute elItemName = new Attribute(_elGroupMarkerAttributName, _name);
    _elGroupMarker.setAttribute(elItemId);
    _elGroupMarker.setAttribute(elItemName);
  }

  /**
   * Generiert die Objektreprsentation eines GroupMarkers aus einem <code>Element</code>. Es wird hierzu kein Clone
   * erstellt, folglich bleiben Eltern und Kinder des Elements erhalten.
   * 
   * @param el
   *          <code>Element</code>, zu dem die Objektreprsentation erstellt wird.
   */
  public void generiereElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elGroupMarkerName) && el.getAttributes().size() == 2)
    {
      _elGroupMarker = el;
      _id = el.getAttributeValue(_elGroupMarkerAttributId);
      _name = el.getAttributeValue(_elGroupMarkerAttributName);
    }
  }

  /**
   * Generiert die Objektreprsentation eines GroupMarkers aus einem <code>Element</code>. Es handelt sich hierbei um
   * eine flache Kopie, Eltern- und Kindelemente gehen verloren.
   * 
   * @param el
   *          <code>Element</code>, von dem eine flache Kopie erstellt werden soll.
   */
  public void generiereAusElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elGroupMarkerName) && el.getAttributes().size() == 2)
    {
      _id = el.getAttributeValue(_elGroupMarkerAttributId);
      _name = el.getAttributeValue(_elGroupMarkerAttributName);
      generiereElement();
    }
  }

  /**
   * Liefert das <code>Element</code> des GroupMarkers.
   * 
   * @return Das <code>Element</code> des GroupMarkers.
   */
  public Element getElementItem()
  {
    return _elGroupMarker;
  }

  /**
   * Liefert eine String-Reprsentation des GroupMarkers.
   * 
   * @return Stringreprsentation des GroupMarkers.
   */
  public String toString()
  {
    String returnString = "Element: " + _elGroupMarkerName + " Id: " + _id + " Name: " + _name;
    return returnString;
  }

}