/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.xml.datenstruktur;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Reprsentation eines "Mens". Die Reprsentation eines Mens richtet sich nach <code>menubar.dtd</code>.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:05 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class XmlElementMenu
{
  private String _id;

  private Element _elMenu = null;

  private String _elMenuName = XmlBezeichnerMenuSymbol.elementMenu;

  private String _elMenuAttributId = XmlBezeichnerMenuSymbol.attId;

  /**
   * Parameterloser Konstruktor
   * 
   */
  public XmlElementMenu()
  {
  }

  /**
   * Konstruktor, erzeugt ein Men ber eine anzugebende Id
   * 
   * @param id
   *          Id des Mens, dass erzeugt werden soll.
   */
  public XmlElementMenu(String id)
  {
    this._id = id;
    generiereElement();
  }

  // public void addItem(Item item)
  // {
  // }
  //
  // public void addSubMenu(Submenu submenu)
  // {
  // }

  /**
   * Generiert das <code>Element</code> fr den <code>JDOM</code>-Baum
   */
  private void generiereElement()
  {
    _elMenu = new Element(_elMenuName);
    Attribute elMenuId = new Attribute(_elMenuAttributId, _id);
    _elMenu.setAttribute(elMenuId);
  }

  /**
   * Generiert die Objektreprsentation eines Mens aus einem <code>Element</code>. Es wird hierzu kein Clone
   * erstellt, folglich bleiben Eltern und Kinder des Elements erhalten.
   * 
   * @param el
   *          <code>Element</code>, zu dem die Objektreprsentation erstellt wird.
   */
  public void generiereElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elMenuName) && el.getAttributes().size() == 1)
    {
      _elMenu = el;
      _id = el.getAttributeValue(_elMenuAttributId);
    }
  }

  /**
   * Generiert die Objektreprsentation eines Mens aus einem <code>Element</code>. Es handelt sich hierbei um eine
   * flache Kopie, Eltern- und Kindelemente gehen verloren.
   * 
   * @param el
   *          <code>Element</code>, von dem eine flache Kopie erstellt werden soll.
   */
  public void generiereAusElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elMenuName) && el.getAttributes().size() == 1)
    {
      _id = el.getAttributeValue(_elMenuAttributId);
      generiereElement();
    }
  }

  /**
   * Liefert das <code>Element</code> des Mens.
   * 
   * @return Das <code>Element</code> des Mens.
   */
  public Element getElementMenu()
  {
    return _elMenu;
  }

  /**
   * Liefert eine String-Reprsentation des Mens.
   * 
   * @return Stringreprsentation des Mens.
   */
  public String toString()
  {
    String returnString = "Element: " + _elMenuName + " Id: " + _id;
    return returnString;
  }

}
