/*
 * This file is part of Rahmenwerk.
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * 
 * Copyright (c) 2012 Swarco Traffic Systems GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * Swarco Traffic Systems GmbH - initial API and implementation
 * 
 * 
 * 
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau 
 * T. +49 (0)7225 / 64-03
 * F. +49 (0)7225 / 64-300 
 * E. office.sts@swarco.de
 */
package de.sts.esptool.views;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.sts.esptool.Activator;
import de.sts.esptool.dlgs.DaVKommunikationExt;
import de.sts.esptool.dlgs.DialogBenutzerWaehler;
import de.sts.esptool.dlgs.EinstellungenWaehlerExt;
import de.sts.esptool.dlgs.EnumSpeicherOrt;
import de.sts.esptool.dlgs.EnumSubjekt;
import de.sts.esptool.dlgs.LadeSpeicherDialogMenu;
import de.sts.esptool.dlgs.SpeicherInfo;
import de.sts.esptool.dlgs.LadeSpeicherDialogMenu.EnumDialogSpeichernLaden;

/**
 * Mit Hilfe dieser View lassen sich die Inhalte des Einstellungenspeichers
 * Anzeigen und selektiv lschen. Das Editieren von Inhalten ist in dieser
 * Version noch nicht mglich.
 * 
 * @author HofmannJ
 * 
 */
public class ViewOverview extends ViewPart
{
	public static final String ID = "de.sts.esptool.views.ViewOverview";
	private Group _grpOverview;
	private Group _grpDetail;
	private TableViewer _tv;
	private Text _textDetail;
	private EinstellungenSpeicher _esp;
	private Composite _top;

	@Override
	public void createPartControl(Composite parent)
	{
		_top = new Composite(parent, SWT.NONE);
		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(_top, Activator.PLUGIN_ID +".espTool"); 

		/*
		 * Layout
		 */
		_top.setLayout(new org.eclipse.swt.layout.GridLayout(1, false));

		GridData gdGrpOverview = new GridData();
		gdGrpOverview.grabExcessHorizontalSpace = true;
		gdGrpOverview.grabExcessVerticalSpace = true;
		gdGrpOverview.horizontalAlignment = GridData.FILL;
		gdGrpOverview.verticalAlignment = GridData.FILL;
		getGrpOverview(_top).setLayoutData(gdGrpOverview);

		GridData gdGrpDetail = new GridData();
		gdGrpDetail.grabExcessHorizontalSpace = true;
		gdGrpDetail.grabExcessVerticalSpace = true;
		gdGrpDetail.horizontalAlignment = GridData.FILL;
		gdGrpDetail.verticalAlignment = GridData.FILL;
		getGrpDetail(_top).setLayoutData(gdGrpDetail);

		genActions();
		verankereKontextMenu();
	}

	private Group getGrpOverview(Composite parent)
	{
		if (_grpOverview == null)
		{
			_grpOverview = new Group(parent, SWT.NONE);
			_grpOverview.setText("bersicht");

			_tv = new TableViewer(_grpOverview, SWT.FULL_SELECTION | SWT.MULTI
					| SWT.H_SCROLL | SWT.V_SCROLL);

			_tv.addSelectionChangedListener(new ISelectionChangedListener()
			{

				@Override
				public void selectionChanged(SelectionChangedEvent event)
				{
					ISelection selection = event.getSelection();

					if ((!selection.isEmpty())
							&& (selection instanceof StructuredSelection))
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object firstElement = ss.getFirstElement();
						if (firstElement instanceof EspParameter)
						{
							EspParameter par = (EspParameter) firstElement;
							_textDetail.setText(par.getValue());
						}
					} else
					{
						_textDetail.setText("");
					}

				}
			});

			Table table = _tv.getTable();

			TableColumn tcKey = new TableColumn(table, SWT.LEFT);
			tcKey.setText("Key");
			tcKey.setWidth(200);

			TableColumn tcValue = new TableColumn(table, SWT.LEFT);
			tcValue.setText("Value");
			tcValue.setWidth(400);

			table.setHeaderVisible(true);
			table.setLinesVisible(true);

			_tv.setContentProvider(new CP());
			_tv.setLabelProvider(new LP());

			/*
			 * Layout
			 */
			_grpOverview.setLayout(new FillLayout());
		}

		return _grpOverview;
	}

	private Group getGrpDetail(Composite parent)
	{
		if (_grpDetail == null)
		{
			_grpDetail = new Group(parent, SWT.NONE);
			_grpDetail.setText("Detail");

			_textDetail = new Text(_grpDetail, SWT.H_SCROLL | SWT.V_SCROLL);
			_textDetail.setEditable(false);

			/*
			 * Layout
			 */
			_grpDetail.setLayout(new FillLayout());
		}

		return _grpDetail;
	}

	private void verankereKontextMenu()
	{

		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{
				fuelleKontextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_tv.getControl());
		_tv.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, _tv);
	}

	private void fuelleKontextMenu(IMenuManager manager)
	{

		/*
		 * Hier erfolgt die Intitialisierung der Actions, die bereitgestellt werden
		 * solle. Mittels ((IStructuredSelection)
		 * _treeViewer.getSelection()).getFirstElement() wird das aktuell
		 * selektierte Element im TreeViewer bestimmt.
		 */

		// Action neu aus Datei laden
		Action actionLoeschen = new Action()
		{
			public void run()
			{
				aktionSelektionAusEspLoeschen();
			}
		};
		actionLoeschen.setText("&Lschen");
		actionLoeschen
				.setToolTipText("Lscht die selektierten Parameter aus dem Einstellungenspeicher.");
		manager.add(actionLoeschen);
	}

	private void genActions()
	{
		IMenuManager mm = getViewSite().getActionBars().getMenuManager();

		Action aLaden = new Action("&Laden")
		{

			@Override
			public void run()
			{
				aktionLaden();
			}

		};
		aLaden.setId(ID + ".aLaden");
		mm.add(aLaden);

		Action aSpeichern = new Action("&Speichern")
		{
			@Override
			public void run()
			{
				aktionSpeichern();
			}
		};
		aSpeichern.setId(ID + ".aSpeichern");
		mm.add(aSpeichern);

	}

	@Override
	public void setFocus()
	{
		if(_top!=null)
		{
			_top.setFocus();
		}
	}

	private void aktionLaden()
	{
		LadeSpeicherDialogMenu ladeDlg = new LadeSpeicherDialogMenu(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());
		SpeicherInfo speicherInfo = ladeDlg.oeffne(EnumDialogSpeichernLaden.laden);
		EnumSpeicherOrt speicherOrt = speicherInfo.getSpeicherOrt();
		EnumSubjekt subjekt = speicherInfo.getSubjekt();

		EinstellungenSpeicher espLaden = null;
		if (subjekt != null)
		{
			if (subjekt == EnumSubjekt.allgemein)
			{
				if (speicherOrt == EnumSpeicherOrt.allgemeinNetzweit)
				{
					espLaden = EinstellungenWaehlerExt.getInstanz().getEinstellungen(
							EinstellungsArt.allgemeinNetzweit);
				} else if (speicherOrt == EnumSpeicherOrt.allgemeinLokal)
				{
					espLaden = EinstellungenWaehlerExt.getInstanz().getEinstellungen(
							EinstellungsArt.allgemeinLokal);
				}
			}

			if ((subjekt == EnumSubjekt.benutzerAngemeldet)
					|| (subjekt == EnumSubjekt.benutzerAuswahl))
			{
				String benutzer = BenutzerAngemeldet.getInstanz().getName();

				if (subjekt == EnumSubjekt.benutzerAuswahl)
				{
					
					DialogBenutzerWaehler dlg = new DialogBenutzerWaehler(PlatformUI
							.getWorkbench().getDisplay().getActiveShell());
					benutzer = dlg.oeffne();
				}

				if (benutzer != null)
				{
					if (speicherOrt == EnumSpeicherOrt.benutzerNetzweit)
					{
						espLaden = EinstellungenWaehlerExt.getInstanz()
								.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);
					} else if (speicherOrt == EnumSpeicherOrt.benutzerLokal)
					{
						espLaden = EinstellungenWaehlerExt.getInstanz()
						.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);
					}
				}
			}
		}

		if (espLaden != null)
		{
			_esp = new EinstellungenSpeicher();
			_esp.einlesenXmlAusString(espLaden.ausgabeXmlFormatiert(true));
		} else
		{
			_esp = new EinstellungenSpeicher();
		}
		_tv.setInput(_esp);
	}

	private void aktionSpeichern()
	{
		if (_esp != null)
		{
			LadeSpeicherDialogMenu ladeDlg = new LadeSpeicherDialogMenu(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			SpeicherInfo speicherInfo = ladeDlg
					.oeffne(EnumDialogSpeichernLaden.speichern);
			EnumSpeicherOrt speicherOrt = speicherInfo.getSpeicherOrt();
			EnumSubjekt subjekt = speicherInfo.getSubjekt();

			if (subjekt != null)
			{
				if (subjekt == EnumSubjekt.allgemein)
				{
					if (speicherOrt == EnumSpeicherOrt.allgemeinNetzweit)
					{
						DaVKommunikationExt.getInstanz()
								.setVoreinstellungenAllgemeinNetzweit(
										_esp.ausgabeXmlFormatiert(true));
					} else if (speicherOrt == EnumSpeicherOrt.allgemeinLokal)
					{
						EinstellungenWaehlerExt.getInstanz().getEinstellungen(
								EinstellungsArt.allgemeinLokal);
						EinstellungenWaehlerExt.getInstanz().sichereSpeicherAllgemeinLokal(
								_esp);
					}
				}

				if ((subjekt == EnumSubjekt.benutzerAngemeldet)
						|| (subjekt == EnumSubjekt.benutzerAuswahl))
				{
					String benutzer = BenutzerAngemeldet.getInstanz().getName();

					if (subjekt == EnumSubjekt.benutzerAuswahl)
					{
						DialogBenutzerWaehler dlg = new DialogBenutzerWaehler(PlatformUI
								.getWorkbench().getDisplay().getActiveShell());
						benutzer = dlg.oeffne();
					}

					if (benutzer != null)
					{
						if (speicherOrt == EnumSpeicherOrt.benutzerNetzweit)
						{
							EinstellungenWaehlerExt.getInstanz()
									.sichereEinstellungenSpeicherBenutzerBestimmtNetzweit(
											benutzer, _esp);
						} else if (speicherOrt == EnumSpeicherOrt.benutzerLokal)
						{
							EinstellungenWaehlerExt.getInstanz()
									.sichereEinstellungenSpeicherBenutzerBestimmtLokal(benutzer,
											_esp);
						}
					}
				}
			}

		}
	}

	@SuppressWarnings("unchecked")
	private void aktionSelektionAusEspLoeschen()
	{
		if (_tv != null)
		{
			LinkedList<EspParameter> listeParameterLoeschen = new LinkedList<EspParameter>();

			ISelection selection = _tv.getSelection();
			if (selection instanceof StructuredSelection)
			{
				StructuredSelection ss = (StructuredSelection) selection;
				if (!ss.isEmpty())
				{
					Iterator it = ss.iterator();
					while (it.hasNext())
					{
						Object obj = it.next();
						if ((obj != null) && (obj instanceof EspParameter))
						{
							listeParameterLoeschen.add((EspParameter) obj);
						}
					}
				}
			}

			if ((_esp != null) && (listeParameterLoeschen.size() > 0))
			{
				for (EspParameter espParameter : listeParameterLoeschen)
				{
					_esp.loescheParameter(espParameter.getKey());
				}

				_tv.refresh();
			}
		}
	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			LinkedList<EspParameter> liste = new LinkedList<EspParameter>();

			if (inputElement instanceof EinstellungenSpeicher)
			{
				EinstellungenSpeicher esp = (EinstellungenSpeicher) inputElement;

				HashMap<String, String> mapParameter = esp.getAlleParameter();

				Iterator<Entry<String, String>> it = mapParameter.entrySet().iterator();
				while (it.hasNext())
				{
					Entry<String, String> entry = it.next();
					String key = entry.getKey();
					String value = entry.getValue();

					EspParameter espParameter = new EspParameter(key, value);
					liste.add(espParameter);
				}
			}

			return liste.toArray(new EspParameter[liste.size()]);
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if ((element != null) && (element instanceof EspParameter))
			{
				EspParameter esp = (EspParameter) element;

				switch (columnIndex)
				{
				case 0:
					if (esp.getKey() != null)
					{
						re = esp.getKey();
					}
					break;
				case 1:
					if (esp.getValue() != null)
					{
						re = esp.getValue();
					}
					break;
				}
			}

			if (re == null)
			{
				re = "";
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	private class EspParameter
	{
		private String _key = null;
		private String _value = null;

		public EspParameter(String key, String value)
		{
			this._key = key;
			this._value = value;
		}

		public String getKey()
		{
			return _key;
		}

		public String getValue()
		{
			return _value;
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();

			sb.append("Key=\"");
			if (getKey() != null)
			{
				sb.append(getKey());
			} else
			{
				sb.append("-");
			}

			sb.append("\", Value=\"");
			if (getValue() != null)
			{
				sb.append(getValue());
			} else
			{
				sb.append("-");
			}
			sb.append("\"");

			return sb.toString();
		}

	}

}
