/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2005 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.dav.dav.
 * 
 * de.bsvrz.dav.dav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.dav.dav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.dav.dav; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.dav.dav.main;

import de.bsvrz.dav.daf.communication.lowLevel.AuthentificationProcess;
import de.bsvrz.dav.daf.communication.lowLevel.ConnectionInterface;
import de.bsvrz.dav.daf.communication.lowLevel.LowLevelCommunication;
import de.bsvrz.dav.daf.communication.lowLevel.ServerConnectionInterface;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.ApplicationDataTelegram;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.AttributeGroupAspectCombination;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.BaseSubscriptionInfo;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.ReceiveSubscriptionInfo;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.ReceiveSubscriptionTelegram;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.ReceiveUnsubscriptionTelegram;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.SendSubscriptionInfo;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.SendSubscriptionTelegram;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.SendUnsubscriptionTelegram;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterDataSubscription;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterDataSubscriptionReceipt;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterDataTelegram;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterDataUnsubscription;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterListsDeliveryUnsubscription;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterListsSubscription;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterListsUnsubscription;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterListsUpdate;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterSubscriptionsConstants;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.InitialisationNotCompleteException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.NormalCloser;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.SystemTerminator;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.impl.CommunicationConstant;
import de.bsvrz.dav.daf.main.impl.ConfigurationManager;
import de.bsvrz.dav.daf.main.impl.config.DafDataModel;
import de.bsvrz.dav.daf.main.impl.config.telegrams.TransmitterConnectionInfo;
import de.bsvrz.dav.daf.main.impl.config.telegrams.TransmitterInfo;
import de.bsvrz.dav.dav.communication.appProtocol.T_A_HighLevelCommunication;
import de.bsvrz.dav.dav.communication.appProtocol.T_A_HighLevelCommunicationInterface;
import de.bsvrz.dav.dav.communication.davProtocol.T_T_HighLevelCommunication;
import de.bsvrz.dav.dav.communication.davProtocol.T_T_HighLevelCommunicationInterface;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * Diese Komponente startet alle notwendigen Aktionen, um eine bestimmte Aufgabe zu erledigen durch den Aufruf einer ihrer Methoden. Die Aktionen werden
 * natrlich nur durchgefhrt, wenn der Initiator auch eine Berechtigung dafr hat. Die Berechtigung entnimmt man der Initiatorrolle. Wenn die Rolle nicht lokal
 * vorhanden ist, wird sie aus der Konfiguration ausgelesen.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 6342 $
 */
public class ConnectionsManager implements ConnectionsManagerInterface, DistributionInterface {

	private static final Debug _debug = Debug.getLogger();

	private static final short DEFAULT_WEIGHT = 1;

	/** Die Startparameters dieses Datenverteilers */
	private final ServerDavParameters _serverDavParameters;

	/** Die Startparameters dieses Datenverteilers als Applikation */
	private final ClientDavParameters _clientDavParameters;

	/** Die Serverkommunikationskomponente der Datenverteiler, die Applikationsverbindungen akzeptiert. */
	private final ServerConnectionInterface _applicationsServerConnection;

	/** Die Serverkommunikationskomponente der Datenverteiler, die Datenverteilersverbindungen akzeptiert. */
	private ServerConnectionInterface _transmittersServerConnection;

	/** Der Authentifikationskomponente */
	private final AuthentificationComponent _authentificationComponent;

	/** Interne Thread zur Kommunikation */
	private TransmitterConnectionsSubscriber _transmitterConnectionsSubscriber;

	/** Interne Thread zur Kommunikation */
	private ApplicationConnectionsSubscriber _applicationConnectionsSubscriber;

	/** Liste der Datenverteilerverbindungen */
	private List<T_T_HighLevelCommunication> _transmitterConnections;

	/** Liste der Applikationsverbindungen */
	private final List<T_A_HighLevelCommunication> _applicationConnections = new CopyOnWriteArrayList<T_A_HighLevelCommunication>();

	/**
	 * Dieser Thread verschickt die Statusinformationen ber angemeldete Applikationen. Nur wenn alle bentigten ATG: _pidAngemeldeteApplikationen,
	 * _pidAngemeldeteDatenidentifikationen, _pidTelegrammLaufzeiten vorhanden sind, wird der Thread initialisiert und gestartet.
	 * <p/>
	 * Ist die Variable <code>null</code>, so wurde der Thread nicht initialisiert(die genannten ATGs fehlen) und der Mechanismus steht nicht zur Verfgung.
	 */
	private ApplicationStatusUpdater _applicationStatusUpdater = null;

	/** Liste der auf eine positive Quitung wartenden Anmeldungen, die ber einen neuen weg umgeleitet werden mssen. */
	private ArrayList _pendingSubscriptionToRedirect;

	/** Information ob die Konfigurationsverbindung erfolgreich ist. */
	private boolean _configurationAvaillable;

	/** Die Id des Datenverteilers */
	private final long _transmitterId;

	/** Die Netztopologie dieses Datenverteilers */
	private TransmitterConnectionInfo _transmitterConnectionInfos[];

	/** Die Liste der nicht etablierten Verbindungen */
	private Vector missedConnectionInfos;

	/** berwacht die Nachbarnverbindungen und leitet Ersatzverbindungen wenn ntig */
	private TransmitterConnectionsMonitor _connectionsMonitor;

	/** Interne Variable (Closing state) */
	private volatile boolean _closing = false;

	/** Die eigene DAF-Verbindung */
	ClientDavConnection selfClientDavConnection;

	/** Das DataModel zur Konfigurationsabfrage */
	DataModel dataModel = null;

	/** Der Anmeldemanager */
	private final SubscriptionsManager subscriptionsManager;

	/** Der Objekte-Attributesgruppen-Aspekte-Listen-Manager */
	private final ListsManager listsManager;

	/** Der Bestwege-Manager */
	private final BestWayManager bestWayManager;

	/** Der Cache Manager */
	final CacheManager cacheManager;

	/** Der Authentifikationsmanager */
	final AuthentificationManager authentificationManager;

	/** Die ID des Konfigurationsverantwortlichen der Konfiguration. */
	private long _configurationId;

	/**
	 * ATG, die bentigt wird um einen Datensatz zu verschicken, der alle angemeldeten Applikationen enthlt. Ist diese ATG nicht vorhanden, so wird der
	 * Datenverteiler gestartet ohne diese Funktionalitt zur Verfgung zu stellen.
	 */
	private final String _pidAngemeldeteApplikationen = "atg.angemeldeteApplikationen";

	/**
	 * ATG, die bentigt wird um einen Datensatz zu verschicken, der die angemeldeten Daten aller angemeldeten Applikationen enthlt. Ist diese ATG nicht
	 * vorhanden, so wird der Datenverteiler gestartet ohne diese Funktionalitt zur Verfgung zu stellen.
	 */
	private final String _pidAngemeldeteDatenidentifikationen = "atg.angemeldeteDatenidentifikationen";

	/**
	 * ATG, die bentigt wird um einen Datensatz zu verschicken, der die Telegrammlaufzeiten aller angemeldeten Applikationen enthlt. Ist diese ATG nicht
	 * vorhanden, so wird der Datenverteiler gestartet ohne diese Funktionalitt zur Verfgung zu stellen.
	 */
	private final String _pidTelegrammLaufzeiten = "atg.telegrammLaufzeiten";


	/** Dieser Konstruktor wird fr Tests bentigt. */
	public ConnectionsManager() {
		subscriptionsManager = null;
		listsManager = null;
		bestWayManager = null;
		cacheManager = null;
		authentificationManager = null;
		_transmitterId = -12;
		_serverDavParameters = null;
		_clientDavParameters = null;
		_applicationsServerConnection = null;
		_authentificationComponent = null;
	}

	/**
	 * Erzeugt eine neue Verbindungsverwaltung fr den Datenverteiler.
	 *
	 * @param serverDavParameters Die Parameter sind u. a. die Adressen und Subadressen der Kommunikationskanle
	 *
	 * @throws CommunicationError         wenn bei der initialen Kommunikation mit dem Datenverteiler Fehler aufgetreten sind
	 * @throws MissingParameterException  wenn notwendige Verbindungsparameter nicht spezifiziert wurden
	 * @throws InconsistentLoginException bei einem fehlgeschlagenen Authentifizierungsversuch
	 * @throws ConnectionException        wenn eine Verbindung nicht aufgebaut werden konnte
	 */
	public ConnectionsManager(ServerDavParameters serverDavParameters)
			throws CommunicationError, MissingParameterException, InconsistentLoginException, ConnectionException {

		// Initialisation
		_serverDavParameters = serverDavParameters;
		_clientDavParameters = _serverDavParameters.getClientDavParameters();
		_transmitterId = _serverDavParameters.getDataTransmitterId();

		_transmitterConnections = new CopyOnWriteArrayList<T_T_HighLevelCommunication>();
		missedConnectionInfos = new Vector();
		_configurationAvaillable = false;
		subscriptionsManager = new SubscriptionsManager(this);
		listsManager = new ListsManager(this);
		bestWayManager = new BestWayManager(this.getTransmitterId(), this, listsManager);
		listsManager.setBestWayManager(bestWayManager);
		cacheManager = new CacheManager();
		authentificationManager = new AuthentificationManager(this);
		_pendingSubscriptionToRedirect = new ArrayList();

		try {
			// Get the low level protocoll
			String communicationProtocolName = _serverDavParameters.getLowLevelCommunicationName();
			if(communicationProtocolName == null) {
				throw new InitialisationNotCompleteException("Kommunikationsprotokollname ungltig.");
			}
			Class communicationProtocolClass = Class.forName(communicationProtocolName);
			if(communicationProtocolClass == null) {
				throw new InitialisationNotCompleteException("Kommunikationsprotokollname ungltig.");
			}

			//Start the listener at the application port
			_applicationsServerConnection = (ServerConnectionInterface)communicationProtocolClass.newInstance();
			_applicationsServerConnection.connect(_serverDavParameters.getApplicationConnectionsSubAddress());

			// Get the authentification process
			String authentificationName = _serverDavParameters.getAuthentificationProcessName();
			if(authentificationName == null) {
				throw new InitialisationNotCompleteException("Unbekanntes Authentifikationsverfahren.");
			}
			Class authentificationClass = Class.forName(authentificationName);
			if(authentificationClass == null) {
				throw new InitialisationNotCompleteException("Unbekanntes Authentifikationsverfahren.");
			}
			AuthentificationProcess authentificationProcess = (AuthentificationProcess)authentificationClass.newInstance();
			_authentificationComponent = new AuthentificationComponent(authentificationProcess);

			// Start the application connections receiver thread
			_applicationConnectionsSubscriber = new ApplicationConnectionsSubscriber();
			_applicationConnectionsSubscriber.start();

			// Start the client application of this transmitter
			selfClientDavConnection = new ClientDavConnection(_clientDavParameters);
			long waitingTime = 0, startTime = System.currentTimeMillis();
			long maxWaitingTime = CommunicationConstant.MAX_WAITING_TIME_FOR_CONNECTION;
			while(waitingTime < maxWaitingTime) {
				try {
					selfClientDavConnection.connect();
					selfClientDavConnection.setCloseHandler(new NormalCloser());
					// try to login. if login successfull then a configuration is found
					// otherwise keep trying till the threshold time is overdue
					selfClientDavConnection.login();
					dataModel = selfClientDavConnection.getDataModel();
					_configurationId = selfClientDavConnection.getLocalConfigurationAuthority().getId();
					_configurationAvaillable = true;
					break;
				}
				catch(CommunicationError ex) {
					_debug.warning("Es konnte keine Verbindung zur Konfiguration hergestellt werden", ex);
					try {
						selfClientDavConnection.disconnect(false, "");
						Thread.sleep(CommunicationConstant.SLEEP_TIME_WAITING_FOR_CONNECTION);
						waitingTime = System.currentTimeMillis() - startTime;
					}
					catch(InterruptedException e) {
						close(true, e.getMessage());
						return;
					}
				}
				catch(ConfigurationException ex) {
					_debug.error("Es konnte keine Verbindung zur Konfiguration hergestellt werden", ex);
					try {
						selfClientDavConnection.disconnect(false, "");
						Thread.sleep(CommunicationConstant.SLEEP_TIME_WAITING_FOR_CONNECTION);
						waitingTime = System.currentTimeMillis() - startTime;
					}
					catch(InterruptedException e) {
						close(true, e.getMessage());
						return;
					}
				}
			}
			// if no configuration is found then close everything.
			if(waitingTime > maxWaitingTime) {
				close(true, "Zeitgrenze berschritten. Es konnte keine Konfigurationsverbindung aufgebaut werden");
				return;
			}

			// Otherwise tell every waiting connection that they can continue there authentification process.

			selfClientDavConnection.setCloseHandler(new SystemTerminator());

			// Prfen, ob eine gltige Datenverteiler-ID angegeben wurde

			final SystemObject davObject = dataModel.getObject(_transmitterId);
			if(davObject == null || !davObject.isOfType("typ.datenverteiler")) {
				final StringBuilder message = new StringBuilder();
				message.append("Das Konfigurationsobjekt zu der via Aufrufparameter -datenverteilerId= angegebenen Objekt-ID ");
				message.append(_transmitterId);
				message.append(" ");
				if(davObject == null) {
					message.append("wurde nicht gefunden");
				}
				else {
					message.append("ist nicht vom typ.datenverteiler, sondern vom ").append(davObject.getType().getPidOrNameOrId());
				}
				message.append(".\nFolgende Datenverteiler sind der Konfiguration bekannt:\n");
				final SystemObjectType davType = dataModel.getType("typ.datenverteiler");
				final List<SystemObject> davs = davType.getElements();
				Formatter formatter = new Formatter();
				formatter.format("%40s %22s %s\n", "PID", "ID", "NAME");
				for(SystemObject dav : davs) {
					formatter.format("%40s %22d %s\n", dav.getPid(), dav.getId(), dav.getName());
				}
				message.append(formatter.toString());
				_debug.error(message.toString());
				try {
					Thread.sleep(2000);
				}
				catch(InterruptedException ignored) {
				}
				throw new IllegalArgumentException("Ungltige Datenverteiler-ID " + _transmitterId);
			}

			_debug.fine("Verbindungsaufbau fr normale Applikationen ermglichen");

			if(dataModel.getObject(_pidAngemeldeteApplikationen) == null || dataModel.getObject(_pidAngemeldeteDatenidentifikationen) == null
			   || dataModel.getObject(_pidTelegrammLaufzeiten) == null) {
				// Eine ATG fehlt -> Der Status der angemeldeten Applikationen kann nicht zur Verfgung gestellt werden.
				_debug.warning(
						"Eine Attributgruppe zum versenden des Status einer Applikation ist nicht vorhanden. Der Status der Applikationen kann nicht zur Verfgung gestellt werden, der Datenverteiler wird ohne diese Funktion gestartet. Fr diese Funktion sind folgende Attributgruppen notwendig: "
						+ _pidAngemeldeteApplikationen + " " + _pidAngemeldeteDatenidentifikationen + " " + _pidTelegrammLaufzeiten + " ."
				);
			}
			else {
				_applicationStatusUpdater = new ApplicationStatusUpdater();
				_applicationStatusUpdater.start();
			}


			synchronized(_applicationConnections) {
				for(int i = 0; i < _applicationConnections.size(); ++i) {
					T_A_HighLevelCommunicationInterface connection = _applicationConnections.get(
							i
					);
					if(connection != null) {
						connection.continueAuthentification();
					}
				}
			}

			// Informationen zum Distributionspaket des Datenverteilers werden publiziert
			try {
				final AttributeGroup infoAtg = dataModel.getAttributeGroup("atg.distributionspaketReleaseInfo");
				if(infoAtg == null) {
					_debug.info("Informationen zum Distributionspaket des Datenverteilers knnen nicht publiziert werden, weil das Datenmodell "
					            + "(kb.systemModellGlobal) noch nicht aktualisiert wurde.");
				}
				else {
					final Aspect aspect = dataModel.getAspect("asp.standard");
					final DataDescription dataDescription = new DataDescription(infoAtg, aspect);
					final String packageName = "de.bsvrz.dav.dav";
					String release = "unbekannt";
					String revision = "unbekannt";
					String compileTime = "unbekannt";
					String licence = "unbekannt";
					String dependsOnCompiled = "unbekannt";
					String dependsOnSource = "unbekannt";
					String dependsOnLib = "unbekannt";
					try {
						final Class<?> infoClass = Class.forName(packageName + ".PackageRuntimeInfo");
						release = (String)infoClass.getMethod("getRelease").invoke(null);
						revision = (String)infoClass.getMethod("getRevision").invoke(null);
						compileTime = (String)infoClass.getMethod("getCompileTime").invoke(null);
						licence = (String)infoClass.getMethod("getLicence").invoke(null);
						dependsOnCompiled = (String)infoClass.getMethod("getDependsOnCompiled").invoke(null);
						dependsOnSource = (String)infoClass.getMethod("getDependsOnSource").invoke(null);
						dependsOnLib = (String)infoClass.getMethod("getDependsOnLib").invoke(null);
					}
					catch(Exception e) {
						_debug.info("Informationen zum Distributionspaket des Datenverteilers konnten nicht ermittelt werden", e);
					}
					final Data data = selfClientDavConnection.createData(infoAtg);
					data.getTextValue("Name").setText(packageName);
					data.getTextValue("Release").setText(release);
					data.getTextValue("Version").setText(revision);
					data.getTextValue("Stand").setText(compileTime);
					data.getTextValue("Lizenz").setText(licence);
					data.getTextValue("Abhngigkeiten").setText(dependsOnCompiled);
					data.getTextValue("QuellcodeAbhngigkeiten").setText(dependsOnSource);
					data.getTextValue("BibliothekAbhngigkeiten").setText(dependsOnLib);
					final ResultData result = new ResultData(davObject, dataDescription, System.currentTimeMillis(), data);
					_debug.info("Informationen zum Distributionspaket des Datenverteilers werden publiziert", result);
					final ClientSenderInterface senderInterface = new ClientSenderInterface() {
						public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
						}
						public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
							return false;
						}
					};
					selfClientDavConnection.subscribeSource(senderInterface, result);
				}
			}
			catch(Exception e) {
				_debug.info("Informationen zum Distributionspaket des Datenverteilers konnten nicht publiziert werden", e);
			}

			try {
				final long delay = _serverDavParameters.getInitialInterDavServerDelay();
				_debug.config("Verbindungen von und zu anderen Datenverteilern werden in " + delay + " ms zugelassen");
				Thread.sleep(delay);
			}
			catch(InterruptedException e) {
				throw new RuntimeException(e); 
			}

			_debug.finer("Datenverteiler-ID", _transmitterId);
			_debug.config("Verbindungen von und zu anderen Datenverteilern werden jetzt zugelassen");

			try {
				_serverDavParameters.setConfigurationId(_configurationId);
//				serverDavParameters.setConfigurationId(dataModel.getConfigurationAuthorityId());
				// Get the net topologie
				ConfigurationManager configurationManager = ((DafDataModel)dataModel).getConfigurationManager();
				if(configurationManager != null) {
					int davDavSubadress = -1;
					_transmitterConnectionInfos = configurationManager.getTransmitterConnectionInfo(_transmitterId);
					for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
						_debug.finer("Datenverteilerverbindung", _transmitterConnectionInfos[i].parseToString());
						if(_transmitterConnectionInfos[i] != null) {
							TransmitterInfo transmitterInfo_1 = _transmitterConnectionInfos[i].getTransmitter_1();
							TransmitterInfo transmitterInfo_2 = _transmitterConnectionInfos[i].getTransmitter_2();
							long id1 = transmitterInfo_1.getTransmitterId();
							long id2 = transmitterInfo_2.getTransmitterId();
							if(id1 == id2) {
								close(true, "Inkonsistente Netztopologie (Verbindung von Datenverteiler[" + id1 + "] zu sich selbst");
							}
							int subAdresse1 = transmitterInfo_1.getSubAdress();
							int subAdresse2 = transmitterInfo_2.getSubAdress();
							for(int j = i + 1; j < _transmitterConnectionInfos.length; ++j) {
								if(_transmitterConnectionInfos[j] != null) {
									long tmpId1 = _transmitterConnectionInfos[j].getTransmitter_1().getTransmitterId();
									long tmpId2 = _transmitterConnectionInfos[j].getTransmitter_2().getTransmitterId();
									if((id1 == tmpId1) && (id2 == tmpId2)) {
										close(
												true,
												"Inkonsistente Netztopologie (Mehrfache Verbindung zwichen Datenverteiler[" + id1 + "] und Datenverteiler["
												+ id2 + "] mglich)"
										);
									}
								}
							}
							if(id1 == _transmitterId) {
								if(davDavSubadress == -1) {
									davDavSubadress = subAdresse1;
								}
								else if(davDavSubadress != subAdresse1) {
									close(
											true, "Inkonsistente Netztopologie (Es wurden dem Datenverteiler[" + id1 + "] verschiedene Subadressen zugewiesen"
									);
								}
							}
							if(id2 == _transmitterId) {
								if(davDavSubadress == -1) {
									davDavSubadress = subAdresse2;
								}
								else if(davDavSubadress != subAdresse2) {
									close(
											true, "Inkonsistente Netztopologie (Es wurden dem Datenverteiler[" + id2 + "] verschiedene Subadressen zugewiesen"
									);
								}
							}
						}
					}

					if(davDavSubadress == -1) {
						davDavSubadress = this._serverDavParameters.getTransmitterConnectionsSubAddress();
					}
					//Start the listener at the transmitter port
					_transmittersServerConnection = (ServerConnectionInterface)communicationProtocolClass.newInstance();

					int subAddressToListenFor = davDavSubadress;
					if(subAddressToListenFor == 100000) {
						// Zu Testzwecken wird die Portnummer mit dem Wert 100000 serverseitig durch 8088 und clientseitig durch 8081 ersetzt
						subAddressToListenFor = 8088;
					}

					subAddressToListenFor += _serverDavParameters.getTransmitterConnectionsSubAddressOffset();

					_transmittersServerConnection.connect(subAddressToListenFor);
					// Start the transmitter connections receiver thread
					_transmitterConnectionsSubscriber = new TransmitterConnectionsSubscriber();
					_transmitterConnectionsSubscriber.start();

					connectToNeighbours();
					_connectionsMonitor = new TransmitterConnectionsMonitor();
					_connectionsMonitor.start();
				}
			}
			catch(RuntimeException ex) {
				close(true, ex.getMessage());
			}
		}
		catch(ClassNotFoundException ex) {
			throw new InitialisationNotCompleteException("Fehler beim Erzeugen der Datenverteilerkommunikation.");
		}
		catch(InstantiationException ex) {
			throw new InitialisationNotCompleteException("Fehler beim Erzeugen der Datenverteilerkommunikation.");
		}
		catch(IllegalAccessException ex) {
			throw new InitialisationNotCompleteException("Fehler beim Erzeugen der Datenverteilerkommunikation.");
		}
	}

	public final long getTransmitterId() {
		return _transmitterId;
	}

	/** Verbindet mit den Nachbarndatenverteiler */
	private void connectToNeighbours() {
		if(_transmitterConnectionInfos == null) {
			return;
		}
		synchronized(missedConnectionInfos) {
			for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
				if(_transmitterConnectionInfos[i].isExchangeConnection()) {
					continue;
				}
				if(!_transmitterConnectionInfos[i].isActiveConnection()) {
					continue;
				}
				TransmitterInfo t1 = _transmitterConnectionInfos[i].getTransmitter_1();
				if(t1.getTransmitterId() == _transmitterId) {
					if(!connectToMainTransmitter(_transmitterConnectionInfos[i])) {
						connectToAlternativeTransmitters(_transmitterConnectionInfos[i]);
						missedConnectionInfos.add(_transmitterConnectionInfos[i]);
					}
				}
			}
		}
	}

	/**
	 * Startet den Verbindungsaufbau zwischen zwei direkt benachbarten Datenverteilern. Beim Verbindungsaufbau zwischen zwei DAV werden durch die Angabe der beiden
	 * Kommunikationspartner, die Wichtung der Verbindung, die Angabe, welche(r) Datenverteiler die Verbindung aufbaut und die Spezifikation von Ersatzverbindungen
	 * festgelegt, um welche Art von Verrbindung es sich handelt.
	 *
	 * @param transmitterConnectionInfo Enthlt Informationen zu der Verbindungart zwischen zwei Datenverteilern.
	 *
	 * @return true: Verbindung hergestellt, false: Verbindung nicht hergestellt
	 *
	 * @see #connectToTransmitter(de.bsvrz.dav.daf.main.impl.config.telegrams.TransmitterInfo,short,long,String)
	 */
	private boolean connectToMainTransmitter(TransmitterConnectionInfo transmitterConnectionInfo) {
		TransmitterInfo t2 = transmitterConnectionInfo.getTransmitter_2();
		short weight = transmitterConnectionInfo.getWeight();
		long waitingTime = transmitterConnectionInfo.getConnectionTimeThreshold();
		return connectToTransmitter(t2, weight, waitingTime, transmitterConnectionInfo.getUserName());
	}

	/**
	 * Startet den Ersatzverbindungsaufbau zwischen zwei nicht direkt benachbarten Datenverteilern. Beim Verbindungsaufbau zwischen zwei DAV werden durch die
	 * Angabe der beiden Kommunikationspartner, die Wichtung der Verbindung, die Angabe, welche(r) Datenverteiler die Verbindung aufbaut und die Spezifikation von
	 * Ersatzverbindungen festgelegt, um welche Art von Verrbindung es sich handelt. Ob Ersatzverbindungen automatisch etabliert werden sollen, wird durch das
	 * autoExchangeTransmitterDetection Flag festgelegt.
	 *
	 * @param transmitterConnectionInfo Enthlt Informationen zu der Verbindungart zwischen zwei Datenverteilern.
	 *
	 * @see #connectToTransmitter(de.bsvrz.dav.daf.main.impl.config.telegrams.TransmitterInfo,short,long,String)
	 */
	private void connectToAlternativeTransmitters(TransmitterConnectionInfo transmitterConnectionInfo) {
		TransmitterInfo t2 = transmitterConnectionInfo.getTransmitter_2();
		if(transmitterConnectionInfo.isAutoExchangeTransmitterDetectionOn()) {
			TransmitterConnectionInfo infos[] = getInvolvedTransmitters(t2);
			if(infos != null) {
				for(int j = 0; j < infos.length; ++j) {
					TransmitterInfo transmitterInfo = infos[j].getTransmitter_2();
					short weight = infos[j].getWeight();
					long time = infos[j].getConnectionTimeThreshold();
					if(transmitterInfo != null) {
						connectToTransmitter(transmitterInfo, weight, time, transmitterConnectionInfo.getUserName());
					}
				}
			}
		}
		else {
			TransmitterInfo infos[] = transmitterConnectionInfo.getExchangeTransmitterList();
			if(infos != null) {
				for(int j = 0; j < infos.length; ++j) {
					TransmitterConnectionInfo tmpTransmitterConnectionInfo = null;
					for(int k = 0; k < _transmitterConnectionInfos.length; ++k) {
						if(_transmitterConnectionInfos[k].isExchangeConnection()
						   && (_transmitterConnectionInfos[k].getTransmitter_1().getTransmitterId() == _transmitterId) && (
								_transmitterConnectionInfos[k].getTransmitter_2().getTransmitterId() == infos[j].getTransmitterId())) {
							tmpTransmitterConnectionInfo = _transmitterConnectionInfos[k];
							break;
						}
					}
					if(tmpTransmitterConnectionInfo != null) {
						short weight = tmpTransmitterConnectionInfo.getWeight();
						long time = tmpTransmitterConnectionInfo.getConnectionTimeThreshold();
						connectToTransmitter(infos[j], weight, time, tmpTransmitterConnectionInfo.getUserName());
					}
				}
			}
		}
	}

	/**
	 * Startet den Verbindungsaufbau zwischen zwei Datenverteilern. Falls keine Verbindung etabliert werden konnte, wird eine entsprechende Exception gefangen
	 *
	 * @param t_info   Information zum Datenverteiler
	 * @param weight   Die Information wird von der Wegverwaltung benutzt, wenn eine Verbindung bewertet wird.
	 * @param time     Zeitspanne in der versucht werden soll eine Verbindung aufzubauen, in Millisekunden. Maximale Wartezeit eine Sekunde.
	 * @param userName Benutzername mit dem die Authentifizierung durchgefhrt werden soll.
	 *
	 * @return true, wenn Verbindung hergestellt werden konnte; false, wenn Verbindung nicht hergestellt werden konnte.
	 *
	 * @see #connectToTransmitter(de.bsvrz.dav.daf.main.impl.config.telegrams.TransmitterInfo,short,String,String)
	 */
	private boolean connectToTransmitter(TransmitterInfo t_info, short weight, long time, String userName) {
		String password;
		if(userName.equals("")) {
			userName = _serverDavParameters.getUserName();
			password = _serverDavParameters.getUserPassword();
		}
		else {
			password = _serverDavParameters.getStoredPassword(userName);
			if(password == null) {
				_debug.error(
						"Passwort des Benutzers " + userName + " konnte nicht ermittelt werden. Es wird gebraucht fr Datenverteilerkopplung mit " + t_info
				);
				return false;
			}
		}
		_debug.info("Starte Datenverteilerkopplung als Benutzer " + userName + " zu ", t_info);
		_debug.finer(" time", time);
		_debug.finer(" weight", weight);
		long waitingTime = time;
		if(waitingTime < 1000) {
			waitingTime = 1000;
		}
		long startTime = System.currentTimeMillis();
		do {
			try {
				connectToTransmitter(t_info, weight, userName, password);
				return true;
			}
			catch(ConnectionException ex) {
				_debug.warning("Verbindung zum " + t_info + " konnte nicht aufgebaut werden", ex);
				try {
					Thread.sleep(1000);
				}
				catch(InterruptedException e) {
					return false;
				}
				waitingTime -= (System.currentTimeMillis() - startTime);
			}
			catch(CommunicationError ex) {
				_debug.warning("Verbindung zum " + t_info + " konnte nicht aufgebaut werden", ex);
				try {
					Thread.sleep(1000);
				}
				catch(InterruptedException e) {
					return false;
				}
				waitingTime -= (System.currentTimeMillis() - startTime);
			}
		}
		while(waitingTime > 0);
		return false;
	}

	/**
	 * Etabliert Verbindung zwischen zwei Datenverteilern. Falls keine Verbindung aufgebaut werden konnte, wird eine entsprechende Ausnahme geworfen.
	 *
	 * @param t_info   Informationen zum Datenverteiler.
	 * @param weight   Die Information wird von der Wegverwaltung benutzt, wenn eine Verbindung bewertet wird.
	 * @param userName Benutzername mit dem die Authentifizierung durchgefhrt werden soll.
	 * @param password Passwort des Benutzers mit dem die Authentifizierung durchgefhrt werden soll.
	 *
	 * @throws ConnectionException wenn eine Verbindung nicht aufgebaut werden konnte
	 * @throws CommunicationError  wenn bei der initialen Kommunikation mit dem Datenverteiler Fehler aufgetreten sind
	 */
	private void connectToTransmitter(TransmitterInfo t_info, short weight, final String userName, final String password)
			throws ConnectionException, CommunicationError {
		if(_transmittersServerConnection == null) {
			throw new IllegalArgumentException("Die Verwaltung ist nicht richtig initialisiert.");
		}

		long tId = t_info.getTransmitterId();

		int subAddressToConnectTo = t_info.getSubAdress();
		if(subAddressToConnectTo == 100000) {
			// Zu Testzwecken wird die Portnummer mit dem Wert 100000 serverseitig durch 8088 und clientseitig durch 8081 ersetzt
			subAddressToConnectTo = 8081;
		}
		subAddressToConnectTo += _serverDavParameters.getTransmitterConnectionsSubAddressOffset();

		for(T_T_HighLevelCommunication transmitterConnection : _transmitterConnections) {
			if(transmitterConnection != null) {
				if(transmitterConnection.getId() == tId) {
					return;
				}
				else {
					String adress = transmitterConnection.getRemoteAdress();
					int subAdress = transmitterConnection.getRemoteSubadress();
					if((adress != null) && (adress.equals(t_info.getAdress())) && (subAddressToConnectTo == subAdress)) {
						return;
					}
				}
			}
		}
		ConnectionInterface connection = _transmittersServerConnection.getPlainConnection();
		LowLevelCommunication lowLevelCommunication = new LowLevelCommunication(
				connection,
				_serverDavParameters.getCommunicationOutputBufferSize(),
				_serverDavParameters.getCommunicationInputBufferSize(),
				_serverDavParameters.getSendKeepAliveTimeout(),
				_serverDavParameters.getReceiveKeepAliveTimeout(),
				LowLevelCommunication.NORMAL_MODE,
				false
		);
		lowLevelCommunication.connect(t_info.getAdress(), subAddressToConnectTo);
		ServerConnectionProperties properties = new ServerConnectionProperties(
				lowLevelCommunication, _authentificationComponent, _serverDavParameters
		);
		T_T_HighLevelCommunication highLevelCommunication = new T_T_HighLevelCommunication(
				properties, subscriptionsManager, ConnectionsManager.this, bestWayManager, weight, !_configurationAvaillable, userName, password
		);
		highLevelCommunication.connect();
		_transmitterConnections.add(highLevelCommunication);
		highLevelCommunication.completeInitialisation();

		_debug.info("Verbindungsaufbau zum " + t_info + " war erfolgreich");
	}

	/**
	 * Erstellt ein Array, das die Informationen ber die benachbarten Datenverteiler des bergebenen Datenverteilers enthlt.
	 *
	 * @param t_info Information zum Datenverteiler
	 *
	 * @return Liste mit benachbarten Datenverteilern
	 */
	private TransmitterConnectionInfo[] getInvolvedTransmitters(TransmitterInfo t_info) {
		ArrayList list = new ArrayList();
		for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
			if((_transmitterConnectionInfos[i] == null) || _transmitterConnectionInfos[i].isExchangeConnection()) {
				continue;
			}
			TransmitterInfo t1 = _transmitterConnectionInfos[i].getTransmitter_1();
			if(t1.getTransmitterId() == t_info.getTransmitterId()) {
				TransmitterInfo t2 = _transmitterConnectionInfos[i].getTransmitter_2();
				if(t2 != null) {
					for(int j = 0; j < _transmitterConnectionInfos.length; ++j) {
						if(_transmitterConnectionInfos[j] == null) {
							continue;
						}
						if(_transmitterConnectionInfos[j].isExchangeConnection()) {
							TransmitterInfo _t1 = _transmitterConnectionInfos[j].getTransmitter_1();
							TransmitterInfo _t2 = _transmitterConnectionInfos[j].getTransmitter_2();
							if((_t1.getTransmitterId() == _transmitterId) && (_t2.getTransmitterId() == t2.getTransmitterId())) {
								list.add(_transmitterConnectionInfos[j]);
								break;
							}
						}
					}
				}
			}
		}
		int size = list.size();
		if(size == 0) {
			return null;
		}
		TransmitterConnectionInfo array[] = new TransmitterConnectionInfo[size];
		for(int i = 0; i < size; ++i) {
			array[i] = (TransmitterConnectionInfo)list.get(i);
		}
		return array;
	}

	/**
	 * Gibt die ID der Applikation zurck
	 *
	 * @param applicationTypePid die Pid des Applikationstyps
	 * @param applicationName    der Applikationsname
	 *
	 * @return die Applikation ID
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationChangeException
	 *          signalisiert Fehler bei Konfigurationsnderungen
	 */
	public long getApplicationId(String applicationTypePid, String applicationName) throws ConfigurationChangeException {
		if((applicationTypePid == null) || (applicationName == null)) {
			throw new IllegalArgumentException("bergabeparameter ist null");
		}
		if(applicationTypePid.equals(_clientDavParameters.getApplicationTypePid()) && applicationName.equals(_clientDavParameters.getApplicationName())) {
			return _transmitterId;
		}
		// Ask the configuration
		if(dataModel != null) {
			SystemObjectType type = dataModel.getType(applicationTypePid);
			if(type != null) {
				DynamicObject application = dataModel.createDynamicObject(type, "", applicationName);
				if(application != null) {
					return application.getId();
				}
			}
		}
		return -1L;
	}

	/**
	 * Gibt die ID der Konfiguration mit der gegebenen Pid zurck
	 *
	 * @param configurationPid Die Pid der Konfiguration
	 *
	 * @return die Id der Konfiguration
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationException signalisiert Fehler bei Konfigurationsnderungen
	 */
	public long getConfigurationId(String configurationPid) throws ConfigurationException {
		// if local configuration
		Object objects[] = _serverDavParameters.getLocalModeParameter();
		if(objects != null) {
			String _configurationPid = (String)objects[0];
			if(configurationPid.equals(_configurationPid)) {
				return ((Long)objects[1]).longValue();
			}
		}
		if(CommunicationConstant.LOCALE_CONFIGURATION_PID_ALIASE.equals(configurationPid)) {
			if(dataModel == null) {
				throw new IllegalStateException("Konfiguration ist noch nicht angemeldet.");
			}
			return _configurationId;
//			return dataModel.getConfigurationAuthorityId();
		}
		// Ask the configuration
		if(dataModel != null) {
			SystemObject configuration = dataModel.getObject(configurationPid);
			if(configuration != null) {
				return configuration.getId();
			}
		}
		return -1L;
	}

	/**
	 * Fragt die Konfiguration, ob der bergegebene User mit dem Passwort ein berechtigter Benutzer ist oder nicht.
	 *
	 * @param userName                Benutzername
	 * @param encriptedPassword       verschlsseltes Passwort
	 * @param authentificationText    Zufallstext
	 * @param authentificationProcess Namen des Authentifizierungsverfahrens
	 * @param typePid                 ID des Typs
	 *
	 * @return die Benutzerid wenn er berechtigt ist sonst -1
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationException
	 *          Fehler bei Konfigurationsnderungen
	 */
	public long isValidUser(
			String userName, byte[] encriptedPassword, String authentificationText, AuthentificationProcess authentificationProcess, String typePid)
			throws ConfigurationException {
		// if local configuration
		if(CommunicationConstant.CONFIGURATION_TYPE_PID.equals(typePid)) {
			if(userName.equals(_serverDavParameters.getConfigurationUserName())) {
				String password = _serverDavParameters.getConfigurationUserPassword();
				byte _encriptedPassword[] = authentificationProcess.encrypt(password, authentificationText);
				if(Arrays.equals(encriptedPassword, _encriptedPassword)) {
					return 0;
				}
			}
		}
		else if(CommunicationConstant.PARAMETER_TYPE_PID.equals(typePid)) {
			if(userName.equals(_serverDavParameters.getParameterUserName())) {
				String password = _serverDavParameters.getParameterUserPassword();
				byte _encriptedPassword[] = authentificationProcess.encrypt(password, authentificationText);
				if(Arrays.equals(encriptedPassword, _encriptedPassword)) {
					return 0;
				}
			}
		}
		else if(_serverDavParameters.isLocalMode()) {
			if(userName.equals(_clientDavParameters.getUserName())) {
				String password = _clientDavParameters.getUserPassword();
				byte _encriptedPassword[] = authentificationProcess.encrypt(password, authentificationText);
				if(Arrays.equals(encriptedPassword, _encriptedPassword)) {
					return _transmitterId;
				}
			}
		}
		// Ask the configuration
		if(dataModel != null) {
			ConfigurationManager configurationManager = ((DafDataModel)dataModel).getConfigurationManager();
			if(configurationManager == null) {
				throw new ConfigurationException("Der Konfigurationsmanager ist null");
			}
			else {

				// In diesem Fall wird die Klasse verschickt, die benutzt wurde um das Passwort zu verschlsseln.
				// Auf der Gegenseite mte dann genau diese Klasse verwendet werden, um ebenfalls das Passwort
				// zu verschlsseln.
				// Dies wurde entfernt (Achim Wullenkord), da das Verschlsslungsverfahren unverschlsselt bertragen
				// wird und somit Problemlos gendert werden knnte und dann auf der Gegenseite eine Klasse benutzt werden
				// knnte, die immer "true" zurckgibt.
				// Jetzt wird das Verfahren bertragen und die Konfiguration bestimmt selbst (ber eine Factory) welche
				// Klasse zum verschlsseln benutzt wird (das bertragenen Verschlsslungsverfahren bestimmt dabei die Klasse
				// die die Factory erzeugt)
//				long userId = configurationManager.isValidUser(
//				        userName, encriptedPassword, authentificationText, authentificationProcess.getClass().getName()
//				);

				// Es wird nicht die Klasse mit Klassennamen verschickt, sondern das angewandte Verfahren
				long userId = configurationManager.isValidUser(
						userName, encriptedPassword, authentificationText, authentificationProcess.getName()
				);

				if(userId > -1) {
					authentificationManager.addUser(userId);
				}
				return userId;
			}
		}
		return -1L;
	}

	/** Setzt dass die Konfigurationsverbindung erfolgreich hergestellt ist */
	public final void setLocalConfigurationAvaillable() {
		//configurationAvaillable = true;
		synchronized(_applicationConnections) {
			for(int i = 0; i < _applicationConnections.size(); ++i) {
				T_A_HighLevelCommunicationInterface connection = _applicationConnections.get(i);
				if(connection != null) {
					String applicationTypePid = connection.getApplicationTypePid();
					String applicationName = connection.getApplicationName();
					if((applicationTypePid == null) || (applicationName == null)) {
						continue;
					}
					if(applicationTypePid.equals(_clientDavParameters.getApplicationTypePid())
					   && applicationName.equals(_clientDavParameters.getApplicationName())) {
						connection.continueAuthentification();
						break;
					}
				}
			}
		}
	}

	/**
	 * Aktualisiert die Anmeldungen eines Datenverteiler oder einer Applikation bezglich der neuen Rechte.
	 *
	 * @param userId Identitt des Benutzers
	 */
	final synchronized void handleUserRightsChanged(long userId) {
		OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
		List applications = getApplicationConnections(userId);
		if((applications != null) && (applications.size() > 0)) {
			for(int j = applications.size() - 1; j > -1; --j) {
				T_A_HighLevelCommunication connection = (T_A_HighLevelCommunication)applications.get(j);
				if(connection != null) {
					long connectionId = connection.getId();
					T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)connection.getSubscriptionComponent();
					if(subscriptionComponent != null) {
						InAndOutSubscription subscriptions[] = subscriptionsManager.getSubscriptionsFromSource(connectionId);
						if((subscriptions == null) || (subscriptions.length == 0)) {
							continue;
						}
						for(int k = 0; k < subscriptions.length; ++k) {
							InAndOutSubscription subscription = subscriptions[k];
							if(subscription == null) {
								continue;
							}
							BaseSubscriptionInfo info = subscription.baseSubscriptionInfo;
							byte state = subscription.subscriptionState;
							if(state == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
								if(subscriptionComponent.canSend(info)) {
									state += 2;
								}
							}
							else if(state == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
								if(subscriptionComponent.canReceive(info)) {
									state += 2;
								}
							}

							boolean notMainTransmitter =
									(subscriptionsManager.getSendingComponent(info) == null) && (subscriptionsManager.getReceivingComponent(
											info
									) == null);

							byte newReceip = authentificationManager.isActionAllowed(userId, info, state);
							if(newReceip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
								byte oldReceip = subscription.receip;
								subscription.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT;
								if(oldReceip == subscription.receip) {
									continue;
								}
								if(subscription.subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
									ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.getReceiveSubscription(info);
									if(receiveSubscriptionInfo != null) {
										long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
										ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
												info, dataNumber, false, (byte)3, null, null, 1, 0, System.currentTimeMillis()
										);
										connection.sendData(dataToSend);
										subscriptionsManager.removeSubscriptionRequest(subscription);
										if(subscriptionComponent != null) {
											subscriptionComponent.unsubscribeReceiveData(subscription.baseSubscriptionInfo);
										}
									}
								}
								else {
									SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(info);
									if(sendSubscriptionInfo != null) {
										connection.triggerSender(info, (byte)2); // Stop sending. No rights for this action
										subscriptionsManager.removeSubscriptionRequest(subscription);
										if(subscriptionComponent != null) {
											subscriptionComponent.unsubscribeSendData(subscription.baseSubscriptionInfo);
										}
									}
								}
								cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
								if(subscription.outSubscriptions != null) {
									for(int h = 0; h < subscription.outSubscriptions.length; ++h) {
										if(subscription.outSubscriptions[h] != null) {
											subscriptionHelper.handleOutUnsubscription(subscription, subscription.outSubscriptions[h]);
										}
									}
									subscription.outSubscriptions = null;
									subscription.actualTransmitterToConsider = null;
								}
								InAndOutSubscription involvedSubscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
										subscription
								);
								if(involvedSubscriptions != null) {
									if(notMainTransmitter) { // not main transmitter
										for(int h = 0; h < involvedSubscriptions.length; ++h) {
											if((involvedSubscriptions[h] != null) && (involvedSubscriptions[h] != subscription)) {
												cleanPendingSubscriptionToRedirect(involvedSubscriptions[h], subscriptionHelper);
												workInvolvedSubscription(involvedSubscriptions[h], subscriptionHelper);
											}
										}
									}
								}
							}
							else {
								byte oldReceip = subscription.receip;
								subscription.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP;
								if(notMainTransmitter) { // not main transmitter
									cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
									workInvolvedSubscription(subscription, subscriptionHelper);
								}
								else {
									SubscriptionComponent sendingSubscriptionComponents[] = subscriptionsManager.getInterrestedSendingComponent(
											info
									);
									if(sendingSubscriptionComponents != null) {
										for(int i = 0; i < sendingSubscriptionComponents.length; ++i) {
											if((sendingSubscriptionComponents[i] != null) && (sendingSubscriptionComponents[i].getType()
											                                                  == SubscriptionComponent.T_A)) {
												SendSubscriptionInfo _sendSubscriptionInfo = ((T_A_SubscriptionComponent)sendingSubscriptionComponents[i]).getSendSubscription(
														info
												);
												if((_sendSubscriptionInfo != null) && (_sendSubscriptionInfo.isRequestSupported())) {
													if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
														T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)sendingSubscriptionComponents[i].getConnection();
														if(_connection != null) {
															_sendSubscriptionInfo.setLastTriggerRequest((byte)0);
															_connection.triggerSender(info, (byte)0);// start sending
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}

		List transmitters = getTransmitterConnections(userId);
		if((transmitters != null) && (transmitters.size() > 0)) {
			for(int j = transmitters.size() - 1; j > -1; --j) {
				T_T_HighLevelCommunication connection = (T_T_HighLevelCommunication)transmitters.get(j);
				if(connection != null) {
					long connectionId = connection.getId();
					InAndOutSubscription subscriptions[] = subscriptionsManager.getSubscriptionsFromSource(connectionId);
					if((subscriptions == null) || (subscriptions.length == 0)) {
						continue;
					}
					for(int k = 0; k < subscriptions.length; ++k) {
						InAndOutSubscription subscription = subscriptions[k];
						if(subscription == null) {
							continue;
						}
						BaseSubscriptionInfo info = subscription.baseSubscriptionInfo;
						boolean notMainTransmitter = (subscriptionsManager.getSendingComponent(info) == null) && (subscriptionsManager.getReceivingComponent(
								info
						) == null);
						byte newReceip = authentificationManager.isActionAllowed(userId, info, subscription.subscriptionState);
						if(newReceip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
							byte oldReceip = subscription.receip;
							subscription.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT;
							if(oldReceip == subscription.receip) {
								continue;
							}
							TransmitterDataSubscriptionReceipt noRightsReceipt = new TransmitterDataSubscriptionReceipt(
									info,
									subscription.subscriptionState,
									subscription.receip,
									subscription.mainTransmitterId,
									subscription.transmitterToConsider
							);
							connection.sendTelegram(noRightsReceipt);
							cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
							if(subscription.outSubscriptions != null) {
								for(int h = 0; h < subscription.outSubscriptions.length; ++h) {
									if(subscription.outSubscriptions[h] != null) {
										subscriptionHelper.handleOutUnsubscription(subscription, subscription.outSubscriptions[h]);
									}
								}
								subscription.outSubscriptions = null;
								subscription.actualTransmitterToConsider = null;
							}
							InAndOutSubscription involvedSubscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
									subscription
							);
							if(involvedSubscriptions != null) {
								if(notMainTransmitter) {
									for(int h = 0; h < involvedSubscriptions.length; ++h) {
										if((involvedSubscriptions[h] != null) && (involvedSubscriptions[h] != subscription)) {
											cleanPendingSubscriptionToRedirect(involvedSubscriptions[h], subscriptionHelper);
											workInvolvedSubscription(involvedSubscriptions[h], subscriptionHelper);
										}
									}
								}
							}
						}
						else {
							byte oldReceip = subscription.receip;
							subscription.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP;
							if(notMainTransmitter) { // not main transmitter
								cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
								workInvolvedSubscription(subscription, subscriptionHelper);
							}
							else {
								if(subscription.transmitterToConsider == null) {
									continue;
								}
								boolean found = false;
								for(int kk = 0; kk < subscription.transmitterToConsider.length; ++kk) {
									if(subscription.transmitterToConsider[kk] == _transmitterId) {
										found = true;
									}
								}
								if(found) {
									TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
											subscription.baseSubscriptionInfo,
											subscription.subscriptionState,
											TransmitterSubscriptionsConstants.POSITIV_RECEIP,
											_transmitterId,
											subscription.transmitterToConsider
									);
									T_T_HighLevelCommunication _connection = getTransmitterConnection(subscription.sourceTransmitterId);
									if(_connection != null) {
										_connection.sendTelegram(positivReceipt);
										if(oldReceip != TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
											T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
											if(_subscriptionComponent != null) {
												TransmitterDataSubscription transmitterDataSubscription = new TransmitterDataSubscription(
														subscription.baseSubscriptionInfo, subscription.subscriptionState, subscription.transmitterToConsider
												);
												if(subscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
													_subscriptionComponent.subscribeSendData(transmitterDataSubscription);
												}
												else {
													_subscriptionComponent.subscribeReceiveData(transmitterDataSubscription);
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		subscriptionHelper.flushOutUnSubscription(this);
	}

	/**
	 * Leitet die Anmeldungen an den beteiligten Datenverteiler weiter.
	 * <p/>
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen um ein Anmeldungstelegramm eines Datenverteilers abzuarbeiten. Ein Eintrag mit der
	 * Anmeldungsinformationen wird erstellt und in eine Verwaltungstabelle aufgenommen.
	 *
	 * @param connection   Schnittstelle "Datenverteiler-Datenverteiler"
	 * @param subscription Anmeldung von Daten des DAV fr Empfnger oder Sender
	 */
	public final synchronized void handleTransmitterSubscription(
			T_T_HighLevelCommunicationInterface connection, TransmitterDataSubscription subscription) {
		byte subscriptionState = subscription.getSubscriptionState();
		BaseSubscriptionInfo info = subscription.getBaseSubscriptionInfo();
		long ids[] = subscription.getTransmitters();
		if(ids == null) {
			TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
					info, subscriptionState, TransmitterSubscriptionsConstants.NEGATIV_RECEIP, -1, ids
			);
			connection.sendTelegram(negativReceipt);
		}
		else {
			long sourceTransmitter = connection.getId();
			long sourceUserId = connection.getRemoteUserId();
			InAndOutSubscription request = new InAndOutSubscription(sourceTransmitter, subscription);
			// if there is a subscription posted to the source then unsubscribe it
			InAndOutSubscription postedSubscriptions[] = subscriptionsManager.getSameSubscriptionPostedTo(
					info, subscriptionState, sourceTransmitter, ids
			);
			if(postedSubscriptions != null) {
				for(int i = 0; i < postedSubscriptions.length; ++i) {
					if((postedSubscriptions[i] != null) && (postedSubscriptions[i].outSubscriptions != null)) {
						cleanPendingSubscriptionToRedirect(postedSubscriptions[i]);
						for(int j = 0; j < postedSubscriptions[i].outSubscriptions.length; ++j) {
							if(postedSubscriptions[i].outSubscriptions[j] != null) {
								TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
										postedSubscriptions[i].baseSubscriptionInfo,
										postedSubscriptions[i].subscriptionState,
										postedSubscriptions[i].outSubscriptions[j].transmitterToConsider
								);
								T_T_HighLevelCommunication _connection = getTransmitterConnection(
										postedSubscriptions[i].outSubscriptions[j].targetTransmitterId
								);
								if(_connection != null) {
									_connection.sendTelegram(_unsubscription);
								}
							}
						}
						postedSubscriptions[i].outSubscriptions = null;
						postedSubscriptions[i].actualTransmitterToConsider = null;
						postedSubscriptions[i].receip = -1;
					}
				}
			}
			else {
				for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
					ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
							i
					);
					if(exchangeSubscriptionAction != null) {
						InAndOutSubscription newRequest = exchangeSubscriptionAction.newSubscription;
						if((newRequest != null) && (newRequest.baseSubscriptionInfo.equals(info)) && (newRequest.outSubscriptions != null)) {
							for(int j = 0; j < newRequest.outSubscriptions.length; ++j) {
								if(newRequest.outSubscriptions[j] != null) {
									if(newRequest.outSubscriptions[j].targetTransmitterId == sourceTransmitter) {
										if(newRequest.outSubscriptions[j].transmitterToConsider.length == ids.length) {
											boolean subSet = true;
											for(int k = 0; k < ids.length; ++k) {
												boolean found = false;
												for(int h = 0; h < newRequest.outSubscriptions[j].transmitterToConsider.length; ++h) {
													if(newRequest.outSubscriptions[j].transmitterToConsider[h] == ids[k]) {
														found = true;
														break;
													}
												}
												if(!found) {
													subSet = false;
													break;
												}
											}
											if(subSet) {
												cleanPendingSubscriptionToRedirect(exchangeSubscriptionAction.oldSubscription);
												break;
											}
										}
									}
								}
							}
						}
					}
				}
			}
			InAndOutSubscription _request = subscriptionsManager.addSubscriptionRequest(request, _transmitterId);
			if(_request != request) {
				if(_request.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
					TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
							info, subscriptionState, TransmitterSubscriptionsConstants.POSITIV_RECEIP, _request.mainTransmitterId, ids
					);
					connection.sendTelegram(positivReceipt);
				}
				else if(_request.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
					TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
							info, subscriptionState, TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT, _request.mainTransmitterId, ids
					);
					connection.sendTelegram(positivReceipt);
				}
				else if(_request.receip == TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP) {
					TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
							info, subscriptionState, TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP, _request.mainTransmitterId, ids
					);
					connection.sendTelegram(positivReceipt);
				}
				else if(_request.receip == TransmitterSubscriptionsConstants.NEGATIV_RECEIP) {
					TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
							info, subscriptionState, TransmitterSubscriptionsConstants.NEGATIV_RECEIP, -1, ids
					);
					connection.sendTelegram(negativReceipt);
				}
				return;
			}

			boolean found = false;
			for(int i = 0; i < ids.length; ++i) {
				if(ids[i] == _transmitterId) {
					found = true;
					break;
				}
			}
			// Main transmitter
			if(found) {
				T_A_SubscriptionComponent subscriptionComponent = subscriptionsManager.getReceivingComponent(info);
				if(subscriptionComponent == null) {
					subscriptionComponent = subscriptionsManager.getSendingComponent(info);
				}
				if(subscriptionComponent != null) {
					byte receip = authentificationManager.isActionAllowed(sourceUserId, info, subscriptionState);
					TransmitterDataSubscriptionReceipt receiptTelegram = new TransmitterDataSubscriptionReceipt(
							info, subscriptionState, receip, _transmitterId, ids
					);
					connection.sendTelegram(receiptTelegram);
					request.receip = receip;
					if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
						request.mainTransmitterId = _transmitterId;
						if(!subscriptionsManager.isSubscriptionRequestAvaillable(request)) {
							T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)connection.getSubscriptionComponent();
							if(_subscriptionComponent != null) {
								if(subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending ?
									_subscriptionComponent.subscribeSendData(subscription);
								}
								else { // receiving ?
									_subscriptionComponent.subscribeReceiveData(subscription);
									SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(info);
									if(sendSubscriptionInfo != null) {
										if(sendSubscriptionInfo.isRequestSupported()) {
											if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
												T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)subscriptionComponent.getConnection();
												if(_connection != null) {
													if(authentificationManager.isActionAllowed(
															_connection.getRemoteUserId(), info, subscriptionComponent.canSend(info) ? (byte)0 : (byte)2
													) == 1) {
														sendSubscriptionInfo.setLastTriggerRequest((byte)0);
														_connection.triggerSender(info, (byte)0);
													}
												}
											}
										}
									}
									TransmitterDataTelegram dataToSend[] = cacheManager.getCurrentDataForTransmitter(info);
									if(dataToSend != null) {
										connection.sendTelegrams(dataToSend);
									}
								}
							}
						}
					}
					return;
				}
			}
			InAndOutSubscription inAndOutSubscription = subscriptionsManager.isSuccessfullySubscribed(
					info, subscriptionState, _transmitterId, ids
			);
			byte receip = 0;
			if(inAndOutSubscription != null) {
				receip = inAndOutSubscription.receip;
			}
			if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
				request.receip = receip;
				request.mainTransmitterId = inAndOutSubscription.mainTransmitterId;
				TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
						info, subscriptionState, receip, inAndOutSubscription.mainTransmitterId, ids
				);
				connection.sendTelegram(positivReceipt);
				if(!subscriptionsManager.isSubscriptionRequestAvaillable(request)) {
					T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)connection.getSubscriptionComponent();
					if(_subscriptionComponent != null) {
						if(subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending
							_subscriptionComponent.subscribeSendData(subscription);
						}
						else { // receiving
							_subscriptionComponent.subscribeReceiveData(subscription);
							TransmitterDataTelegram dataToSend[] = cacheManager.getCurrentDataForTransmitter(info);
							if(dataToSend != null) {
								connection.sendTelegrams(dataToSend);
							}
						}
					}
				}
			}
			else if(subscriptionsManager.isUnsuccessfullySubscribed(info, subscriptionState, _transmitterId, ids)) {
				TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
						info, subscriptionState, TransmitterSubscriptionsConstants.NEGATIV_RECEIP, -1, ids
				);
				connection.sendTelegram(negativReceipt);
				request.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
				request.mainTransmitterId = -1;
			}
			else {
				makeOutgoingRequests(request);
				request.mainTransmitterId = -1;
				if(request.outSubscriptions == null) {
					request.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
					TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
							info, subscriptionState, TransmitterSubscriptionsConstants.NEGATIV_RECEIP, -1, ids
					);
					connection.sendTelegram(negativReceipt);
				}
				else if(subscriptionsManager.needToSendSubscription(_transmitterId, request)) {
					request.receip = (byte)-1;
					for(int j = 0; j < request.outSubscriptions.length; ++j) {
						if(request.outSubscriptions[j] != null) {
							TransmitterDataSubscription _subscription = new TransmitterDataSubscription(
									info, subscriptionState, request.outSubscriptions[j].transmitterToConsider
							);
							T_T_HighLevelCommunication _connection = getTransmitterConnection(
									request.outSubscriptions[j].targetTransmitterId
							);
							if(_connection != null) {
								_connection.sendTelegram(_subscription);
							}
						}
					}
				}
				else {
					request.outSubscriptions = null;
				}
			}
		}
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, um ein Abmeldungstelegramm eines Datenverteilers zu bearbeiten.Der Eintrag mit den
	 * Abmeldungsinformationen wird aus der Verwaltungstabelle entfernt. Wenn ein anderer Eintrag mit den gleichen Anmeldeinformationen vorhanden ist, werden
	 * diesem die Anmeldeinformationen des abzumeldenden Eintrags u?bertragen. Diese Informationen geben unter anderem Aufschluss daru?ber, zu welchem anderen
	 * Datenverteiler eine Folgeanmeldung gesendet wurde. Auch wenn eine Umleitung fr den abzumeldenden Ein-trag besteht, muss diese durch den neuen Eintrag
	 * ersetzt werden.
	 *
	 * @param connection     Schnittstelle "Datenverteiler-Datenverteiler"
	 * @param unsubscription Abmeldung von Daten des DAV fr Empfnger oder Sender
	 */
	public final synchronized void handleTransmitterUnsubscription(
			T_T_HighLevelCommunicationInterface connection, TransmitterDataUnsubscription unsubscription) {
		long sourceTransmitterId = connection.getId();
		byte subscriptionState = unsubscription.getSubscriptionState();
		BaseSubscriptionInfo info = unsubscription.getBaseSubscriptionInfo();
		long transmitterIds[] = unsubscription.getTransmitters();
		if((info != null) && (transmitterIds != null)) {
			InAndOutSubscription subscription = subscriptionsManager.removeSubscriptionRequest(
					sourceTransmitterId, info, subscriptionState, transmitterIds, _transmitterId
			);
			if(subscription != null) {
				boolean done = false;
				if(subscription.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
					InAndOutSubscription alternativeSubscription = subscriptionsManager.getAlternativeSubscriptionRequest(
							subscription
					);
					if(alternativeSubscription != null) {
						alternativeSubscription.outSubscriptions = subscription.outSubscriptions;
						alternativeSubscription.receip = subscription.receip;
						alternativeSubscription.mainTransmitterId = subscription.mainTransmitterId;
						alternativeSubscription.transmitterToConsider = subscription.transmitterToConsider;
						alternativeSubscription.actualTransmitterToConsider = subscription.actualTransmitterToConsider;
						for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
							ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
									i
							);
							if(exchangeSubscriptionAction != null) {
								if(exchangeSubscriptionAction.oldSubscription == subscription) {
									exchangeSubscriptionAction.oldSubscription = alternativeSubscription;
								}
							}
						}
						done = true;
					}
					if(!subscriptionsManager.isSubscriptionRequestAvaillable(subscription)) {
						T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)connection.getSubscriptionComponent();
						if(_subscriptionComponent != null) {
							if(subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
								_subscriptionComponent.unsubscribeSendData(info);
							}
							else {
								_subscriptionComponent.unsubscribeReceiveData(info);
							}
						}
					}
				}

				// no need to process: an alternative subscription exists
				if(done) {
					return;
				}

				OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
				cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
				if(subscription.outSubscriptions != null) {
					for(int i = 0; i < subscription.outSubscriptions.length; ++i) {
						if(subscription.outSubscriptions[i] != null) {
							subscriptionHelper.handleOutUnsubscription(subscription, subscription.outSubscriptions[i]);
						}
					}
					subscription.outSubscriptions = null;
					subscription.actualTransmitterToConsider = null;
				}
				if((subscriptionsManager.getSendingComponent(info) == null) && (subscriptionsManager.getReceivingComponent(info)
				                                                                == null)) { // not main transmitter
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(subscription);
					if(subscriptions != null) {
						for(int i = 0; i < subscriptions.length; ++i) {
							if(subscriptions[i] != null) {
								workInvolvedSubscription(subscriptions[i], subscriptionHelper);
							}
						}
					}
				}
				else {
					SubscriptionComponent receivingSubscriptionComponents[] = subscriptionsManager.getInterrestedReceivingComponent(
							info
					);
					SubscriptionComponent sendingSubscriptionComponents[] = subscriptionsManager.getInterrestedSendingComponent(info);
					if((receivingSubscriptionComponents == null) && (sendingSubscriptionComponents != null)) { // no one interrested
						for(int i = 0; i < sendingSubscriptionComponents.length; ++i) {
							if((sendingSubscriptionComponents[i] != null) && (sendingSubscriptionComponents[i].getType() == SubscriptionComponent.T_A)) {
								SendSubscriptionInfo _sendSubscriptionInfo = ((T_A_SubscriptionComponent)sendingSubscriptionComponents[i]).getSendSubscription(
										info
								);
								if((_sendSubscriptionInfo != null) && (_sendSubscriptionInfo.isRequestSupported())) {
									if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
										T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)sendingSubscriptionComponents[i].getConnection();
										if(_connection != null) {
											_sendSubscriptionInfo.setLastTriggerRequest((byte)1);
											_connection.triggerSender(info, (byte)1);// stop sending
										}
									}
								}
							}
						}
					}
					else if((sendingSubscriptionComponents == null) && (receivingSubscriptionComponents != null)) { // no one interrested
						long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
						ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
								info, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
						);
						for(int i = 0; i < receivingSubscriptionComponents.length; ++i) {
							if((receivingSubscriptionComponents[i] != null) && (receivingSubscriptionComponents[i].getType() == SubscriptionComponent.T_A)) {
								ReceiveSubscriptionInfo _receiveSubscriptionInfo = ((T_A_SubscriptionComponent)receivingSubscriptionComponents[i]).getReceiveSubscription(
										info
								);
								if(_receiveSubscriptionInfo != null) {
									if(_receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
										if(_receiveSubscriptionInfo.getLastErrorState() == 2) {
											continue;
										}
									}
									T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)receivingSubscriptionComponents[i].getConnection();
									if(_connection != null) {
										_receiveSubscriptionInfo.setLastDataIndex(dataNumber);
										_receiveSubscriptionInfo.setLastErrorState((byte)2);
										_connection.sendData(dataToSend);
									}
								}
							}
						}
					}
				}
				subscriptionHelper.flushOutUnSubscription(this);

				// if no one interrested delete cached Data
				if(!subscriptionsManager.isSubscriptionRequestAvaillable(info)) {
					cacheManager.clean(info);
				}
			}
		}
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, um eine anstehende Anmeldung zu quittieren.
	 *
	 * @param connection      Schnittstelle "Datenverteiler-Datenverteiler"
	 * @param receiptTelegram Quittung des DAV fr eine an ihn gerichtete Datenanmeldung.
	 */
	public final synchronized void handleTransmitterSubscriptionReceip(
			T_T_HighLevelCommunicationInterface connection, TransmitterDataSubscriptionReceipt receiptTelegram) {
		long targetId = connection.getId();
		InAndOutSubscription subscriptions[] = subscriptionsManager.getSubscriptionRequestToUpdate(targetId, receiptTelegram);
		if(subscriptions == null) {
			handleTransmitterSubscriptionReceip(
					null,
					targetId,
					receiptTelegram.getBaseSubscriptionInfo(),
					receiptTelegram.getMainTransmitterId(),
					receiptTelegram.getTransmitters(),
					receiptTelegram.getSubscriptionState(),
					receiptTelegram.getReceipt()
			);
		}
		else {
			for(int i = 0; i < subscriptions.length; ++i) {
				handleTransmitterSubscriptionReceip(
						subscriptions[i],
						targetId,
						receiptTelegram.getBaseSubscriptionInfo(),
						receiptTelegram.getMainTransmitterId(),
						receiptTelegram.getTransmitters(),
						receiptTelegram.getSubscriptionState(),
						receiptTelegram.getReceipt()
				);
			}
			handleTransmitterSubscriptionReceip(
					null,
					targetId,
					receiptTelegram.getBaseSubscriptionInfo(),
					receiptTelegram.getMainTransmitterId(),
					receiptTelegram.getTransmitters(),
					receiptTelegram.getSubscriptionState(),
					receiptTelegram.getReceipt()
			);
		}
	}

	/**
	 * Hilfsmethode zu <code>final synchronized void handleTransmitterSubscriptionReceip(T_T_HighLevelCommunication connection, TransmitterDataSubscriptionReceipt
	 * receipTelegram) </code>
	 *
	 * @param subscription      Anmeldung TBD weitere Beschreibung, da InAndOutSubscription noch nicht kommentiert
	 * @param targetId          Id des Kommunikationpartners
	 * @param info              Datenidentifikation
	 * @param mainTransmitterId Identitt des Haupt-DAV
	 * @param transmitterIds    Liste von erreichbaren transmittern
	 * @param subscriptionState Status der Anmeldung
	 * @param receip            Status der Quittung
	 *
	 * @see #handleTransmitterSubscriptionReceip(de.bsvrz.dav.dav.communication.davProtocol.T_T_HighLevelCommunicationInterface, de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterDataSubscriptionReceipt)
	 */
	final void handleTransmitterSubscriptionReceip(
			InAndOutSubscription subscription,
			long targetId,
			BaseSubscriptionInfo info,
			long mainTransmitterId,
			long transmitterIds[],
			byte subscriptionState,
			byte receip) {
		if(Transmitter._debugLevel > 20) {
			System.err.println("subscription = " + subscription);
			System.err.println("targetId = " + targetId);
			System.err.println("info = " + info);
			System.err.println("mainTransmitterId = " + mainTransmitterId);
//			System.err.println("transmitterIds = " + Arrays.asList(transmitterIds));
			System.err.println("subscriptionState = " + subscriptionState);
			System.err.println("receip = " + receip);
		}
		if(subscription == null) {
			if(Transmitter._debugLevel > 20) {
				System.err.println("pendingSubscriptionToRedirect.size () = " + _pendingSubscriptionToRedirect.size());
			}
			for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
				ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
						i
				);
				if(exchangeSubscriptionAction != null) {
					if(subscriptionsManager.updateExchangeSubscriptionRequest(
							exchangeSubscriptionAction.newSubscription, targetId, info, transmitterIds, mainTransmitterId, subscriptionState, receip
					)) {
						InAndOutSubscription request = exchangeSubscriptionAction.newSubscription;
						if(request != null) {
							if(receip == TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP) {
								_pendingSubscriptionToRedirect.remove(i);
								InAndOutSubscription oldSubscription = exchangeSubscriptionAction.oldSubscription;
								InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
										oldSubscription.baseSubscriptionInfo
								);
								if(subscriptions != null) {
									for(int j = 0; j < subscriptions.length; ++j) {
										if(subscriptions[j] != null) {
											handleInvalidSubscription(subscriptions[j], mainTransmitterId);
										}
									}
								}
								return;
							}

							if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
								request.mainTransmitterId = mainTransmitterId;
							}

							boolean allReceipted = true;
							int postiveResponces = 0;
							for(int j = 0; j < request.outSubscriptions.length; ++j) {
								if(request.outSubscriptions[j] != null) {
									if(request.outSubscriptions[j].receip == -1) {
										allReceipted = false;
										break;
									}
									else if(request.outSubscriptions[j].receip == 1) {
										++postiveResponces;
									}
								}
							}

							if(allReceipted) {
								if(postiveResponces == 1) {
									request.receip = receip;
									if(request.outSubscriptions != null) {
										for(int j = request.outSubscriptions.length - 1; j > -1; --j) {
											if((request.outSubscriptions[j] != null) && (request.outSubscriptions[j].receip != 1)) {
												TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
														request.baseSubscriptionInfo,
														request.subscriptionState,
														request.outSubscriptions[j].transmitterToConsider
												);
												T_T_HighLevelCommunication _connection = getTransmitterConnection(
														request.outSubscriptions[j].targetTransmitterId
												);
												if(_connection != null) {
													_connection.sendTelegram(_unsubscription);
												}
												request.outSubscriptions[j] = null;
											}
										}
									}
									if(exchangeSubscriptionAction.isRedirectionSafe()) {// sender or source
										_pendingSubscriptionToRedirect.remove(i);
										InAndOutSubscription oldSubscription = exchangeSubscriptionAction.oldSubscription;
										if(oldSubscription != null) {
											for(int j = 0; j < oldSubscription.outSubscriptions.length; ++j) {
												if(oldSubscription.outSubscriptions[j] != null) {
													TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
															oldSubscription.baseSubscriptionInfo,
															oldSubscription.subscriptionState,
															oldSubscription.outSubscriptions[j].transmitterToConsider
													);
													T_T_HighLevelCommunication _connection = getTransmitterConnection(
															oldSubscription.outSubscriptions[j].targetTransmitterId
													);
													if(_connection != null) {
														_connection.sendTelegram(_unsubscription);
													}
												}
											}
											oldSubscription.receip = exchangeSubscriptionAction.newSubscription.receip;
											oldSubscription.mainTransmitterId = exchangeSubscriptionAction.newSubscription.mainTransmitterId;
											oldSubscription.actualTransmitterToConsider = exchangeSubscriptionAction.newSubscription.actualTransmitterToConsider;
											oldSubscription.outSubscriptions = exchangeSubscriptionAction.newSubscription.outSubscriptions;
										}
									}
								}
								else if(postiveResponces > 1) {
									_pendingSubscriptionToRedirect.remove(i);
									InAndOutSubscription oldSubscription = exchangeSubscriptionAction.oldSubscription;
									InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
											oldSubscription.baseSubscriptionInfo
									);
									if(subscriptions != null) {
										for(int j = 0; j < subscriptions.length; ++j) {
											if(subscriptions[j] != null) {
												handleInvalidSubscription(subscriptions[j], mainTransmitterId);
											}
										}
									}
								}
								else {
									_pendingSubscriptionToRedirect.remove(i);
									InAndOutSubscription newSubscription = exchangeSubscriptionAction.newSubscription;
									if(newSubscription != null) {
										for(int j = 0; j < newSubscription.outSubscriptions.length; ++j) {
											if(newSubscription.outSubscriptions[j] != null) {
												TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
														newSubscription.baseSubscriptionInfo,
														newSubscription.subscriptionState,
														newSubscription.outSubscriptions[j].transmitterToConsider
												);
												T_T_HighLevelCommunication _connection = getTransmitterConnection(
														newSubscription.outSubscriptions[j].targetTransmitterId
												);
												if(_connection != null) {
													_connection.sendTelegram(_unsubscription);
												}
											}
										}
									}
								}
							}
						}
						return;
					}
				}
			}
		}
		else {
			if(receip == TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP) {
				InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
						subscription.baseSubscriptionInfo
				);
				if(subscriptions != null) {
					if(Transmitter._debugLevel > 20) {
						System.err.println("E subscriptions = " + Arrays.asList(subscriptions));
					}
					for(int i = 0; i < subscriptions.length; ++i) {
						if(subscriptions[i] != null) {
							handleInvalidSubscription(subscriptions[i], mainTransmitterId);
						}
					}
				}
				return;
			}

			boolean allReceipted = true;
			byte _receip = -1;
			int postiveResponces = 0;
			for(int j = 0; j < subscription.outSubscriptions.length; ++j) {
				if(subscription.outSubscriptions[j] != null) {
					if(subscription.outSubscriptions[j].receip == -1) {
						allReceipted = false;
						break;
					}
					else if(subscription.outSubscriptions[j].receip == 1) {
						++postiveResponces;
						_receip = _receip < 1 ? 1 : _receip;
					}
					else if(subscription.outSubscriptions[j].receip == 2) {
						++postiveResponces;
						_receip = _receip < 2 ? 2 : _receip;
					}
					else if(subscription.outSubscriptions[j].receip == 3) {
						++postiveResponces;
						_receip = _receip < 3 ? 3 : _receip;
					}
				}
			}

			if(Transmitter._debugLevel > 20) {
				System.err.println("allReceipted = " + allReceipted);
				System.err.println("_receip = " + _receip);
				System.err.println("receip = " + receip);
				System.err.println("postiveResponces = " + postiveResponces);
			}
			if(allReceipted) {
				if(postiveResponces == 1) {
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(subscription, true);
					if(subscriptions != null) {
						if(Transmitter._debugLevel > 20) {
							System.err.println("C subscriptions = " + Arrays.asList(subscriptions));
						}
						for(int i = 0; i < subscriptions.length; ++i) {
							if(subscriptions[i] != null) {
								handlePositiveReceip(subscriptions[i], _receip, mainTransmitterId);
							}
						}
					}
				}
				else if(postiveResponces > 1) {
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
							subscription.baseSubscriptionInfo
					);
					if(subscriptions != null) {
						if(Transmitter._debugLevel > 20) {
							System.err.println("D subscriptions = " + Arrays.asList(subscriptions));
						}
						for(int i = 0; i < subscriptions.length; ++i) {
							if(subscriptions[i] != null) {
								handleInvalidSubscription(subscriptions[i], mainTransmitterId);
							}
						}
					}
				}
				else { // negativ receip
					if(Transmitter._debugLevel > 20) {
						System.err.println("#######################################################################");
						System.err.println("subscription.receip= " + subscription.receip);
						System.err.println("receip= " + receip);
					}
					if(subscription.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
						// rs: von einem anderen DAV negative Quittung zu einer vorher positiv quittierten Anmeldung
						// rs: erhalten, also als abgemeldet vermerken
						if(subscription.isTransmitterConnection) {
							T_T_HighLevelCommunication _connection = getTransmitterConnection(subscription.sourceTransmitterId);
							if(_connection != null) {
								if(subscription.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
									if(!subscriptionsManager.isSubscriptionRequestAvaillable(subscription)) {
										T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
										if(_subscriptionComponent != null) {
											if(subscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
												if(Transmitter._debugLevel > 20) System.err.println("unsubscribeSendData");
												_subscriptionComponent.unsubscribeSendData(subscription.baseSubscriptionInfo);
											}
											else {
												if(Transmitter._debugLevel > 20) System.err.println("unsubscribeReceiveData");
												_subscriptionComponent.unsubscribeReceiveData(subscription.baseSubscriptionInfo);
											}
										}
									}
								}
							}
						}
						InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedPositiveSubscriptionRequests(
								subscription
						);
						if(subscriptions != null) {
							if(Transmitter._debugLevel > 20) {
								System.err.println("A subscriptions = " + Arrays.asList(subscriptions));
							}
							OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
							for(int i = 0; i < subscriptions.length; ++i) {
								if(subscriptions[i] != null) {
									cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
									workInvolvedSubscription(subscriptions[i], subscriptionHelper);
								}
							}
							subscriptionHelper.flushOutUnSubscription(this);
						}
						//  Logik (else) invertiert: Folgender Block sorgt fr "keine Quelle" Datenstze wenn ein anderer DAV nicht mehr liefern kann
						//}
						//else {
						//	InAndOutSubscription[] subscriptions;
						subscriptions = subscriptionsManager.getInvolvedSubscriptionRequests(subscription, true);
						if(subscriptions != null) {
							if(Transmitter._debugLevel > 20) {
								System.err.println("B subscriptions = " + Arrays.asList(subscriptions));
							}
							for(int i = 0; i < subscriptions.length; ++i) {
								if(subscriptions[i] != null) {
									handleNegativeReceip(subscriptions[i]);
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Hilfsmethode zu <code>final void handleTransmitterSubscriptionReceip(InAndOutSubscription subscription, long targetId, BaseSubscriptionInfo info, long
	 * mainTransmitterId, long transmitterIds[],byte subscriptionState,byte receip)</code>. Behandelt eine negative Quittung einer Anmeldung.
	 *
	 * @param subscription Anmeldung TBD weitere Beschreibung
	 *
	 * @see #handleTransmitterSubscriptionReceip(InAndOutSubscription,long,de.bsvrz.dav.daf.communication.lowLevel.telegrams.BaseSubscriptionInfo,long,long[],byte,byte)
	 */
	private final void handleNegativeReceip(InAndOutSubscription subscription) {
		if(Transmitter._debugLevel > 20) System.err.println("++handleNegativeReceip subscription = " + subscription);
		if(subscription == null) {
			return;
		}
		if(subscription.receip == TransmitterSubscriptionsConstants.NEGATIV_RECEIP) {
			return;
		}
		subscription.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
		subscription.mainTransmitterId = -1;
		if(subscription.isTransmitterConnection) {
			TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
					subscription.baseSubscriptionInfo,
					subscription.subscriptionState,
					TransmitterSubscriptionsConstants.NEGATIV_RECEIP,
					-1L,
					subscription.transmitterToConsider
			);
			T_T_HighLevelCommunication _connection = getTransmitterConnection(subscription.sourceTransmitterId);
			if(_connection != null) {
				_connection.sendTelegram(negativReceipt);
			}
		}
		else {
			T_A_HighLevelCommunication _connection = getApplicationConnection(subscription.sourceTransmitterId);
			if(_connection != null) {
				T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
				if(subscriptionComponent != null) {
					if(subscription.subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
						ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.getReceiveSubscription(
								subscription.baseSubscriptionInfo
						);
						if(receiveSubscriptionInfo != null) {
							long dataNumber = cacheManager.getCurrentDataIndex(subscription.baseSubscriptionInfo) + 1;
							if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
								if(receiveSubscriptionInfo.getLastErrorState() == 2) {
									return;
								}
							}
							ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
									subscription.baseSubscriptionInfo, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
							);
							receiveSubscriptionInfo.setLastDataIndex(dataNumber);
							receiveSubscriptionInfo.setLastErrorState((byte)2);
							_connection.sendData(dataToSend);
						}
					}
					else {
						SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(
								subscription.baseSubscriptionInfo
						);
						if(sendSubscriptionInfo != null) {
							if(sendSubscriptionInfo.isRequestSupported()) {
								if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
									sendSubscriptionInfo.setLastTriggerRequest((byte)1);
									_connection.triggerSender(subscription.baseSubscriptionInfo, (byte)1); // stop Sending no one interrested
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Hilfsmehode zu <code>private final void handleNegativeReceip(...) </code> Behandelt eine mehr als einmal positiv quittierte Anmeldung
	 *
	 * @param subscription      Anmeldung TBD weitere Beschreibung, da InAndOutSubscription noch nicht kommentiert
	 * @param mainTransmitterId ID des Zulieferer-Datenverteiler
	 *
	 * @see #handleTransmitterSubscriptionReceip(InAndOutSubscription,long,de.bsvrz.dav.daf.communication.lowLevel.telegrams.BaseSubscriptionInfo,long,long[],byte,byte)
	 */
	private void handleInvalidSubscription(InAndOutSubscription subscription, long mainTransmitterId) {
		if(subscription == null) {
			return;
		}

		if(subscription.receip == TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP) {
			return;
		}
		if(subscription.outSubscriptions != null) {
			for(int j = subscription.outSubscriptions.length - 1; j > -1; --j) {
				if(subscription.outSubscriptions[j] != null) {
					TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
							subscription.baseSubscriptionInfo, subscription.subscriptionState, subscription.outSubscriptions[j].transmitterToConsider
					);
					T_T_HighLevelCommunication _connection = getTransmitterConnection(
							subscription.outSubscriptions[j].targetTransmitterId
					);
					if(_connection != null) {
						_connection.sendTelegram(_unsubscription);
					}
				}
			}
		}
		subscription.outSubscriptions = null;
		subscription.actualTransmitterToConsider = null;

		subscription.receip = TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP;
		subscription.mainTransmitterId = mainTransmitterId;
		if(subscription.isTransmitterConnection) {
			TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
					subscription.baseSubscriptionInfo,
					subscription.subscriptionState,
					TransmitterSubscriptionsConstants.MORE_THAN_ONE_POSITIV_RECEIP,
					subscription.mainTransmitterId,
					subscription.transmitterToConsider
			);
			T_T_HighLevelCommunication _connection = getTransmitterConnection(subscription.sourceTransmitterId);
			if(_connection != null) {
				_connection.sendTelegram(positivReceipt);
			}
		}
		else {
			T_A_HighLevelCommunication _connection = getApplicationConnection(subscription.sourceTransmitterId);
			if(_connection != null) {
				T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
				if(subscriptionComponent != null) {
					if(subscription.subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
						ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.getReceiveSubscription(
								subscription.baseSubscriptionInfo
						);
						if(receiveSubscriptionInfo != null) {
							long dataNumber = cacheManager.getCurrentDataIndex(subscription.baseSubscriptionInfo) + 1;
							ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
									subscription.baseSubscriptionInfo, dataNumber, false, (byte)8, null, null, 1, 0, System.currentTimeMillis()
							);
							_connection.sendData(dataToSend);
							subscriptionsManager.removeSubscriptionRequest(subscription);
							if(subscriptionComponent != null) {
								subscriptionComponent.unsubscribeReceiveData(subscription.baseSubscriptionInfo);
							}
						}
					}
					else {
						SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(
								subscription.baseSubscriptionInfo
						);
						if(sendSubscriptionInfo != null) {
							_connection.triggerSender(subscription.baseSubscriptionInfo, (byte)3);
							subscriptionsManager.removeSubscriptionRequest(subscription);
							if(subscriptionComponent != null) {
								subscriptionComponent.unsubscribeSendData(subscription.baseSubscriptionInfo);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Hilfsmethode zur Behandelung einer positive Quittung einer Anmeldung.
	 *
	 * @param subscription      Anmeldung TBD weitere Beschreibung, da InAndOutSubscription noch nicht kommentiert
	 * @param receip            Status der Quittung
	 * @param mainTransmitterId ID des Zulieferer-Datenverteiler
	 *
	 * @see #handleTransmitterSubscriptionReceip(InAndOutSubscription,long,de.bsvrz.dav.daf.communication.lowLevel.telegrams.BaseSubscriptionInfo,long,long[],byte,byte)
	 */
	private void handlePositiveReceip(InAndOutSubscription subscription, byte receip, long mainTransmitterId) {
		if(subscription == null) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(subscription.receip == receip) {
			return;
		}
		subscription.receip = receip;
		subscription.mainTransmitterId = mainTransmitterId;
		if(subscription.outSubscriptions != null) {
			if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
				for(int i = subscription.outSubscriptions.length - 1; i > -1; --i) {
					if((subscription.outSubscriptions[i] != null) && (subscription.outSubscriptions[i].receip != 1)) {
						TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
								subscription.baseSubscriptionInfo, subscription.subscriptionState, subscription.outSubscriptions[i].transmitterToConsider
						);
						T_T_HighLevelCommunication _connection = getTransmitterConnection(
								subscription.outSubscriptions[i].targetTransmitterId
						);
						if(_connection != null) {
							_connection.sendTelegram(_unsubscription);
						}
						subscription.outSubscriptions[i] = null;
					}
				}
			}
			else if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
				for(int i = subscription.outSubscriptions.length - 1; i > -1; --i) {
					if(subscription.outSubscriptions[i] != null) {
						TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
								subscription.baseSubscriptionInfo, subscription.subscriptionState, subscription.outSubscriptions[i].transmitterToConsider
						);
						T_T_HighLevelCommunication _connection = getTransmitterConnection(
								subscription.outSubscriptions[i].targetTransmitterId
						);
						if(_connection != null) {
							_connection.sendTelegram(_unsubscription);
						}
					}
				}
				subscription.outSubscriptions = null;
				subscription.actualTransmitterToConsider = null;
			}
		}
		if(subscription.isTransmitterConnection) {
			TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
					subscription.baseSubscriptionInfo,
					subscription.subscriptionState,
					receip,
					subscription.mainTransmitterId,
					subscription.transmitterToConsider
			);
			T_T_HighLevelCommunication _connection = getTransmitterConnection(subscription.sourceTransmitterId);
			if(_connection != null) {
				_connection.sendTelegram(positivReceipt);
				if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
					T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
					if(_subscriptionComponent != null) {
						TransmitterDataSubscription transmitterDataSubscription = new TransmitterDataSubscription(
								subscription.baseSubscriptionInfo, subscription.subscriptionState, subscription.transmitterToConsider
						);
						if(subscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
							_subscriptionComponent.subscribeSendData(transmitterDataSubscription);
						}
						else {
							_subscriptionComponent.subscribeReceiveData(transmitterDataSubscription);
						}
					}
				}
			}
		}
		else {
			T_A_HighLevelCommunication _connection = getApplicationConnection(subscription.sourceTransmitterId);
			if(_connection != null) {
				T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
				if(subscriptionComponent != null) {
					if(subscription.subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) { // receiver
						if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
							ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.getReceiveSubscription(
									subscription.baseSubscriptionInfo
							);
							if(receiveSubscriptionInfo != null) {
								long dataNumber = cacheManager.getCurrentDataIndex(subscription.baseSubscriptionInfo) + 1;
								if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
									if(receiveSubscriptionInfo.getLastErrorState() == 3) {
										return;
									}
								}
								receiveSubscriptionInfo.setLastDataIndex(dataNumber);
								receiveSubscriptionInfo.setLastErrorState((byte)3);
								ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
										subscription.baseSubscriptionInfo, dataNumber, false, (byte)3, null, null, 1, 0, System.currentTimeMillis()
								);
								_connection.sendData(dataToSend);
								subscriptionsManager.removeSubscriptionRequest(subscription);
								if(subscriptionComponent != null) {
									subscriptionComponent.unsubscribeReceiveData(subscription.baseSubscriptionInfo);
								}
							}
						}
					}
					else { // sender
						SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(
								subscription.baseSubscriptionInfo
						);
						if(sendSubscriptionInfo != null) {
							if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
								if(sendSubscriptionInfo.isRequestSupported()) {
									if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
										if(authentificationManager.isActionAllowed(
												_connection.getRemoteUserId(),
												subscription.baseSubscriptionInfo,
												subscriptionComponent.canSend(subscription.baseSubscriptionInfo) ? (byte)0 : (byte)2
										) == 1) {
											sendSubscriptionInfo.setLastTriggerRequest((byte)0);
											_connection.triggerSender(subscription.baseSubscriptionInfo, (byte)0); // start sending
										}
									}
								}
							}
							else if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
								_connection.triggerSender(subscription.baseSubscriptionInfo, (byte)2); // stop sending no right
								subscriptionsManager.removeSubscriptionRequest(subscription);
								if(subscriptionComponent != null) {
									subscriptionComponent.unsubscribeSendData(subscription.baseSubscriptionInfo);
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Leitet die Anmeldungen an den beteiligten Datenverteiler weiter. Diese Methode wird von der Protokollsteuerung DaV-DAF aufgerufen, wenn sich eine
	 * Applikation als Empfnger oder Senke anmmelden mchte. Ist keine andere Quelle oder Senke im System feststellbar, werden die Sendeanmeldungen bearbeitet,
	 * die an diesem Datenverteiler vorliegen.
	 *
	 * @param connection   Schnittstelle zwischen DAV und DAF
	 * @param subscription Anmeldung als Emfnger oder Senke
	 */
	public synchronized void handleApplicationReceiveSubscription(
			T_A_HighLevelCommunication connection, ReceiveSubscriptionTelegram subscription) {
		ReceiveSubscriptionInfo receiveSubscriptionInfo = subscription.getReceiveSubscriptionInfo();
		if(receiveSubscriptionInfo != null) {
			BaseSubscriptionInfo info = receiveSubscriptionInfo.getBaseSubscriptionInfo();
			if(info != null) {
				if(receiveSubscriptionInfo.isDrain()) {
					// Wenn eine andere Quelle oder Senke vorhanden ist, dann leeren Datensatz versenden.
					if((subscriptionsManager.getSendingComponent(info) != null) || (subscriptionsManager.getReceivingComponent(info) != null)) {
						long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
						ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
								info, dataNumber, false, (byte)8, null, null, 1, 0, System.currentTimeMillis()
						);
						connection.sendData(dataToSend);
						return;
					}
				}
				long sourceUserId = connection.getRemoteUserId();
				byte receip = authentificationManager.isActionAllowed(
						sourceUserId, info, receiveSubscriptionInfo.isDrain() ? (byte)3 : (byte)1
				);
				// Keine Zugriffsrechte vorhanden -> Leeren Datensatz versenden
				if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
					long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
					ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
							info, dataNumber, false, (byte)3, null, null, 1, 0, System.currentTimeMillis()
					);
					connection.sendData(dataToSend);
					return;
				}
				// subscribe for receiving
				T_A_SubscriptionComponent _subscriptionComponent = (T_A_SubscriptionComponent)connection.getSubscriptionComponent();
				if(_subscriptionComponent != null) {
					_subscriptionComponent.subscribeReceiveData(subscription);
					// Es wurde erfolgreich eine neue Datenidentifikation durch eine Applikation angemeldet. Dies muss publiziert werden.
					if(_applicationStatusUpdater != null) {
						_applicationStatusUpdater.applicationSubscribedNewConnection(connection);
					}
				}
				long initiator = connection.getId();
				long potentialTransmitters[] = listsManager.getPotentialTransmitters(info);
				InAndOutSubscription request = new InAndOutSubscription(
						initiator, info, TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION, potentialTransmitters
				);
				InAndOutSubscription oldRequest = subscriptionsManager.addSubscriptionRequest(request, _transmitterId);
				if(oldRequest != request) {
					if(oldRequest.outSubscriptions != null) {
						for(int j = 0; j < oldRequest.outSubscriptions.length; ++j) {
							if(oldRequest.outSubscriptions[j] != null) {
								TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
										info, oldRequest.subscriptionState, oldRequest.outSubscriptions[j].transmitterToConsider
								);
								T_T_HighLevelCommunication _connection = getTransmitterConnection(
										oldRequest.outSubscriptions[j].targetTransmitterId
								);
								if(_connection != null) {
									_connection.sendTelegram(_unsubscription);
								}
							}
						}
						oldRequest.outSubscriptions = null;
						oldRequest.actualTransmitterToConsider = null;
					}
				}

				if(receiveSubscriptionInfo.isDrain()) {
					request.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP;
					request.mainTransmitterId = _transmitterId;
					cacheManager.clean(info);
					boolean noOneInterrested = true;
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(info);
					if(subscriptions != null) {
						for(int i = 0; i < subscriptions.length; ++i) {
							// see if there is an other source or drain in the system
							if((subscriptions[i] != null) && (subscriptions[i].outSubscriptions != null) && (subscriptions[i].receip
							                                                                                 == TransmitterSubscriptionsConstants
									.POSITIV_RECEIP)) {
								long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
								ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
										info, dataNumber, false, (byte)8, null, null, 1, 0, System.currentTimeMillis()
								);
								connection.sendData(dataToSend);
								subscriptionsManager.removeSubscriptionRequest(request);
								if(_subscriptionComponent != null) {
									_subscriptionComponent.unsubscribeReceiveData(info);
								}
								return;
							}
						}
						for(int i = 0; i < subscriptions.length; ++i) {
							if(subscriptions[i] != null) {
								cleanPendingSubscriptionToRedirect(subscriptions[i]);
								if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
									noOneInterrested = false;
								}
								if(subscriptions[i].outSubscriptions != null) {
									for(int j = 0; j < subscriptions[i].outSubscriptions.length; ++j) {
										if(subscriptions[i].outSubscriptions[j] != null) {
											TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
													info, subscriptions[i].subscriptionState, subscriptions[i].outSubscriptions[j].transmitterToConsider
											);
											T_T_HighLevelCommunication _connection = getTransmitterConnection(
													subscriptions[i].outSubscriptions[j].targetTransmitterId
											);
											if(_connection != null) {
												_connection.sendTelegram(_unsubscription);
											}
										}
									}
									subscriptions[i].outSubscriptions = null;
									subscriptions[i].actualTransmitterToConsider = null;
								}

								receip = 2;
								subscriptions[i].receip = receip;

								if(subscriptions[i].isTransmitterConnection) { // transmitter
									T_T_HighLevelCommunication _connection = getTransmitterConnection(subscriptions[i].sourceTransmitterId);
									if(_connection != null) {
										receip = authentificationManager.isActionAllowed(
												_connection.getRemoteUserId(), info, subscriptions[i].subscriptionState
										);
										subscriptions[i].receip = receip;
										TransmitterDataSubscriptionReceipt receiptTelegram = new TransmitterDataSubscriptionReceipt(
												info, subscriptions[i].subscriptionState, receip, _transmitterId, subscriptions[i].transmitterToConsider
										);
										_connection.sendTelegram(receiptTelegram);
									}
									if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
										subscriptions[i].mainTransmitterId = _transmitterId;
										T_T_SubscriptionComponent subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
										if(subscriptionComponent != null) {
											TransmitterDataSubscription _subscription = new TransmitterDataSubscription(
													info, subscriptions[i].subscriptionState, subscriptions[i].transmitterToConsider
											);
											if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
												subscriptionComponent.subscribeSendData(_subscription);
											}
											else {
												subscriptionComponent.subscribeReceiveData(_subscription);
											}
										}
									}
								}
								else { // application
									T_A_HighLevelCommunication _connection = getApplicationConnection(subscriptions[i].sourceTransmitterId);
									if(_connection != null) {
										receip = authentificationManager.isActionAllowed(
												_connection.getRemoteUserId(), info, subscriptions[i].subscriptionState
										);
										subscriptions[i].receip = receip;
										if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
											subscriptions[i].mainTransmitterId = _transmitterId;
											if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
												T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
												if(subscriptionComponent != null) {
													SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(info);
													if(sendSubscriptionInfo != null) {
														if(sendSubscriptionInfo.isRequestSupported()) {
															if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
																sendSubscriptionInfo.setLastTriggerRequest((byte)0);
																_connection.triggerSender(info, (byte)0); // start sending
															}
														}
													}
												}
											}
										}
										else if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
											subscriptions[i].mainTransmitterId = _transmitterId;
											T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
											if(subscriptionComponent != null) {
												if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
													SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.getSendSubscription(info);
													if(sendSubscriptionInfo != null) {
														_connection.triggerSender(info, (byte)2); // stop sending no rights
														subscriptionsManager.removeSubscriptionRequest(subscriptions[i]);
														if(subscriptionComponent != null) {
															subscriptionComponent.unsubscribeSendData(subscriptions[i].baseSubscriptionInfo);
														}
													}
												}
												else {
													ReceiveSubscriptionInfo _receiveSubscriptionInfo = subscriptionComponent.getReceiveSubscription(info);
													if(_receiveSubscriptionInfo != null) {
														long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
														ApplicationDataTelegram data = new ApplicationDataTelegram(
																info, dataNumber, false, (byte)3, null, null, 1, 0, System.currentTimeMillis()
														);
														_connection.sendData(data);
														subscriptionsManager.removeSubscriptionRequest(subscriptions[i]);
														if(subscriptionComponent != null) {
															subscriptionComponent.unsubscribeReceiveData(subscriptions[i].baseSubscriptionInfo);
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
					if(noOneInterrested) {
						long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
						if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
							if(receiveSubscriptionInfo.getLastErrorState() == 2) {
								return;
							}
						}
						receiveSubscriptionInfo.setLastDataIndex(dataNumber);
						receiveSubscriptionInfo.setLastErrorState((byte)2);
						ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
								info, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
						);
						connection.sendData(dataToSend);
					}
					listsManager.addInfo(info);
				}
				else {
					// get the sending component (See if main transmitter)
					T_A_SubscriptionComponent subscriptionComponent = subscriptionsManager.getSendingComponent(info);
					if(subscriptionComponent == null) {
						subscriptionComponent = subscriptionsManager.getReceivingComponent(info);
					}
					if(subscriptionComponent == null) {
						InAndOutSubscription inAndOutSubscription = subscriptionsManager.isSuccessfullySubscribed(
								info, TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION
						);
						receip = 0;
						if(inAndOutSubscription != null) {
							receip = inAndOutSubscription.receip;
						}
						if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
							request.receip = receip;
							request.mainTransmitterId = inAndOutSubscription.mainTransmitterId;
							try {
								subscriptionsManager.sendDataToT_A_Component(_subscriptionComponent, info);
							}
							catch(ConfigurationException ex) {
								ex.printStackTrace();
							}
						}
						else if(subscriptionsManager.isUnsuccessfullySubscribed(
								info, TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION
						)) {
							request.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
							request.mainTransmitterId = -1;
							long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
							if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
								if(receiveSubscriptionInfo.getLastErrorState() == 2) {
									return;
								}
							}
							receiveSubscriptionInfo.setLastDataIndex(dataNumber);
							receiveSubscriptionInfo.setLastErrorState((byte)2);
							ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
									info, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
							);
							connection.sendData(dataToSend);
						}
						else {
							makeOutgoingRequests(request);
							request.mainTransmitterId = -1;
							if(request.outSubscriptions == null) {
								request.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
								request.mainTransmitterId = -1;
								long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
								if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
									if(receiveSubscriptionInfo.getLastErrorState() == 2) {
										return;
									}
								}
								receiveSubscriptionInfo.setLastDataIndex(dataNumber);
								receiveSubscriptionInfo.setLastErrorState((byte)2);
								ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
										info, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
								);
								connection.sendData(dataToSend);
							}
							else if(subscriptionsManager.needToSendSubscription(_transmitterId, request)) {
								request.receip = (byte)-1;
								for(int j = 0; j < request.outSubscriptions.length; ++j) {
									if(request.outSubscriptions[j] != null) {
										TransmitterDataSubscription _subscription = new TransmitterDataSubscription(
												info, TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION, request.outSubscriptions[j].transmitterToConsider
										);
										T_T_HighLevelCommunication _connection = getTransmitterConnection(
												request.outSubscriptions[j].targetTransmitterId
										);
										if(_connection != null) {
											_connection.sendTelegram(_subscription);
										}
									}
								}
							}
							else {
								request.outSubscriptions = null;
							}
						}
					}
					else {
						request.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP;
						request.mainTransmitterId = _transmitterId;
						try {
							subscriptionsManager.sendDataToT_A_Component(_subscriptionComponent, info);
						}
						catch(ConfigurationException ex) {
							ex.printStackTrace();
						}

						SubscriptionComponent subscriptionComponents[] = subscriptionsManager.getInterrestedSendingComponent(info);
						if(subscriptionComponents == null) { // Kein sender
							long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
							if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
								if(receiveSubscriptionInfo.getLastErrorState() == 2) {
									return;
								}
							}
							receiveSubscriptionInfo.setLastDataIndex(dataNumber);
							receiveSubscriptionInfo.setLastErrorState((byte)2);
							ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
									info, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
							);
							connection.sendData(dataToSend);
						}

						SubscriptionComponent tmpSubscriptionComponents[] = subscriptionsManager.getInterrestedReceivingComponent(info);
						if((tmpSubscriptionComponents != null) && (tmpSubscriptionComponents.length == 1)) {
							InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(info);
							if(subscriptions != null) {
								for(int i = 0; i < subscriptions.length; ++i) {
									if(subscriptions[i] != null) {
										if(!subscriptions[i].isTransmitterConnection) { // Application
											if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
												T_A_HighLevelCommunication _connection = getApplicationConnection(subscriptions[i].sourceTransmitterId);
												if(_connection != null) {
													T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
													if(tmpSubscriptionComponent != null) {
														SendSubscriptionInfo tmpSendSubscriptionInfo = tmpSubscriptionComponent.getSendSubscription(info);
														if(tmpSendSubscriptionInfo != null) {
															if(tmpSendSubscriptionInfo.isRequestSupported()) {
																if(tmpSendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
																	if(authentificationManager.isActionAllowed(
																			_connection.getRemoteUserId(),
																			info,
																			tmpSubscriptionComponent.canSend(info) ? (byte)0 : (byte)2
																	) == 1) {
																		tmpSendSubscriptionInfo.setLastTriggerRequest((byte)0);
																		_connection.triggerSender(info, (byte)0); // start sending
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Meldet die Daten ab und leitet die Abmeldungen, wenn ntig, an die beteiligten Datenverteiler weiter. Diese Methode wird von der Protokollsteuerung DaV-DAF
	 * aufgerufen, wenn sich eine Applikation als Empfnger oder Senke abmelden mchte. Im Falle einer Senkenabmeldung wird die Basisanmeldeinformation aus der
	 * Zuliefererdatenverwaltung entfernt, so dass dieser Datenverteiler nicht mehr der Zentraldatenverteiler der spezifizierten Daten ist. Danach werden alle von
	 * dieser gelschten Anmeldung abhngigen Anmeldungen bearbeitet. Im Falle einer Empfngerabmeldug existieren zwei Mglichkeiten: Bei einer lokalen Quelle wird
	 * die Quelle aufgefordert das Senden der Daten einzustellen. Wenn die Quell- oder die Sendeapplikation bei einem anderen Datenverteiler angemeldet ist, dann
	 * wird zuerst berprft, ob bei diesem Datenverteiler eine andere Empfangsanmeldung auf diese Daten vorhanden ist. Wenn keine Anmeldung fr die spezifizierten
	 * Daten existiert, dann wird der aktuelle Datensatz aus dem Datenbestand entfernt.
	 *
	 * @param connection     Schnittstelle zwischen DAV und DAF
	 * @param unsubscription Abmeldetelegram der Applikation
	 */
	public synchronized void handleApplicationReceiveUnsubscription(
			T_A_HighLevelCommunication connection, ReceiveUnsubscriptionTelegram unsubscription) {

		long applicationId = connection.getId();
		byte subscriptionState = TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION;
		BaseSubscriptionInfo info = unsubscription.getUnSubscriptionInfo();
		InAndOutSubscription subscription = subscriptionsManager.removeSubscriptionRequest(
				applicationId, info, subscriptionState
		);
		T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)connection.getSubscriptionComponent();
		// Wenn das Objekt null, mache nichts
		if(subscriptionComponent != null) {
			ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.unsubscribeReceiveData(info);
			// Die Anmeldung der Applikation wurde zurck genommen -> Andere darber informieren
			if(_applicationStatusUpdater != null) {
				_applicationStatusUpdater.applicationUnsubscribeConnection(connection);
			}

			// Wenn das Objekt null, mache nichts
			if(receiveSubscriptionInfo != null) {
				if(receiveSubscriptionInfo.isDrain()) {
					listsManager.removeInfo(info);
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(info);
					if(subscriptions != null) {
						OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
						for(int i = 0; i < subscriptions.length; ++i) {
							if(subscriptions[i] != null) {
								cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
								workInvolvedSubscription(subscriptions[i], subscriptionHelper);
							}
						}
						subscriptionHelper.flushOutUnSubscription(this);
					}
				}
				else if(subscription != null) {
					if(subscription.outSubscriptions == null) {
						SubscriptionComponent subscriptionComponents[] = subscriptionsManager.getInterrestedReceivingComponent(info);
						if(subscriptionComponents == null) { // no one interrested
							subscriptionComponents = subscriptionsManager.getInterrestedSendingComponent(info);
							if(subscriptionComponents != null) {
								for(int i = 0; i < subscriptionComponents.length; ++i) {
									if((subscriptionComponents[i] != null) && (subscriptionComponents[i].getType() == SubscriptionComponent.T_A)) {
										SendSubscriptionInfo _sendSubscriptionInfo = ((T_A_SubscriptionComponent)subscriptionComponents[i]).getSendSubscription(
												info
										);
										if((_sendSubscriptionInfo != null) && (_sendSubscriptionInfo.isRequestSupported())) {
											if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
												T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)subscriptionComponents[i].getConnection();
												if(_connection != null) {
													_sendSubscriptionInfo.setLastTriggerRequest((byte)1);
													_connection.triggerSender(info, (byte)1);// stop sending
												}
											}
										}
									}
								}
							}
						}
					}
					else {
						boolean done = false;
						if(subscription.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
							InAndOutSubscription alternativeSubscription = subscriptionsManager.getAlternativeSubscriptionRequest(
									subscription
							);
							if(alternativeSubscription != null) {
								alternativeSubscription.outSubscriptions = subscription.outSubscriptions;
								alternativeSubscription.receip = subscription.receip;
								alternativeSubscription.mainTransmitterId = subscription.mainTransmitterId;
								alternativeSubscription.transmitterToConsider = subscription.transmitterToConsider;
								alternativeSubscription.actualTransmitterToConsider = subscription.actualTransmitterToConsider;
								for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
									ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
											i
									);
									if(exchangeSubscriptionAction != null) {
										if(exchangeSubscriptionAction.oldSubscription == subscription) {
											exchangeSubscriptionAction.oldSubscription = alternativeSubscription;
										}
									}
								}
								done = true;
							}
						}
						if(done) {
							return;
						}

						OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
						for(int i = 0; i < subscription.outSubscriptions.length; ++i) {
							if(subscription.outSubscriptions[i] != null) {
								subscriptionHelper.handleOutUnsubscription(subscription, subscription.outSubscriptions[i]);
							}
						}
						subscription.outSubscriptions = null;
						subscription.actualTransmitterToConsider = null;
						cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
						InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(subscription);
						if(subscriptions != null) {
							for(int i = 0; i < subscriptions.length; ++i) {
								if(subscriptions[i] != null) {
									cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
									workInvolvedSubscription(subscriptions[i], subscriptionHelper);
								}
							}
						}
						subscriptionHelper.flushOutUnSubscription(this);
					}
				}
				// if no one interrested delete cached Data
				if(!subscriptionsManager.isSubscriptionRequestAvaillable(info)) {
					cacheManager.clean(info);
				}
			}
		}
	}

	/**
	 * Leitet die Anmeldungen an den beteiligten Datenverteiler weiter. Diese Methode wird von der Protokollsteuerung DaV-DAF aufgerufen, wenn sich eine
	 * Applikation als Quelle oder Sender anmelden mchte. Wenn sich die Applikation als Quelle anmelden mchte, dann wird zuerst geprft, ob an diesem
	 * Datenverteiler bereits eine andere Applikation als Quelle oder Senke fr das gleiche Datum angemeldet ist. Wenn durch die Empfangsanmeldung Folgeanmeldungen
	 * bei anderen Datenverteiler vorhanden waren, dann werden diese abgemeldet. Stammt diese Empfangsanmeldung von einem anderen Datenverteiler, so wird eine
	 * positive Quittung an diesen Datenverteiler gesendet.
	 *
	 * @param connection   Schnittstelle zwischen DAV und DAF
	 * @param subscription Anmeldetelegramm der Applikation
	 */
	public synchronized void handleApplicationSendSubscription(
			T_A_HighLevelCommunication connection, SendSubscriptionTelegram subscription) {

		if(_applicationStatusUpdater != null) {
			_applicationStatusUpdater.applicationSubscribedNewConnection(connection);
		}

		SendSubscriptionInfo sendSubscriptionInfo = subscription.getSendSubscriptionInfo();
		if(sendSubscriptionInfo != null) {
			BaseSubscriptionInfo info = sendSubscriptionInfo.getBaseSubscriptionInfo();
			if(info != null) {
				if(sendSubscriptionInfo.isSource()) {
					if((subscriptionsManager.getSendingComponent(info) != null) || (subscriptionsManager.getReceivingComponent(info) != null)) {
						connection.triggerSender(info, (byte)3); // Stop sending more than one main application
						return;
					}
				}
				long sourceUserId = connection.getRemoteUserId();
				byte receip = authentificationManager.isActionAllowed(
						sourceUserId, info, sendSubscriptionInfo.isSource() ? (byte)2 : (byte)0
				);
				if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
					connection.triggerSender(info, (byte)2); // Stop sending. No rights for this action
					return;
				}
				// Subscribe for sending
				T_A_SubscriptionComponent _subscriptionComponent = (T_A_SubscriptionComponent)connection.getSubscriptionComponent();
				if(_subscriptionComponent != null) {
					_subscriptionComponent.subscribeSendData(subscription);
					// Es wurde erfolgreich eine neue Datenidentifikation durch eine Applikation angemeldet. Dies muss publiziert werden.
					if(_applicationStatusUpdater != null) {
						_applicationStatusUpdater.applicationSubscribedNewConnection(connection);
					}
				}
				long initiator = connection.getId();
				long potentialTransmitters[] = listsManager.getPotentialTransmitters(info);
				InAndOutSubscription request = new InAndOutSubscription(
						initiator, info, TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION, potentialTransmitters
				);
				InAndOutSubscription oldRequest = subscriptionsManager.addSubscriptionRequest(request, _transmitterId);
				if(oldRequest != request) {
					if(oldRequest.outSubscriptions != null) {
						for(int j = 0; j < oldRequest.outSubscriptions.length; ++j) {
							if(oldRequest.outSubscriptions[j] != null) {
								TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
										info, oldRequest.subscriptionState, oldRequest.outSubscriptions[j].transmitterToConsider
								);
								T_T_HighLevelCommunication _connection = getTransmitterConnection(
										oldRequest.outSubscriptions[j].targetTransmitterId
								);
								if(_connection != null) {
									_connection.sendTelegram(_unsubscription);
								}
							}
						}
						oldRequest.outSubscriptions = null;
						oldRequest.actualTransmitterToConsider = null;
					}
				}
				if(sendSubscriptionInfo.isSource()) {
					request.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP;
					request.mainTransmitterId = _transmitterId;
					cacheManager.clean(info);
					boolean noOneInterrested = true;
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(info);
					for(int i = 0; i < subscriptions.length; ++i) {
						// see if there is an other source or drain in the system
						if((subscriptions[i] != null) && (subscriptions[i].outSubscriptions != null) && (subscriptions[i].receip
						                                                                                 == TransmitterSubscriptionsConstants.POSITIV_RECEIP)) {
							connection.triggerSender(info, (byte)3); // Stop sending more than one main application
							subscriptionsManager.removeSubscriptionRequest(request);
							if(_subscriptionComponent != null) {
								_subscriptionComponent.unsubscribeSendData(info);
							}
							return;
						}
					}
					for(int i = 0; i < subscriptions.length; ++i) {
						if(subscriptions[i] != null) {
							cleanPendingSubscriptionToRedirect(subscriptions[i]);
							if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
								noOneInterrested = false;
							}
							if(subscriptions[i].outSubscriptions != null) {
								for(int j = 0; j < subscriptions[i].outSubscriptions.length; ++j) {
									if(subscriptions[i].outSubscriptions[j] != null) {
										TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
												info, subscriptions[i].subscriptionState, subscriptions[i].outSubscriptions[j].transmitterToConsider
										);
										T_T_HighLevelCommunication _connection = getTransmitterConnection(
												subscriptions[i].outSubscriptions[j].targetTransmitterId
										);
										if(_connection != null) {
											_connection.sendTelegram(_unsubscription);
										}
									}
								}
								subscriptions[i].outSubscriptions = null;
								subscriptions[i].actualTransmitterToConsider = null;
							}

							receip = 2;
							subscriptions[i].receip = receip;

							if(subscriptions[i].isTransmitterConnection) { // transmitter
								T_T_HighLevelCommunication _connection = getTransmitterConnection(subscriptions[i].sourceTransmitterId);
								if(_connection != null) {
									receip = authentificationManager.isActionAllowed(
											_connection.getRemoteUserId(), info, subscriptions[i].subscriptionState
									);
									subscriptions[i].receip = receip;
									TransmitterDataSubscriptionReceipt receiptTelegram = new TransmitterDataSubscriptionReceipt(
											info, subscriptions[i].subscriptionState, receip, _transmitterId, subscriptions[i].transmitterToConsider
									);
									_connection.sendTelegram(receiptTelegram);
								}
								if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
									subscriptions[i].mainTransmitterId = _transmitterId;
									T_T_SubscriptionComponent subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
									if(subscriptionComponent != null) {
										TransmitterDataSubscription _subscription = new TransmitterDataSubscription(
												info, subscriptions[i].subscriptionState, subscriptions[i].transmitterToConsider
										);
										if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
											subscriptionComponent.subscribeSendData(_subscription);
										}
										else {
											subscriptionComponent.subscribeReceiveData(_subscription);
										}
									}
								}
							}
							else { // application
								T_A_HighLevelCommunication _connection = getApplicationConnection(subscriptions[i].sourceTransmitterId);
								if(_connection != null) {
									receip = authentificationManager.isActionAllowed(
											_connection.getRemoteUserId(), info, subscriptions[i].subscriptionState
									);
									subscriptions[i].receip = receip;
									if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
										subscriptions[i].mainTransmitterId = _transmitterId;
										if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
											if(subscriptions.length > 1) {
												T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
												if(subscriptionComponent != null) {
													SendSubscriptionInfo _sendSubscriptionInfo = subscriptionComponent.getSendSubscription(info);
													if(_sendSubscriptionInfo != null) {
														if(_sendSubscriptionInfo.isRequestSupported()) {
															if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
																_sendSubscriptionInfo.setLastTriggerRequest((byte)0);
																_connection.triggerSender(info, (byte)0); // start sending
															}
														}
													}
												}
											}
										}
									}
									else if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP_NO_RIGHT) {
										subscriptions[i].mainTransmitterId = _transmitterId;
										T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
										if(subscriptionComponent != null) {
											if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
												SendSubscriptionInfo _sendSubscriptionInfo = subscriptionComponent.getSendSubscription(info);
												if(_sendSubscriptionInfo != null) {
													_connection.triggerSender(info, (byte)2); // stop sending no rights
													subscriptionsManager.removeSubscriptionRequest(subscriptions[i]);
													if(subscriptionComponent != null) {
														subscriptionComponent.unsubscribeSendData(info);
													}
												}
											}
											else {
												ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.getReceiveSubscription(info);
												if(receiveSubscriptionInfo != null) {
													long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
													ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
															info, dataNumber, false, (byte)3, null, null, 1, 0, System.currentTimeMillis()
													);
													_connection.sendData(dataToSend);
													subscriptionsManager.removeSubscriptionRequest(subscriptions[i]);
													if(subscriptionComponent != null) {
														subscriptionComponent.unsubscribeReceiveData(info);
													}
												}
											}
										}
									}
								}
							}
						}
					}
					if(noOneInterrested) {
						if(sendSubscriptionInfo.isRequestSupported()) {
							if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
								sendSubscriptionInfo.setLastTriggerRequest((byte)1);
								connection.triggerSender(info, (byte)1); // Stop sending no one interrested
							}
						}
					}
					listsManager.addInfo(info);
				}
				else {
					// get the receiving component (See if main transmitter)
					T_A_SubscriptionComponent subscriptionComponent = subscriptionsManager.getReceivingComponent(info);
					if(subscriptionComponent == null) {
						subscriptionComponent = subscriptionsManager.getSendingComponent(info);
					}
					if(subscriptionComponent == null) {
						InAndOutSubscription inAndOutSubscription = subscriptionsManager.isSuccessfullySubscribed(
								info, TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION
						);
						receip = 0;
						if(inAndOutSubscription != null) {
							receip = inAndOutSubscription.receip;
						}
						if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
							request.receip = receip;
							request.mainTransmitterId = inAndOutSubscription.mainTransmitterId;
							if(sendSubscriptionInfo.isSenderRequestSupported()) {
								if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
									sendSubscriptionInfo.setLastTriggerRequest((byte)0);
									connection.triggerSender(info, (byte)0); // Start sending
								}
							}
						}
						else if(subscriptionsManager.isUnsuccessfullySubscribed(
								info, TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION
						)) {
							request.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
							request.mainTransmitterId = -1;
							if(sendSubscriptionInfo.isSenderRequestSupported()) {
								if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
									sendSubscriptionInfo.setLastTriggerRequest((byte)1);
									connection.triggerSender(info, (byte)1); // Stop sending. No one interrested
								}
							}
						}
						else {
							makeOutgoingRequests(request);
							request.mainTransmitterId = -1;
							if(request.outSubscriptions == null) {
								request.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
								if(sendSubscriptionInfo.isSenderRequestSupported()) {
									if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
										sendSubscriptionInfo.setLastTriggerRequest((byte)1);
										connection.triggerSender(info, (byte)1); // Stop sending. No one interrested
									}
								}
							}
							else if(subscriptionsManager.needToSendSubscription(_transmitterId, request)) {
								request.receip = (byte)-1;
								for(int j = 0; j < request.outSubscriptions.length; ++j) {
									if(request.outSubscriptions[j] != null) {
										TransmitterDataSubscription _subscription = new TransmitterDataSubscription(
												info, TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION, request.outSubscriptions[j].transmitterToConsider
										);
										T_T_HighLevelCommunication _connection = getTransmitterConnection(
												request.outSubscriptions[j].targetTransmitterId
										);
										if(_connection != null) {
											_connection.sendTelegram(_subscription);
										}
									}
								}
							}
							else {
								request.outSubscriptions = null;
							}
						}
					}
					else {
						request.receip = TransmitterSubscriptionsConstants.POSITIV_RECEIP;
						request.mainTransmitterId = _transmitterId;
						SubscriptionComponent subscriptionComponents[] = subscriptionsManager.getInterrestedReceivingComponent(info);
						if(subscriptionComponents == null) { // Kein empfnger
							if(sendSubscriptionInfo.isRequestSupported()) {
								if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
									sendSubscriptionInfo.setLastTriggerRequest((byte)1);
									connection.triggerSender(info, (byte)1); // Stop sending no one interrested
								}
							}
						}
						else {
							if(sendSubscriptionInfo.isSenderRequestSupported()) {
								if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
									sendSubscriptionInfo.setLastTriggerRequest((byte)0);
									connection.triggerSender(info, (byte)0); // Start sending
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Meldet die Daten ab und leitet die Abmeldungen, wenn ntig, an den beteiligten Datenverteiler. Diese Methode wird von der Protokollsteuerung DaV-DAF
	 * aufgerufen, wenn sich eine Applikation als Quelle oder Sender abmelden mchte. Im Falle einer Quellenabmeldung wird die Basisanmeldeinformation aus der
	 * Zuliefererdatenverwaltung entfernt, so dass dieser Datenverteiler nicht mehr der Zentraldatenverteiler der spezifizierten Daten ist. Danach werden alle von
	 * dieser gelschten Anmeldung abhngigen Anmeldungen bearbeitet. Im Falle einer Senderabmeldung existieren zwei Mglichkeiten: Bei einer lokalen Senke wird
	 * die Senke benachrichtigt, dass kein Sender mer vorhanden ist. Wenn die Senken- oder die Empfangsapplikation bei einem anderen Datenverteiler angemeldet ist,
	 * dann wird zuerst ?berpru? ob bei diesem Datenverteiler eine andere Sendeanmeldung auf diese Daten vorhanden ist. Wenn keine Anmeldung fr die
	 * spezifizierten Daten existiert, dann wird der aktuelle Datensatz aus dem bestand entfernt.
	 *
	 * @param connection     Schnittstelle zwischen DAV und DAF
	 * @param unsubscription Abmeldetelegram der Applikation
	 */
	public synchronized void handleApplicationSendUnsubscription(
			T_A_HighLevelCommunication connection, SendUnsubscriptionTelegram unsubscription) {

		if(_applicationStatusUpdater != null) {
			_applicationStatusUpdater.applicationUnsubscribeConnection(connection);
		}

		long applicationId = connection.getId();
		byte subscriptionState = TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION;
		BaseSubscriptionInfo info = unsubscription.getUnSubscriptionInfo();
		InAndOutSubscription subscription = subscriptionsManager.removeSubscriptionRequest(
				applicationId, info, subscriptionState
		);

		T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)connection.getSubscriptionComponent();
		if(subscriptionComponent != null) {
			SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.unsubscribeSendData(info);
			// Die Anmeldung der Applikation wurde zurck genommen -> Andere darber informieren
			if(_applicationStatusUpdater != null) {
				_applicationStatusUpdater.applicationUnsubscribeConnection(connection);
			}
			if(sendSubscriptionInfo != null) {
				if(sendSubscriptionInfo.isSource()) {
					listsManager.removeInfo(info);
					InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(info);
					if(subscriptions != null) {
						OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
						for(int i = 0; i < subscriptions.length; ++i) {
							if(subscriptions[i] != null) {
								cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
								workInvolvedSubscription(subscriptions[i], subscriptionHelper);
							}
						}
						subscriptionHelper.flushOutUnSubscription(this);
					}
				}
				else if(subscription != null) {
					if(subscription.outSubscriptions == null) {
						T_A_SubscriptionComponent _subscriptionComponent = (T_A_SubscriptionComponent)subscriptionsManager.getReceivingComponent(
								info
						);
						if(_subscriptionComponent != null) { // a drain?
							SubscriptionComponent _subscriptionComponents[] = subscriptionsManager.getInterrestedSendingComponent(info);
							if(_subscriptionComponents == null) { //the one and only sender?
								T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)_subscriptionComponent.getConnection();
								if(_connection != null) {
									ReceiveSubscriptionInfo _receiveSubscriptionInfo = _subscriptionComponent.getReceiveSubscription(info);
									if(_receiveSubscriptionInfo != null) {
										long dataNumber = cacheManager.getCurrentDataIndex(info) + 1;
										if(_receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
											if(_receiveSubscriptionInfo.getLastErrorState() == 2) {
												return;
											}
										}
										_receiveSubscriptionInfo.setLastDataIndex(dataNumber);
										_receiveSubscriptionInfo.setLastErrorState((byte)2);
										ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
												info, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
										);
										_connection.sendData(dataToSend);
									}
								}
							}
						}
					}
					else {
						boolean done = false;
						if(subscription.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
							InAndOutSubscription alternativeSubscription = subscriptionsManager.getAlternativeSubscriptionRequest(
									subscription
							);
							if(alternativeSubscription != null) {
								alternativeSubscription.outSubscriptions = subscription.outSubscriptions;
								alternativeSubscription.receip = subscription.receip;
								alternativeSubscription.mainTransmitterId = subscription.mainTransmitterId;
								alternativeSubscription.transmitterToConsider = subscription.transmitterToConsider;
								alternativeSubscription.actualTransmitterToConsider = subscription.actualTransmitterToConsider;
								for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
									ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
											i
									);
									if(exchangeSubscriptionAction != null) {
										if(exchangeSubscriptionAction.oldSubscription == subscription) {
											exchangeSubscriptionAction.oldSubscription = alternativeSubscription;
										}
									}
								}
								done = true;
							}
						}
						if(done) {
							return;
						}

						OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
						for(int i = 0; i < subscription.outSubscriptions.length; ++i) {
							if(subscription.outSubscriptions[i] != null) {
								subscriptionHelper.handleOutUnsubscription(subscription, subscription.outSubscriptions[i]);
							}
						}
						subscription.outSubscriptions = null;
						subscription.actualTransmitterToConsider = null;
						cleanPendingSubscriptionToRedirect(subscription, subscriptionHelper);
						InAndOutSubscription subscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(subscription);
						if(subscriptions != null) {
							for(int i = 0; i < subscriptions.length; ++i) {
								if(subscriptions[i] != null) {
									cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
									workInvolvedSubscription(subscriptions[i], subscriptionHelper);
								}
							}
						}
						subscriptionHelper.flushOutUnSubscription(this);
					}
				}
				// if no one interrested delete cached Data
				if(!subscriptionsManager.isSubscriptionRequestAvaillable(info)) {
					cacheManager.clean(info);
				}
			}
		}
	}

	/**
	 * Aktualisiert die Liste der auf eine positive Quitung wartenden Anmeldungen, die ber einen neuen weg umgeleitet werden mssen.
	 *
	 * @param subscription Eintrag mit den Anmeldungsinformationen
	 */
	private void cleanPendingSubscriptionToRedirect(InAndOutSubscription subscription) {
		for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
			ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(i);
			if(exchangeSubscriptionAction != null) {
				if(exchangeSubscriptionAction.oldSubscription == subscription) {
					_pendingSubscriptionToRedirect.remove(i);
					InAndOutSubscription newSubscription = exchangeSubscriptionAction.newSubscription;
					if(newSubscription != null) {
						for(int j = 0; j < newSubscription.outSubscriptions.length; ++j) {
							if(newSubscription.outSubscriptions[j] != null) {
								TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
										newSubscription.baseSubscriptionInfo,
										newSubscription.subscriptionState,
										newSubscription.outSubscriptions[j].transmitterToConsider
								);
								T_T_HighLevelCommunication _connection = getTransmitterConnection(
										newSubscription.outSubscriptions[j].targetTransmitterId
								);
								if(_connection != null) {
									_connection.sendTelegram(_unsubscription);
								}
							}
						}
					}
					break;
				}
			}
		}
	}

	/**
	 * Aktualisiert die Liste mit austehenden Anmeldungen TBD: noch nicht gut beschrieben
	 *
	 * @param subscription       Eintrag mit den Anmeldungsinformationen
	 * @param subscriptionHelper
	 */
	private void cleanPendingSubscriptionToRedirect(
			InAndOutSubscription subscription, OutSubscriptionsHelper subscriptionHelper) {
		if(Transmitter._debugLevel > 20) {
			System.err.println(
					"ConnectionsManager.cleanPendingSubscriptionToRedirect pendingSubscriptionToRedirect.size() = " + _pendingSubscriptionToRedirect.size()
			);
		}
		for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
			ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
					i
			);
			if(exchangeSubscriptionAction != null) {
				if(exchangeSubscriptionAction.oldSubscription == subscription) {
					_pendingSubscriptionToRedirect.remove(i);
					InAndOutSubscription newSubscription = exchangeSubscriptionAction.newSubscription;
					if(newSubscription != null) {
						for(int j = 0; j < newSubscription.outSubscriptions.length; ++j) {
							if(newSubscription.outSubscriptions[j] != null) {
								subscriptionHelper.handleOutUnsubscription(newSubscription, newSubscription.outSubscriptions[j]);
							}
						}
					}
					break;
				}
			}
		}
	}

	/**
	 * Behandelt eine Anmeldung nach einer Abmeldeaktion...
	 *
	 * @param involvedSubscription Eintrag mit den Anmeldungsinformationen
	 * @param subscriptionHelper
	 */
	private void workInvolvedSubscription(
			InAndOutSubscription involvedSubscription, OutSubscriptionsHelper subscriptionHelper) {
		if(Transmitter._debugLevel > 20) {
			System.err.println("ConnectionsManager.workInvolvedSubscription involvedSubscription = " + involvedSubscription);
		}
		if(involvedSubscription != null) {
			if(involvedSubscription.receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
				//positiver status
				if(involvedSubscription.isTransmitterConnection) {
					T_T_HighLevelCommunication _connection = getTransmitterConnection(involvedSubscription.sourceTransmitterId);
					if(_connection != null) {
						T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
						if(_subscriptionComponent != null) {
							if(!subscriptionsManager.isSubscriptionRequestAvaillable(involvedSubscription)) {
								if(involvedSubscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) {
									// anmeldestatus == Senderanmeldung
									_subscriptionComponent.unsubscribeSendData(involvedSubscription.baseSubscriptionInfo);
								}
								else {
									// anmeldestatus == emfngeranmeldung
									_subscriptionComponent.unsubscribeReceiveData(involvedSubscription.baseSubscriptionInfo);
								}
							}
						}
					}
				}
			}

			if(involvedSubscription.outSubscriptions != null) {
				for(int j = 0; j < involvedSubscription.outSubscriptions.length; ++j) {
					if(involvedSubscription.outSubscriptions[j] != null) {
						subscriptionHelper.handleOutUnsubscription(involvedSubscription, involvedSubscription.outSubscriptions[j]);
					}
				}
			}
			involvedSubscription.outSubscriptions = null;
			involvedSubscription.actualTransmitterToConsider = null;
			involvedSubscription.receip = -1;

			long potentialTransmitters[] = null;
			if(involvedSubscription.isTransmitterConnection) {
				long ids[] = involvedSubscription.transmitterToConsider;
				boolean found = false;
				for(int i = 0; i < ids.length; ++i) {
					if(ids[i] == _transmitterId) {
						found = true;
						break;
					}
				}
				if(found) {
					potentialTransmitters = new long[ids.length - 1];
					for(int i = 0, j = 0; i < ids.length; ++i) {
						if(ids[i] == _transmitterId) {
							continue;
						}
						potentialTransmitters[j++] = ids[i];
					}
				}
				else {
					potentialTransmitters = ids;
				}
			}
			else {
				potentialTransmitters = listsManager.getPotentialTransmitters(involvedSubscription.baseSubscriptionInfo);
			}
//			if(Transmitter._debugLevel > 20) {
//				System.err.println("potentialTransmitters = " + Arrays.toString(potentialTransmitters));
//			}
			if(potentialTransmitters == null) {
				involvedSubscription.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
				if(involvedSubscription.isTransmitterConnection) {
					TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
							involvedSubscription.baseSubscriptionInfo,
							involvedSubscription.subscriptionState,
							TransmitterSubscriptionsConstants.NEGATIV_RECEIP,
							-1,
							involvedSubscription.transmitterToConsider
					);
					T_T_HighLevelCommunication _connection = getTransmitterConnection(involvedSubscription.sourceTransmitterId);
					if(_connection != null) {
						_connection.sendTelegram(negativReceipt);
					}
				}
				else {
					T_A_HighLevelCommunication _connection = getApplicationConnection(involvedSubscription.sourceTransmitterId);
					if(_connection != null) {
						T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
						if(tmpSubscriptionComponent != null) {
							if(involvedSubscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending
								SendSubscriptionInfo _sendSubscriptionInfo = tmpSubscriptionComponent.getSendSubscription(
										involvedSubscription.baseSubscriptionInfo
								);
								if(_sendSubscriptionInfo != null) {
									if(_sendSubscriptionInfo.isRequestSupported()) {
										if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
											_sendSubscriptionInfo.setLastTriggerRequest((byte)1);
											_connection.triggerSender(involvedSubscription.baseSubscriptionInfo, (byte)1);// stop sending
										}
									}
								}
							}
							else { // receiving
								ReceiveSubscriptionInfo _receiveSubscriptionInfo = tmpSubscriptionComponent.getReceiveSubscription(
										involvedSubscription.baseSubscriptionInfo
								);
								if(_receiveSubscriptionInfo != null) {
									long dataNumber = cacheManager.getCurrentDataIndex(involvedSubscription.baseSubscriptionInfo) + 1;
									final long lastDataIndex = _receiveSubscriptionInfo.getLastDataIndex();
									if(lastDataIndex == dataNumber) {
										if(_receiveSubscriptionInfo.getLastErrorState() == 2) {
											return;
										}
									}
									_receiveSubscriptionInfo.setLastDataIndex(dataNumber);
									_receiveSubscriptionInfo.setLastErrorState((byte)2);
									ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
											involvedSubscription.baseSubscriptionInfo, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
									);
									_connection.sendData(dataToSend);
								}
							}
						}
					}
				}
				return;
			}

			InAndOutSubscription inAndOutSubscription = subscriptionsManager.isSuccessfullySubscribed(
					involvedSubscription.baseSubscriptionInfo, involvedSubscription.subscriptionState, _transmitterId, potentialTransmitters
			);
			byte receip = 0;
			if(inAndOutSubscription != null) {
				receip = inAndOutSubscription.receip;
			}
			if(receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
				involvedSubscription.receip = receip;
				involvedSubscription.mainTransmitterId = inAndOutSubscription.mainTransmitterId;
				if(involvedSubscription.isTransmitterConnection) {
					TransmitterDataSubscriptionReceipt positivReceipt = new TransmitterDataSubscriptionReceipt(
							involvedSubscription.baseSubscriptionInfo,
							involvedSubscription.subscriptionState,
							receip,
							inAndOutSubscription.mainTransmitterId,
							involvedSubscription.transmitterToConsider
					);
					T_T_HighLevelCommunication _connection = getTransmitterConnection(involvedSubscription.sourceTransmitterId);
					if(_connection != null) {
						_connection.sendTelegram(positivReceipt);
						T_T_SubscriptionComponent _subscriptionComponent = (T_T_SubscriptionComponent)_connection.getSubscriptionComponent();
						if(_subscriptionComponent != null) {
							TransmitterDataSubscription dataSubscription = new TransmitterDataSubscription(
									involvedSubscription.baseSubscriptionInfo,
									involvedSubscription.subscriptionState,
									involvedSubscription.transmitterToConsider
							);
							if(involvedSubscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending
								_subscriptionComponent.subscribeSendData(dataSubscription);
							}
							else { // receiving
								_subscriptionComponent.subscribeReceiveData(dataSubscription);
								TransmitterDataTelegram dataToSend[] = cacheManager.getCurrentDataForTransmitter(
										involvedSubscription.baseSubscriptionInfo
								);
								if(dataToSend != null) {
									_connection.sendTelegrams(dataToSend);
								}
							}
						}
					}
				}
				else {
					if(involvedSubscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending
						T_A_HighLevelCommunication _connection = getApplicationConnection(involvedSubscription.sourceTransmitterId);
						if(_connection != null) {
							T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
							if(tmpSubscriptionComponent != null) {
								SendSubscriptionInfo _sendSubscriptionInfo = tmpSubscriptionComponent.getSendSubscription(
										involvedSubscription.baseSubscriptionInfo
								);
								if(_sendSubscriptionInfo != null) {
									if(_sendSubscriptionInfo.isRequestSupported()) {
										if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)0) {
											if(authentificationManager.isActionAllowed(
													_connection.getRemoteUserId(),
													involvedSubscription.baseSubscriptionInfo,
													tmpSubscriptionComponent.canSend(involvedSubscription.baseSubscriptionInfo) ? (byte)0 : (byte)2
											) == 1) {
												_sendSubscriptionInfo.setLastTriggerRequest((byte)0);
												_connection.triggerSender(involvedSubscription.baseSubscriptionInfo, (byte)0);// start sending
											}
										}
									}
								}
							}
						}
					}
					else { // receiving
						T_A_HighLevelCommunication _connection = getApplicationConnection(involvedSubscription.sourceTransmitterId);
						if(_connection != null) {
							try {
								subscriptionsManager.sendDataToT_A_Component(
										(T_A_SubscriptionComponent)_connection.getSubscriptionComponent(), involvedSubscription.baseSubscriptionInfo
								);
							}
							catch(ConfigurationException ex) {
								ex.printStackTrace();
							}
						}
					}
				}
			}
			else if(subscriptionsManager.isUnsuccessfullySubscribed(
					involvedSubscription.baseSubscriptionInfo, involvedSubscription.subscriptionState, _transmitterId, potentialTransmitters
			)) {
				involvedSubscription.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
				involvedSubscription.mainTransmitterId = -1;
				if(involvedSubscription.isTransmitterConnection) {
					TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
							involvedSubscription.baseSubscriptionInfo,
							involvedSubscription.subscriptionState,
							TransmitterSubscriptionsConstants.NEGATIV_RECEIP,
							-1,
							involvedSubscription.transmitterToConsider
					);
					T_T_HighLevelCommunication _connection = getTransmitterConnection(involvedSubscription.sourceTransmitterId);
					if(_connection != null) {
						_connection.sendTelegram(negativReceipt);
					}
				}
				else {
					T_A_HighLevelCommunication _connection = getApplicationConnection(involvedSubscription.sourceTransmitterId);
					if(_connection != null) {
						T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
						if(tmpSubscriptionComponent != null) {
							if(involvedSubscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending
								SendSubscriptionInfo _sendSubscriptionInfo = tmpSubscriptionComponent.getSendSubscription(
										involvedSubscription.baseSubscriptionInfo
								);
								if(_sendSubscriptionInfo != null) {
									if(_sendSubscriptionInfo.isRequestSupported()) {
										if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
											_sendSubscriptionInfo.setLastTriggerRequest((byte)1);
											_connection.triggerSender(involvedSubscription.baseSubscriptionInfo, (byte)1);// stop sending
										}
									}
								}
							}
							else { // receiving
								ReceiveSubscriptionInfo _receiveSubscriptionInfo = tmpSubscriptionComponent.getReceiveSubscription(
										involvedSubscription.baseSubscriptionInfo
								);
								if(_receiveSubscriptionInfo != null) {
									long dataNumber = cacheManager.getCurrentDataIndex(involvedSubscription.baseSubscriptionInfo) + 1;
									if(_receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
										if(_receiveSubscriptionInfo.getLastErrorState() == 2) {
											return;
										}
									}
									_receiveSubscriptionInfo.setLastDataIndex(dataNumber);
									_receiveSubscriptionInfo.setLastErrorState((byte)2);
									ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
											involvedSubscription.baseSubscriptionInfo, dataNumber, false, (byte)2, null, null, 1, 0, System.currentTimeMillis()
									);
									_connection.sendData(dataToSend);
								}
							}
						}
					}
				}
			}
			else {
				makeOutgoingRequests(involvedSubscription);
				involvedSubscription.mainTransmitterId = -1;
				if(involvedSubscription.outSubscriptions == null) {
					involvedSubscription.actualTransmitterToConsider = null;
					involvedSubscription.receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
					if(involvedSubscription.isTransmitterConnection) {
						// rs: Anmeldung stammt von einem fremden DaV, also negative Quittung versenden
						TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
								involvedSubscription.baseSubscriptionInfo,
								involvedSubscription.subscriptionState,
								TransmitterSubscriptionsConstants.NEGATIV_RECEIP,
								-1,
								involvedSubscription.transmitterToConsider
						);
						T_T_HighLevelCommunication _connection = getTransmitterConnection(involvedSubscription.sourceTransmitterId);
						if(_connection != null) {
							_connection.sendTelegram(negativReceipt);
						}
					}
					else {
						// rs: Anmeldung stammt von einer lokalen Applikation, also negative Sendesteuerung bzw. "keine Quelle" versenden
						if(Transmitter._debugLevel > 20) System.err.println("ConnectionsManager.workInvolvedSubscription T_A");
						T_A_HighLevelCommunication _connection = getApplicationConnection(involvedSubscription.sourceTransmitterId);
						if(_connection != null) {
							T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
							if(tmpSubscriptionComponent != null) {
								if(involvedSubscription.subscriptionState == TransmitterSubscriptionsConstants.SENDER_SUBSCRIPTION) { // sending
									SendSubscriptionInfo _sendSubscriptionInfo = tmpSubscriptionComponent.getSendSubscription(
											involvedSubscription.baseSubscriptionInfo
									);
									if(_sendSubscriptionInfo != null) {
										if(_sendSubscriptionInfo.isRequestSupported()) {
											if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
												_sendSubscriptionInfo.setLastTriggerRequest((byte)1);
												if(Transmitter._debugLevel > 20) {
													System.err.println("ConnectionsManager.workInvolvedSubscription triggerSender");
												}
												_connection.triggerSender(involvedSubscription.baseSubscriptionInfo, (byte)1);// stop sending
											}
										}
									}
								}
								else { // receiving
									ReceiveSubscriptionInfo _receiveSubscriptionInfo = tmpSubscriptionComponent.getReceiveSubscription(
											involvedSubscription.baseSubscriptionInfo
									);
									if(_receiveSubscriptionInfo != null) {
										long dataNumber = cacheManager.getCurrentDataIndex(involvedSubscription.baseSubscriptionInfo) + 1;
										if(_receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
											if(_receiveSubscriptionInfo.getLastErrorState() == 2) {
												return;
											}
										}
										_receiveSubscriptionInfo.setLastDataIndex(dataNumber);
										_receiveSubscriptionInfo.setLastErrorState((byte)2);
										ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
												involvedSubscription.baseSubscriptionInfo,
												dataNumber,
												false,
												(byte)2,
												null,
												null,
												1,
												0,
												System.currentTimeMillis()
										);
										_connection.sendData(dataToSend);
									}
								}
							}
						}
					}
				}
				else if(subscriptionsManager.needToSendSubscription(_transmitterId, involvedSubscription)) {
					involvedSubscription.receip = (byte)-1;
					for(int j = 0; j < involvedSubscription.outSubscriptions.length; ++j) {
						if(involvedSubscription.outSubscriptions[j] != null) {
							subscriptionHelper.handleOutSubscription(involvedSubscription, involvedSubscription.outSubscriptions[j]);
						}
					}
				}
				else {
					involvedSubscription.outSubscriptions = null;
				}
			}
		}
	}

	/**
	 * Erzeugt die notwendigen Informationen zu den Anmeldungen, die verschickt werden.
	 * <p/>
	 * Es wird eine Liste angelegt, mit allen potentiellen Zulieferdaentenverteiler eines bestimmten Datums. Wenn es sich um eine DAV-DAV Verbindung handelt, wird
	 * die komplette Liste mit Ausnahme der eigenen Adresse(<code>transmitterId</code>) in die Liste der potentiellen DAVs kopiert. Handelt es sich nicht um eine
	 * Transmitterverbindung, so wird die Liste der erreichbaren Transmitter ber <code>listsManager.getPotentialTransmitters()</code> geholt. Ansclieend wird die
	 * Routine <code>bestWayManager</code> aufgerufen, um die besten Wege von DAV zu DAV zu finden. Anscjhlieend werden die von einem DAV erreichbaren DAVs in die
	 * Liste <code>outSubscription</code> gespeichert.
	 *
	 * @param request Anmeldungsinformationen
	 */
	private void makeOutgoingRequests(InAndOutSubscription request) {
		if(request == null) {
			throw new IllegalArgumentException("Argument ist null");
		}
		long[] potentialTransmitters;
		if(request.isTransmitterConnection) {
			long[] ids = request.transmitterToConsider;
			boolean found = false;
			for(int i = 0; i < ids.length; ++i) {
				if(ids[i] == _transmitterId) {
					found = true;
					break;
				}
			}
			if(found) {
				potentialTransmitters = new long[ids.length - 1];
				int j = 0;
				for(int i = 0; i < ids.length; ++i) {
					if(ids[i] == _transmitterId) {
						continue;
					}
					potentialTransmitters[j++] = ids[i];
				}
			}
			else {
				potentialTransmitters = ids;
			}
		}
		else {
			potentialTransmitters = listsManager.getPotentialTransmitters(request.baseSubscriptionInfo);
		}

		request.actualTransmitterToConsider = potentialTransmitters;
		if(potentialTransmitters == null || potentialTransmitters.length == 0) {
			request.outSubscriptions = null;
		}
		else {
			Hashtable tmpTable = new Hashtable();
			ArrayList keys = new ArrayList();
//
			for(int j = 0; j < potentialTransmitters.length; ++j) {
				Long tmpLong = potentialTransmitters[j];
				if(potentialTransmitters[j] == _transmitterId) {
					continue;
				}
				if(potentialTransmitters[j] == request.sourceTransmitterId) {
					continue;
				}
				long hightway = bestWayManager.getBestWay(potentialTransmitters[j]);
				if((hightway < 0) || (getTransmitterConnection(hightway) == null) || (hightway == request.sourceTransmitterId)) {
					continue;
				}
				Long target = new Long(hightway);
				ArrayList list = (ArrayList)tmpTable.get(target);
				if(list == null) {
					list = new ArrayList();
					tmpTable.put(target, list);
					keys.add(target);
				}
				list.add(tmpLong);
			}
			InAndOutSubscription.OutSubscription outSubscriptions[] = null;
			int size = keys.size();
			if(size > 0) {
				outSubscriptions = new InAndOutSubscription.OutSubscription[size];
				for(int j = 0; j < size; ++j) {
					Long key = (Long)keys.get(j);
					ArrayList list = (ArrayList)tmpTable.get(key);
					if(list != null) {
						int listSize = list.size();
						if(listSize > 0) {
							long[] _ids = new long[listSize];
							for(int k = 0; k < listSize; ++k) {
								_ids[k] = ((Long)list.get(k)).longValue();
							}
							outSubscriptions[j] = request.newOutSubscription(key.longValue(), _ids, (byte)-1);
						}
					}
				}
			}
			request.outSubscriptions = outSubscriptions;
			if(outSubscriptions == null) {
				request.actualTransmitterToConsider = null;
			}
		}
	}

	/**
	 * Entfernt der gegebene Applikationsverbindung aus der Verbindungsverwaltung.
	 * <p/>
	 * Diese Methode wird von der Protokollsteuerung DaV-DAF aufgerufen, wenn eine Applikation terminiert. Der Reprsentant der Applikationsverbindung wird aus der
	 * Verbindungsverwaltung entfernt. Handelt es sich bei der Applikation um die lokale Konfiguration, dann wird dieser Datenverteiler terminiert. Die
	 * close-Methode wird aufgerufen. Ansonsten werden zuerst alle von der Applikation stammenden Anmeldungen aus der Anmeldungsverwaltung entfernt und einzelnen
	 * bearbeitet. Wenn eine Anmeldung eine Quell - oder Senderanmeldung ist, dann werden die Aktionen durchgefhrt, die unter handleApplicationSendUnsubscription
	 * beschrieben sind. Handelt es sich um eine Senken- oder Empfangsanmeldung, dann werden die Aktionen durchgefhrt, die unter
	 * handleApplicationReceiveUnsubscription beschrieben sind.
	 *
	 * @param connection die zu entfernende Verbindung zwischen DAV und Application
	 */
	public void unsubscribeConnection(T_A_HighLevelCommunicationInterface connection) {
		if(connection == null) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(_closing) return;
		if(connection.isConfiguration()) {
			close(true, "Die Konfiguration ist nicht mehr vorhanden. Der Datenverteiler wird jetzt beendet");
		}
		else {
			final SystemObject applicationObject;
			synchronized(this) {
				if(_closing) return;
				long source = connection.getId();
				applicationObject = removeApplicationConnection(source);
				InAndOutSubscription subscriptions[] = subscriptionsManager.removeSubscriptionRequests(source);
				T_A_SubscriptionComponent subscriptionComponent = (T_A_SubscriptionComponent)connection.getSubscriptionComponent();
				if(subscriptionComponent != null) {
					if(subscriptions != null) {
						OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
						// Alle Anmeldungen der betroffenen Applikation betrachten
						for(int i = 0; i < subscriptions.length; ++i) {
							if((subscriptions[i] != null) && (subscriptions[i].baseSubscriptionInfo != null)) {
								if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
									// Empfangsanmeldung seitens der betroffenen Anmeldung
									ReceiveSubscriptionInfo receiveSubscriptionInfo = subscriptionComponent.unsubscribeReceiveData(
											subscriptions[i].baseSubscriptionInfo
									);
									if(receiveSubscriptionInfo != null) {
										if(receiveSubscriptionInfo.isDrain()) {
											listsManager.removeInfo(subscriptions[i].baseSubscriptionInfo);
											InAndOutSubscription subscriptionsToWork[] = subscriptionsToWork = subscriptionsManager.getInvolvedSubscriptionRequests(
													subscriptions[i].baseSubscriptionInfo
											);
											if(subscriptionsToWork != null) {
												for(int j = 0; j < subscriptionsToWork.length; ++j) {
													if(subscriptionsToWork[j] != null) {
														cleanPendingSubscriptionToRedirect(subscriptionsToWork[j], subscriptionHelper);
														workInvolvedSubscription(subscriptionsToWork[j], subscriptionHelper);
													}
												}
											}
										}
										else {
											SubscriptionComponent subscriptionComponents[] = subscriptionsManager.getInterrestedReceivingComponent(
													subscriptions[i].baseSubscriptionInfo
											);
											if(subscriptionComponents == null) {
												T_A_SubscriptionComponent senderSubscriptionComponent = (T_A_SubscriptionComponent)subscriptionsManager.getSendingComponent(
														subscriptions[i].baseSubscriptionInfo
												);
												if(senderSubscriptionComponent != null) {
													SendSubscriptionInfo sendSubscriptionInfo = senderSubscriptionComponent.getSendSubscription(
															subscriptions[i].baseSubscriptionInfo
													);
													if((sendSubscriptionInfo != null) && (sendSubscriptionInfo.isRequestSupported())) {
														if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
															T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)senderSubscriptionComponent.getConnection();
															if(_connection != null) {
																sendSubscriptionInfo.setLastTriggerRequest((byte)1);
																_connection.triggerSender(subscriptions[i].baseSubscriptionInfo, (byte)1);
															}
														}
													}
												}
											}
											if(subscriptions[i].outSubscriptions != null) {
												boolean done = false;
												if(subscriptions[i].receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
													InAndOutSubscription alternativeSubscription = subscriptionsManager.getAlternativeSubscriptionRequest(
															subscriptions[i]
													);
													if(alternativeSubscription != null) {
														alternativeSubscription.outSubscriptions = subscriptions[i].outSubscriptions;
														alternativeSubscription.receip = subscriptions[i].receip;
														alternativeSubscription.mainTransmitterId = subscriptions[i].mainTransmitterId;
														alternativeSubscription.transmitterToConsider = subscriptions[i].transmitterToConsider;
														alternativeSubscription.actualTransmitterToConsider = subscriptions[i].actualTransmitterToConsider;
														for(int j = _pendingSubscriptionToRedirect.size() - 1; j > -1; --j) {
															ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
																	j
															);
															if(exchangeSubscriptionAction != null) {
																if(exchangeSubscriptionAction.oldSubscription == subscriptions[i]) {
																	exchangeSubscriptionAction.oldSubscription = alternativeSubscription;
																}
															}
														}
														done = true;
													}
												}
												if(done) {
													continue;
												}
												for(int j = 0; j < subscriptions[i].outSubscriptions.length; ++j) {
													if(subscriptions[i].outSubscriptions[j] != null) {
														subscriptionHelper.handleOutUnsubscription(subscriptions[i], subscriptions[i].outSubscriptions[j]);
													}
												}
												subscriptions[i].outSubscriptions = null;
												subscriptions[i].actualTransmitterToConsider = null;
												cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
												InAndOutSubscription subscriptionsToWork[] = subscriptionsToWork = subscriptionsManager.getInvolvedSubscriptionRequests(
														subscriptions[i]
												);
												if(subscriptionsToWork != null) {
													for(int j = 0; j < subscriptionsToWork.length; ++j) {
														if(subscriptionsToWork[j] != null) {
															cleanPendingSubscriptionToRedirect(subscriptionsToWork[j], subscriptionHelper);
															workInvolvedSubscription(subscriptionsToWork[j], subscriptionHelper);
														}
													}
												}
											}
										}
									}
								}
								else {
									// Betrachtete Anmeldung der betroffenen Applikation wurde als Quelle oder Sender gemacht
									SendSubscriptionInfo sendSubscriptionInfo = subscriptionComponent.unsubscribeSendData(
											subscriptions[i].baseSubscriptionInfo
									);
									if(sendSubscriptionInfo != null) {
										if(sendSubscriptionInfo.isSource()) {
											// Betrachtete Anmeldung der betroffenen Applikation wurde als Quelle gemacht
											listsManager.removeInfo(subscriptions[i].baseSubscriptionInfo);
											InAndOutSubscription subscriptionsToWork[] = subscriptionsToWork = subscriptionsManager.getInvolvedSubscriptionRequests(
													subscriptions[i].baseSubscriptionInfo
											);
											if(subscriptionsToWork != null) {
												// Abhngige Anmeldungen von anderen Applikationen bzw. Datenverteilern betrachten
												for(int j = 0; j < subscriptionsToWork.length; ++j) {
													if(subscriptionsToWork[j] != null) {
														cleanPendingSubscriptionToRedirect(subscriptionsToWork[j], subscriptionHelper);
														workInvolvedSubscription(subscriptionsToWork[j], subscriptionHelper);
													}
												}
											}
										}
										else {
											if(subscriptions[i].outSubscriptions != null) {
												boolean done = false;
												if(subscriptions[i].receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
													InAndOutSubscription alternativeSubscription = subscriptionsManager.getAlternativeSubscriptionRequest(
															subscriptions[i]
													);
													if(alternativeSubscription != null) {
														alternativeSubscription.outSubscriptions = subscriptions[i].outSubscriptions;
														alternativeSubscription.receip = subscriptions[i].receip;
														alternativeSubscription.mainTransmitterId = subscriptions[i].mainTransmitterId;
														alternativeSubscription.transmitterToConsider = subscriptions[i].transmitterToConsider;
														alternativeSubscription.actualTransmitterToConsider = subscriptions[i].actualTransmitterToConsider;
														for(int j = _pendingSubscriptionToRedirect.size() - 1; j > -1; --j) {
															ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
																	j
															);
															if(exchangeSubscriptionAction != null) {
																if(exchangeSubscriptionAction.oldSubscription == subscriptions[i]) {
																	exchangeSubscriptionAction.oldSubscription = alternativeSubscription;
																}
															}
														}
														done = true;
													}
												}
												if(done) {
													continue;
												}
												for(int j = 0; j < subscriptions[i].outSubscriptions.length; ++j) {
													if(subscriptions[i].outSubscriptions[j] != null) {
														subscriptionHelper.handleOutUnsubscription(subscriptions[i], subscriptions[i].outSubscriptions[j]);
													}
												}
												subscriptions[i].outSubscriptions = null;
												subscriptions[i].actualTransmitterToConsider = null;
												cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
												InAndOutSubscription subscriptionsToWork[] = subscriptionsToWork = subscriptionsManager.getInvolvedSubscriptionRequests(
														subscriptions[i]
												);
												if(subscriptionsToWork != null) {
													for(int j = 0; j < subscriptionsToWork.length; ++j) {
														if(subscriptionsToWork[j] != null) {
															cleanPendingSubscriptionToRedirect(subscriptionsToWork[j], subscriptionHelper);
															workInvolvedSubscription(subscriptionsToWork[j], subscriptionHelper);
														}
													}
												}
											}
										}
									}
								}
								// if no one interrested delete cached Data
								if(!subscriptionsManager.isSubscriptionRequestAvaillable(subscriptions[i].baseSubscriptionInfo)) {
									cacheManager.clean(subscriptions[i].baseSubscriptionInfo);
								}
							}
						}
						subscriptionHelper.flushOutUnSubscription(this);
					}
				}
			}
			if(applicationObject != null) {
				try {
					applicationObject.invalidate();
				}
				catch(ConfigurationChangeException e) {
					_debug.warning("Applikationsobjekt " + applicationObject + " konnte nicht nicht gelscht werden", e);
				}
			}
		}
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, wenn ein Datenverteiler terminiert. Der Reprsentant der  Datenverteilerverbindung wird
	 * aus der Verbindungsverwaltung entfernt. Danach werden alle von der spezifizierten Datenverteiler stammenden Anmeldungen ermittelt und einzeln bearbeitet. Es
	 * werden fr jede einzelne diese Anmeldungen alle abhngigen Anmeldungen ermittelt und jede einzelne von ihnen wird wie folgt bearbeitet: Bei einer
	 * Empfangsanmeldung wird ein spezieller Datensatz generiert, aus dem hervorgeht, dass keine Quelle mehr vorhanden ist und zum Initiator weitergeleitet. Wenn
	 * diese Verbindung eine Hauptverbindung, also keine Ersatzverbindung, ist, dann wird ihr Reprsentant zur Verbindungsverwaltung hinzugefgt. Ein Thread im
	 * Hintergrund startet dann bei Bedarf die Ersatzverbindungen und berwacht, ob diese Hauptverbindung spter wieder verfgbar ist. Dann werden die
	 * Ersatzverbindungen automatisch wieder abgebaut
	 *
	 * @param connection die zu entfernende Verbindung zwischen DAV und DAV
	 */
	public final void unsubscribeConnection(T_T_HighLevelCommunicationInterface connection) {
		if(connection == null) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(_closing || (_transmitterConnections == null)) {
			return;
		}
		long connectedTransmitterId = connection.getId();
		_debug.info("Verbindung zum Datenverteiler " + connectedTransmitterId + " abgebaut");
		synchronized(this) {
			removeTransmitterConnection(connectedTransmitterId);
			bestWayManager.handleDisconnection(connection);

			_debug.fine("Anmeldungen bei Datenverteiler " + connectedTransmitterId + " werden aufgegeben");

			OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
			InAndOutSubscription subscriptions[] = subscriptionsManager.removeSubscriptionRequests(connectedTransmitterId);
			if(subscriptions != null) {
				for(int i = 0; i < subscriptions.length; ++i) {
					if((subscriptions[i] != null) && (subscriptions[i].baseSubscriptionInfo != null)) {
						if(subscriptions[i].outSubscriptions != null) {
							boolean done = false;
							if(subscriptions[i].receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
								InAndOutSubscription alternativeSubscription = subscriptionsManager.getAlternativeSubscriptionRequest(
										subscriptions[i]
								);
								if(alternativeSubscription != null) {
									alternativeSubscription.outSubscriptions = subscriptions[i].outSubscriptions;
									alternativeSubscription.receip = subscriptions[i].receip;
									alternativeSubscription.mainTransmitterId = subscriptions[i].mainTransmitterId;
									alternativeSubscription.transmitterToConsider = subscriptions[i].transmitterToConsider;
									alternativeSubscription.actualTransmitterToConsider = subscriptions[i].actualTransmitterToConsider;
									for(int j = _pendingSubscriptionToRedirect.size() - 1; j > -1; --j) {
										ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
												j
										);
										if(exchangeSubscriptionAction != null) {
											if(exchangeSubscriptionAction.oldSubscription == subscriptions[i]) {
												exchangeSubscriptionAction.oldSubscription = alternativeSubscription;
											}
										}
									}
									done = true;
								}
							}
							if(done) {
								continue;
							}
							for(int j = 0; j < subscriptions[i].outSubscriptions.length; ++j) {
								if(subscriptions[i].outSubscriptions[j] != null) {
									subscriptionHelper.handleOutUnsubscription(subscriptions[i], subscriptions[i].outSubscriptions[j]);
								}
							}
							subscriptions[i].outSubscriptions = null;
							subscriptions[i].actualTransmitterToConsider = null;
						}

						cleanPendingSubscriptionToRedirect(subscriptions[i], subscriptionHelper);
						T_T_SubscriptionComponent subscriptionComponent = (T_T_SubscriptionComponent)connection.getSubscriptionComponent();
						if(subscriptionComponent != null) {
							if(subscriptions[i].receip == TransmitterSubscriptionsConstants.POSITIV_RECEIP) {
								if(subscriptions[i].subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
									subscriptionComponent.unsubscribeReceiveData(subscriptions[i].baseSubscriptionInfo);
								}
								else {
									subscriptionComponent.unsubscribeSendData(subscriptions[i].baseSubscriptionInfo);
								}
							}
						}

						SubscriptionComponent subscriptionComponents[] = subscriptionsManager.getInterrestedReceivingComponent(
								subscriptions[i].baseSubscriptionInfo
						);
						if(subscriptionComponents == null) {
							T_A_SubscriptionComponent senderSubscriptionComponent = (T_A_SubscriptionComponent)subscriptionsManager.getSendingComponent(
									subscriptions[i].baseSubscriptionInfo
							);
							if(senderSubscriptionComponent != null) {
								SendSubscriptionInfo sendSubscriptionInfo = senderSubscriptionComponent.getSendSubscription(
										subscriptions[i].baseSubscriptionInfo
								);
								if((sendSubscriptionInfo != null) && (sendSubscriptionInfo.isRequestSupported())) {
									if(sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
										T_A_HighLevelCommunication _connection = (T_A_HighLevelCommunication)senderSubscriptionComponent.getConnection();
										if(_connection != null) {
											sendSubscriptionInfo.setLastTriggerRequest((byte)1);
											_connection.triggerSender(subscriptions[i].baseSubscriptionInfo, (byte)1);
										}
									}
								}
							}
						}

						if((subscriptionsManager.getSendingComponent(subscriptions[i].baseSubscriptionInfo) == null) && (
								subscriptionsManager.getReceivingComponent(subscriptions[i].baseSubscriptionInfo) == null)) { // not main transmitter
							InAndOutSubscription subscriptionsToWork[] = subscriptionsManager.getInvolvedSubscriptionRequests(
									subscriptions[i]
							);
							if(subscriptionsToWork != null) {
								for(int j = 0; j < subscriptionsToWork.length; ++j) {
									if(subscriptionsToWork[j] != null) {
										cleanPendingSubscriptionToRedirect(subscriptionsToWork[j], subscriptionHelper);
										workInvolvedSubscription(subscriptionsToWork[j], subscriptionHelper);
									}
								}
							}
						}

						// if no one interrested delete cached Data
						if(!subscriptionsManager.isSubscriptionRequestAvaillable(subscriptions[i].baseSubscriptionInfo)) {
							cacheManager.clean(subscriptions[i].baseSubscriptionInfo);
						}
					}
				}
			}

			long connectionId = connection.getId();
			subscriptions = subscriptionsManager.getAffectedOutgoingSubscriptions(connectionId);
			if(subscriptions != null) {
				for(int i = 0; i < subscriptions.length; ++i) {
					if(subscriptions[i] != null) {
						InAndOutSubscription subscriptionsToWork[] = subscriptionsManager.getInvolvedSubscriptionRequests(
								subscriptions[i]
						);
						if(subscriptionsToWork != null) {
							for(int j = 0; j < subscriptionsToWork.length; ++j) {
								if(subscriptionsToWork[j] != null) {
									if(subscriptionsToWork[j].subscriptionState == TransmitterSubscriptionsConstants.RECEIVER_SUBSCRIPTION) {
										if(subscriptionsToWork[j].isTransmitterConnection) {
											T_T_HighLevelCommunication _connection = getTransmitterConnection(
													subscriptionsToWork[j].sourceTransmitterId
											);
											if(_connection != null) {
												long dataNumber = cacheManager.getCurrentDataIndex(subscriptionsToWork[j].baseSubscriptionInfo) + 1;
												TransmitterDataTelegram dataToSend = new TransmitterDataTelegram(
														subscriptionsToWork[j].baseSubscriptionInfo,
														dataNumber,
														false,
														(byte)2,
														null,
														null,
														1,
														0,
														System.currentTimeMillis(),
														(byte)1
												);
												_connection.sendTelegram(dataToSend);
											}
										}
										else {
											T_A_HighLevelCommunication _connection = getApplicationConnection(
													subscriptionsToWork[j].sourceTransmitterId
											);
											if(_connection != null) {
												T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
												if(tmpSubscriptionComponent != null) {
													ReceiveSubscriptionInfo receiveSubscriptionInfo = tmpSubscriptionComponent.getReceiveSubscription(
															subscriptionsToWork[j].baseSubscriptionInfo
													);
													if(receiveSubscriptionInfo != null) {
														long dataNumber = cacheManager.getCurrentDataIndex(subscriptionsToWork[j].baseSubscriptionInfo) + 1;
														if(receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
															if(receiveSubscriptionInfo.getLastErrorState() == 2) {
																continue;
															}
														}
														receiveSubscriptionInfo.setLastDataIndex(dataNumber);
														receiveSubscriptionInfo.setLastErrorState((byte)2);
														ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
																subscriptionsToWork[j].baseSubscriptionInfo,
																dataNumber,
																false,
																(byte)2,
																null,
																null,
																1,
																0,
																System.currentTimeMillis()
														);
														_connection.sendData(dataToSend);
													}
												}
											}
										}
									}

									cleanPendingSubscriptionToRedirect(subscriptionsToWork[j], subscriptionHelper);
									workInvolvedSubscription(subscriptionsToWork[j], subscriptionHelper);
								}
							}
						}
					}
				}
			}
			subscriptionHelper.flushOutUnSubscription(this);
		}

		_debug.fine("Liste der unterbrochenen Verbindungen wird aktualisiert fr Datenverteiler " + connectedTransmitterId);

		synchronized(missedConnectionInfos) {
			if(_transmitterConnectionInfos != null) {
				// see if we need to make a substitute for this connection
				for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
					if((_transmitterConnectionInfos[i] == null) || _transmitterConnectionInfos[i].isExchangeConnection()) {
						continue;
					}
					TransmitterInfo t1 = _transmitterConnectionInfos[i].getTransmitter_1();
					if((t1 != null) && (t1.getTransmitterId() == _transmitterId)) {
						TransmitterInfo t2 = _transmitterConnectionInfos[i].getTransmitter_2();
						if((t2 != null) && (t2.getTransmitterId() == connectedTransmitterId)) {
							missedConnectionInfos.add(_transmitterConnectionInfos[i]);
							missedConnectionInfos.notifyAll();
							break;
						}
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc} Diese Methode wird von der Protokollsteuerung DaV-DaV, der Anmeldungsverwaltung und von dieser Komponente selber aufgerufen. Sie dient als
	 * Hilfsfunktion, um eine in der Verbindungsverwaltung registrierte Datenverteilerverbindung ausfindig zu machen.
	 */
	public T_T_HighLevelCommunication getTransmitterConnection(long communicationTransmitterId) {
		for(T_T_HighLevelCommunication transmitterConnection : _transmitterConnections) {
			if(transmitterConnection != null) {
				if(transmitterConnection.getId() == communicationTransmitterId) {
					return transmitterConnection;
				}
			}
		}
		return null;
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DAF, der Anmeldungsverwaltung und von dieser Komponente selber aufgerufen. Sie dient als Hilfsfunktion, um
	 * eine in der Verbindungsverwaltung registrierte Applikationsverbindung ausfindig zu machen.
	 *
	 * @param applicationId die Id der verbundenen Applikation
	 *
	 * @return registrierte Applikationsverbindung
	 */
	final T_A_HighLevelCommunication getApplicationConnection(long applicationId) {
		for(T_A_HighLevelCommunication applicationConnection : _applicationConnections) {
			if(applicationConnection != null) {
				if(applicationConnection.getId() == applicationId) {
					return applicationConnection;
				}
			}
		}
		return null;
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, wenn eine Anmeldung einer Zuliefererinformation eines Datenverteilers angekommen ist. Sie
	 * ruft die Methode subscribe der Zuliefererdatenverwaltung auf.
	 *
	 * @param connection                   Verbindung zwischen DAV und DAV
	 * @param transmitterListsSubscription AnmeldungsListe des Nachbardatenverteilers
	 */
	public final synchronized void handleListsSubscription(
			T_T_HighLevelCommunicationInterface connection, TransmitterListsSubscription transmitterListsSubscription) {
		if((connection == null) || (transmitterListsSubscription == null)) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(listsManager != null) {
			listsManager.subscribe(connection.getId(), transmitterListsSubscription.getTransmitterList());
		}
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, wenn eine Abmeldung einer Zuliefererinformation eines Datenverteilers angekommen ist. Sie
	 * ruft die Methode unsubscribe der Zuliefererdatenverwaltung auf.
	 *
	 * @param connection                     Verbindung zwischen DAV und DAV
	 * @param transmitterListsUnsubscription AbmeldungsListe des Nachbardatenverteilers
	 */
	public final synchronized void handleListsUnsubscription(
			T_T_HighLevelCommunicationInterface connection, TransmitterListsUnsubscription transmitterListsUnsubscription) {
		if((connection == null) || (transmitterListsUnsubscription == null)) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(listsManager != null) {
			listsManager.unsubscribe(connection.getId(), transmitterListsUnsubscription.getTransmitterList());
		}
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, wenn eine Kndigung einer Anmeldung einer
	 * Zuliefererinformation eines Datenverteilers angekommen ist. Sie ruft die Methode unsubscribeDeliverer der Anmeldelistenverwaltung auf.
	 *
	 * @param connection Verbindung zwischen DAV und DAV
	 * @param transmitterListsDeliveryUnsubscription
	 *                   Aufforderung zur Abmeldung der Zulieferdaten
	 */
	public final synchronized void handleListsDeliveryUnsubscription(
			T_T_HighLevelCommunicationInterface connection, TransmitterListsDeliveryUnsubscription transmitterListsDeliveryUnsubscription) {
		if((connection == null) || (transmitterListsDeliveryUnsubscription == null)) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(listsManager != null) {
			listsManager.unsubscribeDeliverer(connection.getId(), transmitterListsDeliveryUnsubscription.getTransmitterList());
		}
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung DaV-DaV aufgerufen, wenn eine Aktualisierung der Zuliefererinformationen von einen Datenverteiler angekommen
	 * ist. Zuerst wird die Methode updateEntry der Zuliefererverwaltung aufgerufen. Als Rckgabewert liefert diese Methode Listen der neu hinzugekommenen Objekte
	 * und Attributgruppen- und Aspektkombinationen. Dann werden alle nicht positiv quittierten Anmeldungen ermittelt, deren Basisanmeldeinformation mit den neuen
	 * registrierten Objekten und Kombinationen Gemeinsamkeiten hat. Fr jede einzelne Anmeldung werden dann die potentiellen Zentraldatenverteiler ermittelt. Hat
	 * sich die Liste der Zentraldatenverteiler verndert, dann werden die Folgeanmeldungen bei den nicht mehr relevanten Zentraldatenverteilern abgemeldet und bei
	 * den neu ermittelten angemeldet.
	 *
	 * @param connection             Verbindung zwischen DAV und DAV
	 * @param transmitterListsUpdate Aufforderung zur Aktualisierung der Zulieferinformaion
	 */
	public final synchronized void handleListsUpdate(
			T_T_HighLevelCommunicationInterface connection, TransmitterListsUpdate transmitterListsUpdate) {
		if((connection == null) || (transmitterListsUpdate == null)) {
			throw new IllegalArgumentException("Argument ist null");
		}
		if(listsManager != null) {
			listsManager.updateEntry(transmitterListsUpdate);
			long ids[] = transmitterListsUpdate.getObjectsToAdd();
			AttributeGroupAspectCombination combinations[] = transmitterListsUpdate.getAttributeGroupAspectsToAdd();
			if((ids != null) || (combinations != null)) {
				OutSubscriptionsHelper subscriptionHelper = new OutSubscriptionsHelper();
				InAndOutSubscription involvedSubscriptions[] = subscriptionsManager.getInvolvedSubscriptionRequests(
						ids, combinations
				);
				if(involvedSubscriptions != null) {
					for(int i = 0; i < involvedSubscriptions.length; ++i) {
						if(involvedSubscriptions[i] != null) {
							cleanPendingSubscriptionToRedirect(involvedSubscriptions[i]);
							boolean done = false;
							if((involvedSubscriptions[i].outSubscriptions != null) && (involvedSubscriptions[i].receip <= TransmitterSubscriptionsConstants
									.NEGATIV_RECEIP)) {
								InAndOutSubscription subscriptionCopy = new InAndOutSubscription(involvedSubscriptions[i]);
								makeOutgoingRequests(subscriptionCopy);
								if(subscriptionCopy.outSubscriptions == null) {
									for(int j = 0; j < involvedSubscriptions[i].outSubscriptions.length; ++j) {
										if(involvedSubscriptions[i].outSubscriptions[j] != null) {
											subscriptionHelper.handleOutUnsubscription(
													involvedSubscriptions[i], involvedSubscriptions[i].outSubscriptions[j]
											);
										}
									}
									involvedSubscriptions[i].outSubscriptions = null;
									involvedSubscriptions[i].actualTransmitterToConsider = null;
									involvedSubscriptions[i].receip = -1;
									involvedSubscriptions[i].mainTransmitterId = -1;
									done = true;
								}
								else {
									boolean changed = false;
									ArrayList outSubscriptionsUnion = new ArrayList();
									for(int j = 0; j < involvedSubscriptions[i].outSubscriptions.length; ++j) {
										if(involvedSubscriptions[i].outSubscriptions[j] != null) {
											boolean notFound = true;
											for(int k = 0; k < subscriptionCopy.outSubscriptions.length; ++k) {
												if(subscriptionCopy.outSubscriptions[k] != null) {
													// to do : elaborate the transmitter to consider list.
													if((involvedSubscriptions[i].outSubscriptions[j].targetTransmitterId == subscriptionCopy.outSubscriptions[k]
															.targetTransmitterId) && Arrays.equals(
															involvedSubscriptions[i].outSubscriptions[j].transmitterToConsider,
															subscriptionCopy.outSubscriptions[k].transmitterToConsider
													)) {
														subscriptionCopy.outSubscriptions[k] = null;
														notFound = false;
														break;
													}
												}
											}
											if(notFound) {
												subscriptionHelper.handleOutUnsubscription(
														involvedSubscriptions[i], involvedSubscriptions[i].outSubscriptions[j]
												);
												changed = true;
											}
											else {
												outSubscriptionsUnion.add(involvedSubscriptions[i].outSubscriptions[j]);
											}
										}
									}
									for(int j = 0; j < subscriptionCopy.outSubscriptions.length; ++j) {
										if(subscriptionCopy.outSubscriptions[j] != null) {
											outSubscriptionsUnion.add(subscriptionCopy.outSubscriptions[j]);
											subscriptionHelper.handleOutSubscription(subscriptionCopy, subscriptionCopy.outSubscriptions[j]);
											changed = true;
										}
									}
									int size = outSubscriptionsUnion.size();
									if(size > 0) {
										involvedSubscriptions[i].outSubscriptions = new InAndOutSubscription.OutSubscription[size];
										for(int j = 0; j < size; ++j) {
											involvedSubscriptions[i].outSubscriptions[j] = (InAndOutSubscription.OutSubscription)outSubscriptionsUnion.get(
													j
											);
										}
									}
									if(changed) {
										involvedSubscriptions[i].actualTransmitterToConsider = subscriptionCopy.actualTransmitterToConsider;
										involvedSubscriptions[i].receip = -1;
									}
									done = true;
								}
							}
							if(done) {
								continue;
							}

							makeOutgoingRequests(involvedSubscriptions[i]);
							involvedSubscriptions[i].mainTransmitterId = -1;
							if(involvedSubscriptions[i].outSubscriptions == null) {
								involvedSubscriptions[i].actualTransmitterToConsider = null;
								if(involvedSubscriptions[i].receip != TransmitterSubscriptionsConstants.NEGATIV_RECEIP) {
									involvedSubscriptions[i].receip = TransmitterSubscriptionsConstants.NEGATIV_RECEIP;
									if(involvedSubscriptions[i].isTransmitterConnection) {
										TransmitterDataSubscriptionReceipt negativReceipt = new TransmitterDataSubscriptionReceipt(
												involvedSubscriptions[i].baseSubscriptionInfo,
												involvedSubscriptions[i].subscriptionState,
												TransmitterSubscriptionsConstants.NEGATIV_RECEIP,
												-1,
												involvedSubscriptions[i].transmitterToConsider
										);
										T_T_HighLevelCommunication _connection = getTransmitterConnection(
												involvedSubscriptions[i].sourceTransmitterId
										);
										if(_connection != null) {
											_connection.sendTelegram(negativReceipt);
										}
									}
									else {
										T_A_HighLevelCommunication _connection = getApplicationConnection(
												involvedSubscriptions[i].sourceTransmitterId
										);
										if(_connection != null) {
											T_A_SubscriptionComponent tmpSubscriptionComponent = (T_A_SubscriptionComponent)_connection.getSubscriptionComponent();
											if(tmpSubscriptionComponent != null) {
												if(involvedSubscriptions[i].subscriptionState == TransmitterSubscriptionsConstants
														.SENDER_SUBSCRIPTION) { // sending
													SendSubscriptionInfo _sendSubscriptionInfo = tmpSubscriptionComponent.getSendSubscription(
															involvedSubscriptions[i].baseSubscriptionInfo
													);
													if(_sendSubscriptionInfo != null) {
														if(_sendSubscriptionInfo.isRequestSupported()) {
															if(_sendSubscriptionInfo.getLastTriggerRequest() != (byte)1) {
																_sendSubscriptionInfo.setLastTriggerRequest((byte)1);
																_connection.triggerSender(
																		involvedSubscriptions[i].baseSubscriptionInfo, (byte)1
																);// stop sending
															}
														}
													}
												}
												else { // receiving
													ReceiveSubscriptionInfo _receiveSubscriptionInfo = tmpSubscriptionComponent.getReceiveSubscription(
															involvedSubscriptions[i].baseSubscriptionInfo
													);
													if(_receiveSubscriptionInfo != null) {
														long dataNumber = cacheManager.getCurrentDataIndex(involvedSubscriptions[i].baseSubscriptionInfo) + 1;
														if(_receiveSubscriptionInfo.getLastDataIndex() == dataNumber) {
															if(_receiveSubscriptionInfo.getLastErrorState() == 2) {
																continue;
															}
														}
														_receiveSubscriptionInfo.setLastDataIndex(dataNumber);
														_receiveSubscriptionInfo.setLastErrorState((byte)2);
														ApplicationDataTelegram dataToSend = new ApplicationDataTelegram(
																involvedSubscriptions[i].baseSubscriptionInfo,
																dataNumber,
																false,
																(byte)2,
																null,
																null,
																1,
																0,
																System.currentTimeMillis()
														);
														_connection.sendData(dataToSend);
													}
												}
											}
										}
									}
								}
							}
							else if(subscriptionsManager.needToSendSubscription(_transmitterId, involvedSubscriptions[i])) {
								involvedSubscriptions[i].receip = (byte)-1;
								for(int j = 0; j < involvedSubscriptions[i].outSubscriptions.length; ++j) {
									if(involvedSubscriptions[i].outSubscriptions[j] != null) {
										subscriptionHelper.handleOutSubscription(
												involvedSubscriptions[i], involvedSubscriptions[i].outSubscriptions[j]
										);
									}
								}
							}
							else {
								involvedSubscriptions[i].outSubscriptions = null;
							}
						}
					}
				}

				// missed redirections
				involvedSubscriptions = subscriptionsManager.getInvolvedPositiveSubscriptionRequests(ids, combinations);
				if(involvedSubscriptions != null) {
					for(int i = 0; i < involvedSubscriptions.length; ++i) {
						if(involvedSubscriptions[i] != null) {
							long oldWay = -1;
							long bestWay = bestWayManager.getBestWay(involvedSubscriptions[i].mainTransmitterId);

							boolean mustBeRedirected = false;
							for(int j = 0; j < involvedSubscriptions[i].outSubscriptions.length; ++j) {
								if((involvedSubscriptions[i].outSubscriptions[j] != null) && (involvedSubscriptions[i].outSubscriptions[j].targetTransmitterId
								                                                              != bestWay)) {
									oldWay = involvedSubscriptions[i].outSubscriptions[j].targetTransmitterId;
									mustBeRedirected = true;
									break;
								}
							}
							if(mustBeRedirected) {
								for(int j = _pendingSubscriptionToRedirect.size() - 1; j > -1; --j) {
									ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
											j
									);
									if(exchangeSubscriptionAction != null) {
										if(exchangeSubscriptionAction.oldSubscription == involvedSubscriptions[i]) {
											if(bestWay == exchangeSubscriptionAction.newRootingTransmitterId) {
												mustBeRedirected = false;
											}
											else {
												_pendingSubscriptionToRedirect.remove(j);
												for(int k = 0; k < exchangeSubscriptionAction.newSubscription.outSubscriptions.length; ++k) {
													if(exchangeSubscriptionAction.newSubscription.outSubscriptions[k] != null) {
														subscriptionHelper.handleOutUnsubscription(
																exchangeSubscriptionAction.newSubscription,
																exchangeSubscriptionAction.newSubscription.outSubscriptions[k]
														);
													}
												}
											}
											break;
										}
									}
								}
								if(mustBeRedirected) {
									InAndOutSubscription request = new InAndOutSubscription(involvedSubscriptions[i]);
									makeOutgoingRequests(request);
									if(request.outSubscriptions != null) {
										long dataIndex = cacheManager.getCurrentDataIndex(involvedSubscriptions[i].baseSubscriptionInfo);
										_pendingSubscriptionToRedirect.add(
												new ExchangeSubscriptionAction(
														involvedSubscriptions[i],
														request,
														oldWay,
														bestWay,
														involvedSubscriptions[i].mainTransmitterId,
														dataIndex
												)
										);
										for(int j = 0; j < request.outSubscriptions.length; ++j) {
											if(request.outSubscriptions[j] != null) {
												subscriptionHelper.handleOutSubscription(request, request.outSubscriptions[j]);
											}
										}
									}
								}
							}
						}
					}
				}
				subscriptionHelper.flushOutUnSubscription(this);
			}
		}
	}

	/**
	 * Diese Methode wird von der Anmeldungsverwaltung aufgerufen, wenn ein neuer Datensatz angekommen ist. Hier wird berprft, ob ein Umleitungsantrag auf dem
	 * spezifizierten Datenverteiler connectedTransmitterId basiert, d. h., dass entweder der alte oder der neue Weg ber diesen Datenverteiler luft. Der Antrag
	 * wird dann mit dem Index des Datensatzes dataIndex aktualisiert. Wenn der Datensatzindex ber den neuen Weg gleich oder um eins hher als der Datensatzindex
	 * ber den alten Weg ist, dann wird die Umleitung als erfolgreich aufgebaut angesehen. Die Folgeanmeldungen ber den alten Weg werden abgemeldet, und der
	 * Umleitungsantrag wird aus der Umleitungsverwaltung entfernt.
	 *
	 * @param info                   Datenidentifikation
	 * @param connectedTransmitterId ID des DAV
	 * @param dataIndex              Datensatzindex
	 *
	 * @return true wenn Datensatzindex gleich oder um eins hher als der Datensatzindex ber den alten Weg ist, sonst false.
	 */
	final synchronized boolean updatePendingSubscription(
			BaseSubscriptionInfo info, long connectedTransmitterId, long dataIndex) {
		for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
			ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
					i
			);
			if(exchangeSubscriptionAction != null) {
				if(exchangeSubscriptionAction.oldSubscription.baseSubscriptionInfo.equals(info)) {
					boolean canSend = exchangeSubscriptionAction.setDataIndex(connectedTransmitterId, dataIndex);
					if(exchangeSubscriptionAction.isRedirectionSafe()) {
						_pendingSubscriptionToRedirect.remove(i);
						InAndOutSubscription oldSubscription = exchangeSubscriptionAction.oldSubscription;
						if(oldSubscription != null) {
							for(int j = 0; j < oldSubscription.outSubscriptions.length; ++j) {
								if(oldSubscription.outSubscriptions[j] != null) {
									TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
											oldSubscription.baseSubscriptionInfo,
											oldSubscription.subscriptionState,
											oldSubscription.outSubscriptions[j].transmitterToConsider
									);
									T_T_HighLevelCommunication _connection = getTransmitterConnection(
											oldSubscription.outSubscriptions[j].targetTransmitterId
									);
									if(_connection != null) {
										_connection.sendTelegram(_unsubscription);
									}
								}
							}
							oldSubscription.receip = exchangeSubscriptionAction.newSubscription.receip;
							oldSubscription.mainTransmitterId = exchangeSubscriptionAction.newSubscription.mainTransmitterId;
							oldSubscription.actualTransmitterToConsider = exchangeSubscriptionAction.newSubscription.actualTransmitterToConsider;
							oldSubscription.outSubscriptions = exchangeSubscriptionAction.newSubscription.outSubscriptions;
						}
						return true;
					}
					return canSend;
				}
			}
		}
		return true;
	}

	/**
	 * Diese Methode wird von der Protokollsteuerung aufgerufen, um einer Verbindung ein Gewicht zuzuweisen. Die Information wird von der Wegverwaltung benutzt,
	 * wenn eine Verbindung bewertet wird.
	 *
	 * @param connectedTransmitterId ID des DAV
	 *
	 * @return Gewichtung der Verbindung
	 */
	public final short getWeight(long connectedTransmitterId) {
		short weight = DEFAULT_WEIGHT;
		if(_transmitterConnectionInfos != null) {
			for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
				TransmitterInfo t1 = _transmitterConnectionInfos[i].getTransmitter_1();
				if(t1 != null) {
					if(t1.getTransmitterId() == _transmitterId) {
						TransmitterInfo t2 = _transmitterConnectionInfos[i].getTransmitter_2();
						if((t2 != null) && (t2.getTransmitterId() == connectedTransmitterId)) {
							weight = _transmitterConnectionInfos[i].getWeight();
						}
					}
					else if(t1.getTransmitterId() == connectedTransmitterId) {
						TransmitterInfo t2 = _transmitterConnectionInfos[i].getTransmitter_2();
						if((t2 != null) && (t2.getTransmitterId() == _transmitterId)) {
							weight = _transmitterConnectionInfos[i].getWeight();
						}
					}
				}
			}
		}
		return weight;
	}

	/**
	 * Bestimmt die Verbindungsinformationen fr eine Verbindung von diesem Datenverteiler zum angegebenen Datenverteiler.
	 *
	 * @param connectedTransmitterId ID des DAV
	 *
	 * @return Verbindungsinformationen
	 */
	private final TransmitterConnectionInfo getTransmitterConnectionInfo(long connectedTransmitterId) {
		if(_transmitterConnectionInfos != null) {
			for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
				_debug.finest("getTransmitterConnectionInfo: prfe", _transmitterConnectionInfos[i].parseToString());
				TransmitterInfo t1 = _transmitterConnectionInfos[i].getTransmitter_1();
				if(t1.getTransmitterId() == _transmitterId) {
					TransmitterInfo t2 = _transmitterConnectionInfos[i].getTransmitter_2();
					if(t2.getTransmitterId() == connectedTransmitterId) {
						_debug.finer("getTransmitterConnectionInfo: gefunden", _transmitterConnectionInfos[i].parseToString());
						return _transmitterConnectionInfos[i];
					}
				}
			}
		}
		_debug.finer("getTransmitterConnectionInfo: nicht gefunden", connectedTransmitterId);
		return null;
	}

	/**
	 * Bestimmt die Verbindungsinformationen fr eine Verbindung vom angegebenen Datenverteiler zu diesem Datenverteiler.
	 *
	 * @param connectedTransmitterId ID des DAV
	 *
	 * @return Verbindungsinformationen
	 */
	private final TransmitterConnectionInfo getRemoteTransmitterConnectionInfo(long connectedTransmitterId) {
		if(_transmitterConnectionInfos != null) {
			for(int i = 0; i < _transmitterConnectionInfos.length; ++i) {
				_debug.finest("getRemoteTransmitterConnectionInfo: prfe", _transmitterConnectionInfos[i].parseToString());
				TransmitterInfo t1 = _transmitterConnectionInfos[i].getTransmitter_1();
				TransmitterInfo t2 = _transmitterConnectionInfos[i].getTransmitter_2();
				if(t2.getTransmitterId() == _transmitterId && t1.getTransmitterId() == connectedTransmitterId) {
					_debug.finer("getRemoteTransmitterConnectionInfo: gefunden", _transmitterConnectionInfos[i].parseToString());
					return _transmitterConnectionInfos[i];
				}
			}
		}
		_debug.finer("getRemoteTransmitterConnectionInfo: nicht gefunden", connectedTransmitterId);
		return null;
	}

	/**
	 * Bestimmt den Benutzername der zur Authentifizierung beim angegebenen Datenverteiler benutzt werden soll. Wenn der Benutzername in der Topologie nicht
	 * vorgegeben ist, dann wird der Standardbenutzername des Datenverteilers benutzt.
	 *
	 * @param connectedTransmitterId Objekt-ID des anderen Datenverteilers.
	 *
	 * @return Benutzername fr die Authentifizierung beim anderen Datenverteiler.
	 */
	public String getUserNameForAuthentification(long connectedTransmitterId) {
		String userName = "";
		final TransmitterConnectionInfo info = getTransmitterConnectionInfo(connectedTransmitterId);
		if(info != null) {
			userName = info.getUserName();
		}
		else {
			final TransmitterConnectionInfo remoteInfo = getRemoteTransmitterConnectionInfo(connectedTransmitterId);
			if(remoteInfo != null) {
				userName = remoteInfo.getRemoteUserName();
			}
			else {
				_debug.warning("Keine Verbindungsinfo fr Verbindung zum Datenverteiler " + connectedTransmitterId + " gefunden.");
			}
		}
		if(userName.equals("")) {
			userName = _serverDavParameters.getUserName();
			_debug.warning(
					"Kein Benutzername fr die Authentifizierung beim Datenverteiler " + connectedTransmitterId + " gefunden."
					+ " Es wird der Default-Benutzername " + userName + " benutzt."
			);
		}
		return userName;
	}

	/**
	 * Bestimmt das Benutzerpasswort das zur Authentifizierung beim angegebenen Datenverteiler benutzt werden soll. Wenn der Benutzername in der Topologie nicht
	 * vorgegeben ist, dann wird das Passwort des Standardbenutzers des Datenverteilers zurckgegeben.
	 *
	 * @param connectedTransmitterId Objekt-ID des anderen Datenverteilers.
	 *
	 * @return Passwort fr die Authentifizierung beim anderen Datenverteiler.
	 */
	public String getPasswordForAuthentification(long connectedTransmitterId) {
		String userName = "";
		final TransmitterConnectionInfo info = getTransmitterConnectionInfo(connectedTransmitterId);
		if(info != null) {
			userName = info.getUserName();
		}
		else {
			final TransmitterConnectionInfo remoteInfo = getRemoteTransmitterConnectionInfo(connectedTransmitterId);
			if(remoteInfo != null) {
				userName = remoteInfo.getRemoteUserName();
			}
		}
		if(userName.equals("")) {
			return _serverDavParameters.getUserPassword();
		}
		return _serverDavParameters.getStoredPassword(userName);
	}

	public synchronized final void updateDestinationRoute(
			long _transmitterId, RoutingConnectionInterface oldConnection, RoutingConnectionInterface newConnection) {
		if(oldConnection == null) {
			return;
		}
		if(newConnection == null) {
			return;
		}

		long oldRouterId = oldConnection.getRemoteNodeId();
		long newRouterId = newConnection.getRemoteNodeId();

		if(oldRouterId == newRouterId) {
			return;
		}

		for(int i = _pendingSubscriptionToRedirect.size() - 1; i > -1; --i) {
			ExchangeSubscriptionAction exchangeSubscriptionAction = (ExchangeSubscriptionAction)_pendingSubscriptionToRedirect.get(
					i
			);
			if(exchangeSubscriptionAction != null) {
				if(exchangeSubscriptionAction.transmitterId == _transmitterId) {
					_pendingSubscriptionToRedirect.remove(i);
					InAndOutSubscription request = exchangeSubscriptionAction.newSubscription;
					if(request != null) {
						for(int j = 0; j < request.outSubscriptions.length; ++j) {
							if(request.outSubscriptions[j] != null) {
								TransmitterDataUnsubscription _unsubscription = new TransmitterDataUnsubscription(
										request.baseSubscriptionInfo, request.subscriptionState, request.outSubscriptions[j].transmitterToConsider
								);
								T_T_HighLevelCommunication _connection = getTransmitterConnection(
										request.outSubscriptions[j].targetTransmitterId
								);
								if(_connection != null) {
									_connection.sendTelegram(_unsubscription);
								}
							}
						}
					}
				}
			}
		}

		InAndOutSubscription involvedSubscriptions[] = subscriptionsManager.getSubscriptionRequestsToRedirect(
				_transmitterId, oldRouterId
		);
		if(involvedSubscriptions != null) {
			for(int i = 0; i < involvedSubscriptions.length; ++i) {
				if(involvedSubscriptions[i] != null) {
					InAndOutSubscription request = new InAndOutSubscription(involvedSubscriptions[i]);
					makeOutgoingRequests(request);
					if(request.outSubscriptions != null) {
						long dataIndex = cacheManager.getCurrentDataIndex(involvedSubscriptions[i].baseSubscriptionInfo);
						_pendingSubscriptionToRedirect.add(
								new ExchangeSubscriptionAction(
										involvedSubscriptions[i], request, oldRouterId, newRouterId, _transmitterId, dataIndex
								)
						);
						for(int j = 0; j < request.outSubscriptions.length; ++j) {
							if(request.outSubscriptions[j] != null) {
								TransmitterDataSubscription _subscription = new TransmitterDataSubscription(
										request.baseSubscriptionInfo, request.subscriptionState, request.outSubscriptions[j].transmitterToConsider
								);
								T_T_HighLevelCommunication _connection = getTransmitterConnection(
										request.outSubscriptions[j].targetTransmitterId
								);
								if(_connection != null) {
									_connection.sendTelegram(_subscription);
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Diese Methode wird von der Datenverteilerapplikation aufgerufen wenn ein Datenverteiler heruntergefahren wird. Zuerst werden die Threads terminiert, die fr
	 * das Erstellen der Kommunikationskanle zwischen Applikation bzw. Datenverteiler und diesem Datenverteiler zustndig sind. Danach werden alle bereits
	 * vorhandenen Verbindungen zu anderen Applikationen und Datenverteilern terminiert, so dass diese auch informiert werden, dass dieser Datenverteiler nicht
	 * mehr zur Verfgung steht.
	 *
	 * @param error   True: ein Fehler ist aufgetreten
	 * @param message genauere Beschreibung des Fehlers
	 */
	public final void close(boolean error, String message) {
		if(_closing) return;
		_closing = true;
		synchronized(this) {
			String debugMessage = "Der Datenverteiler wird beendet. Ursache: " + message;
			if(error) {
				_debug.error(debugMessage);
			}
			else {
				_debug.warning(debugMessage);
			}
			if(_applicationConnectionsSubscriber != null) {
				_applicationConnectionsSubscriber.interrupt();
				_applicationConnectionsSubscriber = null;
			}
			if(_transmitterConnectionsSubscriber != null) {
				_transmitterConnectionsSubscriber.interrupt();
				_transmitterConnectionsSubscriber = null;
			}
			if(_applicationsServerConnection != null) {
				_applicationsServerConnection.disconnect();
			}
			if(_transmittersServerConnection != null) {
				_transmittersServerConnection.disconnect();
			}

			if(listsManager != null) {
				listsManager.close();
			}

			synchronized(_transmitterConnections) {
				for(T_T_HighLevelCommunication transmitterConnection : _transmitterConnections) {
					if(transmitterConnection != null) {
						transmitterConnection.terminate(error, message);
					}
				}
				_transmitterConnections.clear();
			}

			synchronized(_applicationConnections) {
				for(T_A_HighLevelCommunication connection : _applicationConnections) {
					if(connection != null) {
						connection.terminate(error, message);
					}
				}
				_applicationConnections.clear();
			}
		}
	}


	/**
	 * Erstellt eine Liste der DAV-DAV-Verbindungen fr den entsperchenden Benutzer.
	 *
	 * @param userId ID des Benutzers
	 *
	 * @return Liste der Verbindungen
	 */
	private List getTransmitterConnections(long userId) {
		ArrayList result = new ArrayList();
		for(T_T_HighLevelCommunication transmitterConnection : _transmitterConnections) {
			if(transmitterConnection != null) {
				if(transmitterConnection.getRemoteUserId() == userId) {
					result.add(transmitterConnection);
				}
			}
		}
		return result;
	}

	/**
	 * Erstellt eine Liste der DAV-DAF-Verbindungen fr den entsprechenden Benutzer.
	 *
	 * @param userId ID des Benutzers
	 *
	 * @return Liste der Applikationsverbindungen
	 */
	private final List getApplicationConnections(long userId) {
		ArrayList result = new ArrayList();
		for(T_A_HighLevelCommunication applicationConnection : _applicationConnections) {
			if(applicationConnection != null) {
				if(applicationConnection.getRemoteUserId() == userId) {
					result.add(applicationConnection);
				}
			}
		}
		return result;
	}

	/**
	 * Entfernt Verbindung zwischen zwei DAV
	 *
	 * @param communicationTransmitterId Id des verbundenen Datenverteilers
	 */
	private void removeTransmitterConnection(long communicationTransmitterId) {
		synchronized(_transmitterConnections) {
			for(int i = _transmitterConnections.size() - 1; i > -1; --i) {
				T_T_HighLevelCommunication highLevelCommunication = (T_T_HighLevelCommunication)_transmitterConnections.get(i);
				if(highLevelCommunication != null) {
					if(highLevelCommunication.getId() == communicationTransmitterId) {
						_transmitterConnections.remove(i);
						authentificationManager.removeUser(highLevelCommunication.getRemoteUserId());
						break;
					}
				}
			}
		}
	}

	/**
	 * Entfernt Verbindung zwischen DAV und DAF
	 *
	 * @param applicationId Id der verbundenen Applikation
	 * @return Applikationsobjekt, das der angegebenen Applikation zugeordnet ist.
	 */
	private final SystemObject removeApplicationConnection(long applicationId) {
		boolean foundApplication = false;
		synchronized(_applicationConnections) {
			for(int i = _applicationConnections.size() - 1; i > -1; --i) {
				T_A_HighLevelCommunication highLevelCommunication = _applicationConnections.get(i);
				if(highLevelCommunication != null) {
					if(highLevelCommunication.getId() == applicationId) {
						final T_A_HighLevelCommunication removedHighLevelCommunication = _applicationConnections.remove(i);
						authentificationManager.removeUser(highLevelCommunication.getRemoteUserId());

						// Nur wenn der Thread auch vorhanden ist
						if(_applicationStatusUpdater != null) {
							_applicationStatusUpdater.applicationRemoved(removedHighLevelCommunication);
							_applicationConnections.notifyAll();
						}
						foundApplication = true;
						break;
					}
				}
			}
		}
		if(foundApplication) {
			if(dataModel != null && applicationId != 0 && applicationId != -1) {
				final SystemObject applicationObject = dataModel.getObject(applicationId);
				if(applicationObject != null && applicationObject instanceof ClientApplication) {
					return applicationObject;
				}
			}
		}
		return null;
	}

	/** Diese Methode ruft <code>subscriptionsManager.printSubscriptions</code> auf, welche die Anmeldeinformationen in den OutputSream schreibt. */
	public void printSubscriptions() {
		subscriptionsManager.printSubscriptions();
	}

	/**
	 * Setzt den Datenverteilersbetriebsmodus auf den Lokalen Modus.
	 *
	 * @param configurationPid Pid der Konfigurationsapplikation
	 * @param configurationId  Id der Konfigurationsapplikation
	 */
	public void setLocaleModeParameter(final String configurationPid, final long configurationId) {
		_serverDavParameters.setLocalModeParameter(configurationPid, configurationId);
	}

	/**
	 * Gibt die configurationPid des Lokalen Modus zurck.
	 *
	 * @return <code>configurationPid</code>
	 */
	public String getLocaleModeConfigurationPid() {
		final Object[] localeModeParameter = _serverDavParameters.getLocalModeParameter();
		if(localeModeParameter == null) return "";
		return (String)localeModeParameter[0];
	}

	/**
	 * Gibt die configurationId des Lokalen Modus zurck.
	 *
	 * @return <code>configurationId</code>
	 */
	public long getLocaleModeConfigurationId() {
		final Object[] localeModeParameter = _serverDavParameters.getLocalModeParameter();
		if(localeModeParameter == null) return -1;
		return (Long)localeModeParameter[1];
	}

	/** Diese Subklasse startet einen Thread, der eine Application bei einem Datenverteiler anmeldet. */
	class ApplicationConnectionsSubscriber extends Thread {

		public ApplicationConnectionsSubscriber() {
			super("ApplicationConnectionsSubscriber");
			//XXX setPriority(Thread.MIN_PRIORITY);
		}

		/** The run method that loops through */
		public final void run() {
			if(_applicationsServerConnection == null) {
				return;
			}
			while(!isInterrupted()) {
				// Blocks until a connection occurs:
				final ConnectionInterface connection = _applicationsServerConnection.accept();
				Runnable runnable = new Runnable() {
					public void run() {
						if(connection == null) {
							return;
						}
						else {
							try {
								LowLevelCommunication lowLevelCommunication = new LowLevelCommunication(
										connection,
										_serverDavParameters.getCommunicationOutputBufferSize(),
										_serverDavParameters.getCommunicationInputBufferSize(),
										_serverDavParameters.getSendKeepAliveTimeout(),
										_serverDavParameters.getReceiveKeepAliveTimeout(),
										LowLevelCommunication.NORMAL_MODE,
										true
								);
								ServerConnectionProperties properties = new ServerConnectionProperties(
										lowLevelCommunication, _authentificationComponent, _serverDavParameters
								);
								T_A_HighLevelCommunication highLevelCommunication = new T_A_HighLevelCommunication(
										properties, subscriptionsManager, ConnectionsManager.this, !_configurationAvaillable
								);
								highLevelCommunication.setApplicationStatusUpdater(_applicationStatusUpdater);

								_applicationConnections.add(highLevelCommunication);
							}
							catch(ConnectionException ex) {
								ex.printStackTrace();
							}
						}
					}
				};
				Thread thread = new Thread(runnable);
				thread.start();
				yield();
			}
		}
	}

	/** Diese Subklasse startet eine Thread, der eine DAV bei einem DAV anmeldet. */

	class TransmitterConnectionsSubscriber extends Thread {

		public TransmitterConnectionsSubscriber() {
			super("TransmitterConnectionsSubscriber");
			//XXX setPriority(Thread.MIN_PRIORITY);
		}

		/** The run method that loops through */
		public final void run() {
			if(_transmittersServerConnection == null) {
				return;
			}
			while(!isInterrupted()) {
				// Blocks until a connection occurs:
				final ConnectionInterface connection = _transmittersServerConnection.accept();
				Runnable runnable = new Runnable() {
					public void run() {
						if(connection == null) {
							return;
						}
						else {
							try {
								boolean connectionExists = false;
								for(T_T_HighLevelCommunication transmitterConnection : _transmitterConnections) {
									if(transmitterConnection != null) {
										String adress = transmitterConnection.getRemoteAdress();
										int subAdress = transmitterConnection.getRemoteSubadress();
										if((adress != null) && (adress.equals(connection.getMainAdress())) && (subAdress
										                                                                       == connection.getSubAdressNumber())) {
											connectionExists = true;
											break;
										}
									}
								}
								if(!connectionExists) {
									LowLevelCommunication lowLevelCommunication = new LowLevelCommunication(
											connection,
											_serverDavParameters.getCommunicationOutputBufferSize(),
											_serverDavParameters.getCommunicationInputBufferSize(),
											_serverDavParameters.getSendKeepAliveTimeout(),
											_serverDavParameters.getReceiveKeepAliveTimeout(),
											LowLevelCommunication.NORMAL_MODE,
											true
									);
									ServerConnectionProperties properties = new ServerConnectionProperties(
											lowLevelCommunication, _authentificationComponent, _serverDavParameters
									);
									T_T_HighLevelCommunication highLevelCommunication = new T_T_HighLevelCommunication(
											properties,
											subscriptionsManager,
											ConnectionsManager.this,
											bestWayManager,
											DEFAULT_WEIGHT,
											!_configurationAvaillable,
											"",
											""
									);
									_transmitterConnections.add(highLevelCommunication);
								}
							}
							catch(ConnectionException ex) {
								ex.printStackTrace();
							}
						}
					}
				};
				Thread thread = new Thread(runnable);
				thread.start();
				yield();
			}
		}
	}

	class TransmitterConnectionsMonitor extends Thread {

		public TransmitterConnectionsMonitor() {
			super("TransmitterConnectionsMonitor");
			//XXX setPriority(Thread.MIN_PRIORITY);
		}

		/** The run method that loops through */
		public final void run() {
			if(missedConnectionInfos == null) {
				return;
			}
			while(!isInterrupted()) {
				try {
					synchronized(missedConnectionInfos) {
						for(int i = missedConnectionInfos.size() - 1; i > -1; --i) {
							TransmitterConnectionInfo transmitterConnectionInfo = (TransmitterConnectionInfo)missedConnectionInfos.get(i);
							if(transmitterConnectionInfo != null) {
								if(connectToMainTransmitter(transmitterConnectionInfo)) {
									missedConnectionInfos.remove(i);

									if(transmitterConnectionInfo.isAutoExchangeTransmitterDetectionOn()) {
										TransmitterInfo t2 = transmitterConnectionInfo.getTransmitter_2();
										TransmitterConnectionInfo infos[] = getInvolvedTransmitters(t2);
										if(infos != null) {
											for(int j = 0; j < infos.length; ++j) {
												try {
													TransmitterInfo transmitterInfo = infos[j].getTransmitter_2();
													if(transmitterInfo != null) {
														T_T_HighLevelCommunication connection = getTransmitterConnection(transmitterInfo.getTransmitterId());
														if((connection != null) && (!connection.isAcceptedConnection())) {
															connection.terminate(
																	false,
																	"Automatisch ermittelte Ersatzverbindung wird nicht mehr bentigt, weil ursprngliche Verbindung wiederhergestellt wurde"
															);
														}
													}
												}
												catch(Exception e) {
													e.printStackTrace();
												}
											}
										}
									}
									else {
										TransmitterInfo infos[] = transmitterConnectionInfo.getExchangeTransmitterList();
										if(infos != null) {
											for(int j = 0; j < infos.length; ++j) {
												TransmitterConnectionInfo transmitterConnectionInfoToDisconnect = null;
												for(int k = 0; k < _transmitterConnectionInfos.length; ++k) {
													if(_transmitterConnectionInfos[k].isExchangeConnection()
													   && (_transmitterConnectionInfos[k].getTransmitter_1().getTransmitterId() == _transmitterId) && (
															_transmitterConnectionInfos[k].getTransmitter_2().getTransmitterId()
															== infos[j].getTransmitterId())) {
														transmitterConnectionInfoToDisconnect = _transmitterConnectionInfos[k];
														break;
													}
												}
												if(transmitterConnectionInfoToDisconnect != null) {
													try {
														T_T_HighLevelCommunication connection = getTransmitterConnection(infos[j].getTransmitterId());
														if((connection != null) && (!connection.isAcceptedConnection())) {
															connection.terminate(
																	false,
																	"Konfigurierte Ersatzverbindung wird nicht mehr bentigt, weil ursprngliche Verbindung wiederhergestellt wurde"
															);
														}
													}
													catch(Exception e) {
														e.printStackTrace();
													}
												}
											}
										}
									}
								}
								else {
									connectToAlternativeTransmitters(transmitterConnectionInfo);
								}
							}
						}
						missedConnectionInfos.wait(10000);
					}
				}
				catch(InterruptedException ex) {
					return;
				}
				yield();
			}
		}
	}

	/**
	 * Bestimmt die Einstellungen des Datenverteilers, die ber Aufrufargumente verndert werden knnen.
	 *
	 * @return Einstellungen des Datenverteilers.
	 */
	public ServerDavParameters getServerDavParameters() {
		return _serverDavParameters;
	}

	/**
	 * Dieser Thread verschickt den Status(angemeldete Appliktionen, Telegrammlaufzeiten, durch Applikationen angemeldete Datenidentifikationen) aller angemeldeten
	 * Applikationen.
	 */
	public final class ApplicationStatusUpdater extends Thread {

		/** Enthlt alle Applikationen, die an/ab gemeldet werden sollen. Die Elemente werden nach Fifo bearbeitet. */
		private final List<ApplicationConnection> _application = Collections.synchronizedList(new ArrayList<ApplicationConnection>());

		private final SystemObject _davObject;

		/** Zum verschicken aller angemeldeten Applikationen */
		private final SourceApplicationUpdater _applicationUpdater = new SourceApplicationUpdater(this);

		/** DataDescription, mit der eine Quelle zum verschicken aller angemeldeten Applikationen angemeldet wird. */
		private final DataDescription _applicationUpdaterDataDescription;

		/** Zum verschicken aller Telegrammlaufzeiten der angemeldeten Applikationen */
		private final SourceApplicationUpdater _applicationRuntime = new SourceApplicationUpdater(this);

		/** DataDescription, mit der eine Quelle zum verschicken der Telegrammlaufzeiten der angemeldeten Applikationen. */
		private final DataDescription _applicationRuntimeDataDescription;

		/** Die DataDescription, die zum Versandt von Anmeldungen einer Applikation benutzt wird. */
		private final DataDescription _applicationDataDescriptionDD;

		/** Verschickt zyklisch alle angemeldeten Applikationen und deren Telegrammlaufzeiten. */
		private final Timer _timer = new Timer("Status der Applikation zyklisch verschicken", true);

		/**
		 * Wird fr eine Applikation eine neue Datenidentifikation angemeldet, so wird dies ber einen TimerTask publiziert. Der TimerTask, der diese Aufgabe
		 * bernimmt, wird hier gespeichert. Der TimerTask kann jederzeit unterbrochen werden und durch einen neuen TimerTask ersetzt werden. Auch wenn der TimerTask
		 * bereits ausgefhrt wurde befindet er sich weiterhin in dieser Map und kann durch einen neuen TimerTask ersetzt werden.
		 * <p/>
		 * Als Schlssel dient die HighLevelCommu, dies entspricht der Applikation, deren Anmeldungen verschickt werden sollen. Als Value wird ein TimerTask
		 * zurckgegeben, der alle angemeldeten Datenidentifikationen der Applikation verschickt, sobald der Thread ausgefhrt wird. Der Thread kann bereits
		 * ausgefhrt worden sein oder aber noch ausgefhrt werden (oder befindet sich in Bearbeitung). Wurde noch kein TimerTask angelegt, so wird <code>null</code>
		 * zurckgegeben.
		 */
		private final Map<T_A_HighLevelCommunication, TimerTask> _threadsForDataIdentificationUpdates = new HashMap<T_A_HighLevelCommunication, TimerTask>();

		/**
		 * Key = Applikation, die bereits Daten angemeldet hat; Value = ClientSenderInterface(SourceApplicationUpdater). Damit kann spter die Verbindung wieder
		 * abgemeldet werden und es wird ein mehrfaches anmelden verhindert. Gleichzeit kann das Objekt befragt werden, ob berhaupt gesendet werden soll.
		 */
		private final Map<T_A_HighLevelCommunication, SourceApplicationUpdater> _subcribedDataIdentifications = new HashMap<T_A_HighLevelCommunication, SourceApplicationUpdater>();

		public ApplicationStatusUpdater() {
			try {
				this.setDaemon(true);

				setName("Applikationsstatus-Updater");

				_davObject = dataModel.getObject(_transmitterId);

				// Als Quelle fr alle angemeldeten Applikationen anmelden

				final AttributeGroup applicationUpdaterATG = dataModel.getAttributeGroup(_pidAngemeldeteApplikationen);
				final Aspect applicationUpdaterAspect = dataModel.getAspect("asp.standard");
				_applicationUpdaterDataDescription = new DataDescription(applicationUpdaterATG, applicationUpdaterAspect);
				selfClientDavConnection.subscribeSender(_applicationUpdater, _davObject, _applicationUpdaterDataDescription, SenderRole.source());

				// Als Quelle fr Telegrammlaufzeiten zu allen angemeldeten Applikationen anmelden

				final AttributeGroup applicationTelegramRuntimeATG = dataModel.getAttributeGroup(_pidTelegrammLaufzeiten);
				final Aspect applicationTelegramRuntimeAspect = dataModel.getAspect("asp.messwerte");
				_applicationRuntimeDataDescription = new DataDescription(applicationTelegramRuntimeATG, applicationTelegramRuntimeAspect);
				selfClientDavConnection.subscribeSender(_applicationRuntime, _davObject, _applicationRuntimeDataDescription, SenderRole.source());

				// Datenidentifikation zum verschicken von "Datenidentifikationen, die zu einer Applikation gehren" anlegen
				_applicationDataDescriptionDD = new DataDescription(
						dataModel.getAttributeGroup(_pidAngemeldeteDatenidentifikationen), dataModel.getAspect("asp.standard")
				);
			}
			catch(OneSubscriptionPerSendData oneSubscriptionPerSendData) {
				final String errorMessage = "Der Thread zur Publizierung des Status aller angemeldeten Applikationen konnte nicht gestartet werden.";
				_debug.error(
						errorMessage, oneSubscriptionPerSendData
				);
				throw new IllegalStateException(errorMessage);
			}
		}


		/**
		 * Prft, ob das bergebene Objekt gleich dem Objekt ist, das die Applikation des DaV darstellt, die sich selbst beim DaV anmeldet oder aber die
		 * Konfiguration.
		 * <p/>
		 * Wurde bisher noch kein DaV-Objekt gesucht/gefunden und das bergebene Objekt ist das DaV-Objekt, so wird das Objekt einen internen Variablen zugewiesen und
		 * <code>true</code> zurck gegeben.
		 *
		 * @param unknownObject Objekt, das vielleicht das DaV-Objekt darstellen knnte.
		 *
		 * @return true = Es handelt sich um das DaV-Objekt oder das Konfigurationsobjekt -> Das Objekt muss ausgeblendet werden; false = sonst
		 */
		private boolean isSpecialTreatedApplicationObject(T_A_HighLevelCommunication unknownObject) {

			// 1) Die Konfiguration besitzt weder ein Systemobjekt noch einen Benutzer
			// 2) Das Applikationsobjekt des Datenverteilers ist vom Typ "typ.datenverteiler"

			final long applicationId = unknownObject.getId();
			if(applicationId == 0 || applicationId == -1) return true;
			final SystemObject unknownSystemobject = dataModel.getObject(applicationId);
			final SystemObject unknownUser = dataModel.getObject(unknownObject.getRemoteUserId());

			if(unknownSystemobject == null) {
				if(unknownUser == null) {
					return true;
				}
				else {
					_debug.error("Es gibt kein Systemobjekt, aber einen Benutzer: " + unknownUser);
					return true;
				}
			}
			else {
				// Das Systemobjekt ist vorhanden, hat es den richtigen Typ ?
				if(unknownSystemobject instanceof ClientApplication) {
					return false;
				}
				else {
					return true;
				}
			}
		}

		/** Es wird ein Datensatz verschickt, der alle Applikationen enthlt, die gerade am DaV angemeldet sind. */
		private void sendApplicationUpdate() {
			ResultData resultData = null;
			Vector<T_A_HighLevelCommunication> applicationConnectionsCopy = null;
			if(_applicationUpdater.sendData() == true) {
				// Es gibt einen Empfnger fr die Daten
				applicationConnectionsCopy = new Vector<T_A_HighLevelCommunication>(_applicationConnections);
			}
			if(applicationConnectionsCopy != null) {
				// Es gibt einen Empfnger fr die Daten
				final Data data = selfClientDavConnection.createData(_applicationUpdaterDataDescription.getAttributeGroup());

				// Alle Applikationen eintragen, die derzeit angemeldet sind.
				final Data.Array subscribedApplications = data.getItem("angemeldeteApplikation").asArray();
				// Diese gre pat vielleicht nicht. Das Applikationsobjekt fr die Konfiguration und das Applikationsobjekt fr den DaV muss noch entfernt werden.
				subscribedApplications.setLength(applicationConnectionsCopy.size());

				// Index, an dem ein neues Element in das Array eingefgt werden muss
				int dataIndex = 0;

				for(T_A_HighLevelCommunication applicationConnection : applicationConnectionsCopy) {
					if(isSpecialTreatedApplicationObject(applicationConnection) == false) {
						// atl.angemeldeteApplikation
						final Data listEntry = subscribedApplications.getItem(dataIndex);

						final SystemObject subscribedApplicationSystemObject = dataModel.getObject(applicationConnection.getId());
						listEntry.getItem("applikation").asReferenceValue().setSystemObject(subscribedApplicationSystemObject);

						final SystemObject subscribedUserSystemObject = dataModel.getObject(applicationConnection.getRemoteUserId());
						listEntry.getItem("benutzer").asReferenceValue().setSystemObject(subscribedUserSystemObject);

						listEntry.getItem("seit").asTimeValue().setMillis(applicationConnection.getConnectionCreatedTime());

						listEntry.getItem("sendepufferzustand").asTextValue().setText(applicationConnection.getSendBufferState());

						dataIndex++;
					}
					else {
						// Dieses Applikationsobjekt darf nicht mit verschickt werden. Also das ursprngliche Array verkleinern.
						subscribedApplications.setLength(subscribedApplications.getLength() - 1);
					}
				} // for, ber alle angemeldeten Applikationen

				resultData = new ResultData(_davObject, _applicationUpdaterDataDescription, System.currentTimeMillis(), data);
				try {
					selfClientDavConnection.sendData(resultData);
				}
				catch(SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
                    String message = "Telegramm konnte wegen fehlender Sendesteuerung nicht versendet werden." +
                            " Dies kann hier wegen der Anmeldung als Quelle nicht vorkommen";
                    _debug.error(message, sendSubscriptionNotConfirmed);
                    throw new RuntimeException(message, sendSubscriptionNotConfirmed);
				}
			}
		}

		/** Verschickt ein Telegramm, das fr alle angemeldeten Applikationen die Telegrammlaufzeit vom DaV zur Applikation enthlt. */
		private void sendApplicationTelegramRuntimeUpdate() {
			if(_applicationRuntime.sendData() == true) {
				// Es gibt einen Empfnger fr die Daten

				final ArrayList<T_A_HighLevelCommunication> applicationConnections;
				applicationConnections = new ArrayList<T_A_HighLevelCommunication>(_applicationConnections);

				final Data data = selfClientDavConnection.createData(_applicationRuntimeDataDescription.getAttributeGroup());
				final Data.Array subscribedApplications = data.getItem("telegrammLaufzeit").asArray();
				subscribedApplications.setLength(applicationConnections.size());

				// Index, an dem ein neues Element in das Array eingefgt werden muss
				int dataIndex = 0;

				for(T_A_HighLevelCommunication applicationConnection : applicationConnections) {
					if(isSpecialTreatedApplicationObject(applicationConnection) == false) {
						// atl.telegrammLaufzeit
						final Data listEntry = subscribedApplications.getItem(dataIndex);

						final SystemObject subscribedApplicationSystemObject = dataModel.getObject(applicationConnection.getId());
						listEntry.getItem("applikation").asReferenceValue().setSystemObject(subscribedApplicationSystemObject);

						final int maxWaitingTime = 30000;
						long roundTripTime = 0;
						try {
							roundTripTime = applicationConnection.getTelegrammTime(maxWaitingTime);
							if(roundTripTime < 0) roundTripTime = maxWaitingTime;
						}
						catch(CommunicationError communicationError) {
							roundTripTime = maxWaitingTime;
							_debug.warning(
									"Fehler bei der Ermittlung der Telegrammlaufzeit: betroffene Applikation: "
									+ dataModel.getObject(applicationConnection.getId())
							);
						}
						listEntry.getItem("laufzeit").asUnscaledValue().set(roundTripTime);
						dataIndex++;
					}
					else {
						// Dieses Applikationsobjekt darf nicht mit verschickt werden. Also das ursprngliche Array verkleinern.
						subscribedApplications.setLength(subscribedApplications.getLength() - 1);
					}
				} // for, ber alle angemeldeten Applikationen

				final ResultData resultData = new ResultData(_davObject, _applicationRuntimeDataDescription, System.currentTimeMillis(), data);
				try {
					selfClientDavConnection.sendData(resultData);
				}
				catch(SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
                    String message = "Telegramm konnte wegen fehlender Sendesteuerung nicht versendet werden." +
                            " Dies kann hier wegen der Anmeldung als Quelle nicht vorkommen";
                    _debug.error(message, sendSubscriptionNotConfirmed);
                    throw new RuntimeException(message, sendSubscriptionNotConfirmed);
				}
			}
		}

		/**
		 * Verschickt einen Datensatz mit der ATG "atg.angemeldeteDatenidentifikationen".
		 *
		 * @param application Applikation, deren angemeldete Datenidentifiaktionen propagiert werden sollen.
		 */
		private void sendDataDescriptionUpdate(T_A_HighLevelCommunication application) {
			SourceApplicationUpdater updater = null;
			synchronized(_subcribedDataIdentifications) {
				// Dieses Objekt hat die Sendesteuerung
				updater = _subcribedDataIdentifications.get(application);
			}
			if(updater != null) {
				if(updater.sendData() == true) {
					// Es gibt Empfnger fr die Daten

					// Dieses Objekt besitzt alle Anmeldungen (Sender/Empfnger) und die entsprechenden Rollen(Quelle,Senke,Sender,Empfnger)
					final SubscriptionComponent subscriptionComponent = application.getSubscriptionComponent();

					// T_T sollte nicht gebraucht werden, da nur angemeldete Applikationen auf dem neusten Stand gehalten werden sollen.
					if(subscriptionComponent instanceof T_A_SubscriptionComponent) {

						final Data data = selfClientDavConnection.createData(_applicationDataDescriptionDD.getAttributeGroup());
						final Data.Array subscribedApplications = data.getItem("angemeldeteDatenidentifikation").asArray();

						// Alle angemldeten Datenidentifikation der Applikation anfordern

						final T_A_SubscriptionComponent t_a_subscriptionComponent = (T_A_SubscriptionComponent)subscriptionComponent;

						// Alle Sendeanmeldungen (Quelle oder Sender)
						final List<SendSubscriptionInfo> allSendingSubscriptions = t_a_subscriptionComponent.getSendingSubscriptions();
						final List<ReceiveSubscriptionInfo> allReceivingSubscription = t_a_subscriptionComponent.getReceivingSubscription();

						subscribedApplications.setLength(allSendingSubscriptions.size() + allReceivingSubscription.size());

						// Index, an dem ein neues Element in das Array eingefgt werden muss
						int dataIndex = 0;

						// Alle Anmeldungen der Applikation eintrage, die Daten empfangen.
						for(ReceiveSubscriptionInfo receiveSubscriptionInfo : allReceivingSubscription) {

							// Es handelt sich um Anmeldungen, die Daten empfangen. Handelt es sich um eine Senke oder einen Empfnger?

							// Den nchsten Eintrag aus der Liste whlen
							final Data listEntry = subscribedApplications.getItem(dataIndex);
							dataIndex++;

							final String role;
							if(receiveSubscriptionInfo.isDrain() == true) {
								role = "Senke";
							}
							else {
								role = "Empfnger";
							}

							inscribeDataDescription(listEntry, receiveSubscriptionInfo.getBaseSubscriptionInfo(), role);
						}// for ber alle Empfangsanmeldungen

						for(SendSubscriptionInfo sendingSubscription : allSendingSubscriptions) {
							// Es handelt sich um eine Anmeldung, die Daten verschickt. Sender oder Quelle ?

							// Den nchsten Eintrag aus der Liste whlen
							final Data listEntry = subscribedApplications.getItem(dataIndex);
							dataIndex++;

							final String role;
							if(sendingSubscription.isSource() == true) {
								role = "Quelle";
							}
							else {
								role = "Sender";
							}

							inscribeDataDescription(listEntry, sendingSubscription.getBaseSubscriptionInfo(), role);
						}// for ber alle Sendeanmeldungen

						// Es sind alle Anmeldungen am Data vermerkt, der Datensatz kann verschickt werden.

						final SystemObject systemObjectApplication = dataModel.getObject(application.getId());

						final ResultData resultData = new ResultData(
								systemObjectApplication, _applicationDataDescriptionDD, System.currentTimeMillis(), data
						);

						try {
							selfClientDavConnection.sendData(resultData);
						}
						catch(SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
							String message = "Telegramm konnte wegen fehlender Sendesteuerung nicht versendet werden." +
                            " Dies kann hier wegen der Anmeldung als Quelle nicht vorkommen";
							_debug.error(message, sendSubscriptionNotConfirmed);
							throw new RuntimeException(message, sendSubscriptionNotConfirmed);
						}
					}
				}
			}
		}

		/**
		 * Schreibt in den bergebenen Datensatz(atl.angemeldeteDatenidentifikation) alle Daten bentigten Daten.
		 *
		 * @param data                 Datensatz (atl.angemeldeteDatenidentifikation)
		 * @param baseSubscriptionInfo Enthlt das SystemObject(Id), die verwendete ATGV (Id) und die Simulationsvariante
		 * @param role                 Verwendete Rolle, siehe auch att.rolleAnmeldung. Zulssige Werte: Quelle, Sender, Senke, Empfnger
		 */
		private void inscribeDataDescription(
				final Data data, final BaseSubscriptionInfo baseSubscriptionInfo, final String role) {
			final SystemObject subscribedObject = dataModel.getObject(baseSubscriptionInfo.getObjectID());
			data.getItem("objekt").asReferenceValue().setSystemObject(subscribedObject);

			final AttributeGroupUsage usage = dataModel.getAttributeGroupUsage(baseSubscriptionInfo.getUsageIdentification());
			data.getItem("attributgruppenverwendung").asReferenceValue().setSystemObject(usage);

			data.getItem("simulationsvariante").asUnscaledValue().set(baseSubscriptionInfo.getSimulationVariant());

			data.getItem("rolle").asTextValue().setText(role);
		}

		/**
		 * Erstellt einen TimerTask, der alls 60 Sekunden alle angemeldeten Applikationen und deren Telegrammlaufzeiten verschickt (wenn ein Empfnger angemeldet
		 * ist).
		 */
		private void createPeriodicUpdateTask() {
			final TimerTask task = new TimerTask() {
				public void run() {
					try {
						sendApplicationUpdate();
					}
					catch(Exception e) {
						_debug.warning("Fehler beim Versand der Statusinformation mit angemeldeten Applikationen", e);
						return;
					}
					try {
						sendApplicationTelegramRuntimeUpdate();
					}
					catch(Exception e) {
						_debug.warning("Fehler beim Versand der Statusinformation mit den Telegrammlaufzeiten der Applikationen", e);
					}
				}
			};
			// In 60 Sekunden starten, alle 60 Sekunden wiederholen
			_timer.schedule(task, 60000, 60000);
		}

		/**
		 * Meldet fr eine neue Applikation eine Quelle an, die Datenstze vom mit der ATG "atg.angemeldeteDatenidentifikationen" verschickt.
		 * <p/>
		 * Gleichzeitig wird das Objekt in alle bentigten Datenstrukturen eingetragen.
		 * <p/>
		 * Hat bereits eine Anmeldung stattgefunden, wird nichts gemacht (zum Beispiel, wenn weiter DataDescription fr eine Applikation angemeldet werden).
		 *
		 * @param newApplication enthlt alle Daten zum anmelden.
		 */
		private void subscribeDataDescriptionSource(T_A_HighLevelCommunication newApplication) {
			synchronized(_subcribedDataIdentifications) {
				if(_subcribedDataIdentifications.containsKey(newApplication) == false) {
					// Die besonderen Applikationen(Konfiguration, Objekt, das den DaV darstellt) verschicken ihren Status nicht
					if(isSpecialTreatedApplicationObject(newApplication) == false) {
						// Bisher hat keine Anmeldung stattgefunden.
						final AttributeGroup attributeGroup = dataModel.getAttributeGroup(_pidAngemeldeteDatenidentifikationen);
						final Aspect aspect = dataModel.getAspect("asp.standard");

						final DataDescription dataDescription = new DataDescription(attributeGroup, aspect);

						final SystemObject applicationObject = dataModel.getObject(newApplication.getId());

						final SourceApplicationUpdater sourceApplicationUpdater = new SourceApplicationUpdater(this);

						try {
							selfClientDavConnection.subscribeSender(sourceApplicationUpdater, applicationObject, dataDescription, SenderRole.source());
							_subcribedDataIdentifications.put(newApplication, sourceApplicationUpdater);
						}
						catch(OneSubscriptionPerSendData oneSubscriptionPerSendData) {
							_debug.error(
									"Fr eine Applikation kann keine Quelle angemeldet werden, die alle angemeldeten Datenidentifikationen der Applikation publiziert.",
									oneSubscriptionPerSendData
							);
						}
					}
				}
			}
		}

		/**
		 * Meldet die Datenstze der ATG "atg.angemeldeteDatenidentifikationen" wieder ab und entfernt die Objekte aus allen Datenstrukturen. Wurden die Daten bereits
		 * abgemeldet wird nichts gemacht.
		 * <p/>
		 * Eventuell angemeldete TimerTasks werden unterbrochen und entfernt.
		 *
		 * @param removedApplication Objekt, deren Quellenanmeldung zurckgenommen werden soll.
		 */
		private void unsubscribeDataDescriptionSource(T_A_HighLevelCommunication removedApplication) {
			SourceApplicationUpdater updater = null;
			synchronized(_subcribedDataIdentifications) {
				updater = _subcribedDataIdentifications.remove(removedApplication);
			}
			if(updater != null) {
				// Das Objekt war noch vorhanden, also knnen die Daten abgemeldet werden.
				final AttributeGroup attributeGroup = dataModel.getAttributeGroup(_pidAngemeldeteDatenidentifikationen);
				final Aspect aspect = dataModel.getAspect("asp.standard");

				final DataDescription dataDescription = new DataDescription(attributeGroup, aspect);

				final SystemObject applicationObject = dataModel.getObject(removedApplication.getId());

				selfClientDavConnection.unsubscribeSender(updater, applicationObject, dataDescription);
			}

			// Falls es noch einen TimerTask gibt, der die angemeldeten Datenidentifikationen verschicken will, wird dieser beendet.
			// Dies ist ntig, weil die Applikation sich vom DaV abgemeldet hat.
			synchronized(_threadsForDataIdentificationUpdates) {
				final TimerTask taskForDataIdentificationUpdate = _threadsForDataIdentificationUpdates.remove(removedApplication);
				if(taskForDataIdentificationUpdate != null) {
					taskForDataIdentificationUpdate.cancel();
				}
			}
		}

		public void run() {

			// Alle angemeldeten Applikationen und die Telegrammlaufzeit werden zyklisch alle 60 Sekunden verschickt.
			createPeriodicUpdateTask();

			while(isInterrupted() == false) {
				final ApplicationConnection newApplicationDetected;
				synchronized(_applicationConnections) {
					// Solange warten, bis es eine Applikation gibt, die bearbeitet werden muss
					while(_application.size() == 0) {
						// Wenn <code>_applicationConnections</code> sich ndert, wird in _application die Applikation abgelegt, die gendert wurde.
						try {
							_applicationConnections.wait();
						}
						catch(InterruptedException e) {
							_debug.error("Der Thread wurde mit Interrupt beendet", e);
							// Da der Thread beendet wurde, wird die while-Schleife verlassen ohne Daten zu verschicken (vielleicht gibt es zu diesem
							// Zeitpunkt auch nichts, was verschickt werden soll).
							return;
						}
					} // while, nichts zu tun
					//Fifo
					newApplicationDetected = _application.remove(0);
				} // synch neue Applikationen

				final T_A_HighLevelCommunication newApplicationConnection = newApplicationDetected.getApplicationConnection();

				if(newApplicationDetected.isAdded()) {
					// 1) Es wurde eine Applikation hinzugefgt -> Quelle fr die angemeldetenDatenidentifikationen der Applikation anmelden.
					subscribeDataDescriptionSource(newApplicationConnection);
				}
				else {
					// 1) Quelle fr angemeldetenDatenidentifikationen dieser Applikation abmelden und den TimerTask beenden(falls vorhanden)
					unsubscribeDataDescriptionSource(newApplicationConnection);
				}

				// Alle benachrichtigen, dass eine Applikation hinzugefgt oder entfernt wurde
				sendApplicationUpdate();
			}
		}

		/**
		 * Fgt eine neue Applikation den Datenstrukturen hinzu und der Thread, der Aktualisierungsdatenstze verschickt, wird aufgeweckt.
		 *
		 * @param applicationConnection Neue Applikation
		 */
		public void applicationAdded(final T_A_HighLevelCommunication applicationConnection) {
			synchronized(_applicationConnections) {
				_application.add(new ApplicationConnection(applicationConnection, true));
				// Es wurde eine Applikation hinzugefgt, der Thread muss aufwachen und ein Telegramm verschicken
				_applicationConnections.notifyAll();
			}
		}

		/**
		 * Speichert die entfernte Applikation und weckt den wartende Thread auf, der daraufhin eine Datensatz mit den aktuell angemeldeten Applikationen verschickt.
		 *
		 * @param applicationConnection Applikation, die entfernt wurde
		 */
		public void applicationRemoved(final T_A_HighLevelCommunication applicationConnection) {
			synchronized(_applicationConnections) {
				_application.add(new ApplicationConnection(applicationConnection, false));
				// Es wurde eine Applikation entfernt, der Thread muss aufwachen und ein Telegramm verschicken
				_applicationConnections.notifyAll();
			}
		}

		/**
		 * Diese Methode wird aufgerufen, wenn eine Applikation eine neue Datenidentifikation anmeldet.
		 * <p/>
		 * Fr diese Applikation wird ein Datensatz verschickt, der alle angemeldeten Datenidentifikationen enthlt.
		 *
		 * @param application Applikation, die neue Daten anmeldet.
		 */
		public void applicationSubscribedNewConnection(final T_A_HighLevelCommunication application) {
			publishConnectionChanged(application);
		}

		/**
		 * Diese Methode wird aufgerufen, wenn eine Applikation eine  Datenidentifikation abmeldet.
		 * <p/>
		 * Fr diese Applikation wird ein Datensatz verschickt, der alle angemeldeten Datenidentifikationen enthlt.
		 *
		 * @param application Applikation, die eine Datenidentifikation abmeldet.
		 */
		public void applicationUnsubscribeConnection(final T_A_HighLevelCommunication application) {
			publishConnectionChanged(application);
		}

		/**
		 * Wird aufgerufen, wenn eine Applikation einen Datenidentifikation an/abmeldet. Es wird ein TimerTask angelegt, der nach einer gewissen Zeit alle
		 * angemeldeten Datenidentifikationen der bergebene Applikation publiziert. Gibt es bereits einen TimerTask, der diese Aufgabe bernehmen mchte, so wird
		 * dieser gestoppt und durch einen neuen TimerTask ersetzt, der wieder eine gewisse Zeit wartet.
		 * <p/>
		 * Dadruch wird verhindert, dass sehr viele Anmeldungen eine Flut von Update-Telegrammen auslst. Jede neue Anmeldung verzgert das Update-Telegramm, erst
		 * wenn alle Anmeldungen durchgefhrt wurden, wird das Telegramm verschickt.
		 *
		 * @param application Applikation, die neue Daten an oder abmeldet.
		 */
		private void publishConnectionChanged(final T_A_HighLevelCommunication application) {
			synchronized(_threadsForDataIdentificationUpdates) {
				final TimerTask taskForThisApplication = _threadsForDataIdentificationUpdates.get(application);

				if(taskForThisApplication != null) {
					// Es gibt bereits einen Task, dieser kann bereits abgelaufen sein oder sich noch in Bearbeitung befinden.
					// (Auch wenn der Task bereits ausgefhrt wurde, schadet dieser Aufruf nicht)
					taskForThisApplication.cancel();
				}

				// Es muss ein neuer Task angelegt werden
				final TimerTask task = new TimerTask() {
					public void run() {
						try {
							sendDataDescriptionUpdate(application);
						}
						catch(Exception e) {
							_debug.warning("Probleme beim Versenden von Zustandsinformation ber angemeldete Datenidentifikationen", e);
						}
					}
				};
				_threadsForDataIdentificationUpdates.put(application, task);
				// In 5 Sekunden den Task starten, findet eine weitere nderung statt, wird dieser Task mit cancel gestoppt und ein neuer angelegt
				_timer.schedule(task, 5000);
			}
		}
	}

	private static final class ApplicationConnection {

		final boolean _added;

		final T_A_HighLevelCommunication _applicationConnection;

		/**
		 * @param applicationConnection Verbindung der Applikation
		 * @param added                 <code>true</code>, wenn die Verbindung zum Datenverteiler hinzugefgt wurde. <code>false</code>, sonst.
		 */
		public ApplicationConnection(final T_A_HighLevelCommunication applicationConnection, final boolean added) {
			_applicationConnection = applicationConnection;
			_added = added;
		}


		public boolean isAdded() {
			return _added;
		}

		public T_A_HighLevelCommunication getApplicationConnection() {
			return _applicationConnection;
		}
	}

	private final class SourceApplicationUpdater implements ClientSenderInterface {

		private byte _state = ClientSenderInterface.STOP_SENDING;

		private final ApplicationStatusUpdater _sender;


		public SourceApplicationUpdater(final ApplicationStatusUpdater sender) {
			_sender = sender;
		}

		public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
			synchronized(this) {

				if(_state != ClientSenderInterface.START_SENDING && state == ClientSenderInterface.START_SENDING) {
					// Der Zustand wird von "nicht senden" auf "senden" gendert -> Daten verschicken
					if(dataDescription.getAttributeGroup().getPid().equals(_pidAngemeldeteApplikationen)) {
						final Thread helper = new Thread("StatusinfoversandApplikationen") {
							public void run() {
								try {
									_sender.sendApplicationUpdate();
								}
								catch(Exception e) {
									_debug.warning("Fehler beim Versand der Statusinformation bzgl. der angemeldeten Applikationen", e);
								}
							}
						};
						helper.setDaemon(true);
						helper.start();
					}
					else if(dataDescription.getAttributeGroup().getPid().equals(_pidTelegrammLaufzeiten)) {
						final Thread helper = new Thread("StatusinfoversandTelegrammlaufzeiten") {
							public void run() {
								try {
									_sender.sendApplicationTelegramRuntimeUpdate();
								}
								catch(Exception e) {
									_debug.warning("Fehler beim Versand der Statusinformation bzgl. der Telegrammlaufzeiten", e);
								}
							}
						};
						helper.setDaemon(true);
						helper.start();
					}
					else if(dataDescription.getAttributeGroup().getPid().equals(_pidAngemeldeteDatenidentifikationen)) {
						// Die beteiligte T_A_HighLevel finden
						final T_A_HighLevelCommunication application = getApplicationConnection(object.getId());
						// Abfrage, falls die Application entfernt wurde
						if(application != null) {
							final Thread helper = new Thread("StatusinfoversandAnmeldungen") {
								public void run() {
									try {
										_sender.sendDataDescriptionUpdate(application);
									}
									catch(Exception e) {
										_debug.warning("Fehler beim Versand der Statusinformation bzgl. der Anmeldungen einer Applikation", e);
									}
								}
							};
							helper.setDaemon(true);
							helper.start();
						}
					}
				}
				_state = state;
			}
		}

		public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
			return true;
		}

		public boolean sendData() {
			synchronized(this) {
				if(_state == ClientSenderInterface.START_SENDING) {
					return true;
				}
				else {
					return false;
				}
			}
		}
	}
}
